#include "stdafx.h"
#include <aygshell.h>  // New in PocketPC version

//#define STRICT
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include "resource.h"
#include "seislog.h"
#include "library.h"
#include "process.h"
#include "devices.h"
#include "eventlog.h"

#define WINCE

struct SIDB {
   DEVICE_INFO *Devices;
   CHAR NumDevices;
};


//
// Prototype definitions of functions used only from this file..
//
LRESULT CALLBACK AddDeviceDlgProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK DeviceInfoDlgProc(HWND, UINT, WPARAM, LPARAM);
HANDLE StartDevice(PTCHAR, DWORD, BOOL, WORD, DWORD*);
BOOL RemoveDevice(PTCHAR);




//
// Theese are global variables used by functions in this file
//
extern HINSTANCE hInst;
extern DWORD MainThreadID;
extern DWORD ProcKeeperThreadID;
extern DWORD SysClkSyncThreadID;
extern DEVICE_INFO Devices[];
extern CHAR NumRegisteredDevices;
extern TCHAR DeviceDir[];



//
// Structure used by functions in this file
//
struct DEV_CNF_FILE_HDR {
   WORD DeviceID;             // An identifier for this particular model.
   BYTE NumChannels;          // The number of channels the device support.
};







/*******************************************************************/
/*    Dialog box procedure for the 'Administer devices' dialog.    */
/*******************************************************************/
LRESULT CALLBACK AdministerDevicesDlgProc(HWND hWndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
#ifdef WINCE
   SHINITDLGINFO shidi; // New in PocketPC version
#endif
   HWND hWndDlgItem;
   TCHAR Index, string85[86];
   TCHAR MessageText[100];
   INT Status;
   DWORD dwIndex;


   switch(message)  {
      case WM_INITDIALOG:
#ifdef WINCE
	 // Create a Done button and size it (new in PocketPC version).
	 shidi.dwMask = SHIDIM_FLAGS;
	 shidi.dwFlags = SHIDIF_DONEBUTTON | SHIDIF_SIPDOWN | SHIDIF_SIZEDLGFULLSCREEN;
	 shidi.hDlg = hWndDlg;
	 SHInitDialog(&shidi);
#endif
      // Initialize the 'Registered Devices' list box.
      hWndDlgItem=GetDlgItem(hWndDlg, IDC_LIST1);
      for (Index=0;Index<NumRegisteredDevices;Index++) {
         wcscpy(string85, Devices[Index].UniqueDeviceName);
         dwIndex=SendMessage(hWndDlgItem, LB_ADDSTRING, 0, (LPARAM)(LPCTSTR)string85);
         SendMessage(hWndDlgItem, LB_SETITEMDATA, (WPARAM)dwIndex, (LPARAM)Devices[Index].DeviceID);
         if (Index==0) SendMessage(hWndDlgItem, LB_SETCURSEL, dwIndex, 0);
      }

      // Disable buttons if needed.
      if (!NumRegisteredDevices) {
         EnableWindow(GetDlgItem(hWndDlg, IDC_BUTTON2), FALSE); 
         EnableWindow(GetDlgItem(hWndDlg, IDC_BUTTON3), FALSE); 
      }
#ifndef WINCE
      // Bring window to front.
      BringWindowToTop(hWndDlg);
#endif
      return TRUE; 

      case WM_COMMAND:
      switch(LOWORD(wParam)) {
         case IDC_BUTTON1:
         //
         // This is the 'Add' button. The user wants to add a new device.
         //

         // Check that there are room for more devices.
         if (NumRegisteredDevices==MAX_NUM_OF_DEVICES) {
            wcscpy(MessageText, TEXT("Cant add any more devices.\nThe program limit has been reached."));
            MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONHAND);
            return FALSE;
         }


         // Transfer control to the 'Add new device' dialog box.
         Status=DialogBox(hInst, MAKEINTRESOURCE(IDD_ADD_DEVICE), hWndDlg, (DLGPROC)AddDeviceDlgProc);      

         // If user has added a new device, update the list box.
         if (Status==1) { 
            // Get handle to listbox.
            hWndDlgItem=GetDlgItem(hWndDlg, IDC_LIST1);

            // Remove existing contents.
            SendMessage(hWndDlgItem, LB_RESETCONTENT , 0, 0);

            // Update the 'Registered Devices' list box.
            for (Index=0;Index<NumRegisteredDevices;Index++) {
               wcscpy(string85, Devices[Index].UniqueDeviceName);
               dwIndex=SendMessage(hWndDlgItem, LB_ADDSTRING, 0, (LPARAM)(LPCTSTR)string85);
               SendMessage(hWndDlgItem, LB_SETITEMDATA, (WPARAM)dwIndex, (LPARAM)Devices[Index].DeviceID);
               if (Index==0) SendMessage(hWndDlgItem, LB_SETCURSEL, dwIndex, 0);
            }

            // Inform user that it is nessesarry start and stop
            // the system for the changes to take effect.
            if (IsSystemRunning()) {
               wcscpy(MessageText, TEXT("You must restart the system for changes to take effect."));
               MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONINFORMATION);
            }

            // Enable buttons.
            if (NumRegisteredDevices) {
               EnableWindow(GetDlgItem(hWndDlg, IDC_BUTTON2), TRUE); 
               EnableWindow(GetDlgItem(hWndDlg, IDC_BUTTON3), TRUE); 
            }
         }
         break;
         
         case IDC_BUTTON2:
         //
         // This is the 'Remove' button. The user wants to remove a device.
         //

         // Check that there are any devices to remove.
         if (!NumRegisteredDevices) {
            MessageBeep(MB_ICONHAND);
            return FALSE;
         }

         // Get selection from list box. (Get the device description).
         hWndDlgItem=GetDlgItem(hWndDlg, IDC_LIST1);
         dwIndex=SendMessage(hWndDlgItem, LB_GETCURSEL, 0, 0);
         SendMessage(hWndDlgItem, LB_GETTEXT, (WPARAM)dwIndex, (LPARAM)(LPCTSTR)string85);

         // User must confirm that he want to remove the device.
         wcscpy(MessageText, TEXT("Are you sure you want to remove the device '"));
         wcscat(MessageText, string85);
         wcscat(MessageText, TEXT("' ?"));
         Status=MessageBox(hWndDlg, MessageText, TEXT("Seislog"), MB_ICONQUESTION|MB_YESNO|MB_DEFBUTTON2);
         if (Status==IDYES) { 
          
            // Remove the device.
            RemoveDevice(string85);

            // Update the 'Registered Devices' list box.
            SendMessage(hWndDlgItem, LB_RESETCONTENT , 0, 0);
            for (Index=0;Index<NumRegisteredDevices;Index++) {
               wcscpy(string85, Devices[Index].UniqueDeviceName);
               dwIndex=SendMessage(hWndDlgItem, LB_ADDSTRING, 0, (LPARAM)(LPCTSTR)string85);
               SendMessage(hWndDlgItem, LB_SETITEMDATA, (WPARAM)dwIndex, (LPARAM)Devices[Index].DeviceID);
               if (Index==0) SendMessage(hWndDlgItem, LB_SETCURSEL, dwIndex, 0);
            }

            // Inform user that it is nessesarry to start and
            // stop the system for the changes to take effect.
            if (IsSystemRunning()) {
               wcscpy(MessageText, TEXT("You must restart the system for changes to take effect."));
               MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONINFORMATION);
            }

            // Disable buttons if needed.
            if (!NumRegisteredDevices) {
               EnableWindow(GetDlgItem(hWndDlg, IDC_BUTTON2), FALSE); 
               EnableWindow(GetDlgItem(hWndDlg, IDC_BUTTON3), FALSE); 
            }
         }
         break;

         case IDC_BUTTON3:
         //
         // This is the 'Deviceinfo' button.
         //

         // Get selection from list box. (Get the device description).
         hWndDlgItem=GetDlgItem(hWndDlg, IDC_LIST1);
         dwIndex=SendMessage(hWndDlgItem, LB_GETCURSEL, 0, 0);
         SendMessage(hWndDlgItem, LB_GETTEXT, (WPARAM)dwIndex, (LPARAM)(LPCTSTR)string85);

         // Transfer control to the 'Device Info' dialog box.
         Status=DialogBoxParam(hInst, MAKEINTRESOURCE(IDD_DEVICEINFO), hWndDlg, (DLGPROC)DeviceInfoDlgProc, (LPARAM)string85);
         break;

         case IDOK: // Changed in PocketPC version. ID is IDC_BUTTON4 in Win9X version.
         // This is the 'Done' button ('Close' button in Win9X version).
         // Close the dialog box
         EndDialog(hWndDlg , 0);
         break;
      }
      return FALSE;
   }
   return FALSE;
}





/*******************************************************************/
/* Dialog box procedure for the 'Add new device' dialog.           */
/*******************************************************************/
LRESULT CALLBACK AddDeviceDlgProc(HWND hWndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
#ifdef WINCE
   SHINITDLGINFO shidi; // New in PocketPC version
#endif
   TCHAR WorkDir[MAX_PATH], Directory[MAX_PATH];
   BYTE DevIndex;
   HWND hWndDlgItem;
   WORD DeviceID;
   DWORD dwIndex, ThreadID;
   HANDLE hEvent, hThread;
   MSG_PARAM MsgParam;

   switch(message)  {
      case WM_INITDIALOG:
#ifdef WINCE
	  // Size dialog (new in PocketPC version).
	  shidi.dwMask = SHIDIM_FLAGS;
	  shidi.dwFlags = SHIDIF_SIPDOWN | SHIDIF_SIZEDLGFULLSCREEN;
	  shidi.hDlg = hWndDlg;
	  SHInitDialog(&shidi);
#endif
      //
      // Initialize the 'Registered Devices' list box. All devices that
      // are supported must be filled in here, with manufacturer name and 
      // model name. The DeviceID for the device must also be inserted as ITEMDATA.
      //
      hWndDlgItem=GetDlgItem(hWndDlg, IDC_LIST1);
      dwIndex=SendMessage(hWndDlgItem, LB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("Internal Wave Generator"));
      SendMessage(hWndDlgItem, LB_SETITEMDATA, (WPARAM)dwIndex, (LPARAM)10);
      dwIndex=SendMessage(hWndDlgItem, LB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("Earth Data Ltd - EDM006"));
      SendMessage(hWndDlgItem, LB_SETITEMDATA, (WPARAM)dwIndex, (LPARAM)100);
      dwIndex=SendMessage(hWndDlgItem, LB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("Earth Data Ltd - PS2400"));
      SendMessage(hWndDlgItem, LB_SETITEMDATA, (WPARAM)dwIndex, (LPARAM)105);
      dwIndex=SendMessage(hWndDlgItem, LB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("GeoSIG AG - GBVx16"));
      SendMessage(hWndDlgItem, LB_SETITEMDATA, (WPARAM)dwIndex, (LPARAM)200);
	  dwIndex=SendMessage(hWndDlgItem, LB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("SeisProd - SeisAD18"));
      SendMessage(hWndDlgItem, LB_SETITEMDATA, (WPARAM)dwIndex, (LPARAM)210);
      dwIndex=SendMessage(hWndDlgItem, LB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("Nanometrics - RD3/RD6"));
      SendMessage(hWndDlgItem, LB_SETITEMDATA, (WPARAM)dwIndex, (LPARAM)300);
	  dwIndex=SendMessage(hWndDlgItem, LB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("Kinemetrics - K2"));
      SendMessage(hWndDlgItem, LB_SETITEMDATA, (WPARAM)dwIndex, (LPARAM)500);
	  dwIndex=SendMessage(hWndDlgItem, LB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("M.Mariotti - SADC10/18/20"));
      SendMessage(hWndDlgItem, LB_SETITEMDATA, (WPARAM)dwIndex, (LPARAM)510);
#ifndef WINCE
      dwIndex=SendMessage(hWndDlgItem, LB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("Nanometrics - HRD24"));
      SendMessage(hWndDlgItem, LB_SETITEMDATA, (WPARAM)dwIndex, (LPARAM)310);
      dwIndex=SendMessage(hWndDlgItem, LB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("ComputerBoards - CIO-DAS6402/16"));
      SendMessage(hWndDlgItem, LB_SETITEMDATA, (WPARAM)dwIndex, (LPARAM)400);
#endif
      SendMessage(hWndDlgItem, LB_SETCURSEL, 0, 0);

      // Bring window to front.
      BringWindowToTop(hWndDlg);
      return TRUE; 


      case WM_COMMAND:
      switch(LOWORD(wParam)) {
         case IDC_CONFIGURE:
         // 
         // This is the 'Configure' button. The user wants to add a new device.
         //

         // Get selection from list box (Get the device-id).
         hWndDlgItem=GetDlgItem(hWndDlg, IDC_LIST1);
         dwIndex=SendMessage(hWndDlgItem, LB_GETCURSEL, 0, 0);
         DeviceID=(WORD)SendMessage(hWndDlgItem, LB_GETITEMDATA, (WPARAM)dwIndex, 0);

         // Create a device-directory for this device-instance.
         // The directory will be called 'New Device' 
         // because we do not know the unique device name yet.
         wcscpy(WorkDir, DeviceDir);
         StripBackslashFromPath(WorkDir);
         wcscat(WorkDir, TEXT("\\New Device"));
         CreateDirectory(WorkDir, NULL);

         // Start the device-manager up in cofigure mode, 
         // using the newly created directory as it's working directory.
         // The thread-id for the thread is returned in the ThreadID parameter.
         hThread=StartDevice(WorkDir, MainThreadID, TRUE, DeviceID, &ThreadID);


         // Send 'TM_CONFIGURE' message to the device manager.
         // This will make the device-manager open it's configure 
         // dialogbox, thereby allowing the user to configure the device.
         hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
         PostThreadMessage(ThreadID, TM_CONFIGURE, (WPARAM)hEvent, (LPARAM)&MsgParam); 
         WaitForSingleObject(hEvent, INFINITE); // Wait for the user to finish before continuing.
         CloseHandle(hEvent);

		 // Activate window so the (OK) button is displayed
		 SetForegroundWindow(hWndDlg);


         //
         // If the user pressed the OK button in the device-manager
         // window, then a 'device.cnf' file should now exist in
         // the work directory (ex: c:\\seislog\\devices\\New Device).
         //

         // Check the return values to see if the user has cancelled the operation.
         if (MsgParam.bStatus==FALSE) {
            //
            // The user has cancelled the operation.
            //
            
            // Stop the device manager thread.
            PostThreadMessage(ThreadID, TM_EXIT, 0, 0);
            WaitForSingleObject(hThread, INFINITE); // Wait for thread to end.

            // Clean up and remove the temporary work-directory.
            ClearDirectory(WorkDir);
            RemoveDirectory(WorkDir);
         
            // Close the dialogbox and return zero.
            EndDialog(hWndDlg , 0);
            return FALSE;
         }

         
         // So far the user has finished the device configuration
         // dialog and has created a 'device.cnf' file in the
         // work directory.

         // Get a unique name for this instance of the device manager.
         // This string is supplied by the device manager. The string
         // will be returned in the MsgParam.cStatus member.
         hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
         PostThreadMessage(ThreadID, TM_GET_UNIQUE_DEVICE_NAME, (WPARAM)hEvent, (LPARAM)&MsgParam); 
         WaitForSingleObject(hEvent, INFINITE);
         CloseHandle(hEvent);

         // Stop the device manager thread.
         PostThreadMessage(ThreadID, TM_EXIT, 0, 0);
         WaitForSingleObject(hThread, INFINITE);

         // Check for conflicts with existing devices.
         if (ExistDevice(MsgParam.cStatus, &DevIndex)) {
            // A conflicting device with the same name already exist.
            // Inform user, and abort operation.
            TCHAR MsgText[150];
            wcscpy(MsgText, TEXT("Unable to add device.\n"));
            wcscat(MsgText, TEXT("A conflict with an existing device has been detected.\n"));
            wcscat(MsgText, TEXT("This may happen if two devices are identically configured."));
            MessageBox(hWndDlg, MsgText, TEXT("Seislog message"), MB_OK|MB_ICONHAND);
            
            // Clean up and remove the temporary work-directory.
            ClearDirectory(WorkDir);
            RemoveDirectory(WorkDir);
         
            // Close the dialogbox and return zero.
            EndDialog(hWndDlg , 0);
            return FALSE;
         }


         // Rename the work directory to it's correct name.
         wcscpy(Directory, DeviceDir);
         StripBackslashFromPath(Directory);
         wcscat(Directory, TEXT("\\"));
         wcscat(Directory, MsgParam.cStatus);
         MoveFile(WorkDir, Directory);
 
         // Recreate the 'Devices'table.
         RegisterDevices();

         // Close the dialog box and exit.
         EndDialog(hWndDlg , 1);
         break;
         
         case IDCANCEL:
         // This is the 'Cancel' button.
         // User wants to cancel the operation. Close the dialog box and exit.
         EndDialog(hWndDlg , 0);
         break;
      }
      return FALSE;
   }
   return FALSE;
}





/*******************************************************************/
/*    Dialog box procedure for the 'Device Info' dialog.           */
/*******************************************************************/
LRESULT CALLBACK DeviceInfoDlgProc(HWND hWndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
#ifdef WINCE
   SHINITDLGINFO shidi; // New in PocketPC version
#endif
   TCHAR string5[6], DeviceName[65];
   BYTE DevIndex;
   HANDLE hEvent;
   MSG_PARAM MsgParams;
   DEVMNGR_INTERF_STRUCT *DS;
   static DWORD DeviceThreadID;
   static BOOL DeviceStarted;

   switch(message)  {
      case WM_INITDIALOG:
#ifdef WINCE
	  // Size dialog (new in PocketPC version).
	 shidi.dwMask = SHIDIM_FLAGS;
	 shidi.dwFlags = SHIDIF_DONEBUTTON | SHIDIF_SIPDOWN | SHIDIF_SIZEDLGFULLSCREEN;
	 shidi.hDlg = hWndDlg;
	 SHInitDialog(&shidi);
#endif
	  // Get and set device name.
      wcscpy(DeviceName, (PTCHAR)lParam);
      SetDlgItemText(hWndDlg, IDC_STATIC1, DeviceName);

      // Make sure that the device-manager is running.
      DeviceStarted=FALSE;
      DeviceThreadID=GetProcessThreadIDByName(DeviceName);
      if (DeviceThreadID==0) {
         // Start the device.
         DeviceStarted=TRUE;
         if (ExistDevice(DeviceName, &DevIndex)) {
            DeviceThreadID=StartDeviceManager(FALSE, DevIndex);
         }
      }

      //
      // Get pointer to device-manager's interface structure.
      // 
      hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);      // Create an event object.
      PostThreadMessage(DeviceThreadID, TM_GET_INTERFACE, (WPARAM)hEvent, (LPARAM)&MsgParams); 
      WaitForSingleObject(hEvent, INFINITE); // Wait for the thread  to process the messsage.
      DS=(DEVMNGR_INTERF_STRUCT*)MsgParams.vPointer;
      CloseHandle(hEvent);

      // Set Manufacturer name.
      SetDlgItemText(hWndDlg, IDC_STATIC2, DS->Manufacturer);

      // Set Model.
      SetDlgItemText(hWndDlg, IDC_STATIC3, DS->Model);

      // Set Number of channels.
      SetDlgItemText(hWndDlg, IDC_STATIC4, _itow((INT)DS->NumChannels, string5, 10));

      // Set Sample rate.
      SetDlgItemText(hWndDlg, IDC_STATIC5, _itow((INT)DS->SampleRate, string5, 10));

      // Get and set resource string.
      hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);      // Create an event object.
      PostThreadMessage(DeviceThreadID, TM_GET_RESOURCE_STRING, (WPARAM)hEvent, (LPARAM)&MsgParams); 
      WaitForSingleObject(hEvent, INFINITE); // Wait for the thread  to process the messsage.
      SetDlgItemText(hWndDlg, IDC_STATIC6, MsgParams.cStatus);
      CloseHandle(hEvent);

      return TRUE; 

      case WM_COMMAND:
      switch(LOWORD(wParam)) {
         case IDC_MORE:
         // This is the 'More info' button.
         // Send TM_SHOW_CONFIG message to manager thread.
         PostThreadMessage(DeviceThreadID, TM_SHOW_CONFIG, 0, 0);
		 
		 //[PocketPC] 
		 MessageBox(hWndDlg,
			        TEXT("More information was\nwritten in Event Log"),
			        TEXT("Seislog message"),
					MB_OK | MB_ICONINFORMATION);
         break;


         case IDOK: // Changed in PocketPC version. ID is IDC_CLOSE in Win9X version.
         // This is the 'Done' button.   
         // If started, then stop device manager thread.
         if (DeviceStarted) {
            PostThreadMessage(DeviceThreadID, TM_EXIT, 0, 0); 
            while (PostThreadMessage(DeviceThreadID, TM_DUMMY, 0, 0)) Sleep(100);
         }

         // Close the dialog box
         EndDialog(hWndDlg , 0);
         break;
         
      }
      return FALSE;
   }
   return FALSE;
}







/*******************************************************************************/
/* Reads and load information from disk about all registered devices.          */
/* Save number of devices found in the global variable NumRegisteredDevices.   */
/*                                                                             */
/* [PocketPC] This function is paritially rewritten. In Win9X version,         */
/*            this function initially skips the '.' and '..' direcories.       */
/*            PocketPC does not have  the concept of current and previous      */
/*            directory, so the first file found will be the first device      */
/*******************************************************************************/
VOID RegisterDevices(VOID)
{
   // Browse through all the subdirectories in the device directory.
   // Register all directories that has a 'device.cnf' file as
   // a device, with name equal to the directory name. Directories
   // which do not has a 'device.cnf' are considered to be invalid,
   // and will therefore be deleted together with all existing files.

   TCHAR FileSpec[MAX_PATH+1], FileSpec2[MAX_PATH+1], Index; 
   TCHAR Message[100];
   HANDLE SearchHndl;
   DEV_CNF_FILE_HDR DeviceInfo;
   WIN32_FIND_DATA FileData;

   //
   // Clear/initialize the 'Devices' array.
   //
   for (Index=0;Index<MAX_NUM_OF_DEVICES;Index++) {
      Devices[Index].DeviceID=0;
      wcscpy(Devices[Index].UniqueDeviceName, TEXT(""));
   }
   NumRegisteredDevices=0;


   //
   // Register devices.
   //
   //
   // Build file-search-specification string.
   wcscpy(FileSpec, DeviceDir);
   StripBackslashFromPath(FileSpec);
   wcscat(FileSpec, TEXT("\\*.*"));

   // Find the first file/directory.

   SearchHndl=FindFirstFile(FileSpec, &FileData);
   if (SearchHndl==INVALID_HANDLE_VALUE) return;

	 if ((FileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)) {
	 // We have found a directory.
	 // Create a string containing the full path to this directory.
	 AddPathAndFilename(FileSpec, DeviceDir, FileData.cFileName);

	 // Check if the direcory contains a 'device.cnf' file.
	 if (ExistFile(FileSpec, TEXT("device.cnf"))) {
		if (NumRegisteredDevices<MAX_NUM_OF_DEVICES) {
		   // Register this device.
		   NumRegisteredDevices++;

		   // Get device-ID and number of channnels 
		   // for this device. This information should be 
		   // found at the start of the 'device.cnf' file.
		   wcscpy(FileSpec2, FileSpec);
		   wcscat(FileSpec2, TEXT("\\device.cnf"));
		   ReadFileToMemory(FileSpec2, &DeviceInfo, (DWORD)sizeof(DEV_CNF_FILE_HDR));
		   Devices[NumRegisteredDevices-1].DeviceID=DeviceInfo.DeviceID;
		   Devices[NumRegisteredDevices-1].NumChannels=DeviceInfo.NumChannels;

		   // Get the unique name (logical name) for this device.
		   wcscpy(Devices[NumRegisteredDevices-1].UniqueDeviceName, FileData.cFileName);
		}
	 } else {
		// Remove this directory.
		ClearDirectory(FileSpec);
		RemoveDirectory(FileSpec);

		// Update program log
		wcscpy(Message, TEXT("Seislog: No 'device.cnf' file found in '"));
		wcscat(Message, FileSpec);
		wcscat(Message, TEXT("'. Device has been deleted."));
		WriteSystemLog(Message, 2);

	 }
	 
	}


   // Find and process the rest of the directories.
   while (FindNextFile(SearchHndl, &FileData)) {
      if ((FileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)) {
         // We have found a directory.
         // Create a string containing the full path to this directory.
         AddPathAndFilename(FileSpec, DeviceDir, FileData.cFileName);
      
         // Check if the direcory contains a 'device.cnf' file.
         if (ExistFile(FileSpec, TEXT("device.cnf"))) {
            if (NumRegisteredDevices<MAX_NUM_OF_DEVICES) {
               // Register this device.
               NumRegisteredDevices++;

               // Get device-ID and number of channnels 
               // for this device. This information should be 
               // found at the start of the 'device.cnf' file.
               wcscpy(FileSpec2, FileSpec);
               wcscat(FileSpec2, TEXT("\\device.cnf"));
               ReadFileToMemory(FileSpec2, &DeviceInfo, (DWORD)sizeof(DEV_CNF_FILE_HDR));
               Devices[NumRegisteredDevices-1].DeviceID=DeviceInfo.DeviceID;
               Devices[NumRegisteredDevices-1].NumChannels=DeviceInfo.NumChannels;

               // Get the unique name (logical name) for this device.
               wcscpy(Devices[NumRegisteredDevices-1].UniqueDeviceName, FileData.cFileName);
            }
         } else {
            // Remove this directory.
            ClearDirectory(FileSpec);
            RemoveDirectory(FileSpec);

            // Update program log
            wcscpy(Message, TEXT("Seislog: No 'device.cnf' file found in '"));
            wcscat(Message, FileSpec);
            wcscat(Message, TEXT("'. Device has been deleted."));
            WriteSystemLog(Message, 2);

         }
      }
   }

   // Close the handle.
   FindClose(SearchHndl);

   return;  

}



/**************************************************************************/
/* Starts all device manager threads.                                     */
/**************************************************************************/
VOID LoadDevices(VOID)
{
   CHAR Index;
   TCHAR Message[100];
   DWORD ThreadID;



   // Check the parameter values.
   if (DeviceDir==(PTCHAR)NULL || MainThreadID==0) return;

   // If no devices is registered, then we have nothing to do.
   if (!NumRegisteredDevices) return;

   // Go trough the 'Devices' array and start all devices.
   for (Index=0;Index<NumRegisteredDevices;Index++) {

      // Start the device manager thread for this device.
      ThreadID=StartDeviceManager(FALSE, Index);
      if (ThreadID==0) {
         // Startup failed update the program log.
         wcscpy(Message, TEXT("Seislog: "));
         wcscat(Message, Devices[Index].UniqueDeviceName);
         wcscat(Message, TEXT(", device failed to start"));
         WriteSystemLog(Message, 3);
      }
   }

   return;
}



/**************************************************************************/
/* Send TM_START message to all device manager threads.                   */
/**************************************************************************/
VOID StartDevices(VOID)
{
   CHAR NumDevices, Index;
   HANDLE hEvent;
   MSG_PARAM MsgParam;
   DWORD DeviceManagers[MAX_NUM_OF_DEVICES];

   // Get a list of all device-managers running.
   MsgParam.vPointer=(PVOID)&DeviceManagers;
   hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
   PostThreadMessage(ProcKeeperThreadID, TM_GET_DEVICEMANAGERS, (WPARAM)hEvent, (LPARAM)&MsgParam); 
   WaitForSingleObject(hEvent, INFINITE);
   NumDevices=(char)MsgParam.dwStatus;
   CloseHandle(hEvent);

   // Send TM_START message to all threads.
   for (Index=0;Index<NumDevices;Index++) PostThreadMessage(DeviceManagers[Index], TM_START, 0, 0); 

}




/*******************************************************************************************/
/* Starts a device manager thread. Returns thread-id of started thread.                    */
/* 'Index' is a pointer into the 'Devices'array.                                           */
/*******************************************************************************************/
DWORD StartDeviceManager(BOOL ConfigureMode, TCHAR Index)
{
   static STARTINFO_DEVMANAGER SIDM;
   DWORD ThreadID;
   HANDLE ThreadHandle;
   TCHAR WorkDirectory[MAX_PATH+1];

   // Create full path to device manager's working directory.
   wcscpy(WorkDirectory, DeviceDir);
   StripBackslashFromPath(WorkDirectory);
   wcscat(WorkDirectory, TEXT("\\"));  
   wcscat(WorkDirectory, Devices[Index].UniqueDeviceName);  

   // Initialize the startupinfo structure for the device manager thread.
   SIDM.hInstance=hInst;
   SIDM.ThreadID=MainThreadID;
   SIDM.ProcKeeprThreadID=ProcKeeperThreadID;
   wcscpy(SIDM.DeviceDir, WorkDirectory);
   SIDM.ConfigOnly=ConfigureMode;
   SIDM.StartEvent=CreateEvent(NULL, FALSE, FALSE, NULL);

   // Launch the correct device-manager thread based on the 'DeviceID' parameter.
   // The thread will be created, but will not start running before 'ResumeThread' is called.
   if (Devices[Index].DeviceID==10) ThreadHandle=CreateThread(NULL, 0, NullDevMngrThrd, (PVOID)&SIDM, CREATE_SUSPENDED, &ThreadID);
   if (Devices[Index].DeviceID==100) ThreadHandle=CreateThread(NULL, 0, EDM006MngrThrd, (PVOID)&SIDM, CREATE_SUSPENDED, &ThreadID);
   if (Devices[Index].DeviceID==105) ThreadHandle=CreateThread(NULL, 0, PS2400MngrThrd, (PVOID)&SIDM, CREATE_SUSPENDED, &ThreadID);
   if (Devices[Index].DeviceID==200) ThreadHandle=CreateThread(NULL, 0, GBVx16MngrThrd, (PVOID)&SIDM, CREATE_SUSPENDED, &ThreadID);
   if (Devices[Index].DeviceID==210) ThreadHandle=CreateThread(NULL, 0, SeisAD18MngrThrd, (PVOID)&SIDM, CREATE_SUSPENDED, &ThreadID);
   if (Devices[Index].DeviceID==300) ThreadHandle=CreateThread(NULL, 0, NMRDxMngrThrd, (PVOID)&SIDM, CREATE_SUSPENDED, &ThreadID);
   if (Devices[Index].DeviceID==500) ThreadHandle=CreateThread(NULL, 0, KMK2MngrThrd, (PVOID)&SIDM, CREATE_SUSPENDED, &ThreadID);
   if (Devices[Index].DeviceID==510) ThreadHandle=CreateThread(NULL, 0, SADCMngrThrd, (PVOID)&SIDM, CREATE_SUSPENDED, &ThreadID);
#ifndef WINCE
   if (Devices[Index].DeviceID==310) ThreadHandle=CreateThread(NULL, 0, NMHRD24MngrThrd, (PVOID)&SIDM, CREATE_SUSPENDED, &ThreadID);
   if (Devices[Index].DeviceID==400) ThreadHandle=CreateThread(NULL, 0, CBDAS640216MngrThrd, (PVOID)&SIDM, CREATE_SUSPENDED, &ThreadID);
#endif

   // Check if 'CreateThread' succeeded.
   if (ThreadHandle==NULL) return 0;

   // Set the thread's priority.
   SetThreadPriority(ThreadHandle, THREAD_PRIORITY_NORMAL);

   // Resume thread. The thread will now start running.
   ResumeThread(ThreadHandle);

	// Close thread handle.
	CloseHandle(ThreadHandle);

   // Do not continue before the thread is properly started.
   WaitForSingleObject(SIDM.StartEvent, INFINITE);
   
   // Delete the event object.
   CloseHandle(SIDM.StartEvent);

   // Return the thread-ID of the started thread.
   return ThreadID;
}





/*******************************************************************************************/
/* Starts a device manager thread.                                                         */
/*******************************************************************************************/
HANDLE StartDevice(PTCHAR DeviceDir, DWORD MainThreadID, BOOL ConfigureMode, WORD DeviceID, DWORD *ThreadID)
{
   static STARTINFO_DEVMANAGER SIDM;
   HANDLE hThread;
   TCHAR WorkDirectory[MAX_PATH+1];

   // Create full path to device manager's working directory.
   wcscpy(WorkDirectory, DeviceDir);
   StripBackslashFromPath(WorkDirectory);

   // Initialize the startupinfo structure for the device manager thread.
   SIDM.hInstance=hInst;
   SIDM.ThreadID=MainThreadID;
   SIDM.ProcKeeprThreadID=ProcKeeperThreadID;
   wcscpy(SIDM.DeviceDir, WorkDirectory);
   SIDM.ConfigOnly=ConfigureMode;
   SIDM.StartEvent=CreateEvent(NULL, FALSE, FALSE, NULL);

   // Launch the correct device-manager thread based on the 'DeviceID' parameter.
   // The thread will be created, but will not start running before 'ResumeThread' is called.
   if (DeviceID==10) hThread=CreateThread(NULL, 0, NullDevMngrThrd, (PVOID)&SIDM, CREATE_SUSPENDED, ThreadID);
   if (DeviceID==100) hThread=CreateThread(NULL, 0, EDM006MngrThrd, (PVOID)&SIDM, CREATE_SUSPENDED, ThreadID);
   if (DeviceID==105) hThread=CreateThread(NULL, 0, PS2400MngrThrd, (PVOID)&SIDM, CREATE_SUSPENDED, ThreadID);
   if (DeviceID==200) hThread=CreateThread(NULL, 0, GBVx16MngrThrd, (PVOID)&SIDM, CREATE_SUSPENDED, ThreadID);
   if (DeviceID==210) hThread=CreateThread(NULL, 0, SeisAD18MngrThrd, (PVOID)&SIDM, CREATE_SUSPENDED, ThreadID);
   if (DeviceID==300) hThread=CreateThread(NULL, 0, NMRDxMngrThrd, (PVOID)&SIDM, CREATE_SUSPENDED, ThreadID);
   if (DeviceID==500) hThread=CreateThread(NULL, 0, KMK2MngrThrd, (PVOID)&SIDM, CREATE_SUSPENDED, ThreadID);
   if (DeviceID==510) hThread=CreateThread(NULL, 0, SADCMngrThrd, (PVOID)&SIDM, CREATE_SUSPENDED, ThreadID);
#ifndef WINCE
   if (DeviceID==310) hThread=CreateThread(NULL, 0, NMHRD24MngrThrd, (PVOID)&SIDM, CREATE_SUSPENDED, ThreadID);
   if (DeviceID==400) hThread=CreateThread(NULL, 0, CBDAS640216MngrThrd, (PVOID)&SIDM, CREATE_SUSPENDED, ThreadID);
#endif

   // Check if 'CreateThread' succeeded.
   if (hThread==NULL) return 0;

   // Set the thread's priority.
   SetThreadPriority(hThread, THREAD_PRIORITY_NORMAL);

   // Resume thread. The thread will now start running.
   ResumeThread(hThread);

   // Do not continue before the thread is properly started.
   WaitForSingleObject(SIDM.StartEvent, INFINITE);
   Sleep(100);
   
   // Delete the event object.
   CloseHandle(SIDM.StartEvent);

   return hThread;
}



/*************************************************************************/
/* Remove a device identified by name from the system.                   */      
/*************************************************************************/
BOOL RemoveDevice(PTCHAR DeviceName)
{
   BOOL Status;
   TCHAR Directory[MAX_PATH+1];
   
   // Remove the device directory.
   wcscpy(Directory, DeviceDir);
   StripBackslashFromPath(Directory);
   wcscat(Directory, TEXT("\\"));
   wcscat(Directory, DeviceName);
   Status=TRUE;
   if (!ClearDirectory(Directory)) Status=FALSE;
   if (!RemoveDirectory(Directory)) Status=FALSE;

   // Recreate the 'Devices'table.
   RegisterDevices();

   return Status;
}


/****************************************************************************/
/* Returns TRUE if a device with the given name already exist in 'Devices'. */      
/* If found, also returns the index for the device into the 'Devices' table.*/      
/****************************************************************************/
BOOL ExistDevice(TCHAR *DeviceName, PBYTE DevIndex)
{
   BYTE Index;

   // Search through the 'Devices' array for device.
   for (Index=0;Index<NumRegisteredDevices;Index++) {
      if (!_wcsicmp(Devices[Index].UniqueDeviceName, DeviceName)) {
         *DevIndex=(BYTE)Index;
         return TRUE;   
      }
   }
   
   *DevIndex=255;
   return FALSE;
}


