/* ************************************************** */
/* Implementation of the fifobuffer class.            */
/* ************************************************** */

#include "stdafx.h"

//#define STRICT
#include <windows.h>
#include "fifobuffer.h"


FifoBuffer::FifoBuffer() {
   PtrBuffer=(BYTE*)NULL;
   OffsFirstByte=OffsFirstFree=0;
   BytesInBuffer=0;
   BufferSize=0;
}


FifoBuffer::~FifoBuffer() {
   // Free allocated memory.
   if (PtrBuffer) GlobalFree((HGLOBAL)PtrBuffer);
}


BOOL FifoBuffer::InitBuffer(DWORD Size) {
//
// Allocates memory for the buffer and
// initiates all private variables. The
// 'Size' parameter sets the size of the 
// buffer. This member function must be
// called before any other member function.
//

   // Check input parameter 'Size'.
   if (Size==0) return false;

   // If buffer exists already, destroy it.
   if (PtrBuffer) {
      GlobalFree((HGLOBAL)PtrBuffer);
      PtrBuffer=(BYTE*)NULL;
      OffsFirstByte=OffsFirstFree=0;
      BytesInBuffer=0;
      BufferSize=0;
   } 

   // Allocate memory.
   PtrBuffer=(BYTE*)GlobalAlloc(GMEM_FIXED, Size);
   if (!PtrBuffer) return false;

   // Set up buffer pointers.
   OffsFirstByte=OffsFirstFree=0;

   // Save buffersize value.
   BufferSize=Size;

   // Update bytecount..
   BytesInBuffer=0;

   return true;
}


BOOL FifoBuffer::InsertBytes(DWORD Count, BYTE *Source) {

   // Check that buffer exists.
   if (!PtrBuffer) return false;

   // Check that there is room in the buffer.
   if ((BufferSize-BytesInBuffer)<Count) return false;

   // Insert bytes into buffer.
   for (Index=0;Index<Count;Index++) {
      *(PtrBuffer+OffsFirstFree)=*(Source+Index);
      OffsFirstFree++;
      if (OffsFirstFree==BufferSize) OffsFirstFree=0;
   }

   // Update bytecount.
   BytesInBuffer=BytesInBuffer+Count;

   return true;
}



BOOL FifoBuffer::InsertBytesFirst(DWORD Count, BYTE *Source) 
{

   // Check that buffer exists.
   if (PtrBuffer==NULL) return false;

   // Check in values.
   if (Source==NULL) return false;
   if (Count<=0) return false;

   // Check that there is room in the buffer.
   if (BufferSize-BytesInBuffer<Count) return false;

   // Insert bytes into buffer.
   for (Index=1;Index<=Count;Index++) {
      if (OffsFirstByte==0) {
         OffsFirstByte=BufferSize-1;
      } else {
         OffsFirstByte--;
      }
      *(PtrBuffer+OffsFirstByte)=*(Source+(Count-Index));
   }

   // Update bytecount.
   BytesInBuffer=BytesInBuffer+Count;

   return true;
}



BOOL FifoBuffer::RemoveBytes(DWORD Count, BYTE *Dest) {
   
   // Check that buffer exists.
   if (!PtrBuffer) return false;

   // Check that there is enogh bytes in the buffer.
   if (BytesInBuffer<Count) return false;

   // Remove bytes from buffer.
   for (Index=0;Index<Count;Index++) {
      *(Dest+Index)=*(PtrBuffer+OffsFirstByte);
      OffsFirstByte++;
      if (OffsFirstByte==BufferSize) OffsFirstByte=0;
   }

   // Update bytecount.
   BytesInBuffer=BytesInBuffer-Count;

   return true;
}


void FifoBuffer::ClearBuffer() {

   BytesInBuffer=0;
   OffsFirstByte=OffsFirstFree=0;
}


DWORD FifoBuffer::GetNumBytes() {

   return BytesInBuffer;
}


DWORD FifoBuffer::GetNumFreeBytes() {

   return BufferSize-BytesInBuffer;
}

