// GBVx16.cpp

#include "stdafx.h"

#include <aygshell.h>

//#define STRICT
#include <windows.h>
#include <stdio.h>
#include "seislog.h"
#include "library.h"
#include "msglist.h"
#include "resource.h"
#include "process.h"
#include "eventlog.h"


#define WINCE

//
// This structure is used to pass startup- 
// information to the serial-reader thread.
//
struct STARTINFO_DEVREADER { 
   HANDLE hComm;              // Handle of communications device 
   TCHAR CommSpeed[7];		   // Communication speed.
   DWORD ThreadID;            // Thread-ID for parent thread.
   BOOL *SerialRead;          // The serial-reader thread will exit if this becomes FALSE.
   DEVMNGR_INTERF_STRUCT *DS; // Pointer to the exported structure.
   DWORD* BlockNr;            // Pointer to variable who are to receive block numbers.
   DOUBLE TimeCorrection;     // Time correction (seconds).
   BOOL UsePCClock;           // If TRUE then time tagging is based on the PC clock.
   BOOL SyncPCClock;          // If TRUE then device reader thread will syncronize the PC clock.
   BYTE SyncStatus;           // The current time syncronization status.
   HANDLE hSyncStatusMutex;   // Handle to a mutex object used to control access to the
                              // SyncStatus variable in this structure.
};


//
// This structure is read from the binary device config file.
//
struct DEVICE_CONFIG {
   WORD DeviceID;          // MUST be '200'.
   BYTE NumChannels;       // Can be 1 or 3.
   TCHAR PortName[6];       // COM1, COM2 ...
   TCHAR CommSpeed[7];		// Communication speed.
   TCHAR Model[8];          // Can be 'GBV-116' or 'GBV-316'.
   BYTE SampleRate;        // Can be 25, 50, 100 or 200. 
   DOUBLE TimeCorrection;  // Time correction (seconds).
   BOOL UsePCClock;        // If TRUE then time tagging is based on the PC clock.
   BOOL SyncPCClock;       // If TRUE then device driver will syncronize the system clock.
};


//
// This sructure represents the data block from the device.
//
struct DEVICE_BLOCK {
   BYTE SyncTCHARs[2];         // Sync TCHARacters ("GB").
   BYTE DeviceID[2];          // Device ID.
   BYTE FirmwareRev[2];       // Firmware revision.
   BYTE StatInfo[2];          // Status information.
   BYTE DeviceInfo[32];       // Device information.
   BYTE BlockNum[4];          // Block number.
   BYTE BytesPrPacket[2];     // Bytes per packet.
   BYTE ADConvIndicator;      // A/D converter indicator.
   BYTE ActChIndicatorLow;    // Active channel indicator (Low).
   BYTE ActChIndicatorHigh;   // Active channel indicator (High).
   BYTE TimeSysIndicator;     // Time system indicator.
   BYTE Year[2];              // Current year.
   BYTE Month;                // Current month.
   BYTE Day;                  // Current day.
   BYTE Hour;                 // Current hour.
   BYTE Minute;               // Current minute.
   BYTE Second;               // Current second.
   BYTE MilliSecond[2];       // Current millisecond.
   BYTE Samples[1800];        // Sample data.
   BYTE CheckSum1;            // Checksum - high byte.
   BYTE CheckSum2;            // Checksum - low byte.
   BYTE CheckSum3;            // NOT USED.
   BYTE CheckSum4;            // NOT USED.
	BYTE Reserved1;            // Reserved.
	BYTE Reserved2;            // Reserved.
};


//
// Global variables.
//
extern BYTE DebugLevel;
extern BOOL StartGPSThread;
extern HWND hWndFrame;
extern DWORD SysClkSyncThreadID;


//
// Function prototyping.
//
DWORD WINAPI ReaderThreadGBVx16(PVOID);
BOOL CALLBACK ConfigDlgProcGBVx16(HWND, UINT, WPARAM, LPARAM);
BOOL GBVx16BytesOK(PBYTE, DWORD);
BYTE GetGBVx16SyncStatus(PBYTE);


/* *********************************************************************** */
/*  Manager thread for the GeoSIG GBV116/GBV316 devices. DeviceID=200.     */
/* *********************************************************************** */
DWORD WINAPI GBVx16MngrThrd(PVOID StartupInfo) 
{
   MSG msg;
   int Status;
   BYTE SyncStatus, FixStatus;
   TCHAR ConfigFile[MAX_PATH+1], ChNr, UniqueDeviceName[65], Message[100];
   TCHAR DeviceResources[256], str10[11];
   WORD BlockSize, NumBlocksPrChannel, SampleRate;
   HANDLE hReaderThread, hComm;
   STARTINFO_DEVMANAGER SI;
   STARTINFO_DEVREADER StartInfo;
   DEVICE_CONFIG DeviceConf, DevConfDlg;
   DEVMNGR_INTERF_STRUCT DS;
   DWORD ParentThreadID, CurrentThreadID, ReaderThreadID;
   DWORD ChannelSize, status, BlockNr;
   MSG_PARAM ID, OD;
   MsgList NotifyList;
   BOOL SerialRead, MemAlloc, ConfigRead;
   

   
   //
   // Perform thread initialization.
   //
   FixStatus=0;
   SyncStatus=2;
   hReaderThread=(HANDLE)NULL;                       // The serial-reader thread is not running.
   CopyMemory((PVOID)&SI, StartupInfo, sizeof(SI));  // Copy startup information to local buffer.
   ParentThreadID=SI.ThreadID;                       // ThreadID for parent thread.
   CurrentThreadID=GetCurrentThreadId();             // ThreadID for this thread.
   

   //
   // Handle device configuration.
   //
   // Read configuration file.
   ConfigRead=FALSE;
   wcscpy(ConfigFile, SI.DeviceDir); 
   wcscat(ConfigFile, TEXT("\\device.cnf"));
   if (ExistFile(SI.DeviceDir, TEXT("device.cnf"))) {
      // Read device config from config file.
      ConfigRead=ReadFileToMemory(ConfigFile, (PVOID)&DeviceConf, (DWORD)sizeof(DeviceConf));
   }
   if (!ConfigRead) {
      // A valid config file could not be read.
      // Set up a default digitizer configuration.
      DeviceConf.DeviceID=200;               // GBVx16 uses device-id 200.
      wcscpy(DeviceConf.Model, TEXT("GBV-316"));   // Assume model GBV-316.
      DeviceConf.NumChannels=3;              // Assume model GBV-316.
      wcscpy(DeviceConf.PortName, TEXT("COM1"));   // Assume digitizer on COM1.
		wcscpy(DeviceConf.CommSpeed, TEXT("38400")); // Assume comm speed of 38400 baud.
      DeviceConf.SampleRate=100;             // Assume 100hz sample rate.  
      DeviceConf.TimeCorrection=0.0;         // Time correction is off by default.
      DeviceConf.UsePCClock=FALSE;           // Use device time.
      DeviceConf.SyncPCClock=TRUE;           // Syncronize the system clock.

      // Write settings to config file.
      WriteFileFromMemory(ConfigFile, (PVOID)&DeviceConf, (DWORD)sizeof(DeviceConf));
   }


   // 
   // Create a unique device instance name.
   //
   wcscpy(UniqueDeviceName, DeviceConf.Model);
   wcscat(UniqueDeviceName, TEXT(" at "));
   wcscat(UniqueDeviceName, DeviceConf.PortName);


   //
   // Register this process in the process database.
   //
   if (!SI.ConfigOnly) RegisterProcess(UniqueDeviceName, 1, CurrentThreadID);


   //
   // Create device resource string.
   // 
   wcscpy(DeviceResources, TEXT("Serial ports: "));
   wcscat(DeviceResources, DeviceConf.PortName);


   //
   // Handle time syncronization stuff.
   //
   StartInfo.SyncPCClock=FALSE;
   if (!SI.ConfigOnly && DeviceConf.SyncPCClock) {
      // Device is configured to syncronize the system clock.
      // Check if any other thread is already doing this.
      if (SysClkSyncThreadID) {
         // Turn off time syncronication. Clock is 
         // already beeing syncronized by another thread.
         DeviceConf.SyncPCClock=FALSE;
         // Write warning in the system log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Cannot syncronize system clock. Clock already syncronized."));
         WriteSystemLog(Message, 2);
      } else {
         // System clock is not already syncronized.
         // This thread will attempt to syncronize it.
         StartInfo.SyncPCClock=TRUE;
         SysClkSyncThreadID=CurrentThreadID;
         // Write text on status bar.
         DrawStatusBarText(SyncStatus, UniqueDeviceName);
         // Write message in the system log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Syncronizing system clock."));
         WriteSystemLog(Message, 1);
      }
   }


   //
   // Calculate sample-rate, blocksize, channelsize and number of blocks pr. channel.
   //
   SampleRate=(WORD)DeviceConf.SampleRate;
   BlockSize=(WORD)(sizeof(BLOCK_HEADER)+(SampleRate*4));
   NumBlocksPrChannel=60;
   ChannelSize=NumBlocksPrChannel*BlockSize;



   //
   // Open the serial port.
   //
   if (!SI.ConfigOnly) {
#ifdef WINCE
	   // All ports in Windows CE must end with ':'
	   TCHAR CEPortName[7];
	   wcscpy(CEPortName, DeviceConf.PortName);
	   wcscat(CEPortName, TEXT(":"));

	   hComm=CreateFile(CEPortName, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, NULL);
#else
       hComm=CreateFile(DeviceConf.PortName, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, NULL);
#endif
      if (hComm==INVALID_HANDLE_VALUE) {
         // Update program log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Unable to open communication port "));
         wcscat(Message, DeviceConf.PortName);
         wcscat(Message, TEXT("."));
         WriteSystemLog(Message, 3);
      }
   }

   
   //
   // Initialize the interface structure.
   //
   // Insert manufacturer name and model.
   wcscpy(DS.Manufacturer, TEXT("GeoSIG AG"));
   wcscpy(DS.Model, DeviceConf.Model);

   // Insert samplerate.
   DS.SampleRate=SampleRate;

   // Insert number of samples in each block.
   DS.SamplesPrBlock=SampleRate;

   // Insert number of channels
   DS.NumChannels=DeviceConf.NumChannels;

   // Insert block size, block header included.
   DS.BlockSize=BlockSize;

   // Insert block size measured in milliseconds.
   DS.BlockSizeInMS=1000;

   // Insert number of blocks pr channel.
   DS.NumBlocksPrChannel=NumBlocksPrChannel;

   // Make sure all pointers to channel data is NULL.
   for (ChNr=0;ChNr<MAX_CHANNELS_PR_DEVICE;ChNr++) DS.ChannelData[ChNr]=(PVOID)NULL;
   
   
   //
   // Allocate memory for the channel buffers, and set up the pointers to the buffers.
   //
   MemAlloc=TRUE;
   if ((DS.ChannelData[0]=(PVOID)LocalAlloc(LPTR, (UINT)ChannelSize))==NULL) MemAlloc=FALSE;
   if (DeviceConf.NumChannels==3) {
      if ((DS.ChannelData[1]=(PVOID)LocalAlloc(LPTR, (UINT)ChannelSize))==NULL) MemAlloc=FALSE;
      if ((DS.ChannelData[2]=(PVOID)LocalAlloc(LPTR, (UINT)ChannelSize))==NULL) MemAlloc=FALSE;
   }
   
   // Check that memory allocation succeded.
   if (!MemAlloc) {
         // Update program log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(":"));
         wcscat(Message, TEXT(" Could not allocate memory."));
         WriteSystemLog(Message, 3);
   }
   

   //
   // If running in debug mode, write a message to the log.
   //
   if (DebugLevel) {
      wcscpy(Message, UniqueDeviceName);
      wcscat(Message, TEXT(": Thread started."));
      WriteSystemLog(Message, 1);
   }
   

   //
   // Signal that the thread is started.
   //
   SetEvent(SI.StartEvent);


   //
   // Enter the thread message loop. The thread will now start to process messages.
   //
   while (GetMessage(&msg, (HWND)NULL, 0, 0)) 
   { 
   
      // If debuglevel is 2 then log all messages recieved.
      if (DebugLevel>1) {
         TCHAR str10[11];
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Recieved message "));
         _itow(msg.message, str10, 10);
         wcscat(Message, str10);
         WriteSystemLog(Message, 1);
      }

      // Save message parameters.
      ZeroMemory((PVOID)&ID, (DWORD)sizeof(ID));
      if (msg.lParam) ID=*((MSG_PARAM*)msg.lParam);
      
      // Initialize return-message parameter to default values.
      ZeroMemory((PVOID)&OD, (DWORD)sizeof(OD));
      OD.bStatus=TRUE;
      OD.idThread=CurrentThreadID;


      // Process message.
      switch (msg.message) {
         case TM_START:                            
         //
         // This message is sendt from a higher level 
         // thread to start reading the serial port.
         //
         if (hReaderThread==NULL && hComm!=INVALID_HANDLE_VALUE && MemAlloc && !SI.ConfigOnly) {
            if (SysClkSyncThreadID && SysClkSyncThreadID!=CurrentThreadID) {
               // The system clock is syncronized by some other thread.
               // Subscribe to the TM_TIMESYNC_STATUS message from thread.
               Subscribe(TM_TIMESYNC_STATUS, SysClkSyncThreadID, CurrentThreadID, (HWND)NULL);

               // Get the current syncronization status.
               StartInfo.SyncStatus=GetCurrentTimeSyncStatus(SysClkSyncThreadID);
            } else {
               // The system clock is not syncronized and
               // there is no other syncronization possibilities.
               StartInfo.SyncStatus=0;
               // Create a mutex used to access the 'StartInfo.SyncStatus' variable.
               StartInfo.hSyncStatusMutex=CreateMutex(NULL, FALSE, NULL);
            }

            // Set up startup information for the serial reader thread routine.
            SerialRead=TRUE;
            StartInfo.hComm=hComm;
				wcscpy(StartInfo.CommSpeed, DeviceConf.CommSpeed);
            StartInfo.ThreadID=CurrentThreadID;
            StartInfo.SerialRead=&SerialRead;
            StartInfo.DS=&DS;
            StartInfo.BlockNr=&BlockNr;
            StartInfo.TimeCorrection=DeviceConf.TimeCorrection;
            StartInfo.UsePCClock=DeviceConf.UsePCClock;

            // Start the thread.
            hReaderThread=CreateThread(NULL, 0, ReaderThreadGBVx16, (VOID*)&StartInfo, CREATE_SUSPENDED, &ReaderThreadID);
            SetThreadPriority(hReaderThread, THREAD_PRIORITY_ABOVE_NORMAL);  // Set priority above normal.
            ResumeThread(hReaderThread); 
         }

         // Set return values. 
         if (!hReaderThread) {
            OD.bStatus=FALSE;
            OD.wStatus=1;
         }
         break;


         
         case TM_STOP:
         //
         // This message is sendt from a higher level thread to stop
         // reading the serial port. Returns the default return values.
         //
         if (hReaderThread) {
            // Make the serial thread loop exit, thereby stopping the thread.
            SerialRead=FALSE;

            // Wait up to 2 seconds for the thread to stop.
            status=WaitForSingleObject(hReaderThread, 2000);
            if (status==WAIT_TIMEOUT) TerminateThread(hReaderThread, 0);

            // Unsubscribe to the TM_TIMESYNC_STATUS message as needed.
            if (SysClkSyncThreadID && SysClkSyncThreadID!=CurrentThreadID) {
               UnSubscribe(TM_TIMESYNC_STATUS, SysClkSyncThreadID, CurrentThreadID, NULL);
               CloseHandle(StartInfo.hSyncStatusMutex); // Delete mutex object.
            }

            // The thread is now stopped, invalidate the thread handle.
				CloseHandle(hReaderThread);
            hReaderThread=NULL;
         }
         break;


         case TM_CONFIGURE:            
         //
         // This message is sendt from a higher level thread
         // when it wants to configure the digitizer settings.
         //
         if (hReaderThread==NULL) {
            // Let user work on a copy of the current device settings.
            DevConfDlg=DeviceConf;

            // Display the dialog box.
            Status=DialogBoxParam(SI.hInstance, MAKEINTRESOURCE(IDD_CNF_GBVx16), NULL, (DLGPROC)ConfigDlgProcGBVx16, (LPARAM)&DevConfDlg);   

            // Save settings if requested.
            if (Status==1) {
               // Save the new configuration.
               DeviceConf=DevConfDlg;

               // Set correct number of channels based on selected model.
               if (!wcscmp(DeviceConf.Model, TEXT("GBV-116"))) DeviceConf.NumChannels=1;
               if (!wcscmp(DeviceConf.Model, TEXT("GBV-316"))) DeviceConf.NumChannels=3;

               // Write configuration to disk.
               WriteFileFromMemory(ConfigFile, (PVOID)&DeviceConf, (DWORD)sizeof(DEVICE_CONFIG));

               // Recreate the unique name of this device instance.
               wcscpy(UniqueDeviceName, DeviceConf.Model);
               wcscat(UniqueDeviceName, TEXT(" at "));
               wcscat(UniqueDeviceName, DeviceConf.PortName);
            }

            // Set return values. 
            if (Status==0) {
               OD.bStatus=FALSE;
               OD.wStatus=100;
            }
         }
         break;


         case TM_SHOW_CONFIG:
            //
            // When this message is received, dump all configuration
            // parameters that is not given in the device resource 
            // string to the system log.
            //
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(" Configuration: Time Correction="));
            swprintf(str10,TEXT("%.3f"), DeviceConf.TimeCorrection);
            wcscat(Message, str10);
            WriteSystemLog(Message, 1);
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(" Configuration: Syncronize system clock="));
            if (DeviceConf.SyncPCClock) wcscat(Message, TEXT("TRUE"));
            if (!DeviceConf.SyncPCClock) wcscat(Message, TEXT("FALSE"));
            WriteSystemLog(Message, 1);
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(" Configuration: Communication speed="));
            wcscat(Message, DeviceConf.CommSpeed);
				wcscat(Message, TEXT(" baud"));
            WriteSystemLog(Message, 1);
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(" Configuration: Use system clock for time tagging="));
            if (DeviceConf.UsePCClock) wcscat(Message, TEXT("TRUE"));
            if (!DeviceConf.UsePCClock) wcscat(Message, TEXT("FALSE"));
            WriteSystemLog(Message, 1);
         break;


         case TM_GET_INTERFACE:
         //
         // This message is used to get a pointer to the interface structure.
         //
         OD.vPointer=(PVOID)&DS;
         break;   


         case TM_GET_UNIQUE_DEVICE_NAME:
         //
         // This message is used to get a string, 
         // identifying this unique device instance.
         //
         wcscpy(OD.cStatus, UniqueDeviceName);
         break;   


         case TM_GET_RESOURCE_STRING:
         //
         // This message is used to get a string, 
         // identifying the system resources used by this device.
         //
         wcscpy(OD.cStatus, DeviceResources);
         break;   


         case TM_SUBSCRIBE:
         //
         // This message is sendt from a window or thread
         // Who wants to be informed about an event.
         // This message must be accompanied by a MSG_PARAM structure.
         // The MSG_PARAM structure must have a pointer to a valid
         // SUBSCRIBE structure.
         //
         //
         // Add window/thread to notify list.
         if (ID.vPointer) if (!NotifyList.Subscribe((SUBSCRIBE*)ID.vPointer)) {
            OD.bStatus=FALSE;
            OD.wStatus=3;
         }
         break;   


         case TM_UNSUBSCRIBE:
         //
         // This message is sendt from a window or thread who
         // wants to remove a notification from the notify list.
         // This message must be accompanied by a MSG_PARAM structure.
         // The MSG_PARAM structure must have a pointer to a valid
         // SUBSCRIBE structure.
         //
         //
         // Remove notification from list.
         if (ID.vPointer) NotifyList.UnSubscribe((SUBSCRIBE*)ID.vPointer);
         
         break;   



         case TM_EXIT:                 
         //
         // This is a request for the thread to end.
         // Close down all processes that are depandent on this thread.
         // The 'StopDependantThreads' function will return immediatly,
         // and will send a 'TM_DEPTHREADS_STOPPED' message back when
         // all dependant threads have been stopped.
         StopDependantThreads();
         break;



         case TM_DEPTHREADS_STOPPED:
         //
         // This message indicates that all dependent threads have been stopped.
         // The thread will cleanup and exit when this message is recieved.         
         //

         // First stop the device-reader thread.
         if (hReaderThread) {
            SerialRead=FALSE;                                            // Signal the reader thread.
            status=WaitForSingleObject(hReaderThread, 2000);             // Wait for the thread to stop.
            if (status==WAIT_TIMEOUT) TerminateThread(hReaderThread, 0); // If thread wont stop, kill it.
            hReaderThread=NULL;
         }

         if (!SI.ConfigOnly) {
            // Unregister this process from the process database.
            UnregisterProcess(CurrentThreadID);

            // Close the communication port.
            CloseHandle(hComm);

            // Handle time syncronization stuff.
            if (DeviceConf.SyncPCClock && StartInfo.SyncPCClock) {
               // Clear the SysClkSyncThreadID variable.
               SysClkSyncThreadID=0;
               // Clear the status bar.
               PostMessage(hWndFrame, TM_WRITE_STATUSBAR, 0, (LPARAM)TEXT(""));
            }
         }


         // Free memory buffers.
         if (DS.ChannelData[0]) LocalFree((HLOCAL)DS.ChannelData[0]); 
         if (DS.ChannelData[1]) LocalFree((HLOCAL)DS.ChannelData[1]); 
         if (DS.ChannelData[2]) LocalFree((HLOCAL)DS.ChannelData[2]); 

         // End the message loop.
         ClearMessageQueue(); // Delete all messages in message queue.
         PostQuitMessage(0);  // Post WM_QUIT message.   
         break;
   
         
         //
         // Handle messages from device reader thread.
         //
         case TM_BUFFERUPDATED:  // A new block of data has been inserted into buffer.
         // Send Notification to all subscribed threads/windows.
         // The notofication is accompanied with this thread's thread-ID, and the blocknr of the updated block.
         NotifyList.Notify((UINT)TM_BUFFERUPDATED, (WPARAM)CurrentThreadID, (LPARAM)BlockNr);
         break;


         case TM_TIMESYNC_STATUS:
         //
         // This message is sendt from a time syncronization
         // thread (other then this thread or its child) to inform
         // about a change in the syncronization status. This thread
         // subscribe to this message from the other thread.
         WaitForSingleObject(StartInfo.hSyncStatusMutex, INFINITE); 
         StartInfo.SyncStatus=(BYTE)msg.wParam;
         ReleaseMutex(StartInfo.hSyncStatusMutex);
         break;
         
         
         case TM_GET_TIMESYNC_STATUS:
         //
         // This message is sendt from a window or thread who
         // wants to know the status of the GPS time syncronisation.
         // This message must be accompanied by a MSG_PARAM structure.
         //
         //
         // Returns the operational status of the thread.
         // Values are:
         //    0 :System clock is beeing syncronized, no error.
         //    1 :System clock is NOT beeing syncronized, but no error (fix lost).
         //    2 :System clock is NOT beeing syncronized, communication error.
         //
         OD.wStatus=(WORD)SyncStatus;
         break;


         case TM_DEVICEREADER_NODATAINPUT:   
         //
         // The device reader has lost data input.
         //
         // Send Notification to all subscribed threads/windows.
         NotifyList.Notify((UINT)TM_DEVICEREADER_NODATAINPUT, WPARAM(CurrentThreadID), 0);

         // Handle GPS stuff.
         if (DeviceConf.SyncPCClock) {
            SyncStatus=2;
            NotifyList.Notify((UINT)TM_TIMESYNC_STATUS, (WPARAM)SyncStatus, 0);
            DrawStatusBarText(SyncStatus, UniqueDeviceName);
         }

         // Write a message to the log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Not receiving data."));
         WriteSystemLog(Message, 2);
         break;


         case TM_DEVICEREADER_DATAINPUTOK:   
         //
         // The device reader has regained data input.
         //
         // Send Notification to all subscribed threads/windows.
         NotifyList.Notify((UINT)TM_DEVICEREADER_DATAINPUTOK, WPARAM(CurrentThreadID), 0);

         // Handle GPS stuff.
         if (DeviceConf.SyncPCClock) {
            SyncStatus=0;
            if (!FixStatus) SyncStatus=1;
            NotifyList.Notify((UINT)TM_TIMESYNC_STATUS, (WPARAM)SyncStatus, 0);
            DrawStatusBarText(SyncStatus, UniqueDeviceName);
         }

         // Write a message to the log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Receiving data."));
         WriteSystemLog(Message, 1);
         break;


         case TM_DEVICEREADER_BADBLOCK:   
         //
         // A bad block of data was received from the device.
         //
         // Send Notification to all subscibed threads/windows.
         // The notification is accompanied with this thread's thread-ID, and the blocknr of the bad block.
         NotifyList.Notify((UINT)TM_DEVICEREADER_BADBLOCK, (WPARAM)CurrentThreadID, (LPARAM)BlockNr);

         // Write a message to the log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Received a bad block from device."));
         WriteSystemLog(Message, 2);
         break;


         case TM_DEVICEREADER_NOFIX:   
         //
         // The GPS reader-thread indicates that fix has been lost.
         //
			// Handle GPS stuff.
			if (DeviceConf.SyncPCClock) {
				FixStatus=0;
				if (!SyncStatus) SyncStatus=1;
				NotifyList.Notify((UINT)TM_TIMESYNC_STATUS, (WPARAM)SyncStatus, 0);
	         DrawStatusBarText(SyncStatus, UniqueDeviceName);
			}

         // Write a message to the log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Satellite fix lost."));
         WriteSystemLog(Message, 2);
         break;


         case TM_DEVICEREADER_FIXOK:   
         //
         // The GPS reader-thread indicates that fix has been regained.
         //
			// Handle GPS stuff.
			if (DeviceConf.SyncPCClock) {
				FixStatus=1;
				if (SyncStatus==1) SyncStatus=0;
				NotifyList.Notify((UINT)TM_TIMESYNC_STATUS, (WPARAM)SyncStatus, 0);
            DrawStatusBarText(SyncStatus, UniqueDeviceName);
			}
         
         // Write a message to the log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Satellite fix OK."));
         WriteSystemLog(Message, 1);
         break;
      }
      
      // If this is requested by the thread who sent the message
      // then return status values, and release the caller thread.
      if (((MSG_PARAM*)msg.lParam)!=NULL && msg.wParam!=NULL) {
         *(MSG_PARAM*)msg.lParam=OD;
         SetEvent((HANDLE)msg.wParam);
      }
   } 


   // Free up memeory allocated in the MsgList class object.
   NotifyList.~MsgList();

   // If running in debug mode, write a message to the log.
   if (DebugLevel) {
      wcscpy(Message, UniqueDeviceName);
      wcscat(Message, TEXT(": Thread stopped."));
      WriteSystemLog(Message, 1);
   }

   // Exit this thread
   ExitThread(0);
   return 0L;
   
}




/* *********************************************************************** */
/*  Serial port reader procedure for the GeoSIG GBVx16 devices.            */
/* *********************************************************************** */
DWORD WINAPI ReaderThreadGBVx16(PVOID StartupStruct)
{
#ifndef WINCE
   TCHAR DevCntrlStr[50]; // Only used in Win9X version
#endif
   BYTE *bPtr, TimeQuality, FixStatus, OldFixStatus, SyncStatus;  
   WORD BlockSize, NumChannels, NumBlocksPrChannel, SampleRate, SampleNr;
   DWORD BytesRead, BytesToRead, BlockNr, OffsetCurrentSample, OffsetToBlockData;
   DOUBLE TransferTime;
   SYSTEMTIME ReadTime, BlockTime, TimeTag, SysTime;
   VOID *DataBlocksCh1, *DataBlocksCh2, *DataBlocksCh3; 
   DCB dcb;
   COMMTIMEOUTS ct={50, 0, 1900, 0, 0};
   STARTINFO_DEVREADER *SI;
   BLOCK_HEADER DBH;
   DEVICE_BLOCK DtaBlck;
   LONG LIValue;
   BOOL BytesOK, DataLost, ReadStatus;
   extern BOOL StartGPSThread;


   // Copy values from startup structure.
   SI=(STARTINFO_DEVREADER*)StartupStruct;         // Use pointer of correct type.
   NumChannels=SI->DS->NumChannels;                // Number of channels digitizer is using.
   SampleRate=SI->DS->SampleRate;                  // Sample rate.
   NumBlocksPrChannel=SI->DS->NumBlocksPrChannel;  // Number of 1 sec. blocks in buffer.
   BlockSize=SI->DS->BlockSize;                    // Size of 1 sec. block header included
   DataBlocksCh1=SI->DS->ChannelData[0];           // Pointer to first block channel 1.
   DataBlocksCh2=SI->DS->ChannelData[1];           // Pointer to first block channel 2.
   DataBlocksCh3=SI->DS->ChannelData[2];           // Pointer to first block channel 3.

   // Initialize some variables.
   FixStatus=0;       // So far we have no satelite fix.
   OldFixStatus=0;    // We have no previous fix status.
   BlockNr=0;         // Start filling data into first block.
   DataLost=TRUE;     // At this point we have no data input.

   // Find number of bytes to read from device,
	if (SampleRate==25 && NumChannels==1)  BytesToRead=140;
	if (SampleRate==25 && NumChannels==3)  BytesToRead=290;
	if (SampleRate==50 && NumChannels==1)  BytesToRead=215;
	if (SampleRate==50 && NumChannels==3)  BytesToRead=515;
	if (SampleRate==100 && NumChannels==1) BytesToRead=365;
	if (SampleRate==100 && NumChannels==3) BytesToRead=965;
	if (SampleRate==200 && NumChannels==1) BytesToRead=665;
	if (SampleRate==200 && NumChannels==3) BytesToRead=1865;

   // Find the transfer time on the serial port.
	if (!wcscmp(SI->CommSpeed, TEXT("9600"))) TransferTime=(DOUBLE)(BytesToRead)/960.0;
	if (!wcscmp(SI->CommSpeed, TEXT("19200"))) TransferTime=(DOUBLE)(BytesToRead)/1920.0;
	if (!wcscmp(SI->CommSpeed, TEXT("38400"))) TransferTime=(DOUBLE)(BytesToRead)/3840.0;
    
   //Initialize the communications parameters.
   SetupComm(SI->hComm, BytesToRead*2, 0);   // Input buffer will hold 2 seconds of data.
   GetCommState(SI->hComm, &dcb);

#ifdef WINCE
   dcb.BaudRate     = _wtoi(SI->CommSpeed);
   dcb.fParity      = TRUE;
   dcb.Parity       = NOPARITY;
   dcb.StopBits     = ONESTOPBIT;
   dcb.ByteSize     = 8;

   dcb.fInX         = FALSE;
   dcb.fOutX        = FALSE;
   dcb.fOutxDsrFlow = FALSE;
   dcb.fOutxCtsFlow = FALSE;  
   dcb.fDtrControl  = DTR_CONTROL_ENABLE;
   dcb.fRtsControl  = RTS_CONTROL_ENABLE;
#else
   wcscpy(DevCntrlStr, TEXT("baud="));
   wcscat(DevCntrlStr, SI->CommSpeed);
   wcscat(DevCntrlStr, TEXT(" parity=N data=8 stop=1"));
   BuildCommDCB(DevCntrlStr, &dcb);
#endif

   dcb.fAbortOnError=FALSE;
   SetCommState(SI->hComm, &dcb);            // Set serial port mode. No flow control is used.
	SetCommTimeouts(SI->hComm, &ct);       	// Set communication timeouts.
   PurgeComm(SI->hComm, PURGE_RXCLEAR);      // Clear the input buffer.


   // Start reading the serial port.
   while(*SI->SerialRead) {
      // Try reading 'BytesToRead' bytes into buffer.
		ReadStatus=ReadFile(SI->hComm, (PVOID)&DtaBlck, BytesToRead, &BytesRead, NULL);		

      // Save the time when the block was read.
      GetSystemTime(&ReadTime);

      // Process incoming bytes.
      if (BytesRead==BytesToRead) {
         // Correct number of bytes was successfully read. Check if
         // bytes are OK. A checksum will be calculated and compared
         // to the checksum in the block.
         BytesOK=GBVx16BytesOK((PBYTE)&DtaBlck, BytesToRead);

         if (BytesOK) { // The block from the digitizer seems to be OK.
            //
            // Get time from incoming block.
				//
            BlockTime.wYear=(WORD)(bHex2Dec(DtaBlck.Year[0])*100 + bHex2Dec(DtaBlck.Year[1]));
            BlockTime.wMonth=(WORD)bHex2Dec(DtaBlck.Month);
            BlockTime.wDayOfWeek=0;
            BlockTime.wDay=(WORD)bHex2Dec(DtaBlck.Day);
            BlockTime.wHour=(WORD)bHex2Dec(DtaBlck.Hour);
            BlockTime.wMinute=(WORD)bHex2Dec(DtaBlck.Minute);
            BlockTime.wSecond=(WORD)bHex2Dec(DtaBlck.Second);
            BlockTime.wMilliseconds=(WORD)bHex2Dec(DtaBlck.MilliSecond[0]*10);
            BlockTime.wMilliseconds+=(WORD)bHex2Dec(DtaBlck.MilliSecond[1]);

            // Apply user specified time correction to block time.
            if (SI->TimeCorrection>0.0) AddToTime(&BlockTime, SI->TimeCorrection);
            if (SI->TimeCorrection<0.0) SubstractFromTime(&BlockTime, -SI->TimeCorrection);

				//
				// Get the GPS status from the block.
				//
            // Values are:
            // 0 - RTC error  								     
            // 1 - Real time clock only. (No GPS connected, or no fix since PowerON).
            // 2 - GPS has no satellite fix (we have had fix).
            // 3 - GPS has satellite fix.
				SyncStatus=GetGBVx16SyncStatus((PBYTE)&DtaBlck);

				//
				// Do system clock syncronization if enabled.
				//
            if (SI->SyncPCClock) {
               // The system time will be set equal to the
               // block time + transfertime + 1 second.
               SysTime=BlockTime;
               AddToTime(&SysTime, 1.0+TransferTime);
               SetSystemTime(&SysTime);
            }

            //
				// Find the time to be inserted into the block header.
            // This is either the devicetime, or a corrected system time.
				//
            if (SI->UsePCClock) {
               // The system clock will be used for timetagging.
               // Use the current time, and compensate for
               // the transfer time + 1 second digitizer delay.
               TimeTag=ReadTime;                                
               SubstractFromTime(&TimeTag, 1.0+TransferTime);
	            // Apply user specified time correction to block time.
	            if (SI->TimeCorrection>0.0) AddToTime(&TimeTag, SI->TimeCorrection);
		         if (SI->TimeCorrection<0.0) SubstractFromTime(&TimeTag, -SI->TimeCorrection);

					
					// Find the time quality.
               if (SysClkSyncThreadID) {
                  // The clock is syncronized. We will have to 
                  // get the syncstatus from the parent thread.
                  WaitForSingleObject(SI->hSyncStatusMutex, INFINITE); 
                  SyncStatus=SI->SyncStatus;
                  if (SI->SyncStatus==0) TimeQuality=2;
                  if (SI->SyncStatus==1) TimeQuality=1;
                  if (SI->SyncStatus==2) TimeQuality=3;
                  ReleaseMutex(SI->hSyncStatusMutex);
               } else {
                  // The clock is NOT syncronized. 
                  TimeQuality=0;
               }
            } else {
               // The block time will be used for timetagging.
               TimeTag=BlockTime;

					// Find the time quality.
               if (SyncStatus==0) { TimeQuality=3; FixStatus=0; }
               if (SyncStatus==1) { TimeQuality=1; FixStatus=0; }
               if (SyncStatus==2) { TimeQuality=1; FixStatus=0; }
					if (SyncStatus==3) { TimeQuality=2; FixStatus=1; }
            }
            

            // Create a block header and insert in all channels.
            DBH.BlockID=BlockNr;
            DBH.TimeTag=TimeTag;
            DBH.TimeQuality=TimeQuality;
            DBH.NullData=FALSE;
            DBH.DataLost=DataLost;
            DBH.EventDetected=FALSE;
            DBH.EventTime=0;
            bPtr=((BYTE*)DataBlocksCh1)+(BlockSize*BlockNr);
            CopyMemory((VOID*)bPtr, (VOID*)&DBH, (DWORD)sizeof(BLOCK_HEADER));
            if (NumChannels==3) {
               bPtr=((BYTE*)DataBlocksCh2)+(BlockSize*BlockNr);
               CopyMemory((VOID*)bPtr, (VOID*)&DBH, (DWORD)sizeof(BLOCK_HEADER));
               bPtr=((BYTE*)DataBlocksCh3)+(BlockSize*BlockNr);
               CopyMemory((VOID*)bPtr, (VOID*)&DBH, (DWORD)sizeof(BLOCK_HEADER));
            }

            // Calculate offset from start of data buffer,
            // into data part of currrent block.
            OffsetToBlockData=(BlockNr*BlockSize)+sizeof(BLOCK_HEADER);

            // Insert samples from digitizer into current block.
            if (NumChannels==1) { // This code is for 1 channel digitizer.
               OffsetCurrentSample=OffsetToBlockData;
               for (SampleNr=0;SampleNr<SampleRate;SampleNr++) {
                  // Process sample pointed to by 'SampleNr'.
                  bPtr=((BYTE*)DataBlocksCh1)+OffsetCurrentSample;
                  LIValue=(LONG)CreateShortFrom2Bytes(DtaBlck.Samples[SampleNr*3], DtaBlck.Samples[SampleNr*3+1]);
                  *((LONG*)bPtr)=LIValue;
                  // Move to next sample.
                  OffsetCurrentSample=OffsetCurrentSample+4;
               }
            }
            if (NumChannels==3) { // This code is for 3 channel digitizer.
               OffsetCurrentSample=OffsetToBlockData;
               for (SampleNr=0;SampleNr<SampleRate;SampleNr++) {
                  // Process sample pointed to by 'SampleNr' for channel 1.
                  bPtr=((BYTE*)DataBlocksCh1)+OffsetCurrentSample;
                  LIValue=(LONG)CreateShortFrom2Bytes(DtaBlck.Samples[SampleNr*9+0], DtaBlck.Samples[SampleNr*9+1]);
                  *((LONG*)bPtr)=LIValue;
                  // Process sample pointed to by 'SampleNr' for channel 2.
                  bPtr=((BYTE*)DataBlocksCh2)+OffsetCurrentSample;
                  LIValue=(LONG)CreateShortFrom2Bytes(DtaBlck.Samples[SampleNr*9+3], DtaBlck.Samples[SampleNr*9+4]);
                  *((LONG*)bPtr)=LIValue;
                  // Process sample pointed to by 'SampleNr' for channel 3.
                  bPtr=((BYTE*)DataBlocksCh3)+OffsetCurrentSample;
                  LIValue=(LONG)CreateShortFrom2Bytes(DtaBlck.Samples[SampleNr*9+6], DtaBlck.Samples[SampleNr*9+7]);
                  *((LONG*)bPtr)=LIValue;
                  // Move to next sample.
                  OffsetCurrentSample=OffsetCurrentSample+4;
               }
            }

         } else {
            //
            // The block from the digitizer does NOT seem to be OK.
            // Since this is probably a rear happening, we will just
            // fill this block with zeros and mark it with a zero flag
            // in the header. This is done so the channel thread can 
            // recognize it as a zero block, and replace the zeros 
            // with the currently calculated device offset value.
            //

            // Find the timequality factor for this block.
            if (!SI->SyncPCClock) {
               // Another thread or is syncronizing the system clock.
               // or the system clock is unsyncronized.
               if (SysClkSyncThreadID) {
                  // The clock is syncronized. We will have to 
                  // get the syncstatus from the device manager.
                  WaitForSingleObject(SI->hSyncStatusMutex, INFINITE); 
                  SyncStatus=SI->SyncStatus;
                  if (SI->SyncStatus==0) TimeQuality=2;
                  if (SI->SyncStatus==1) TimeQuality=1;
                  if (SI->SyncStatus==2) TimeQuality=3;
                  ReleaseMutex(SI->hSyncStatusMutex);
               } else {
                  // The clock is NOT syncronized. 
                  TimeQuality=0;
               }
            }

            // Find the time to be inserted into the block header.
            // Since this block is bad, the system clock must be 
            // used for timetagging.
            TimeTag=ReadTime;                           // Use the time when the block was read.
            SubstractFromTime(&TimeTag, TransferTime+1.0); // Compensate for the transfer time + 1 sec. 
            // Apply additional user specified time correction.
            if (SI->TimeCorrection>0.0) AddToTime(&TimeTag, SI->TimeCorrection);
            if (SI->TimeCorrection<0.0) SubstractFromTime(&TimeTag, SI->TimeCorrection);

            // Create a block header and insert in all channels.
            DBH.BlockID=BlockNr;
            DBH.TimeTag=TimeTag;
            DBH.TimeQuality=TimeQuality;
            DBH.NullData=TRUE;
            DBH.DataLost=DataLost;
            DBH.EventDetected=FALSE;
            DBH.EventTime=0;
            bPtr=((BYTE*)DataBlocksCh1)+(BlockSize*BlockNr);
            CopyMemory((VOID*)bPtr, (VOID*)&DBH, (DWORD)sizeof(BLOCK_HEADER));
            if (NumChannels==3) {
               bPtr=((BYTE*)DataBlocksCh2)+(BlockSize*BlockNr);
               CopyMemory((VOID*)bPtr, (VOID*)&DBH, (DWORD)sizeof(BLOCK_HEADER));
               bPtr=((BYTE*)DataBlocksCh3)+(BlockSize*BlockNr);
               CopyMemory((VOID*)bPtr, (VOID*)&DBH, (DWORD)sizeof(BLOCK_HEADER));
            }

            // Insert null data into current block.
            if (NumChannels==1) ZeroMemory((PVOID)((PBYTE)DataBlocksCh1+OffsetToBlockData), SampleRate*4);
            if (NumChannels==3) {
               ZeroMemory((PVOID)((PBYTE)DataBlocksCh1+OffsetToBlockData), SampleRate*4);
               ZeroMemory((PVOID)((PBYTE)DataBlocksCh2+OffsetToBlockData), SampleRate*4);
               ZeroMemory((PVOID)((PBYTE)DataBlocksCh3+OffsetToBlockData), SampleRate*4);
            }

            // Inform parent thread that a bad block was received.
            PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_BADBLOCK, 0, 0);
         }


         // Return block nr for updated block.
         *SI->BlockNr=BlockNr;

         // Signal parent process that buffer is updated.
         PostThreadMessage(SI->ThreadID, TM_BUFFERUPDATED, 0, 0); 

         // Advance to the next block.
         BlockNr++; 
         if (BlockNr==NumBlocksPrChannel) BlockNr=0;

         // If fix quality has changed, inform parent process.
         if (FixStatus!=OldFixStatus) {
            OldFixStatus=FixStatus;
            if (FixStatus) PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_FIXOK, 0, 0);
            if (!FixStatus) PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_NOFIX, 0, 0);
         }

         // If data stream has just been restored,then inform parent thread.
         if (DataLost) {
            DataLost=FALSE;
            PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_DATAINPUTOK, 0, 0); 
         }

      } else {
         // Failed to read bytes.The digitizer has been
         // disconnected, or an other comm error has occured.
         if (DataLost==FALSE) {
            // Signal parent process that we have no data input.
            PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_NODATAINPUT, 0, 0); 
            DataLost=TRUE;
         }

         // Wait 50ms before trying a new read.
         if (!ReadStatus) Sleep(50);
      }
   }
   
   ExitThread(0);
   return 0L;
   
}





/*******************************************************************/
/*    Configuration dialog box procedure for GBVx16.               */
/*******************************************************************/
BOOL CALLBACK ConfigDlgProcGBVx16(HWND hWndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
   SHINITDLGINFO shidi; // New in PocketPC version
   HWND hwndDlgItem;
   DWORD dwIndex;
   INT Result;
   static DEVICE_CONFIG *DC;
   TCHAR PortNr, Str20[21], Message[100];
   TCHAR Ports[16][6]={TEXT("COM1"),TEXT("COM2"), TEXT("COM3"), TEXT("COM4"), TEXT("COM5"), TEXT("COM6"), TEXT("COM7"), TEXT("COM8")
                    , TEXT("COM9"),TEXT("COM10"),TEXT("COM11"),TEXT("COM12"),TEXT("COM13"),TEXT("COM14"),TEXT("COM15"),TEXT("COM16")};

   switch(message)  {
      case WM_INITDIALOG:

 	  // Create a Done button and size it. New in PocketPC version.
 	  shidi.dwMask = SHIDIM_FLAGS;
	  shidi.dwFlags = SHIDIF_SIPDOWN | SHIDIF_SIZEDLGFULLSCREEN;
	  shidi.hDlg = hWndDlg;
	  SHInitDialog(&shidi);

      // Create pointer to the device config structure.
      DC=(DEVICE_CONFIG*)lParam;

      // Initialize the 'Model' combo box.
      hwndDlgItem=GetDlgItem(hWndDlg, IDC_MODEL);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("GBV-116"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 116);
      if (!wcscmp(DC->Model, TEXT("GBV-116"))) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("GBV-316"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 316);
      if (!wcscmp(DC->Model, TEXT("GBV-316"))) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);

      // Initialize the 'Device is connected to' combo box.
      hwndDlgItem=GetDlgItem(hWndDlg, IDC_PORT);
      for (PortNr=0;PortNr<16;PortNr++) {
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)Ports[PortNr]);
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, PortNr);
         if (!wcscmp(DC->PortName, Ports[PortNr])) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }

      // Initialize the 'Comm speed' combo box.
      hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMM_SPEED);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("9600"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 9600);
      if (!wcscmp(DC->CommSpeed, TEXT("9600"))) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("19200"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 19200);
      if (!wcscmp(DC->CommSpeed, TEXT("19200"))) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("38400"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 38400);
      if (!wcscmp(DC->CommSpeed, TEXT("38400"))) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);

      // Initialize the 'sample rate' combo box.
      hwndDlgItem=GetDlgItem(hWndDlg, IDC_SAMPLE_RATE);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("25"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 25);
      if (DC->SampleRate==25) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("50"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 50);
      if (DC->SampleRate==50) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("100"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 100);
      if (DC->SampleRate==100) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("200"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 200);
      if (DC->SampleRate==200) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);

      // Initialize the 'time correction' edit box.
      swprintf(Str20, TEXT("%.3f"), DC->TimeCorrection);
      SetDlgItemText(hWndDlg, IDC_TIME_CORR, Str20);

      // Initialize the 'Syncronize system clock.' check box.
      CheckDlgButton(hWndDlg, IDC_SYNC_SYS_CLK, BST_UNCHECKED); 
      if (DC->SyncPCClock) CheckDlgButton(hWndDlg, IDC_SYNC_SYS_CLK, BST_CHECKED); 

      // Initialize the 'Use system clock for time tagging.' check box.
      CheckDlgButton(hWndDlg, IDC_USE_SYS_CLK, BST_UNCHECKED); 
      if (DC->UsePCClock) CheckDlgButton(hWndDlg, IDC_USE_SYS_CLK, BST_CHECKED); 
      
      // Bring window to front.
      //BringWindowToTop(hWndDlg);
	  SetForegroundWindow(hWndDlg);
      return TRUE;


      case WM_COMMAND:
      switch(wParam) {

		   case IDC_SYNC_SYS_CLK:
         // This is the 'Syncronize system clock.' checkbox.
         switch (HIWORD(wParam)) { 
            case BN_CLICKED:
				// If checked, uncheck the 'Use system clock for time tagging.' check box.
            if (IsDlgButtonChecked(hWndDlg, IDC_SYNC_SYS_CLK)==BST_CHECKED) 
					CheckDlgButton(hWndDlg, IDC_USE_SYS_CLK, BST_UNCHECKED);
         }
         break;


			case IDC_USE_SYS_CLK:
         // This is the 'Use system clock for time tagging.' checkbox.
         switch (HIWORD(wParam)) { 
            case BN_CLICKED:
				// If checked, uncheck the 'Syncronize system clock.' check box.
            if (IsDlgButtonChecked(hWndDlg, IDC_USE_SYS_CLK)==BST_CHECKED) 
					CheckDlgButton(hWndDlg, IDC_SYNC_SYS_CLK, BST_UNCHECKED); 
         }
         break;


         case IDOK:  // User has pressed the OK button.

         // Read current choice from the 'Model' combo box.
         hwndDlgItem=GetDlgItem(hWndDlg, IDC_MODEL);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         SendMessage(hwndDlgItem, CB_GETLBTEXT, dwIndex, (LPARAM)&DC->Model);

         // Read current choice from the 'Device is connected to' combo box.
         hwndDlgItem=GetDlgItem(hWndDlg, IDC_PORT);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         SendMessage(hwndDlgItem, CB_GETLBTEXT, dwIndex, (LPARAM)&DC->PortName);

         // Read current choice from the 'Comm speed' combo box.
         hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMM_SPEED);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         SendMessage(hwndDlgItem, CB_GETLBTEXT, dwIndex, (LPARAM)&DC->CommSpeed);
   
         // Read current choice from the sample rate combo box.
         hwndDlgItem=GetDlgItem(hWndDlg, IDC_SAMPLE_RATE);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         DC->SampleRate=(BYTE)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

         // Read current choices from the time-correction edit box.
         GetDlgItemText(hWndDlg, IDC_TIME_CORR, Str20, 20);
         Result=swscanf(Str20, TEXT("%Lf"), &DC->TimeCorrection);
         if (Result==0 || Result==EOF) {
            // A string that cannot be converted to a float value is entered.
            wcscpy(Message, TEXT("The time correction value is missing or invalid.\n"));
            MessageBox(hWndDlg, Message, TEXT("Seislog message"), MB_OK|MB_ICONERROR);

            // Move the input focus to this field.
            SetFocus(GetDlgItem(hWndDlg, IDC_TIME_CORR));
            SendDlgItemMessage(hWndDlg, IDC_TIME_CORR, EM_SETSEL, 0, -1); 

            // Return without closing.
            return TRUE;            
         }

         // Read current choices from the 'Syncronize system clock.' check box.
         DC->SyncPCClock=(IsDlgButtonChecked(hWndDlg, IDC_SYNC_SYS_CLK)==BST_CHECKED); 

         // Read current choices from the 'Use system clock for time tagging.' check box.
         DC->UsePCClock=(IsDlgButtonChecked(hWndDlg, IDC_USE_SYS_CLK)==BST_CHECKED);
 
         // Close the dialog box
         EndDialog(hWndDlg , 1);
         return TRUE;
         
         
         case IDCANCEL:  // User has pressed the Cancel button.
         // Do nothing, just exit.
         EndDialog(hWndDlg , 0);
         return TRUE;      
      }
      return FALSE;

   }
   return FALSE;
}





/*******************************************************************/
/*    Checks if the string from the device looks OK.               */
/*******************************************************************/

BOOL GBVx16BytesOK(PBYTE Block, DWORD BlockSize)
{
   WORD CheckSum, BlockCheckSum, Index;
   
   // Check that string starts with 'GB'.
   if (*(Block+0)!='G') return FALSE;
   if (*(Block+1)!='B') return FALSE;

   // Calculate checksum.
   CheckSum=0;
   for (Index=0;Index<BlockSize-6;Index++) CheckSum=CheckSum+*(Block+Index);

   // Compare with checksum given in the block.
   BlockCheckSum=*(Block+BlockSize-6)*256;
   BlockCheckSum=BlockCheckSum+*(Block+BlockSize-5);
   if (CheckSum!=BlockCheckSum) return FALSE;

   // No problems detected, return TRUE.
   return TRUE;
}


/**************************************************************/
/* Gets the syncronization status from the block and returns  */
/* one of 4 values: 0=RTC error  								     */
/*                  1=RTC only, no fix since power on         */ 
/*                  2=Fix Lost  (we have had fix)             */
/*                  3=Fix OK                                  */
/**************************************************************/
BYTE GetGBVx16SyncStatus(PBYTE Block)
{
   BYTE TSI, RetValue;

   // Get the time-system indication byte.
   TSI=Block[49];

   RetValue=0;
	if (TSI==4) RetValue=1;
   if (TSI==5) RetValue=2;
	if (TSI==6) RetValue=1;
	if (TSI==7) RetValue=3;
   
   // Return. 
   return RetValue;
}


