// Ported from Seislog for Windows to SeislogCE. May 2004 by Jos Aa. Ojeda

#include "stdafx.h"
#include <aygshell.h>

#include <windows.h>
#include <stdio.h>
#include "seislog.h"
#include "kmk2.h"
#include "library.h"
#include "msglist.h"
#include "resource.h"
#include "process.h"
#include "eventlog.h"

#define WINCE
//
// This structure is used to pass startup- 
// information to the serial-reader thread.
//
struct STARTINFO_DEVREADER { 
   DWORD ThreadID;            // Thread-ID for parent thread.
   HANDLE hComm;              // Handle of communications device 
   TCHAR CommSpeed[6];         // Communication speed.
   DOUBLE TimeCorrection;     // Time correction to apply (in seconds).
   BOOL *SerialRead;          // The serial-reader thread will exit if this becomes FALSE.
   DEVMNGR_INTERF_STRUCT *DS; // Pointer to the exported structure.
   DWORD* BlockNr;            // Pointer to variable who are to receive block numbers.
   TCHAR UniqueDeviceName[65]; // Unique name for this instance of the device driver.
   BOOL SyncPCClock;          // If TRUE then device reader thread will synchronize the PC clock.
};


//
// This structure is read from the binary device config file.
//
struct DEVICE_CONFIG {
   WORD DeviceID;          // An identifier for this particular model.
   BYTE NumChannels;       // Number of channels (1 to 12)
   TCHAR PortName[6];       // Name of COM port (COM1 to COM16).
   TCHAR CommSpeed[7];      // Communication speed (9600 to 57600 baud).
   DOUBLE TimeCorrection;  // Time correction (seconds).
   BOOL SyncPCClock;       // If TRUE then device driver will synchronize the system clock.
};


struct PACKET_HEADER {
   /*unsigned*/ TCHAR  typeCode;
   /*unsigned*/ TCHAR  seqNr;
   unsigned short source;
   unsigned short destination;
   unsigned short dataLength;
   /*unsigned*/ TCHAR  checksum;
};


struct PACKET_DATA_HEADER {
   unsigned short SerialNr;
   /*unsigned*/ TCHAR StreamNr;
   /*unsigned*/ TCHAR Status;
   unsigned long DataSeqNr;
   unsigned long TimeTag;
   unsigned short MilliSec;
   unsigned short PadBytes;
};


union L_SAMPLE{
  BYTE byte[4];
  long value;
};




//
// Global variables.
//
extern HWND hWndFrame;
extern BYTE DebugLevel;
extern BOOL StartGPSSyncThread;
extern DWORD SysClkSyncThreadID;


//
// Function prototyping.
//
DWORD WINAPI ReaderThreadKMK2(PVOID);
BOOL CALLBACK ConfigDlgProcKMK2(HWND, UINT, WPARAM, LPARAM);
DWORD ReadK2Packet(HANDLE, PBYTE, WORD, DWORD);
WORD ReplaceEquivalenTCHARs(PBYTE, WORD, PBYTE);
BOOL KMK2BytesOK(PBYTE, DWORD);
BOOL VerifyByteChecksum(PBYTE, BYTE, DWORD);
BOOL VerifyWordChecksum(PBYTE, WORD, DWORD);
VOID ExtractPacketSamples(long*, PBYTE);


/* *********************************************************************** */
/*  Manager thread for device Kinemetrics K2. DeviceID=500.                */
/* *********************************************************************** */
DWORD WINAPI KMK2MngrThrd(PVOID StartupInfo) 
{
   MSG msg;
   INT Status;
   UINT Timer;
   BYTE Index, SyncStatus, FixStatus;
   BYTE PKC_GETSTATUS[14]={0xC0,3,0,0,0,0,0,0,0,3,0,0,0xC0,0xC0};
   TCHAR ConfigFile[MAX_PATH+1], ChNr, UniqueDeviceName[65], Message[100];
   TCHAR DeviceResources[256], str10[11];
   WORD BlockSize, NumBlocksPrChannel, SampleRate;
   DWORD BytesWritten;
   BOOL SerialRead, MemAlloc, ConfigRead;
   HANDLE hReaderThread, hComm;
   STARTINFO_DEVMANAGER SI;
   STARTINFO_DEVREADER StartInfo;
   DEVICE_CONFIG DeviceConf, DevConfDlg;
   DEVMNGR_INTERF_STRUCT DS;
   DWORD ParentThreadID, CurrentThreadID, ReaderThreadID;
   DWORD ChannelSize, status, BlockNr;
   MSG_PARAM ID, OD;
   MsgList NotifyList;
   
   //
   // Perform thread initialization.
   //
   FixStatus=0;
   SyncStatus=2;
   hReaderThread=NULL;                               // The serial-reader thread is not running.
   CopyMemory((PVOID)&SI, StartupInfo, sizeof(SI));  // Copy startup information to local buffer.
   ParentThreadID=SI.ThreadID;                       // ThreadID for parent thread.
   CurrentThreadID=GetCurrentThreadId();             // ThreadID for this thread.
   

   //
   // Handle device configuration.
   //
   // Read configuration file.
   ConfigRead=FALSE;
   wcscpy(ConfigFile, SI.DeviceDir); 
   wcscat(ConfigFile, TEXT("\\device.cnf"));
   if (ExistFile(SI.DeviceDir, TEXT("device.cnf"))) {
      // Read device config from config file.
      ConfigRead=ReadFileToMemory(ConfigFile, (PVOID)&DeviceConf, (DWORD)sizeof(DeviceConf));
   }
   if (!ConfigRead) {
      // A valid config file could not be read.
      // Set up a default digitizer configuration.
      DeviceConf.DeviceID=500;               // K2 uses device id 500.
      DeviceConf.NumChannels=4;              // Assume setup for 4 channels.
      wcscpy(DeviceConf.PortName, TEXT("COM1"));   // Assume digitizer on COM1.
      wcscpy(DeviceConf.CommSpeed, TEXT("38400")); // Assume comm speed of 38400 baud.
      DeviceConf.TimeCorrection=0.0;         // Default time correction (seconds).
      DeviceConf.SyncPCClock=FALSE;          // Assume no syncing of PC clock.

      // Write settings to config file.
      WriteFileFromMemory(ConfigFile, (PVOID)&DeviceConf, (DWORD)sizeof(DeviceConf));
   }


   // 
   // Create a unique device instance name.
   //
   wcscpy(UniqueDeviceName, TEXT("K2 at "));
   wcscat(UniqueDeviceName, DeviceConf.PortName);

   //
   // Register this process in the process database.
   //
   if (!SI.ConfigOnly) RegisterProcess(UniqueDeviceName, 1, CurrentThreadID);

   //
   // Create device resource string.
   // 
   wcscpy(DeviceResources, TEXT("Serial ports: "));
   wcscat(DeviceResources, DeviceConf.PortName);

   //
   // Handle time synchronization stuff.
   //
   StartInfo.SyncPCClock=FALSE;
   if (!SI.ConfigOnly && DeviceConf.SyncPCClock) {
      // Device is configured to synchronize the system clock.
      // Check if any other thread is already doing this.
      if (SysClkSyncThreadID) {
         // Turn off time synchronication. Clock is 
         // already beeing synchronized by another thread.
         DeviceConf.SyncPCClock=FALSE;
         // Write warning in the system log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Cannot synchronize system clock. Clock already synchronized."));
         WriteSystemLog(Message, 2);
      } else {
         // System clock is not synchronized. This thread will attempt to synchronize it.
         StartInfo.SyncPCClock=TRUE;
         SysClkSyncThreadID=CurrentThreadID;
         // Write text on status bar.
         DrawStatusBarText(SyncStatus, UniqueDeviceName);
         // Write message in the system log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Synchronizing system clock."));
         WriteSystemLog(Message, 1);
      }
   }

      
   //
   // Calculate sample-rate, blocksize, channelsize and number of blocks pr. channel.
   //
   SampleRate=100; /* Sample rate is fixed on the K2 */
   BlockSize=(WORD)(sizeof(BLOCK_HEADER)+(SampleRate*4));
   NumBlocksPrChannel=60;
   ChannelSize=NumBlocksPrChannel*BlockSize;


   // Open the serial port.
   if (!SI.ConfigOnly) {
      #ifdef WINCE
      // All ports in Windows CE must end with ':'
      TCHAR CEPortName[7];
      wcscpy(CEPortName, DeviceConf.PortName);
      wcscat(CEPortName, TEXT(":"));
      hComm=CreateFile(CEPortName, GENERIC_READ|GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
      #else 
	  TCHAR SerPort[10];
      wcscpy(SerPort, TEXT("\\\\.\\")); wcscat(SerPort, DeviceConf.PortName);
      hComm=CreateFile(SerPort, GENERIC_READ|GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
      #endif
      if (hComm==INVALID_HANDLE_VALUE) {
         // Update program log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Unable to open communication port "));
         wcscat(Message, DeviceConf.PortName);
         wcscat(Message, TEXT("."));
         WriteSystemLog(Message, 3);
      }
   }

   
   //
   // Initialize the interface structure.
   //
   // Insert manufacturer name and model.
   wcscpy(DS.Manufacturer, TEXT("Kinemetrics Inc."));
   wcscpy(DS.Model, TEXT("K2"));

   // Insert samplerate.
   DS.SampleRate=SampleRate;

   // Insert number of samples in each block.
   DS.SamplesPrBlock=SampleRate;

   // Insert number of channels.
   DS.NumChannels=DeviceConf.NumChannels;

   // Insert block size, block header included.
   DS.BlockSize=BlockSize;

   // Insert block size measured in milliseconds.
   DS.BlockSizeInMS=1000;

   // Insert number of blocks pr channel.
   DS.NumBlocksPrChannel=NumBlocksPrChannel;

   // Make sure all pointers to channel data is NULL.
   for (ChNr=0;ChNr<MAX_CHANNELS_PR_DEVICE;ChNr++) DS.ChannelData[ChNr]=(PVOID)NULL;
   
   
   //
   // Allocate memory for the channel buffers, and set up the pointers to the buffers.
   //
   MemAlloc=TRUE;
   for (Index=0;Index<DeviceConf.NumChannels;Index++) {
      if ((DS.ChannelData[Index]=(PVOID)LocalAlloc(LPTR, (UINT)ChannelSize))==NULL) MemAlloc=FALSE;
   }

   // Check that memory allocation succeded.
   if (!MemAlloc) {
         // Update program log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(":"));
         wcscat(Message, TEXT(" Could not allocate memory."));
         WriteSystemLog(Message, 3);
   }
   

   //
   // If running in debug mode, write a message to the log.
   //
   if (DebugLevel) {
      wcscpy(Message, UniqueDeviceName);
      wcscat(Message, TEXT(": Thread started."));
      WriteSystemLog(Message, 1);
   }
   

   //
   // Signal that the thread is started.
   //
   SetEvent(SI.StartEvent);


   //
   // Enter the thread message loop. The thread will now start to process messages.
   //
   while (GetMessage(&msg, (HWND)NULL, 0, 0)) 
   { 
   
      // If debuglevel is 2 then log all messages recieved.
      if (DebugLevel>1) {
         TCHAR str10[11];
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Recieved message "));
         _itow(msg.message, str10, 10);
         wcscat(Message, str10);
         WriteSystemLog(Message, 1);
      }

      // Save message parameters.
      ZeroMemory((PVOID)&ID, (DWORD)sizeof(ID));
      if (msg.lParam) ID=*((MSG_PARAM*)msg.lParam);
      
      // Initialize return-message parameter to default values.
      ZeroMemory((PVOID)&OD, (DWORD)sizeof(OD));
      OD.bStatus=TRUE;
      OD.idThread=CurrentThreadID;


      // Process message.
      switch (msg.message) {
         case TM_START:                            
         //
         // This message is sendt from a higher level 
         // thread to start reading the serial port.
         //
         if (hReaderThread==NULL && hComm!=INVALID_HANDLE_VALUE && MemAlloc && !SI.ConfigOnly) {

            // Set up startup information for the serial reader thread routine.
            SerialRead=TRUE;
            StartInfo.ThreadID=CurrentThreadID;
            StartInfo.hComm=hComm;
            wcscpy(StartInfo.CommSpeed, DeviceConf.CommSpeed);
            StartInfo.TimeCorrection=DeviceConf.TimeCorrection;
            StartInfo.SerialRead=&SerialRead;
            StartInfo.DS=&DS;
            StartInfo.BlockNr=&BlockNr;
            wcscpy(StartInfo.UniqueDeviceName, UniqueDeviceName);
            
            // Start the thread.
            hReaderThread=CreateThread(NULL, 0, ReaderThreadKMK2, (VOID*)&StartInfo, CREATE_SUSPENDED, &ReaderThreadID);
            SetThreadPriority(hReaderThread, THREAD_PRIORITY_ABOVE_NORMAL);            
            ResumeThread(hReaderThread); 

            // Make Windows send a WM_TIMER message every 3000 ms.
            // When this message is received, a PKC_GETSTATUS packet is
            // sendt to the K2 device. K2 then responds by returning
            // a PKR_STATUS packet, which holds the current K2 system
            // time. This time is then used to synchronize the PC clock.
            Timer=SetTimer((HWND)NULL, 0, 3000, NULL); 
         }

         // Set return values. 
         if (!hReaderThread) {
            OD.bStatus=FALSE;
            OD.wStatus=1;
         }
         break;

         
         case TM_STOP:
         //
         // This message is sendt from a higher level thread to stop
         // reading the serial port. Returns the default return values.
         //
         if (hReaderThread) {
            // Stop the WM_TIMER message.
            KillTimer(NULL, Timer);

            // Make the serial thread loop exit, thereby stopping the thread.
            SerialRead=FALSE;

            // Wait up to 2 seconds for the thread to stop.
            status=WaitForSingleObject(hReaderThread, 2000);
            if (status==WAIT_TIMEOUT) TerminateThread(hReaderThread, 0);

            // The thread is now stopped, invalidate the thread handle.
				CloseHandle(hReaderThread);
            hReaderThread=NULL;
         }
         break;


         case TM_CONFIGURE:            
         //
         // This message is sendt from a higher level thread
         // when it wants to configure the digitizer settings.
         //
         if (hReaderThread==NULL) {
            // Let user work on a copy of the current device settings.
            DevConfDlg=DeviceConf;

            // Display the dialog box.
            Status=DialogBoxParam(SI.hInstance, MAKEINTRESOURCE(IDD_CNF_KMK2), NULL, (DLGPROC)ConfigDlgProcKMK2, (LPARAM)&DevConfDlg);   

            // Save settings if requested.
            if (Status==1) {
               // Save the new configuration.
               DeviceConf=DevConfDlg;

               // Write configuration to disk.
               WriteFileFromMemory(ConfigFile, (PVOID)&DeviceConf, (DWORD)sizeof(DEVICE_CONFIG));

               // Create a unique device instance name.
               wcscpy(UniqueDeviceName, TEXT("K2 at "));
               wcscat(UniqueDeviceName, DeviceConf.PortName);
            }

            // Set return values. 
            if (Status==0) {
               OD.bStatus=FALSE;
               OD.wStatus=100;
            }
         }
         break;


         case TM_SHOW_CONFIG:
            //
            // When this message is received, dump all configuration
            // parameters that is not given in the device resource 
            // string to the system log.
            //
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(": Communication speed="));
            wcscat(Message, DeviceConf.CommSpeed);
            WriteSystemLog(Message, 1);
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(": Time correction="));
            swprintf(str10,TEXT("%.3f"), DeviceConf.TimeCorrection);
            wcscat(Message, str10);
            WriteSystemLog(Message, 1);
         break;


         case WM_TIMER:
         //
         // This message is sent to this thread once every 3000 ms.
         // When this message is received, we ask K2 to send us a 
         // PKR_PARMS packet.
         WriteFile(hComm, (PVOID)&PKC_GETSTATUS, 14, &BytesWritten, NULL); 
         break;


         case TM_GET_INTERFACE:
         //
         // This message is used to get a pointer to the interface structure.
         //
         OD.vPointer=(PVOID)&DS;
         break;   


         case TM_GET_UNIQUE_DEVICE_NAME:
         //
         // This message is used to get a string, 
         // identifying this unique device instance.
         //
         wcscpy(OD.cStatus, UniqueDeviceName);
         break;   


         case TM_GET_RESOURCE_STRING:
         //
         // This message is used to get a string, 
         // identifying the system resources used by this device.
         //
         wcscpy(OD.cStatus, DeviceResources);
         break;   


         case TM_SUBSCRIBE:
         //
         // This message is sendt from a window or thread
         // Who wants to be informed about an event.
         // This message must be accompanied by a MSG_PARAM structure.
         // The MSG_PARAM structure must have a pointer to a valid
         // SUBSCRIBE structure.
         //
         //
         // Add window/thread to notify list.
         if (ID.vPointer) if (!NotifyList.Subscribe((SUBSCRIBE*)ID.vPointer)) {
            OD.bStatus=FALSE;
            OD.wStatus=3;
         }
         break;   


         case TM_UNSUBSCRIBE:
         //
         // This message is sendt from a window or thread who
         // wants to remove a notification from the notify list.
         // This message must be accompanied by a MSG_PARAM structure.
         // The MSG_PARAM structure must have a pointer to a valid
         // SUBSCRIBE structure.
         //
         //
         // Remove notification from list.
         if (ID.vPointer) NotifyList.UnSubscribe((SUBSCRIBE*)ID.vPointer);
         
         break;   

         

         case TM_EXIT:                 
         //
         // This is a request for the thread to end.
         // Close down all processes that are depandent on this thread.
         // The 'StopDependantThreads' function will return immediatly,
         // and will send a 'TM_DEPTHREADS_STOPPED' message back when
         // all dependant threads have been stopped.
         if (!hReaderThread) StopDependantThreads();
         break;



         case TM_DEPTHREADS_STOPPED:
         //
         // This message indicates that all dependent threads have been stopped.
         // The thread will cleanup and exit when this message is recieved.         
         //

         if (!SI.ConfigOnly) {
            // Unregister this process from the process database.
            UnregisterProcess(CurrentThreadID);

            // Close the communication port.
            CloseHandle(hComm);

         }

         // Free memory buffers.
         for (Index=0;Index<DeviceConf.NumChannels;Index++) {
            if (DS.ChannelData[Index]!=NULL) LocalFree((HLOCAL)DS.ChannelData[Index]);
         }

         // End the message loop.
         ClearMessageQueue(); // Delete all messages in message queue.
         PostQuitMessage(0);  // Post WM_QUIT message.   
         break;
   
         
         //
         // Handle messages from device reader thread.
         //
         case TM_BUFFERUPDATED:  // A new block of data has been inserted into buffer.
         // Send Notification to all subscribed threads/windows.
         // The notofication is accompanied with this thread's thread-ID, and the blocknr of the updated block.
         NotifyList.Notify((UINT)TM_BUFFERUPDATED, (WPARAM)CurrentThreadID, (LPARAM)BlockNr);
         break;

  
         case TM_GET_TIMESYNC_STATUS:
         //
         // This message is sendt from a window or thread who
         // wants to know the status of the GPS time synchronization.
         // This message must be accompanied by a MSG_PARAM structure.
         //
         //
         // Returns the operational status of the thread.
         // Values are:
         //    0 :System clock is beeing synchronized, no error.
         //    1 :System clock is NOT beeing synchronized, but no error (fix lost).
         //    2 :System clock is NOT beeing synchronized, communication error.
         //
         OD.wStatus=(WORD)SyncStatus;
         break;
         

         case TM_DEVICEREADER_NODATAINPUT:   
         //
         // The device reader has lost data input.
         //
         // Send Notification to all subscribed threads/windows.
         NotifyList.Notify((UINT)TM_DEVICEREADER_NODATAINPUT, WPARAM(CurrentThreadID), 0);

         // Handle GPS stuff.
         if (DeviceConf.SyncPCClock) {
            SyncStatus=2;
            NotifyList.Notify((UINT)TM_TIMESYNC_STATUS, (WPARAM)SyncStatus, 0);
            DrawStatusBarText(SyncStatus, UniqueDeviceName);
         }

         // Write a message to the log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Not receiving data."));
         WriteSystemLog(Message, 2);
         break;


         case TM_DEVICEREADER_DATAINPUTOK:   
         //
         // The device reader has regained data input.
         //

         // Send Notification to all subscribed threads/windows.
         NotifyList.Notify((UINT)TM_DEVICEREADER_DATAINPUTOK, WPARAM(CurrentThreadID), 0);

         // Handle GPS stuff.
         if (DeviceConf.SyncPCClock) {
            SyncStatus=0;
            if (!FixStatus) SyncStatus=1;
            NotifyList.Notify((UINT)TM_TIMESYNC_STATUS, (WPARAM)SyncStatus, 0);
            DrawStatusBarText(SyncStatus, UniqueDeviceName);
         }

         // Write a message to the log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Receiving data."));
         WriteSystemLog(Message, 1);
         break;


         case TM_DEVICEREADER_BADBLOCK:   
         // A bad block of data was received from the device.
         // Send Notification to all subscribed threads/windows.
         // The notofication is accompanied with this thread's thread-ID, and the blocknr of the bad block.
         NotifyList.Notify((UINT)TM_DEVICEREADER_BADBLOCK, (WPARAM)CurrentThreadID, (LPARAM)BlockNr);

         // Write a message to the log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Received a bad block from device."));
         WriteSystemLog(Message, 2);
         break;


         case TM_DEVICEREADER_NOFIX:   
         //
         // The GPS reader-thread indicates that fix has been lost.
         //
			// Handle GPS stuff.
			if (DeviceConf.SyncPCClock) {
				FixStatus=0;
				if (!SyncStatus) SyncStatus=1;
				NotifyList.Notify((UINT)TM_TIMESYNC_STATUS, (WPARAM)SyncStatus, 0);
	         DrawStatusBarText(SyncStatus, UniqueDeviceName);
			}

         // Write a message to the log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Satellite fix lost."));
         WriteSystemLog(Message, 2);
         break;


         case TM_DEVICEREADER_FIXOK:   
         //
         // The GPS reader-thread indicates that fix has been regained.
         //
			// Handle GPS stuff.
			if (DeviceConf.SyncPCClock) {
				FixStatus=1;
				if (SyncStatus==1) SyncStatus=0;
				NotifyList.Notify((UINT)TM_TIMESYNC_STATUS, (WPARAM)SyncStatus, 0);
            DrawStatusBarText(SyncStatus, UniqueDeviceName);
			}
         
         // Write a message to the log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Satellite fix OK."));
         WriteSystemLog(Message, 1);
         break;
      }
      
      // If this is requested by the thread who sent the message
      // then return status values, and release the caller thread.
      if (((MSG_PARAM*)msg.lParam)!=NULL && msg.wParam!=NULL) {
         *(MSG_PARAM*)msg.lParam=OD;
         SetEvent((HANDLE)msg.wParam);
      }
   } 


   // Free up memeory allocated in the MsgList class object.
   NotifyList.~MsgList();

   // If running in debug mode, write a message to the log.
   if (DebugLevel) {
      wcscpy(Message, UniqueDeviceName);
      wcscat(Message, TEXT(": Thread stopped."));
      WriteSystemLog(Message, 1);
   }

   // Exit this thread
   ExitThread(0);
   return 0L;
   
}




/* *********************************************************************** */
/*  Serial port reader procedure for the Kinemetrics K2 device.            */
/* *********************************************************************** */
DWORD WINAPI ReaderThreadKMK2(PVOID StartupStruct)
{
   DCB dcb;
   VOID *DataBlocks;
   long Samples[100];
   PACKET_HEADER PH;
   BLOCK_HEADER *pDBH;
   PACKET_DATA_HEADER PDH;
   STARTINFO_DEVREADER *SI;
   TCHAR Message[128], CmpResult;
   BYTE BytesPrSample, Buffer[3*1024], Channel; 
   BYTE TimeQuality, FixStatus, OldFixStatus, *bPtr;
   BYTE PKC_GETPARMS[14]={0xC0,1,0,0,1,0,0,0,0,2,0,0,0xC0,0xC0};
   WORD BlockSize, NumChannels, NumBlocksPrChannel;
   WORD SampleNr, SampleRate, CheckSum;
   DWORD BytesWritten, OffsetToBlockData;
   DWORD BytesRead, BlockNr, OffsetSample;
   K2_HEADER Parms;
   STATUS_INFO StatusInfo;
   SYSTEMTIME PacketTime, LastPacketTime, K2Time, CurrTime;
   BOOL HeaderBytesOK, DataLost, PacketOK, DataBytesOK, Started;

   // Initialize some variables.
   BlockNr=0;              // Start filling data into first block. This variable is zero based.
   FixStatus=0;            // So far we have no satelite fix.
   OldFixStatus=0;         // We have no previous fix status.
   TimeQuality=0;
   DataLost=TRUE;          // At this point we have no data input.
   Started=FALSE;          // Do not start until packet for stream nr 0 is read.
   
   // Copy values from startup structure.
   SI=(STARTINFO_DEVREADER*)StartupStruct;         // Use pointer of correct type.
   NumChannels=SI->DS->NumChannels;                // Number of channels digitizer is using.
   BytesPrSample=3;                                // Number of bytes pr sample.
   SampleRate=SI->DS->SampleRate;                  // Sample rate.
   NumBlocksPrChannel=SI->DS->NumBlocksPrChannel;  // Number of 1 sec. blocks in buffer.
   BlockSize=SI->DS->BlockSize;                    // Size of 1 sec. block, header included

   //Initialize the communications parameters. 
   SetupComm(SI->hComm, 3072*2, 100);              // Set size of input/output buffers.
   GetCommState(SI->hComm, &dcb);
   #ifdef WINCE
     dcb.BaudRate  = _wtoi(SI->CommSpeed);
     dcb.Parity    = NOPARITY;
     dcb.StopBits  = ONESTOPBIT;
     dcb.ByteSize  = 8;
   #else
	 TCHAR DevCntrlStr[50];
     wcscpy(DevCntrlStr, TEXT("baud="));
     wcscat(DevCntrlStr, DeviceConf.CommSpeed);
     wcscat(DevCntrlStr, TEXT(" parity=N data=8 stop=1"));
     BuildCommDCB(DevCntrlStr, &dcb);
   #endif      
   dcb.fAbortOnError=FALSE;
   SetCommState(SI->hComm, &dcb);            // Set serial port mode.


   // Ask K2 to send us a PKR_PARMS packet.
   WriteFile(SI->hComm, (PVOID)&PKC_GETPARMS, 14, &BytesWritten, NULL);

   // Start reading the serial port.
   while(*SI->SerialRead) {
      
      // Try reading a packet into buffer. The read function
      // will also replace the special END/ESC EQU sequences.
      // A total read-timeout time of 1000 ms is used.
      BytesRead=ReadK2Packet(SI->hComm, Buffer, 3*1024, 1000);

      // Process packet.
      if (BytesRead) {

         // Extract and check the packet header.
         PacketOK=TRUE;
         CopyMemory((PVOID)&PH, (PVOID)((Buffer)+1), sizeof(PH));
         PH.source=SwapWordBytes(PH.source);
         PH.destination=SwapWordBytes(PH.destination);
         PH.dataLength=SwapWordBytes(PH.dataLength);
         HeaderBytesOK=VerifyByteChecksum((PBYTE)&PH, (BYTE)PH.checksum, 8);
         if (!HeaderBytesOK) PacketOK=FALSE;

         // Unpack the PKR_STRDATA packet.
         if (PacketOK && PH.typeCode==0x8A) {
            // Extract the packetdata header.
            CopyMemory((PVOID)&PDH, (PVOID)((Buffer)+10), sizeof(PDH));
            PDH.SerialNr=SwapWordBytes(PDH.SerialNr);
            PDH.MilliSec=SwapWordBytes(PDH.MilliSec);
            PDH.TimeTag=SwapDWordBytes(PDH.TimeTag);   
            PacketTime.wYear=1980;
            PacketTime.wMonth=1;
            PacketTime.wDay=1;
            PacketTime.wHour=0;
            PacketTime.wMinute=0;
            PacketTime.wSecond=0;
            PacketTime.wMilliseconds=PDH.MilliSec;
            AddToTime(&PacketTime, (DOUBLE)PDH.TimeTag);

            // Apply user specified time correction.
            if (SI->TimeCorrection>0.0) AddToTime(&PacketTime, SI->TimeCorrection);
            if (SI->TimeCorrection<0.0) SubstractFromTime(&PacketTime, SI->TimeCorrection);

            // Do not start until we receive 
            // packet for stream number zero.
            if (!Started && !PDH.StreamNr) {
               // We are starting on a new second, so we 
               // will start to process packets from here.
               Started=TRUE; LastPacketTime=PacketTime;

               // Prepare the NullData member of the block headers.
               // This is done in case any packet for this block is bad.
               OffsetToBlockData=(BlockNr*BlockSize)+sizeof(BLOCK_HEADER);
               for (Channel=0;Channel<NumChannels;Channel++) {
                  DataBlocks=SI->DS->ChannelData[Channel];
                  ZeroMemory((PVOID)((PBYTE)DataBlocks+OffsetToBlockData), SampleRate*4);
                  pDBH=(BLOCK_HEADER*)((PBYTE)DataBlocks+BlockSize*BlockNr);
                  pDBH->NullData=TRUE;
               }

               // Warn user if time looks incorrect.
               if (PacketTime.wYear<2000) {
                  wcscpy(Message, SI->UniqueDeviceName);
                  wcscat(Message, TEXT(": Device clock is incorrect!"));
                  WriteSystemLog(Message, 3);
               }
            }

            // Extract the packet samples.
            ExtractPacketSamples(Samples, (PBYTE)(Buffer+26));

            // Check packetdata bytes.
            CheckSum=*(PWORD)(Buffer+10+PH.dataLength);
            CheckSum=SwapWordBytes(CheckSum);
            DataBytesOK=VerifyWordChecksum((PBYTE)(Buffer+10), CheckSum, PH.dataLength);
            if (!DataBytesOK) PacketOK=FALSE;
         }

         // Process the packet if it is OK.
         if (PacketOK) {
            switch (PH.typeCode) {
               // This is the PKR_STATUS packet. This packet has the K2 system
               // time (nearest whole second). Adjust system clock as requested.
               case 0x83:
               if (SI->SyncPCClock) {
                  // This thread is synchronizing the PC clock.
                  // The accuracy of the synchronization is within 1000ms or so.
                  // Copy parameters to StatusInfo variable.
                  CopyMemory((PVOID)&StatusInfo, (PVOID)((Buffer)+10), sizeof(StatusInfo));
                  // Extract K2 time from status.
                  StatusInfo.systemTime=SwapDWordBytes(StatusInfo.systemTime);   
                  K2Time.wYear=1980; K2Time.wMonth=1;
                  K2Time.wDay=1; K2Time.wHour=0;
                  K2Time.wMinute=0; K2Time.wSecond=0;
                  K2Time.wMilliseconds=PDH.MilliSec;
                  AddToTime(&K2Time, (DOUBLE)StatusInfo.systemTime+0.7);

                  // Adjust system clock 10 ms if needed.
                  GetSystemTime(&CurrTime);
						if (TimeDiff(&CurrTime, &K2Time)<0.6) {
                     // Time difference is less then 600 ms. Adjust clock 10 ms.
							CmpResult=TimeCmp(&CurrTime, &K2Time);
							if (CmpResult==-1) AddToTime(&CurrTime, 0.01);
							if (CmpResult== 1) SubstractFromTime(&CurrTime, 0.01);
							SetSystemTime(&CurrTime);
                  } else {
                     // Time difference is greater then 600 ms. Set clock to K2 time.
                     SetSystemTime(&K2Time);
                  }
               }
               break;

               // This is the PKR_PARMS packet.
               case 0x81:
               // Copy parameters to Params variable.
               CopyMemory((PVOID)&Parms, (PVOID)((Buffer)+10), sizeof(Parms));

               // Get GPS status.
               TimeQuality=1;
               if (Parms.roParms.timing.gpsStatus&1) TimeQuality=0;
               if (!(Parms.roParms.timing.gpsStatus&2)) TimeQuality=0;
               if (Parms.roParms.timing.gpsStatus&4) TimeQuality=3;
               if (!(Parms.roParms.timing.gpsStatus&16)) TimeQuality=2;
               FixStatus=0; 
               if (TimeQuality==2) FixStatus=1;

               // If fix quality has changed, inform parent process.
               if (FixStatus!=OldFixStatus) {
                  OldFixStatus=FixStatus;
                  if (FixStatus) PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_FIXOK, 0, 0);
                  if (!FixStatus) PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_NOFIX, 0, 0);
               }
               break;

               
               // This is the PKR_STRDATA packet.
               case 0x8A:
               if (Started) {
                  // If we have found the start of a new second then send
                  // the old second, and prepare for the next second.
                  if (TimeDiff(&PacketTime, &LastPacketTime)!=0.0) {
                     // We have started on a new second.
                     // Prepare block headers for old second.
                     for (Channel=0;Channel<NumChannels;Channel++) {
                        DataBlocks=SI->DS->ChannelData[Channel];
                        pDBH=(BLOCK_HEADER*)((PBYTE)DataBlocks+BlockSize*BlockNr);
                        pDBH->BlockID=BlockNr;
                        pDBH->TimeTag=LastPacketTime;
                        pDBH->TimeQuality=TimeQuality;
                        pDBH->DataLost=DataLost;
                        pDBH->EventDetected=FALSE;
                        pDBH->EventTime=0;
                      }

                     // Return block nr for updated block.
                     *SI->BlockNr=BlockNr;

                     // Signal parent process that buffer is updated.
                     PostThreadMessage(SI->ThreadID, TM_BUFFERUPDATED, 0, 0); 

                     // Advance to the next block.
                     BlockNr++; 
                     if (BlockNr==NumBlocksPrChannel) BlockNr=0;

                     // Clear the blocks for the next second.
                     OffsetToBlockData=(BlockNr*BlockSize)+sizeof(BLOCK_HEADER);
                     for (Channel=0;Channel<NumChannels;Channel++) {
                        DataBlocks=SI->DS->ChannelData[Channel];
                        ZeroMemory((PVOID)((PBYTE)DataBlocks+OffsetToBlockData), SampleRate*4);
                        pDBH=(BLOCK_HEADER*)((PBYTE)DataBlocks+BlockSize*BlockNr);
                        pDBH->NullData=TRUE;
                     }

                     // Request GPS status information twice pr. minute.
                     if (LastPacketTime.wSecond==0 || LastPacketTime.wSecond==30) {
                        // Ask K2 to send us a PKR_PARMS packet.
                        WriteFile(SI->hComm, (PVOID)&PKC_GETPARMS, 14, &BytesWritten, NULL);
                     }
                  }

                  // Process incoming packet.
                  if (PDH.StreamNr<NumChannels) {
                     // We have received data for a valid stream.
                     DataBlocks=SI->DS->ChannelData[PDH.StreamNr];

                     // Update the NullData menber of the block header.
                     pDBH=(BLOCK_HEADER*)((PBYTE)DataBlocks+BlockSize*BlockNr);
                     pDBH->NullData=FALSE;

                     // Insert samples from digitizer into current block.
                     OffsetSample=BlockNr*BlockSize+sizeof(BLOCK_HEADER);
                     for (SampleNr=0;SampleNr<SampleRate;SampleNr++) {
                        // Process sample pointed to by 'SampleNr'.
                        bPtr=((PBYTE)DataBlocks)+OffsetSample;
                        *((PLONG)bPtr)=Samples[SampleNr];
                        // Move to next sample.
                        OffsetSample=OffsetSample+4;
                     }

                     // Save the time for this packet.
                     LastPacketTime=PacketTime;
                  }

                  // The packet was successfully read and processed.   
                  // If data stream has just been restored,then inform parent thread.
                  if (DataLost) {
                     DataLost=FALSE;
                     PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_DATAINPUTOK, 0, 0); 
                  }
               }
               break;
             }

         } else {
            //
            // The block from the digitizer does NOT seem to be OK.
            // Inform parent thread that a bad block was received.
            PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_BADBLOCK, 0, 0);
         }

      } else {
         // Failed to read bytes.The digitizer have 
         // been disconnected, or power has been lost.
         if (!DataLost) {
            DataLost=TRUE;
            Started=FALSE;
            PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_NODATAINPUT, 0, 0); 
         }
         // Wait 200ms before trying a new read.
         Sleep(200);           
      }
   }
   
   ExitThread(0);
   return 0L;
   
}





/*******************************************************************/
/*    This procedure extract the compressed packet samples.        */
/*******************************************************************/
VOID ExtractPacketSamples(long* Samples, PBYTE Buffer)
{
   BYTE Index, *pByte;
   L_SAMPLE LS;

   pByte=Buffer;
   for (Index=0;Index<100;Index++) {
      if (*pByte>=128) {
         // 24-bit sample.
         LS.byte[2]=*pByte;
         LS.byte[1]=*(pByte+1);
         LS.byte[0]=*(pByte+2);
         if ((LS.byte[2]&0x40)==0x40) {
            LS.byte[3]=0xFF;
         } else {
            LS.byte[3]=0x00;
            LS.byte[2]=LS.byte[2]&0x7F;
         }
         LS.value=LS.value<<1;
         Samples[Index]=LS.value;
         pByte=pByte+3;
      } else {
         // 15-bit sample.
         LS.byte[1]=*pByte;
         LS.byte[0]=*(pByte+1);
         if ((LS.byte[1]&0x40)==0x40) {
            LS.byte[3]=0xFF; 
            LS.byte[2]=0xFF; 
            LS.byte[1]=LS.byte[1]|0x80;
         } else {
            LS.byte[3]=0x00;
            LS.byte[2]=0x00; 
         }
         LS.value=LS.value<<1;
         Samples[Index]=Samples[Index-1]+LS.value;
         pByte=pByte+2;
      }
   }
}




/*******************************************************************/
/*    This procedure reads packets from the K2.                    */
/*******************************************************************/
DWORD ReadK2Packet(HANDLE hComm, PBYTE Buffer, WORD BufSize, DWORD TotalTimeOut)
{
   BYTE Byte, LastByte;
   DWORD NumBytesInBuffer, BytesRead, LastByteReadTicker;
   DWORD StartTicker, CurrentTicker, TimeElapsed;
   COMMTIMEOUTS ct={MAXDWORD, 0, 0, 0, 0};
   BOOL Finished, FindPacketStart;

   // Set comm timeouts. Readfile will return
   // immediatly even if no bytes was read.
   SetCommTimeouts(hComm, &ct);

   // Initialize variables.
   LastByte=0;
   Finished=FALSE;
   NumBytesInBuffer=0;
   FindPacketStart=TRUE;
   StartTicker=GetTickCount();
   

   // Read from comm port in a loop until we have read a complete
   // packet with packet frame TCHARacters at the start and end.
   while (!Finished) {
      
      // Read one byte at a time.
		ReadFile(hComm, (PVOID)&Byte, 1, &BytesRead, NULL);

      // Save the ticker count at this time.
      CurrentTicker=GetTickCount();
         
      // Handle the read result.
		if (BytesRead) {  

			// Save the time when the last byte was read.
			LastByteReadTicker=CurrentTicker;
         
         // Handle incoming byte.
	   	if (FindPacketStart) {
            // Search for the start of the packet.
            if (LastByte==0xC0 && Byte!=0xC0) {
               // Start of packet found.
               Buffer[0]=0xC0;
               Buffer[1]=Byte;
               FindPacketStart=FALSE;
               NumBytesInBuffer=2;
            }
         } else {
            // Insert byte into buffer.
            if (LastByte==0x5C && Byte==0xDD) {
               // END_EQUIVALENT sequence.
               Buffer[NumBytesInBuffer-1]=0xC0;
            } else if (LastByte==0x5C && Byte==0xDC) {
               // ESC_EQUIVALENT sequence.
               Buffer[NumBytesInBuffer-1]=0x5C;
            } else {
               Buffer[NumBytesInBuffer]=Byte;
               NumBytesInBuffer++;
            }
            // Look for end of packet.
            if (LastByte==0xC0 && Byte==0xC0) Finished=TRUE;
         }

         // Save the last byte read.
         LastByte=Byte;

         // Check for buffer overflow.
         if (NumBytesInBuffer==BufSize) {
            // This is an error situation. If this happens, we probably have
            // losed a packetframe TCHARacter. Clear buffer and start over.
            LastByte=0;
            Finished=FALSE;
            FindPacketStart=TRUE;
            NumBytesInBuffer=0;
         }

		} else { 
         // Nothing was read. Wait some ms for 
         // more bytes to arrive on the COM port.
			Sleep(20);
		}

		// We must abort if TotalTimeOut time has been reached.
		TimeElapsed=CalcElapsedTickerTime(StartTicker, CurrentTicker);
      if (TimeElapsed>=TotalTimeOut && !Finished) {
         Finished=TRUE;
         NumBytesInBuffer=0;
      }
	}

   // Return number of bytes read.
   return NumBytesInBuffer;
}




/*******************************************************************/
/*    Configuration dialog box procedure for the K2.               */
/*******************************************************************/
BOOL CALLBACK ConfigDlgProcKMK2(HWND hWndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
   SHINITDLGINFO shidi; // New in PocketPC version
   HWND hwndDlgItem;
   DWORD dwIndex;
   INT Result;
   static DEVICE_CONFIG *DC;
   TCHAR PortNr, Channel, Str20[21], Message[100];
   TCHAR Ports[16][6]={TEXT("COM1"),TEXT("COM2"), TEXT("COM3"), TEXT("COM4"), TEXT("COM5"), TEXT("COM6"), TEXT("COM7"), TEXT("COM8")
                    ,TEXT("COM9"),TEXT("COM10"),TEXT("COM11"),TEXT("COM12"),TEXT("COM13"),TEXT("COM14"),TEXT("COM15"),TEXT("COM16")};

   switch(message)  {
      case WM_INITDIALOG:
      // Create a Done button and size it. New in PocketPC version.
 	  shidi.dwMask = SHIDIM_FLAGS;
	  shidi.dwFlags = SHIDIF_SIPDOWN | SHIDIF_SIZEDLGFULLSCREEN;
	  shidi.hDlg = hWndDlg;
	  SHInitDialog(&shidi);
      // Create pointer to the device config structure.
      DC=(DEVICE_CONFIG*)lParam;
   
      // Initialize the 'Device is connected to port' combo box.
      hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO1);
      for (PortNr=0;PortNr<16;PortNr++) {
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCSTR)Ports[PortNr]);
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, PortNr);
         if (!wcscmp(DC->PortName, Ports[PortNr])) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }

      // Initialize the 'Communication speed is set to' combo box.
      hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO2);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCSTR)TEXT("9600"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 9600);
      if (!wcscmp(DC->CommSpeed, TEXT("9600"))) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCSTR)TEXT("19200"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 19200);
      if (!wcscmp(DC->CommSpeed, TEXT("19200"))) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCSTR)TEXT("38400"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 38400);
      if (!wcscmp(DC->CommSpeed, TEXT("38400"))) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCSTR)TEXT("57600"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 57600);
      if (!wcscmp(DC->CommSpeed, TEXT("57600"))) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);

      // Initialize the 'Number of channels active' combo box.
      hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO3);
      for (Channel=1;Channel<=12;Channel++) {
         _itow(Channel, Str20, 10);
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCSTR)Str20);
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, Channel);
         if (DC->NumChannels==Channel) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }

      // Initialize the 'time correction (seconds)' edit box.
      swprintf(Str20, TEXT("%.3f"), DC->TimeCorrection);
      SetDlgItemText(hWndDlg, IDC_TIME_CORR, Str20);

      // Initialize the 'synchronize system clock.' check box.
      CheckDlgButton(hWndDlg, IDC_CHECK1, BST_UNCHECKED); 
      if (DC->SyncPCClock) CheckDlgButton(hWndDlg, IDC_CHECK1, BST_CHECKED); 
	  // Bring window to front.
#ifdef WINCE
	    SetForegroundWindow(hWndDlg);
#else
      BringWindowToTop(hWndDlg);
#endif
      return TRUE;


      case WM_COMMAND:
      switch(LOWORD(wParam)) {
         case IDC_CHECK1:
         // This is the 'synchronize system clock.' checkbox.
         switch (HIWORD(wParam)) { 
            case BN_CLICKED:
				// If checked, uncheck the 'Use system clock for time tagging.' check box.
            if (IsDlgButtonChecked(hWndDlg, IDC_CHECK1)==BST_CHECKED) 
				       CheckDlgButton(hWndDlg, IDC_CHECK2, BST_UNCHECKED);
         }
         break;

         case IDOK:  // User has pressed the OK button.
         // Read current choice from the 'Device is connected to port' combo box.
         hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO1);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         SendMessage(hwndDlgItem, CB_GETLBTEXT, dwIndex, (LPARAM)&DC->PortName);

         // Read current choice from the 'Communication speed is set to' combo box.
         hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO2);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         SendMessage(hwndDlgItem, CB_GETLBTEXT, dwIndex, (LPARAM)&DC->CommSpeed);

         // Read current choice from the 'Number of channels active' combo box.
         hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO3);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         DC->NumChannels=(BYTE)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

         // Read current choice from the time correction (seconds) edit box.
         GetDlgItemText(hWndDlg, IDC_TIME_CORR, Str20, 20);
         Result=swscanf(Str20, TEXT("%Lf"), &DC->TimeCorrection);
         if (Result==0 || Result==EOF) {
            // A string that cannot be converted to a float value is entered.
            wcscpy(Message, TEXT("The time correction value is missing or invalid.\n"));
            MessageBox(hWndDlg, Message, TEXT("Seislog message"), MB_OK|MB_ICONERROR);

            // Move the input focus to this field.
            SetFocus(GetDlgItem(hWndDlg, IDC_TIME_CORR));
            SendDlgItemMessage(hWndDlg, IDC_TIME_CORR, EM_SETSEL, 0, -1); 

            // Return without closing.
            return TRUE;            
         }

         // Read current choices from the 'synchronize system clock.' check box.
         DC->SyncPCClock=(IsDlgButtonChecked(hWndDlg, IDC_CHECK1)==BST_CHECKED); 

         // Close the dialog box
         EndDialog(hWndDlg , 1);
         return TRUE;
         
         
         case IDCANCEL:  // User has pressed the Cancel button.
         // Do nothing, just exit.
         EndDialog(hWndDlg , 0);
         return TRUE;      
      }
      return FALSE;

   }
   return FALSE;
}



/*******************************************************************/
/*    This procedure verifyes the packet header checksum           */
/*******************************************************************/
BOOL VerifyByteChecksum(PBYTE Buffer, BYTE CheckSum, DWORD NumBytes)
{
   BYTE CheckSum1;
   DWORD StrPtr;

   // Calculate checksum for the first 'NumBytes' TCHARacter in buffer.
   CheckSum1=0;
   for (StrPtr=0;StrPtr<NumBytes;StrPtr++) CheckSum1=CheckSum1+*(Buffer+StrPtr);
   if (CheckSum!=CheckSum1) return FALSE;

   return TRUE;
}



/*******************************************************************/
/*    This procedure verifyes the packet data checksum             */
/*******************************************************************/
BOOL VerifyWordChecksum(PBYTE Buffer, WORD CheckSum, DWORD NumBytes)
{
   WORD CheckSum1;
   DWORD StrPtr;

   // Calculate checksum for the first 'NumBytes' TCHARacter in buffer.
   CheckSum1=0;
   for (StrPtr=0;StrPtr<NumBytes;StrPtr++) CheckSum1=CheckSum1+*(Buffer+StrPtr);
   if (CheckSum!=CheckSum1) return FALSE;

   return TRUE;
}



/*******************************************************************/
/*    Checks if the string from the device is OK.                  */
/*******************************************************************/
BOOL KMK2BytesOK(PBYTE Packet, DWORD PacketSize)
{
   WORD CheckSum, PacketCheckSum, Index;
   
   // Check that string starts with 'MOD\0'.
   if (wcscmp(TEXT("MOD"), (PTCHAR)Packet)) return FALSE;

   // Calculate checksum.
   CheckSum=0;
   for (Index=0;Index<PacketSize-2;Index++) CheckSum=CheckSum+*(Packet+Index);

   // Compare with checksum given in the block.
   PacketCheckSum=*(Packet+PacketSize-1)*256;
   PacketCheckSum=PacketCheckSum+*(Packet+PacketSize-2);
   if (CheckSum!=PacketCheckSum) return FALSE;

   // No problems detected, return TRUE.
   return TRUE;
}
