#include "stdafx.h"

//#define WINCE
// osv

#define POCKETPC

#ifdef POCKETPC
  #define WINCE  
#else
  #define WIN95
#endif

//#define STRICT
#define HEADER_HEIGHT 16

//Variables for Win95
#define XPOSCHNUM 2
#define XPOSSTATION 23
#define XPOSCOMPONENT 66
#define XPOSTIMEDATE 101
#define XPOSINLED 268
#define XPOSDISKLED 287
#define XPOSTRIGLED 306
#define XPOSSYNCLED 325
#define XPOSNETWRKTRIGLED 348
#define XPOSSAMPLE 447
#define XPOSSTA 600
#define XPOSLTA 678
#define XPOSOFFSET 761
#define XPOSRBFILE 769
#define XPOSFILTLO 859
#define XPOSFILTHI 908
#define XPOSDEVBIND 921

//New CE version of variables
#define CE_HEADER_HEIGHT 16	
#define CE_XPOSCHNUM 2			
#define CE_XPOSSTATION 35		
#define CE_XPOSCOMPONENT 89	
#define CE_XPOSINLED 136		
#define CE_XPOSDISKLED 155		
#define CE_XPOSTRIGLED 173		
#define CE_XPOSSYNCLED 192		
#define CE_XPOSTIMEDATE 230	
#define CE_XPOSENDTIMEDATE 390 //position where date & time rect ends
#define CE_XPOSSAMPLE 435		
#define CE_XPOSSTA 500			
#define CE_XPOSLTA 580			
#define CE_XPOSOFFSET 640		
#define CE_XPOSRBFILE 657  //720
#define CE_XPOSENDRBFILE 705		
#define CE_XPOSFILTLO 767		
#define CE_XPOSFILTHI 814		

#include <windows.h>
#include <stdio.h>
#include <string.h>
#include "mainmon.h"
#include "seislog.h"
#include "channel.h"
#include "library.h"
#include "process.h"
#include "eventlog.h"
#include "resource.h"

//
// Author: yvind Natvik 1998.
//
//
//

//
// Definition of structures used in this file only.
//
struct CHAN_PROP {
   CHANNEL_INFO_EX ChInfo;
   CHANNEL_STATUS ChStat;
   BUFF_PROP BuffProp;
   SYSTEMTIME TimeTag;
   signed long Sample;
   TCHAR RBFile[7];
   BYTE InLEDStatus;
   BYTE InLEDTimerCnt;  
   BYTE DiskLEDStatus;
   BYTE DiskLEDTimerCnt;   
   BYTE TrigLEDStatus;
   BYTE SyncLEDStatus;
   INT VertPos;
   DWORD CHThreadID;
   DWORD RBThreadID;
};


struct TRGSET_INFO {
   DWORD ThreadID;                     // ThreadID of the triggerset thread.
   BOOL Status;                        // Network trigger status. FALSE=No event active, TRUE=Event active.
   BOOL Channels[MAX_NUM_OF_CHANNELS]; // Array of bool's identifying which channels are 
                                       // included in the triggerset. Channels[0]=TRUE 
                                       // means Channel01 is included and so on.
};


//
// Theese are global variables used by functions in this file
//
extern HINSTANCE hInst;
extern HWND hWndFrame;
extern HWND hWndMDIClient;
extern HWND hWndMainMon;
extern TCHAR ProgDir[];
extern CHANNEL_INFO Channels[];
extern DWORD ProcKeeperThreadID;
extern BYTE DebugLevel;


// Global variable used in this file.
CHAN_PROP ChData[MAX_NUM_OF_CHANNELS];
TRGSET_INFO TriggerSetInfo[MAX_NUM_OF_TRIGGERSETS];
BYTE      NumRunningDevices;
BYTE      NumRunningChannels;
BYTE      NumRunningTriggersets;
DWORD     RunningDevices[MAX_NUM_OF_DEVICES];
DWORD     RunningChannels[MAX_NUM_OF_CHANNELS];
DWORD     RunningTriggersets[MAX_NUM_OF_TRIGGERSETS];

//
// Prototype declarations.
//
VOID CreateDateAndTimeString(PTCHAR, SYSTEMTIME*);
VOID CreateShortRBFilename(PTCHAR, DWORD);



/************************************************************************/
/* Main-monitor window thread.                                          */
/************************************************************************/
DWORD WINAPI MainMonThrdProc(PVOID StartupInfo) 
{
   MSG msg;
   BYTE Index, TrgSetNumber;
   TCHAR Message[100], UniqueProcName[65];
   DWORD CurrentThreadID;
   HANDLE hEvent;
   BUFF_PROP BuffProp;
   MSG_PARAM MsgParams;
   CHANNEL_INFO_EX ChInfo;
   CHANNEL_STATUS ChStat;
#ifdef WIN95
   TCHAR WindowFile[MAX_PATH+1]
   RECT rect;
   WINDOWPLACEMENT WndPlacement;  
   BOOL bStatus;
#endif
   STARTINFO_MAINMONITOR SI;


   // Initialize some variables.
   CopyMemory((PVOID)&SI, StartupInfo, sizeof(SI));   // Copy startup information to local buffer.
   CurrentThreadID=GetCurrentThreadId();              // Get and save threadID for this thread.


   // Create unique process-name for this process.
   wcscpy(UniqueProcName, TEXT("Activity Monitor"));


   // Get a list of all device managers running.
   MsgParams.vPointer=(PVOID)&RunningDevices;
   hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
   PostThreadMessage(ProcKeeperThreadID, TM_GET_DEVICEMANAGERS, (WPARAM)hEvent, (LPARAM)&MsgParams); 
   WaitForSingleObject(hEvent, INFINITE);
   NumRunningDevices=(BYTE)MsgParams.dwStatus;
   CloseHandle(hEvent);

   // Get a list of all channels running.
   NumRunningChannels=GetRunningChannels(RunningChannels);

   // Get a list of all triggerset threads running.
   MsgParams.vPointer=(PVOID)&RunningTriggersets;
   hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
   PostThreadMessage(ProcKeeperThreadID, TM_GET_EVENTDETECTIONS, (WPARAM)hEvent, (LPARAM)&MsgParams); 
   WaitForSingleObject(hEvent, INFINITE);
   NumRunningTriggersets=(BYTE)MsgParams.dwStatus;
   CloseHandle(hEvent);


   // Initialize the 'ChData' table.
   // This table contains different information about the running channels.
   ZeroMemory((PVOID)&ChData, (DWORD)sizeof(ChData));    // Clear the 'ChData' table.
   hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);         // Create an event object.
   for (Index=0;Index<NumRunningChannels;Index++) {
      // Get and save channel information for this channel.
      MsgParams.vPointer=(PVOID)&ChInfo;
      PostThreadMessage(RunningChannels[Index], TM_GET_CHANNEL_INFO, (WPARAM)hEvent, (LPARAM)&MsgParams); 
      WaitForSingleObject(hEvent, INFINITE);
      ChData[Index].ChInfo=ChInfo;

      // Get and save the channel status for this channel.
      MsgParams.vPointer=(PVOID)&ChStat;
      PostThreadMessage(RunningChannels[Index], TM_GET_CHANNEL_STATUS, (WPARAM)hEvent, (LPARAM)&MsgParams); 
      WaitForSingleObject(hEvent, INFINITE);
      ChData[Index].ChStat=ChStat;

      // Set the IN LED status to indicate that we are not receiving data.
      ChData[Index].InLEDStatus=2;

      // Set the Trig LED status to indicate that channel is not trigging.
      ChData[Index].TrigLEDStatus=0;

      // Set the Sync LED status to indicate that channel is not time syncronized.
      ChData[Index].SyncLEDStatus=0;

      // Save thread-id for channel.
      ChData[Index].CHThreadID=RunningChannels[Index];

      // Get and save information for this channel's memory buffer.
      MsgParams.vPointer=(PVOID)&BuffProp;
      PostThreadMessage(RunningChannels[Index], TM_GET_BUFFER_PROPERTIES, (WPARAM)hEvent, (LPARAM)&MsgParams); 
      WaitForSingleObject(hEvent, INFINITE);
      ChData[Index].BuffProp=BuffProp;

      // Save thread-id for channel's disk-ringbuffer.
      PostThreadMessage(RunningChannels[Index], TM_GET_RINGBUFFER, (WPARAM)hEvent, (LPARAM)&MsgParams); 
      WaitForSingleObject(hEvent, INFINITE);
      ChData[Index].RBThreadID=MsgParams.dwStatus;

      // Save the name of the current rb-file.
      if (ChData[Index].RBThreadID) {
         PostThreadMessage(ChData[Index].RBThreadID, TM_GET_RBFILE, (WPARAM)hEvent, (LPARAM)&MsgParams); 
         WaitForSingleObject(hEvent, INFINITE);
         wcscpy(ChData[Index].RBFile, MsgParams.cStatus);
      } else {
         wcscpy(ChData[Index].RBFile, TEXT("N/A"));
      }

   }
   CloseHandle(hEvent);


   // Initialize the TriggerSetInfo table.
   // This table containes information about the running triggerset threads.
   ZeroMemory((PVOID)&TriggerSetInfo, (DWORD)sizeof(TriggerSetInfo));   // Clear the 'TriggerSetInfo' table.
   hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);                        // Create an event object.
   for (Index=0;Index<NumRunningTriggersets;Index++) {
      // Get the triggerset number.
      PostThreadMessage(RunningTriggersets[Index], TM_GET_TRGSET_NUMBER, (WPARAM)hEvent, (LPARAM)&MsgParams); 
      WaitForSingleObject(hEvent, INFINITE);
      TrgSetNumber=(BYTE)MsgParams.dwStatus;

      // Save the thread-ID for this thread.
      TriggerSetInfo[TrgSetNumber-1].ThreadID=RunningTriggersets[Index];

      // Get the event status.
      PostThreadMessage(RunningTriggersets[Index], TM_GET_EVENT_STATUS, (WPARAM)hEvent, (LPARAM)&MsgParams); 
      WaitForSingleObject(hEvent, INFINITE);
      TriggerSetInfo[TrgSetNumber-1].Status=(BYTE)MsgParams.bStatus;

      // Get list off all (trig)channels included in this triggerset.
      MsgParams.vPointer=(PVOID)&TriggerSetInfo[TrgSetNumber-1].Channels;
      PostThreadMessage(RunningTriggersets[Index], TM_GET_CHANNELS, (WPARAM)hEvent, (LPARAM)&MsgParams); 
      WaitForSingleObject(hEvent, INFINITE);
   }
   CloseHandle(hEvent);

#ifdef WIN95
   // Create the MDI child window.
   GetClientRect(hWndFrame, &rect);
   hWndMainMon=CreateMDIWindow(TEXT("SeislogMainMonClass"), UniqueProcName, 0, 
                         0, 0, 494, 270, hWndMDIClient, hInst, NULL);

   // Open 'MainMonitor.wnd' file and read/set window-placement.
   AddPathAndFilename(WindowFile, ProgDir, TEXT("MainMonitor.wnd"));
   if (ExistFile(ProgDir, TEXT("MainMonitor.wnd"))) {
      bStatus=ReadFileToMemory(WindowFile, (PVOID)&WndPlacement, (DWORD)sizeof(WINDOWPLACEMENT));
      if (bStatus) SetWindowPlacement(hWndMainMon, &WndPlacement);
   } 
#endif

#ifdef WINCE
   // [PocketPC] specific code
	hWndMainMon = CreateWindow(TEXT("SeislogMainMonClass"),
		                             UniqueProcName,
									 WS_HSCROLL | WS_VSCROLL | WS_VISIBLE | WS_CHILD /*| WS_OVERLAPPED*/,
									 0,
									 0,
									 240,
									 140,
									 hWndFrame,
									 NULL,
									 hInst,
									 NULL);
#endif

   
   // Register this process in the process database.
   RegisterProcess(UniqueProcName, 4, CurrentThreadID);

   // Start displaying system activity.
   SendMessage(hWndMainMon, WM_STARTUP, 0 ,0);

   //
   // If running in debug mode, write a message to the log.
   //
   if (DebugLevel) {
      wcscpy(Message, UniqueProcName);
      wcscat(Message, TEXT(": Thread started."));
      WriteSystemLog(Message, 1);
   }

   //
   // Signal that the thread is started.
   //
   SetEvent(SI.StartEvent);
   
   // 
   // Enter the thread message loop. The thread will now start to process messages.
   //
   while (GetMessage(&msg, (HWND)NULL, 0, 0)) {

      // If debuglevel is 2 then log all message recieved.
      if (DebugLevel>1) {
         TCHAR str10[11];
         wcscpy(Message, UniqueProcName);
         wcscat(Message, TEXT(": Recieved message "));
         _itow(msg.message, str10, 10);
         wcscat(Message, str10);
         WriteSystemLog(Message, 1);
      }

      TranslateMessage(&msg); 
      DispatchMessage(&msg);

      // Handle the TM_EXIT message.
      if (msg.message==TM_EXIT)
		  SendMessage(hWndMainMon, WM_CLOSE, 0, 0);  
   } 


   // Unregister this process from the process database.
   UnregisterProcess(CurrentThreadID);

   // If running in debug mode, write a message to the log.
   if (DebugLevel) {
      wcscpy(Message, UniqueProcName);
      wcscat(Message, TEXT(": Thread stopped."));
      WriteSystemLog(Message, 1);
   }
   
   // Exit from thread.
   ExitThread(0);
   return 0L;
   
}









/*********************************************************************************/
/* Main monitor window procedure.                                                */
/*********************************************************************************/

LRESULT CALLBACK MainMonWndProc(HWND hWnd, UINT Message, WPARAM wParam, LPARAM lParam)
{

   HDC               hdc;
   RECT              rect;
   HRGN              hClipRgn;
   PAINTSTRUCT       ps;
   WORD              Index;
   HANDLE            hEvent;
   MSG_PARAM         MsgParams;
   BLOCK_HEADER      *BH;
   TCHAR              String[71], str6[7], str11[15];
#ifdef WIN95
   TCHAR             WindowFile[MAX_PATH+1];
   BYTE              Index2;
   HWND              hWndChildAct, hWndChildDeact;
   WINDOWPLACEMENT   WndPlacement;   
   UINT              OldTxtAlign;
#endif
   static HBITMAP    hBitmap1, hBMPNoLed, hBMPInactiveLed, hBMPRedLed, hBMPGreenLed;
   static int        CharHeight, LogicalWndSizeX, LogicalWndSizeY;
   static int        ClientSizeX, ClientSizeY, OffsetX, OffsetY;
   static int        OldClntSizeX, OldClntSizeY;
   static int        VscrollMax, HscrollMax, xPos, yPos;
   static HFONT      hFont;
   TEXTMETRIC        tm;
   SUBSCRIBE         SubscrData;
   BOOL              bVRedraw, bHRedraw, HaveDrawn;

#ifdef WINCE
   RECT              rectTxt; // Rectangle used to draw text
   LOGFONT           lf;      // Text attributes
#endif

   // In a MDI child window function, the following messages MUST be
   // handled by DefMDIChildProc, whether the child handles them 
   // or not: (MDI not used in POCKETPC version)
   //
   //    WM_CHILDACTIVATE, WM_GETMINMAXINFO, WM_MENUTCHAR,
   //    WM_MOVE, WM_SETFOCUS, WM_SIZE, WM_SYSCOMMAND.
   //
   
   // Process messages.
   switch (Message) {
      case WM_CREATE:

#ifdef WIN95
	  // Create the font used in the window.

	  hFont=CreateFont(15, 0, 0, 0, FW_NORMAL, 0, 0, 0, ANSI_CHARSET, OUT_DEVICE_PRECIS,
                       CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH, TEXT("Courier New"));

#endif
	  //[PocketPC]

#ifdef WINCE

  	  lf.lfHeight = 14;
	  lf.lfWidth = 0;
	  lf.lfEscapement = 0;
	  lf.lfOrientation = 0;
	  lf.lfWeight = FW_NORMAL;
	  lf.lfItalic = 0;
	  lf.lfUnderline = 0;
	  lf.lfStrikeOut = 0;
	  lf.lfCharSet = ANSI_CHARSET;
	  lf.lfOutPrecision = NULL;
	  lf.lfClipPrecision = CLIP_DEFAULT_PRECIS;
	  lf.lfQuality = DEFAULT_QUALITY;
	  lf.lfPitchAndFamily = FIXED_PITCH;
	  //lf.lfFaceName = strFace;
	  hFont = CreateFontIndirect(&lf);
#endif


      // Get the character height of the created font.
      hdc=GetDC(hWnd);
      SelectObject(hdc, hFont);
      GetTextMetrics(hdc, &tm);
      CharHeight=tm.tmHeight+tm.tmExternalLeading;
      ReleaseDC(hWnd, hdc);

      // Initialize some variables.
      OffsetX=0;
      OffsetY=0;
	  LogicalWndSizeX= 814; //1210 old value
      LogicalWndSizeY=HEADER_HEIGHT+NumRunningChannels*CharHeight;

      // Get the size of the window client area.
      GetClientRect(hWnd, &rect);
      ClientSizeX=rect.right+1;
      ClientSizeY=rect.bottom+1;

      // Load bitmaps.
      hBitmap1=LoadBitmap(hInst, MAKEINTRESOURCE(IDB_MAINMON1));
      hBMPNoLed=LoadBitmap(hInst, MAKEINTRESOURCE(IDB_NOLED));
      hBMPInactiveLed=LoadBitmap(hInst, MAKEINTRESOURCE(IDB_INACTIVELED));
      hBMPRedLed=LoadBitmap(hInst, MAKEINTRESOURCE(IDB_REDLED));
      hBMPGreenLed=LoadBitmap(hInst, MAKEINTRESOURCE(IDB_GREENLED));

      // The window is initially hidden.
      ShowWindow(hWnd, SW_HIDE);
      return 0L;


      case WM_STARTUP:
      // Create a timer that will send a message every 333 ms.
      SetTimer(hWnd, 0, 333, NULL); 

      // Create an event object.       
      hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);

      // Subscribe to TM_DISK_WRITE messages from all ringbuffers.
      for (Index=0;Index<NumRunningChannels;Index++) {
         if (ChData[Index].RBThreadID) Subscribe(TM_DISK_WRITE, ChData[Index].RBThreadID, 0, hWnd);
      }

      // Subscribe to TM_RB_FILECHANGE messages from all ringbuffers.
      for (Index=0;Index<NumRunningChannels;Index++) {
         if (ChData[Index].RBThreadID) Subscribe(TM_RB_FILECHANGE, ChData[Index].RBThreadID, 0, hWnd);
      }

      // Subscribe to TM_OPERATION_HALTED messages from all ringbuffers.
      for (Index=0;Index<NumRunningChannels;Index++) {
         if (ChData[Index].RBThreadID) Subscribe(TM_OPERATION_HALTED, ChData[Index].RBThreadID, 0, hWnd);
      }
      
      // Subscribe to TM_BUFFERUPDATED messages from all channels.
      for (Index=0;Index<NumRunningChannels;Index++) {
         Subscribe(TM_BUFFERUPDATED, RunningChannels[Index], 0, hWnd);
      }

      // Subscribe to TM_TRIG_START messages from all channels.
      for (Index=0;Index<NumRunningChannels;Index++) {
         Subscribe(TM_TRIG_START, RunningChannels[Index], 0, hWnd);
      }

      // Subscribe to TM_TRIG_STOP messages from all channels.
      for (Index=0;Index<NumRunningChannels;Index++) {
         Subscribe(TM_TRIG_STOP, RunningChannels[Index], 0, hWnd);
      }

      // Subscribe to TM_DEVICEREADER_NODATAINPUT messages from all device managers.
      for (Index=0;Index<NumRunningDevices;Index++) {
         Subscribe(TM_DEVICEREADER_NODATAINPUT, RunningDevices[Index], 0, hWnd);
      }

      // Subscribe to TM_NETWORK_TRIG_START messages from all network-trigger threads.
      for (Index=0;Index<NumRunningTriggersets;Index++) {
         Subscribe(TM_NETWORK_TRIG_START, RunningTriggersets[Index], 0, hWnd);
      }

      // Subscribe to TM_NETWORK_TRIG_STOP messages from all network-trigger threads.
      for (Index=0;Index<NumRunningTriggersets;Index++) {
         Subscribe(TM_NETWORK_TRIG_STOP, RunningTriggersets[Index], 0, hWnd);
      }

      // Destroy the event object
      CloseHandle(hEvent);
      return 0;


      case TM_RB_FILECHANGE:
      //
      // One of the ringbuffer threads has jumped to the next rb-file.
      //
      yPos=HEADER_HEIGHT;
      HaveDrawn=FALSE;
      for (Index=0;Index<NumRunningChannels;Index++) {
         if (ChData[Index].RBThreadID==(DWORD)wParam) {
            CreateShortRBFilename(str6, (DWORD)lParam);
            wcscpy(ChData[Index].RBFile, str6);
            hdc=GetDC(hWnd);
            SelectObject(hdc, hFont);
#ifdef WIN95
            TextOut(hdc, XPOSRBFILE-OffsetX, yPos-OffsetY, str6, 6);       
#endif

#ifdef WINCE
			 rectTxt.top    = yPos-OffsetY;
 			 rectTxt.bottom = (yPos+CharHeight)-OffsetY;
			 rectTxt.left   =  CE_XPOSRBFILE-OffsetX;
			 rectTxt.right  = CE_XPOSENDRBFILE-OffsetX;
	 		 DrawText(hdc, str6, 6, &rectTxt , DT_RIGHT); 
#endif
            ReleaseDC(hWnd, hdc);
            HaveDrawn=TRUE;
         }

         // Increase the y-position for drawing.
         yPos=yPos+CharHeight;
      }

      // Redraw column header if needed.
      if (HaveDrawn && OffsetY) {
         hdc=GetDC(hWnd);
         DrawBitmap(hdc, hBitmap1, -OffsetX, 0);
         ReleaseDC(hWnd, hdc);
      }
      return 0;


      case TM_DISK_WRITE:
      //
      // One of the ringbuffers have written data to the disk.
      //
      yPos=HEADER_HEIGHT;     // Start drawing at y-position below the column header.
      HaveDrawn=FALSE;
      for (Index=0;Index<NumRunningChannels;Index++) {
         if (ChData[Index].RBThreadID==(DWORD)wParam) {
            if (ChData[Index].DiskLEDStatus!=2) {
               ChData[Index].DiskLEDTimerCnt=2;
               if (ChData[Index].DiskLEDStatus!=1) {
                  ChData[Index].DiskLEDStatus=1;
                  hdc=GetDC(hWnd);
#ifdef WINCE                 
						DrawBitmap(hdc, hBMPGreenLed, CE_XPOSDISKLED-OffsetX, yPos-OffsetY);
#else
						DrawBitmap(hdc, hBMPGreenLed, XPOSDISKLED-OffsetX, yPos-OffsetY);
#endif
                  ReleaseDC(hWnd, hdc);
                  HaveDrawn=TRUE;
               }
            }
         }

         // Increase the y-position for drawing.
         yPos=yPos+CharHeight;
      }

      // Redraw column header if needed.
      if (HaveDrawn && OffsetY) {
         hdc=GetDC(hWnd);
         DrawBitmap(hdc, hBitmap1, -OffsetX, 0);
         ReleaseDC(hWnd, hdc);
      }
      return 0;


      case TM_BUFFERUPDATED:

	  //
      // The channel buffer is updated with a new block of data. 
      //

      // Start drawing at y-position below the column header.
      yPos=HEADER_HEIGHT;                    

      // Get handle to the device context.
      hdc=GetDC(hWnd);           

      // Select the correct font into the DC.
      SelectObject(hdc, hFont);  

      // Search trough the 'ChData' table for entries
      // that matches the thread-id of the channel.
      // that sendt the TM_BUFFERUPDATED message.
      for (Index=0;Index<NumRunningChannels;Index++) {
         if (ChData[Index].CHThreadID==(DWORD)wParam) {
            // A matching entry is found.

            // Get pointer to the new block.
            BH=(BLOCK_HEADER*)((PBYTE)ChData[Index].BuffProp.pBuffer+((DWORD)lParam*ChData[Index].BuffProp.BlocksizeInBytes));

            // Handle the 'In' LED.
            ChData[Index].InLEDTimerCnt=10;
            if (ChData[Index].InLEDStatus!=1) {
               ChData[Index].InLEDStatus=1; // Mark the LED status as ON (GREEN).
#ifdef WINCE              
					DrawBitmap(hdc, hBMPGreenLed, CE_XPOSINLED-OffsetX, yPos-OffsetY);
#else
					DrawBitmap(hdc, hBMPGreenLed, XPOSINLED-OffsetX, yPos-OffsetY);
#endif
            }

            // Handle the 'Sync' LED.
            if (BH->TimeQuality!=ChData[Index].SyncLEDStatus) {
               ChData[Index].SyncLEDStatus=BH->TimeQuality;
#ifdef WINCE
               if (BH->TimeQuality==0) DrawBitmap(hdc, hBMPNoLed, CE_XPOSSYNCLED-OffsetX, yPos-OffsetY);
               if (BH->TimeQuality==1) DrawBitmap(hdc, hBMPInactiveLed, CE_XPOSSYNCLED-OffsetX, yPos-OffsetY);
               if (BH->TimeQuality==2) DrawBitmap(hdc, hBMPGreenLed, CE_XPOSSYNCLED-OffsetX, yPos-OffsetY);
               if (BH->TimeQuality==3) DrawBitmap(hdc, hBMPRedLed, CE_XPOSSYNCLED-OffsetX, yPos-OffsetY);
#else
					
               if (BH->TimeQuality==0) DrawBitmap(hdc, hBMPNoLed, XPOSSYNCLED-OffsetX, yPos-OffsetY);
               if (BH->TimeQuality==1) DrawBitmap(hdc, hBMPInactiveLed, XPOSSYNCLED-OffsetX, yPos-OffsetY);
               if (BH->TimeQuality==2) DrawBitmap(hdc, hBMPGreenLed, XPOSSYNCLED-OffsetX, yPos-OffsetY);
               if (BH->TimeQuality==3) DrawBitmap(hdc, hBMPRedLed, XPOSSYNCLED-OffsetX, yPos-OffsetY);
#endif
            }

            // Write the time for the new block.
            ChData[Index].TimeTag=BH->TimeTag;  // Save new time.
            CreateDateAndTimeString(String, &ChData[Index].TimeTag);

#ifdef WIN95
			TextOut(hdc, XPOSTIMEDATE-OffsetX, yPos-OffsetY, String, wcslen(String));        
#endif

#ifdef WINCE
			 rectTxt.top = yPos-OffsetY;
 			 rectTxt.bottom = (yPos+CharHeight)-OffsetY;
		 	 rectTxt.left =  CE_XPOSTIMEDATE-OffsetX;
			 rectTxt.right = CE_XPOSENDTIMEDATE-OffsetX;
			 DrawText(hdc, String, wcslen(String), &rectTxt , DT_LEFT); 
#endif


            // Write the first sample in the block.
            ChData[Index].Sample=*((signed long*)(((PBYTE)BH)+sizeof(BLOCK_HEADER)));
            swprintf(str11, TEXT("%11d"), ChData[Index].Sample);
#ifdef WIN95
			TextOut(hdc, XPOSSAMPLE-OffsetX, yPos-OffsetY, str11, 11);
#endif

#ifdef WINCE
			rectTxt.left =  CE_XPOSENDTIMEDATE-OffsetX;
			rectTxt.right = CE_XPOSSAMPLE-OffsetX;
	 		DrawText(hdc, str11, 11, &rectTxt , DT_RIGHT); 
#endif

            // Get, save and write LTA/STA if applicable.
            if (ChData[Index].ChInfo.UseTrigging) {
               // Get and save the channel status for this channel.
               MsgParams.vPointer=(PVOID)&ChData[Index].ChStat;
               hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
               PostThreadMessage(RunningChannels[Index], TM_GET_CHANNEL_STATUS, (WPARAM)hEvent, (LPARAM)&MsgParams); 
               WaitForSingleObject(hEvent, INFINITE);
               CloseHandle(hEvent);

			   // Write new values for LTA/STA.
               swprintf(str11, TEXT("%11.1f"), ChData[Index].ChStat.STA);
#ifdef WIN95
               OldTxtAlign=SetTextAlign(hdc, TA_RIGHT);
               TextOut(hdc, XPOSSTA-OffsetX, yPos-OffsetY, str11, wcslen(str11));         
#endif

#ifdef WINCE
			   rectTxt.left =  CE_XPOSSAMPLE-OffsetX;
			   rectTxt.right = CE_XPOSSTA-OffsetX;
	 		   DrawText(hdc, str11, wcslen(str11), &rectTxt , DT_RIGHT); 
#endif
               swprintf(str11, TEXT("%11.1f"), ChData[Index].ChStat.LTA);
#ifdef WIN95
               TextOut(hdc, XPOSLTA-OffsetX, yPos-OffsetY, str11, wcslen(str11));         
               SetTextAlign(hdc, OldTxtAlign);
#endif

#ifdef WINCE
			   rectTxt.left =  CE_XPOSSTA-OffsetX;
			   rectTxt.right = CE_XPOSLTA-OffsetX;
	 		   DrawText(hdc, str11, wcslen(str11), &rectTxt , DT_RIGHT); 
#endif





               // If filtering is not used, write the calculated offset (DC).
               if (!ChData[Index].ChInfo.ApplyFilter) {
                  swprintf(str11, TEXT("%11d"), ChData[Index].ChStat.ChannelOffset);
#ifdef WIN95
                  OldTxtAlign=SetTextAlign(hdc, TA_RIGHT);
                  TextOut(hdc, XPOSOFFSET-OffsetX, yPos-OffsetY, str11, wcslen(str11));
                  SetTextAlign(hdc, OldTxtAlign);
#endif

#ifdef WINCE
  	              rectTxt.left =  CE_XPOSLTA-OffsetX;
 				  rectTxt.right = CE_XPOSOFFSET-OffsetX;
 	 			  DrawText(hdc, str11, wcslen(str11), &rectTxt , DT_RIGHT); 
#endif



               }
            }
         }
      
         // Increase the y-position for drawing.
         yPos=yPos+CharHeight;
      }

      // Redraw column header if needed.
      if (OffsetY) DrawBitmap(hdc, hBitmap1, -OffsetX, 0);

      // Release the device context.
      ReleaseDC(hWnd, hdc);

      return 0;


      
      case TM_OPERATION_HALTED:
      //
      // One of the ringbuffer threads has failed. 
      //
      yPos=HEADER_HEIGHT;     // Start drawing at y-position below the column header.
      HaveDrawn=FALSE;
      for (Index=0;Index<NumRunningChannels;Index++) {
         if (ChData[Index].RBThreadID==(DWORD)wParam) {
            // Handle the 'Disk' LED.
            ChData[Index].DiskLEDTimerCnt=0;
            ChData[Index].DiskLEDStatus=2; // Mark the LED status as RED.
            hdc=GetDC(hWnd);
#ifdef WINCE            
				DrawBitmap(hdc, hBMPRedLed, CE_XPOSDISKLED-OffsetX, yPos-OffsetY);
#else
				DrawBitmap(hdc, hBMPRedLed, XPOSDISKLED-OffsetX, yPos-OffsetY);
#endif
            ReleaseDC(hWnd, hdc);
            HaveDrawn=TRUE;
         }
         // Increase the y-position for drawing.
         yPos=yPos+CharHeight;
      }

      // Redraw column header if needed.
      if (HaveDrawn && OffsetY) {
         hdc=GetDC(hWnd);
         DrawBitmap(hdc, hBitmap1, -OffsetX, 0);
         ReleaseDC(hWnd, hdc);
      }
      return 0;


      case TM_DEVICEREADER_NODATAINPUT:
      //
      // One of the devicemanagers has lost it's data input. 
      //
      yPos=HEADER_HEIGHT;     // Start drawing at y-position below the column header.
      HaveDrawn=FALSE;
      for (Index=0;Index<NumRunningChannels;Index++) {
         if (ChData[Index].ChInfo.DevMngrThreadID==(DWORD)wParam) {
            // Handle the 'In' LED.
            ChData[Index].InLEDTimerCnt=0;
            ChData[Index].InLEDStatus=2; // Mark the LED status as ON(RED).
            hdc=GetDC(hWnd);
#ifdef WINCE            
				DrawBitmap(hdc, hBMPRedLed, CE_XPOSINLED-OffsetX, yPos-OffsetY);
#else
            DrawBitmap(hdc, hBMPRedLed, XPOSINLED-OffsetX, yPos-OffsetY);
#endif
            ReleaseDC(hWnd, hdc);
            HaveDrawn=TRUE;
         }
      
         // Increase the y-position for drawing.
         yPos=yPos+CharHeight;
      }

      // Redraw column header if needed.
      if (HaveDrawn && OffsetY) {
         hdc=GetDC(hWnd);
         DrawBitmap(hdc, hBitmap1, -OffsetX, 0);
         ReleaseDC(hWnd, hdc);
      }
      return 0;


      case TM_TRIG_START:
      //
      // One of the channels indicates start of an event. 
      //
      yPos=HEADER_HEIGHT;     // Start drawing at y-position below the column header.
      for (Index=0;Index<NumRunningChannels;Index++) {
         if (ChData[Index].CHThreadID==(DWORD)wParam) {
            // Handle the Trig-LED.
            ChData[Index].TrigLEDStatus=1; // Mark the LED status as GREEN.
            hdc=GetDC(hWnd);
#ifdef WINCE            
				DrawBitmap(hdc, hBMPGreenLed, CE_XPOSTRIGLED-OffsetX, yPos-OffsetY);
#else            
				DrawBitmap(hdc, hBMPGreenLed, XPOSTRIGLED-OffsetX, yPos-OffsetY);
#endif
				ReleaseDC(hWnd, hdc);
         }
         // Increase the y-position for drawing.
         yPos=yPos+CharHeight;
      }

      // Redraw column header if needed.
      if (OffsetY) {
         hdc=GetDC(hWnd);
         DrawBitmap(hdc, hBitmap1, -OffsetX, 0);
         ReleaseDC(hWnd, hdc);
      }
      return 0;


      case TM_TRIG_STOP:
      //
      // One of the channels indicates the end of an event. 
      //
      yPos=HEADER_HEIGHT;     // Start drawing at y-position below the column header.
      for (Index=0;Index<NumRunningChannels;Index++) {
         if (ChData[Index].CHThreadID==(DWORD)wParam) {
            // Handle the Trig-LED.
            ChData[Index].TrigLEDStatus=0; // Mark the LED status as INACTIVE.
            hdc=GetDC(hWnd);
#ifdef WINCE            
				DrawBitmap(hdc, hBMPInactiveLed, CE_XPOSTRIGLED-OffsetX, yPos-OffsetY);
#else            
				DrawBitmap(hdc, hBMPInactiveLed, XPOSTRIGLED-OffsetX, yPos-OffsetY);
#endif
				ReleaseDC(hWnd, hdc);
         }
         // Increase the y-position for drawing.
         yPos=yPos+CharHeight;
      }

      // Redraw column header if needed.
      if (OffsetY) {
         hdc=GetDC(hWnd);
         DrawBitmap(hdc, hBitmap1, -OffsetX, 0);
         ReleaseDC(hWnd, hdc);
      }
      return 0;

#ifdef WIN95
     case TM_NETWORK_TRIG_START:
      //
      // One of the network-trigger threads indicates start of an event. 
      //
      yPos=HEADER_HEIGHT;  // Start drawing at y-position below the column header.
      for (Index=0;Index<NumRunningTriggersets;Index++) {
         if (TriggerSetInfo[Index].ThreadID==(DWORD)wParam) {
            // Turn on all LED's belonging to this triggerset
            // to visualize that an network event has been triggered.
            hdc=GetDC(hWnd);
            xPos=XPOSNETWRKTRIGLED+Index*19;
            for (Index2=0;Index2<NumRunningChannels;Index2++) {
               if (TriggerSetInfo[Index].Channels[Index2]) {
                  DrawBitmap(hdc, hBMPGreenLed, xPos-OffsetX, yPos-OffsetY);
               }
               // Increase the y-position for drawing.
               yPos=yPos+CharHeight;
            }
            ReleaseDC(hWnd, hdc);
            TriggerSetInfo[Index].Status=TRUE; // Change the event status.
         }
      }

      // Redraw column header if needed.
      if (OffsetY) {
         hdc=GetDC(hWnd);
         DrawBitmap(hdc, hBitmap1, -OffsetX, 0);
         ReleaseDC(hWnd, hdc);
      }
      return 0;


      case TM_NETWORK_TRIG_STOP:
      //
      // One of the network-trigger threads indicates the end of an event. 
      //
      yPos=HEADER_HEIGHT;  // Start drawing at y-position below the column header.
      for (Index=0;Index<NumRunningTriggersets;Index++) {
         if (TriggerSetInfo[Index].ThreadID==(DWORD)wParam) {
            // Turn off all LED's belonging to this triggerset
            // to visualize that the network event has ended.
            hdc=GetDC(hWnd);
            xPos=XPOSNETWRKTRIGLED+Index*19;
            for (Index2=0;Index2<NumRunningChannels;Index2++) {
               if (TriggerSetInfo[Index].Channels[Index2]) {
                  DrawBitmap(hdc, hBMPInactiveLed, xPos-OffsetX, yPos-OffsetY);
               }
               // Increase the y-position for drawing.
               yPos=yPos+CharHeight;
            }
            ReleaseDC(hWnd, hdc);
            TriggerSetInfo[Index].Status=FALSE; // Change the event status.
         }
      }

      // Redraw column header if needed.
      if (OffsetY) {
         hdc=GetDC(hWnd);
         DrawBitmap(hdc, hBitmap1, -OffsetX, 0);
         ReleaseDC(hWnd, hdc);
      }
      return 0;
#endif

      case WM_TIMER:
      //
      // This message is sendt from Windows every 333 ms.   
      //

      // Check if any LED counters should be decreased, and redraw LED's as needed.
      yPos=HEADER_HEIGHT;     // Start drawing at y-position below the column header.
      HaveDrawn=FALSE;
      for (Index=0;Index<NumRunningChannels;Index++) {
         // Handle the 'In' LED.
         if (ChData[Index].InLEDTimerCnt) {
            // Decrease timer count and redraw LED if needed.
            if (ChData[Index].InLEDTimerCnt) ChData[Index].InLEDTimerCnt--;
            if (!ChData[Index].InLEDTimerCnt) {
               ChData[Index].InLEDStatus=0; // Mark the LED status as OFF.
               hdc=GetDC(hWnd);
#ifdef WINCE
               DrawBitmap(hdc, hBMPInactiveLed, CE_XPOSINLED-OffsetX, yPos-OffsetY);
#else
					DrawBitmap(hdc, hBMPInactiveLed, XPOSINLED-OffsetX, yPos-OffsetY);
#endif
               ReleaseDC(hWnd, hdc);
               HaveDrawn=TRUE;
            }
         }

         // Handle the 'Disk' LED.
         if (ChData[Index].DiskLEDStatus!=2) {
            if (ChData[Index].DiskLEDTimerCnt) {
               // Decrease timer count and redraw LED if needed.
               if (ChData[Index].DiskLEDTimerCnt) ChData[Index].DiskLEDTimerCnt--;
               if (!ChData[Index].DiskLEDTimerCnt) {
                  ChData[Index].DiskLEDStatus=0; // Mark the LED status as OFF.
                  hdc=GetDC(hWnd);
#ifdef WINCE
                  DrawBitmap(hdc, hBMPInactiveLed, CE_XPOSDISKLED-OffsetX, yPos-OffsetY);
#else
						DrawBitmap(hdc, hBMPInactiveLed, XPOSDISKLED-OffsetX, yPos-OffsetY);
#endif
                  ReleaseDC(hWnd, hdc);
                  HaveDrawn=TRUE;
               }
            }
         }

         // Increase the y-position for drawing.
         yPos=yPos+CharHeight;
      }

      // Redraw column header if needed.
      if (HaveDrawn && OffsetY) {
         hdc=GetDC(hWnd);
         DrawBitmap(hdc, hBitmap1, -OffsetX, 0);
         ReleaseDC(hWnd, hdc);
      }
      return 0;



      case WM_SIZE:
      // Save old window size.
      OldClntSizeX=ClientSizeX;
      OldClntSizeY=ClientSizeY;

      // Get new window size. 
      ClientSizeX=LOWORD(lParam);
      ClientSizeY=HIWORD(lParam);

      // Handle vertical size.
      bVRedraw=FALSE;
      if (OldClntSizeY<ClientSizeY) {
         if (ClientSizeY>(LogicalWndSizeY-OffsetY)) {
            OffsetY=LogicalWndSizeY-ClientSizeY;
            if (OffsetY<0) OffsetY=0; 
            bVRedraw=TRUE;
         }
         if (ClientSizeY>LogicalWndSizeY) bVRedraw=FALSE;
      }

      // Handle horizontal size.
      bHRedraw=FALSE;
      if (OldClntSizeX<ClientSizeX) {
         bHRedraw=TRUE;
         if (ClientSizeX>(LogicalWndSizeX-OffsetX)) {
            OffsetX=LogicalWndSizeX-ClientSizeX;
            if (OffsetX<0) OffsetX=0; 
         }
      }

      // Update scrollbars.
      VscrollMax=max(0, LogicalWndSizeY-ClientSizeY);
      HscrollMax=max(0, LogicalWndSizeX-ClientSizeX);
      SetScrollRange(hWnd, SB_VERT, 0, VscrollMax, FALSE);
      SetScrollPos(hWnd, SB_VERT, OffsetY, TRUE);
      SetScrollRange(hWnd, SB_HORZ, 0, HscrollMax, FALSE);
      SetScrollPos(hWnd, SB_HORZ, OffsetX, TRUE);

      // Redraw window.
      if (bVRedraw || bHRedraw) {
         InvalidateRect(hWnd, NULL, TRUE);
         UpdateWindow(hWnd);
      }
      break;


      case WM_VSCROLL:
         switch (LOWORD(wParam)) {
         case SB_TOP:
            if (OffsetY>0) {
               OffsetY=0;
               InvalidateRect(hWnd, NULL, TRUE);
               UpdateWindow(hWnd);
            }
            break;
         case SB_BOTTOM:
            VscrollMax=max(0, LogicalWndSizeY-ClientSizeY);
            if (OffsetY<VscrollMax) {
               OffsetY=VscrollMax;
               InvalidateRect(hWnd, NULL, TRUE);
               UpdateWindow(hWnd);
            }
            break;
         case SB_LINEDOWN:
            VscrollMax=max(0, LogicalWndSizeY-ClientSizeY);
            if (OffsetY<VscrollMax) {
               OffsetY=OffsetY+CharHeight;
               if (OffsetY>VscrollMax) OffsetY=VscrollMax; 
               InvalidateRect(hWnd, NULL, TRUE);
               UpdateWindow(hWnd);
            }
            break;
         case SB_LINEUP:
            if (OffsetY>0) {
               OffsetY=OffsetY-CharHeight;
               if (OffsetY<0) OffsetY=0;
               InvalidateRect(hWnd, NULL, TRUE);
               UpdateWindow(hWnd);
            }
            break;
         case SB_THUMBTRACK:
            OffsetY=HIWORD(wParam);
            InvalidateRect(hWnd, NULL, TRUE);
            UpdateWindow(hWnd);
            break;
         case SB_PAGEUP:
            if (OffsetY>0) {
               OffsetY=OffsetY-ClientSizeY;
               if (OffsetY<0) OffsetY=0;
               InvalidateRect(hWnd, NULL, TRUE);
               UpdateWindow(hWnd);
            }
            break;
         case SB_PAGEDOWN:
            VscrollMax=max(0, LogicalWndSizeY-ClientSizeY);
            if (OffsetY<VscrollMax) {
               OffsetY=OffsetY+ClientSizeY;
               if (OffsetY>VscrollMax) OffsetY=VscrollMax; 
               InvalidateRect(hWnd, NULL, TRUE);
               UpdateWindow(hWnd);
            }
            break;
         }
         
         // Set position of scrollbar.
         SetScrollPos(hWnd, SB_VERT, OffsetY, TRUE);
      return 0L;


      case WM_HSCROLL:
         switch (LOWORD(wParam)) {
         case SB_TOP:
            if (OffsetX>0) {
               OffsetX=0;
               InvalidateRect(hWnd, NULL, TRUE);
               UpdateWindow(hWnd);
            }
            break;
         case SB_BOTTOM:
            if (OffsetX<HscrollMax) {
               OffsetX=HscrollMax;
               InvalidateRect(hWnd, NULL, TRUE);
               UpdateWindow(hWnd);
            }
            break;
         case SB_LINERIGHT:
            if (OffsetX<HscrollMax) {
               OffsetX=OffsetX+10;
               if (OffsetX>HscrollMax) OffsetX=HscrollMax; 
               InvalidateRect(hWnd, NULL, TRUE);
               UpdateWindow(hWnd);
            }
            break;

         case SB_LINELEFT:
            if (OffsetX>0) {
               OffsetX=OffsetX-10;
               if (OffsetX<0) OffsetX=0;
               InvalidateRect(hWnd, NULL, TRUE);
               UpdateWindow(hWnd);
            }
            break;
         case SB_THUMBTRACK:
            OffsetX=HIWORD(wParam);

            InvalidateRect(hWnd, NULL, TRUE);
            UpdateWindow(hWnd);
            break;
         case SB_PAGELEFT:
            if (OffsetX>0) {
               OffsetX=OffsetX-ClientSizeX/2;
               if (OffsetX<0) OffsetX=0;
               InvalidateRect(hWnd, NULL, TRUE);
               UpdateWindow(hWnd);
            }
            break;
         case SB_PAGERIGHT:
            if (OffsetX<HscrollMax) {
               OffsetX=OffsetX+ClientSizeX/2;
               if (OffsetX>HscrollMax) OffsetX=HscrollMax; 
               InvalidateRect(hWnd, NULL, TRUE);
               UpdateWindow(hWnd);
            }
            break;
         }

         // Set position of scrollbar.
         SetScrollPos(hWnd, SB_HORZ, OffsetX, TRUE);
      return 0;


      case WM_ERASEBKGND:
      // Erase the whole window except for the header part. 
      // This is  done to avoid flicker when scrolling the window.
      GetClientRect(hWnd, (RECT FAR*)&rect);
      SelectObject((HDC)wParam, (HGDIOBJ)(HPEN)GetStockObject(WHITE_PEN));
      SelectObject((HDC)wParam, (HGDIOBJ)(HBRUSH)GetStockObject(WHITE_BRUSH));
      Rectangle((HDC)wParam, 0, HEADER_HEIGHT, rect.right+1, rect.bottom+1); 
      return 1;

      case WM_PAINT:
      hdc=BeginPaint(hWnd, &ps);
      GetClientRect(hWnd, (RECT FAR*)&rect);

      // Select font.
      SelectObject(hdc, hFont);

      // Set clipping region so that the
      // header will not be overwritten.
      hClipRgn=CreateRectRgn(rect.left, rect.top+HEADER_HEIGHT, rect.right+1, rect.bottom+1); 
      SelectClipRgn(hdc, hClipRgn);
      
      // Start writing in the top of the window.
      yPos=HEADER_HEIGHT;

      // Update window.
      // Draw one line for each channel.
      for (Index=0;Index<NumRunningChannels;Index++) {
         // Write channel number.
         wcscpy(String, ChData[Index].ChInfo.ChannelName);

#ifdef WIN95
         TextOut(hdc, XPOSCHNUM-OffsetX, yPos-OffsetY, &String[7], 2);        
#endif

#ifdef WINCE
		 rectTxt.top = yPos-OffsetY;
		 rectTxt.left = CE_XPOSCHNUM-OffsetX;
		 rectTxt.bottom = (yPos+CharHeight)-OffsetY;
		 rectTxt.right = CE_XPOSSTATION-OffsetX;
		 DrawText(hdc, &String[7], 2, &rectTxt , DT_LEFT); 
#endif

         // Write the station code.
         wcscpy(String, ChData[Index].ChInfo.StationCode);

#ifdef WIN95
         TextOut(hdc, XPOSSTATION-OffsetX, yPos-OffsetY, String, wcslen(String));         
#endif

#ifdef WINCE
		 rectTxt.left = CE_XPOSSTATION-OffsetX;
		 rectTxt.right = CE_XPOSCOMPONENT-OffsetX;
	 	 DrawText(hdc, String, wcslen(String), &rectTxt , DT_LEFT); 
#endif

         // Write the compoment.
         wcscpy(String, ChData[Index].ChInfo.Component);

#ifdef WIN95
		 TextOut(hdc, XPOSCOMPONENT-OffsetX, yPos-OffsetY, String, wcslen(String));       
#endif
#ifdef WINCE
		 // [PocketPC]
		 rectTxt.left = CE_XPOSCOMPONENT-OffsetX;
		 rectTxt.right = CE_XPOSINLED-OffsetX;
	 	 DrawText(hdc, String, wcslen(String), &rectTxt , DT_LEFT); 
#endif


         // Write the time and date.
         CreateDateAndTimeString(String, &ChData[Index].TimeTag);
#ifdef WIN95
		 TextOut(hdc, XPOSTIMEDATE-OffsetX, yPos-OffsetY, String, wcslen(String));        
#endif
#ifdef WINCE
		 rectTxt.left =  CE_XPOSTIMEDATE-OffsetX;
		 rectTxt.right = CE_XPOSENDTIMEDATE-OffsetX;
	 	 DrawText(hdc, String, wcslen(String), &rectTxt , DT_LEFT); 
#endif

#ifdef WINCE
         // Draw the In-LED.
         if (ChData[Index].InLEDStatus==0) DrawBitmap(hdc, hBMPInactiveLed, CE_XPOSINLED-OffsetX, yPos-OffsetY);
         if (ChData[Index].InLEDStatus==1) DrawBitmap(hdc, hBMPGreenLed, CE_XPOSINLED-OffsetX, yPos-OffsetY);
         if (ChData[Index].InLEDStatus==2) DrawBitmap(hdc, hBMPRedLed, CE_XPOSINLED-OffsetX, yPos-OffsetY);
               
         // Draw the Disk-LED.
         if (ChData[Index].ChInfo.LogToDisk) {
            if (ChData[Index].DiskLEDStatus==0) DrawBitmap(hdc, hBMPInactiveLed, CE_XPOSDISKLED-OffsetX, yPos-OffsetY);
            if (ChData[Index].DiskLEDStatus==1) DrawBitmap(hdc, hBMPGreenLed, CE_XPOSDISKLED-OffsetX, yPos-OffsetY);
            if (ChData[Index].DiskLEDStatus==2) DrawBitmap(hdc, hBMPRedLed, CE_XPOSDISKLED-OffsetX, yPos-OffsetY);
         } else {
            DrawBitmap(hdc, hBMPNoLed, CE_XPOSDISKLED-OffsetX, yPos-OffsetY);
         }

         // Draw the Trig-LED.
         if (ChData[Index].ChInfo.UseTrigging) {
            if (ChData[Index].TrigLEDStatus==0) DrawBitmap(hdc, hBMPInactiveLed, CE_XPOSTRIGLED-OffsetX, yPos-OffsetY);
            if (ChData[Index].TrigLEDStatus==1) DrawBitmap(hdc, hBMPGreenLed, CE_XPOSTRIGLED-OffsetX, yPos-OffsetY);
         } else {
            DrawBitmap(hdc, hBMPNoLed, CE_XPOSTRIGLED-OffsetX, yPos-OffsetY);
         }

         // Draw the Sync LED.
         if (ChData[Index].SyncLEDStatus==0) DrawBitmap(hdc, hBMPNoLed, CE_XPOSSYNCLED-OffsetX, yPos-OffsetY);
         if (ChData[Index].SyncLEDStatus==1) DrawBitmap(hdc, hBMPInactiveLed, CE_XPOSSYNCLED-OffsetX, yPos-OffsetY);
         if (ChData[Index].SyncLEDStatus==2) DrawBitmap(hdc, hBMPGreenLed, CE_XPOSSYNCLED-OffsetX, yPos-OffsetY);
         if (ChData[Index].SyncLEDStatus==3) DrawBitmap(hdc, hBMPRedLed, CE_XPOSSYNCLED-OffsetX, yPos-OffsetY);
#else
			// Draw the In-LED.
         if (ChData[Index].InLEDStatus==0) DrawBitmap(hdc, hBMPInactiveLed, XPOSINLED-OffsetX, yPos-OffsetY);
         if (ChData[Index].InLEDStatus==1) DrawBitmap(hdc, hBMPGreenLed, XPOSINLED-OffsetX, yPos-OffsetY);
         if (ChData[Index].InLEDStatus==2) DrawBitmap(hdc, hBMPRedLed, XPOSINLED-OffsetX, yPos-OffsetY);
               
         // Draw the Disk-LED.
         if (ChData[Index].ChInfo.LogToDisk) {
            if (ChData[Index].DiskLEDStatus==0) DrawBitmap(hdc, hBMPInactiveLed, XPOSDISKLED-OffsetX, yPos-OffsetY);
            if (ChData[Index].DiskLEDStatus==1) DrawBitmap(hdc, hBMPGreenLed, XPOSDISKLED-OffsetX, yPos-OffsetY);
            if (ChData[Index].DiskLEDStatus==2) DrawBitmap(hdc, hBMPRedLed, XPOSDISKLED-OffsetX, yPos-OffsetY);
         } else {
            DrawBitmap(hdc, hBMPNoLed, XPOSDISKLED-OffsetX, yPos-OffsetY);
         }

         // Draw the Trig-LED.
         if (ChData[Index].ChInfo.UseTrigging) {
            if (ChData[Index].TrigLEDStatus==0) DrawBitmap(hdc, hBMPInactiveLed, XPOSTRIGLED-OffsetX, yPos-OffsetY);
            if (ChData[Index].TrigLEDStatus==1) DrawBitmap(hdc, hBMPGreenLed, XPOSTRIGLED-OffsetX, yPos-OffsetY);
         } else {
            DrawBitmap(hdc, hBMPNoLed, XPOSTRIGLED-OffsetX, yPos-OffsetY);
         }

         // Draw the Sync LED.
         if (ChData[Index].SyncLEDStatus==0) DrawBitmap(hdc, hBMPNoLed, XPOSSYNCLED-OffsetX, yPos-OffsetY);
         if (ChData[Index].SyncLEDStatus==1) DrawBitmap(hdc, hBMPInactiveLed, XPOSSYNCLED-OffsetX, yPos-OffsetY);
         if (ChData[Index].SyncLEDStatus==2) DrawBitmap(hdc, hBMPGreenLed, XPOSSYNCLED-OffsetX, yPos-OffsetY);
         if (ChData[Index].SyncLEDStatus==3) DrawBitmap(hdc, hBMPRedLed, XPOSSYNCLED-OffsetX, yPos-OffsetY);
#endif

#ifdef WIN95
         // Draw the network trigger LED's.
         xPos=XPOSNETWRKTRIGLED;
         for (Index2=0;Index2<MAX_NUM_OF_TRIGGERSETS;Index2++) {
            if (TriggerSetInfo[Index2].Channels[Index]) {
               if (TriggerSetInfo[Index2].Status) {
                  DrawBitmap(hdc, hBMPGreenLed, xPos-OffsetX, yPos-OffsetY);
               } else {
                  DrawBitmap(hdc, hBMPInactiveLed, xPos-OffsetX, yPos-OffsetY);
               }
            } else {
               DrawBitmap(hdc, hBMPNoLed, xPos-OffsetX, yPos-OffsetY);
            }
            xPos=xPos+19;
         }
#endif

       // Write the sample value.
       swprintf(str11, TEXT("%11d"), ChData[Index].Sample);
#ifdef WIN95
		 TextOut(hdc, XPOSSAMPLE-OffsetX, yPos-OffsetY, str11, 11);        
#endif
#ifdef WINCE
		 rectTxt.left =  CE_XPOSENDTIMEDATE-OffsetX;
		 rectTxt.right = CE_XPOSSAMPLE-OffsetX;
	 	 DrawText(hdc, str11, 11, &rectTxt , DT_RIGHT); 
#endif



         // Write the name of the ringbuffer file.
         if (ChData[Index].ChInfo.LogToDisk) {
            wcscpy(String, ChData[Index].RBFile);
#ifdef WIN95
            TextOut(hdc, XPOSRBFILE-OffsetX, yPos-OffsetY, String, wcslen(String));
#endif
#ifdef WINCE
			 rectTxt.left =  CE_XPOSRBFILE-OffsetX;
			 rectTxt.right = CE_XPOSENDRBFILE-OffsetX;
	 		 DrawText(hdc, String, wcslen(String), &rectTxt , DT_RIGHT); 
#endif


         }

         // Write values for STA/LTA and channel offset (DC).
         if (ChData[Index].ChInfo.UseTrigging) {
            swprintf(str11, TEXT("%11.1f"), ChData[Index].ChStat.STA);
#ifdef WIN95
            OldTxtAlign=SetTextAlign(hdc, TA_RIGHT);
            TextOut(hdc, XPOSSTA-OffsetX, yPos-OffsetY, str11, wcslen(str11));         
#endif

#ifdef WINCE
			 
			 rectTxt.left =  CE_XPOSSAMPLE-OffsetX;
			 rectTxt.right = CE_XPOSSTA-OffsetX;
	 		 DrawText(hdc, str11, wcslen(str11), &rectTxt , DT_RIGHT); 
#endif


            swprintf(str11, TEXT("%11.1f"), ChData[Index].ChStat.LTA);

#ifdef WIN95
            TextOut(hdc, XPOSLTA-OffsetX, yPos-OffsetY, str11, wcslen(str11));         
            SetTextAlign(hdc, OldTxtAlign);
#endif			

#ifdef WINCE			 
			 rectTxt.left =  CE_XPOSSTA-OffsetX;
			 rectTxt.right = CE_XPOSLTA-OffsetX;
	 		 DrawText(hdc, str11, wcslen(str11), &rectTxt , DT_RIGHT); 
#endif

			 

            // Write value for channel offset (DC).
            if (!ChData[Index].ChInfo.ApplyFilter) {

               swprintf(str11, TEXT("%11d"), ChData[Index].ChStat.ChannelOffset);
#ifdef WIN95
			   OldTxtAlign=SetTextAlign(hdc, TA_RIGHT);
               TextOut(hdc, XPOSOFFSET-OffsetX, yPos-OffsetY, str11, wcslen(str11));
               SetTextAlign(hdc, OldTxtAlign);
#endif

#ifdef WINCE
  			   rectTxt.left =  CE_XPOSLTA-OffsetX;
			   rectTxt.right = CE_XPOSOFFSET-OffsetX;
	 		   DrawText(hdc, str11, wcslen(str11), &rectTxt , DT_RIGHT); 
#endif




            }
         }

         // Write filter settings.
         if (ChData[Index].ChInfo.ApplyFilter) {
            swprintf(str11, TEXT("%.2f"), ChData[Index].ChInfo.FilterLow);
#ifdef WIN95 
            OldTxtAlign=SetTextAlign(hdc, TA_RIGHT);
            TextOut(hdc, XPOSFILTLO-OffsetX, yPos-OffsetY, str11, wcslen(str11));         
#endif
#ifdef WINCE
			rectTxt.left =  CE_XPOSRBFILE-OffsetX;
			rectTxt.right = CE_XPOSFILTLO-OffsetX;
	 		DrawText(hdc, str11, wcslen(str11), &rectTxt , DT_RIGHT); 
#endif

            swprintf(str11, TEXT("%.2f"), ChData[Index].ChInfo.FilterHigh);

#ifdef WIN95
			TextOut(hdc, XPOSFILTHI-OffsetX, yPos-OffsetY, str11, wcslen(str11));         
            SetTextAlign(hdc, OldTxtAlign);
#endif

#ifdef WINCE
			rectTxt.left =  CE_XPOSFILTLO-OffsetX;
			rectTxt.right = CE_XPOSFILTHI-OffsetX;
	 		DrawText(hdc, str11, wcslen(str11), &rectTxt , DT_RIGHT); 
#endif




         }

#ifdef WIN95
			// Write device binding.
	      wcscpy(String, ChData[Index].ChInfo.DeviceName);
			swprintf(str11, TEXT("%02d"), ChData[Index].ChInfo.DeviceChannel);
			wcscat(String, TEXT(" (Ch")); wcscat(String, str11); wcscat(String, TEXT(")"));
         TextOut(hdc, XPOSDEVBIND-OffsetX, yPos-OffsetY, String, wcslen(String));       

			rectTxt.left =  XPOSDEVBIND-OffsetX;
			rectTxt.right = 1210-OffsetX;
	 		DrawText(hdc, String, wcslen(String), &rectTxt , DT_LEFT); 
#endif

#ifdef WINCE

#endif


         // Increase vertical draw position
         yPos=yPos+CharHeight;
      }

      // Remove the Clipping region.
      SelectClipRgn(hdc, (HRGN)NULL);
      DeleteObject(hClipRgn);
      
      // Draw the column headers.
      DrawBitmap(hdc, hBitmap1, -OffsetX, 0);
      
      EndPaint(hWnd, &ps);
      return 0L;

#ifdef WIN95

      case WM_CHILDACTIVATE:
      // This window is being activated.
      break;

      
      case WM_MDIACTIVATE:
      // An application sends the WM_MDIACTIVATE message to a 
      // multiple document interface (MDI) client window to 
      // instruct the client window to activate a different MDI child window. 
      hWndChildDeact=(HWND)wParam;  // Child being deactivated. 
      hWndChildAct=(HWND)lParam;    // Child being activated. 
      return 0;

#endif

      case WM_CLOSE:
      //
      // The window is about to be closed.
      //

      // Destroy the font.
      DeleteObject(hFont);

      // Destroy bitmap objects.
      DeleteObject((HGDIOBJ)hBitmap1);
      DeleteObject((HGDIOBJ)hBMPInactiveLed);
      DeleteObject((HGDIOBJ)hBMPRedLed);
      DeleteObject((HGDIOBJ)hBMPGreenLed);

      // Kill the timer object.
      KillTimer(hWnd, 0); 

      // Create an event object.       
      hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
      
      // Unsubscribe to TM_DISK_WRITE messages from all ringbuffers.
      for (Index=0;Index<NumRunningChannels;Index++) {
         if (ChData[Index].RBThreadID) UnSubscribe(TM_DISK_WRITE, ChData[Index].RBThreadID, 0, hWnd);
      }

      // Unsubscribe to TM_RB_FILECHANGE messages from all ringbuffers.
      SubscrData.uiMessage=TM_RB_FILECHANGE;
      SubscrData.idThread=0;
      SubscrData.hWnd=hWnd;
      for (Index=0;Index<NumRunningChannels;Index++) {
         if (ChData[Index].RBThreadID) UnSubscribe(TM_RB_FILECHANGE, ChData[Index].RBThreadID, 0, hWnd);
      }

      // Unsubscribe to TM_OPERATION_HALTED messages from all ringbuffers.
      for (Index=0;Index<NumRunningChannels;Index++) {
         if (ChData[Index].RBThreadID) UnSubscribe(TM_OPERATION_HALTED, ChData[Index].RBThreadID, 0, hWnd);
      }

      // Unsubscribe to TM_BUFFERUPDATED messages from all channels.
      for (Index=0;Index<NumRunningChannels;Index++) {
         UnSubscribe(TM_BUFFERUPDATED, RunningChannels[Index], 0, hWnd);
      }

      // Unsubscribe to TM_TRIG_START messages from all channels.
      for (Index=0;Index<NumRunningChannels;Index++) {
         UnSubscribe(TM_TRIG_START, RunningChannels[Index], 0, hWnd);
      }

      // Unsubscribe to TM_TRIG_STOP messages from all channels.
      for (Index=0;Index<NumRunningChannels;Index++) {
         UnSubscribe(TM_TRIG_STOP, RunningChannels[Index], 0, hWnd);
      }

      // Unsubscribe to TM_DEVICEREADER_NODATAINPUT messages from all device managers.
      for (Index=0;Index<NumRunningDevices;Index++) {
         UnSubscribe(TM_DEVICEREADER_NODATAINPUT, RunningDevices[Index], 0, hWnd);
      }

      // Unsubscribe to TM_NETWORK_TRIG_START messages from all network-trigger threads.
      for (Index=0;Index<NumRunningTriggersets;Index++) {
         UnSubscribe(TM_NETWORK_TRIG_START, RunningTriggersets[Index], 0, hWnd);
      }

      // Unsubscribe to TM_NETWORK_TRIG_STOP messages from all network-trigger threads.
      for (Index=0;Index<NumRunningTriggersets;Index++) {
         UnSubscribe(TM_NETWORK_TRIG_STOP, RunningTriggersets[Index], 0, hWnd);
      }

      // Destroy the event object
      CloseHandle(hEvent);

#ifdef WIN95
      // Save current window-placement to config file.
      AddPathAndFilename(WindowFile, ProgDir, TEXT("MainMonitor.wnd"));
      WndPlacement.length=sizeof(WINDOWPLACEMENT);
      GetWindowPlacement(hWnd, &WndPlacement); 
      WriteFileFromMemory(WindowFile, (PVOID)&WndPlacement, (DWORD)sizeof(WINDOWPLACEMENT));

      // Destroy the MDI child window
      SendMessage(hWndMDIClient, WM_MDIDESTROY, (WPARAM)hWnd, 0); 
#endif
      // End the thread loop.
      ClearMessageQueue(); // Delete all messages in message queue.
      PostQuitMessage(0);
      return 0;
   }
#ifdef WIN95
   // Pass unprocessed messages to the 'DefMDIChildProc' function.
   return DefMDIChildProc(hWnd, Message, wParam, lParam);
#endif

#ifdef WINCE
   // Pass unprocessed messages to the 'DefWindowProc' function.
   return DefWindowProc(hWnd, Message, wParam, lParam);   
#endif
  
   return 0;
}


/*****************************************************/
/* Inserts date and time into the string.            */
/*****************************************************/
VOID CreateDateAndTimeString(PTCHAR String, SYSTEMTIME *Time)
{
   TCHAR str4[5];

   // Add time and date to string.
   swprintf(String, TEXT("%02u"), Time->wDay);
   wcscat(String, TEXT("."));
   swprintf(str4, TEXT("%02u"), Time->wMonth);
   wcscat(String, str4);
   wcscat(String, TEXT("."));
   swprintf(str4, TEXT("%04u"), Time->wYear);
   wcscat(String, str4);
   wcscat(String, TEXT(" "));
   swprintf(str4, TEXT("%02u"), Time->wHour);
   wcscat(String, str4);
   wcscat(String, TEXT(":"));
   swprintf(str4, TEXT("%02u"), Time->wMinute);
   wcscat(String, str4);
   wcscat(String, TEXT(":"));
   swprintf(str4, TEXT("%02u"), Time->wSecond);
   wcscat(String, str4);
   wcscat(String, TEXT("."));
   swprintf(str4, TEXT("%03u"), Time->wMilliseconds);
   wcscat(String, str4);
}

