//#define STRICT



#include "stdafx.h"
#define CLIENT_HEIGHT_MIN 137
#define GRAPH_AREA_HEIGHT 101
#define GSV 60                 // Vertical space between graph lines.
#define SC_PROPERTIES 0xE000
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "seislog.h"
#include "library.h"
#include "resource.h"
#include "monitor.h"
#include "devices.h"
#include "channel.h"
#include "eventlog.h"
#include "process.h"

#include <Aygshell.h>

#define WINCE
//
// Author: yvind Natvik 1998.
//
//
//
//
extern HINSTANCE hInst;
extern HWND hWndFrame;
extern HWND hWndMDIClient;
extern TCHAR ChannelDir[];
extern DEVICE_INFO Devices[];
extern CHANNEL_INFO Channels[];
extern DWORD ProcKeeperThreadID;
extern BYTE DebugLevel;



//
// This structure is used to provide static
// storage for the window procedure.
//
struct MON_WND_DATA {
   DWORD ChThreadID;          // ThreadID of the channel thread used by this window.
   TCHAR Station[5+1];         // This channel's station code.
   TCHAR Component[4+1];       // This channel's component.
   TCHAR UniqueProcName[65];   // The name of this process.
   VOID *pChBuffer;           // Pointer to channel's buffer.
   WORD ClntHeight;           // Height of client area.
   WORD ClntWidth;            // Width of client area.
   WORD ClientAreaMinWidth;   // The minimum allowed width of the client area of the window.
   POINT MinTrackSize;        // The minimum trackingsize for the window.
   WORD GraphWidthInPixels;   // The width of the graph area (measured in pixels).
   WORD GraphWidthInBlocks;   // The width of the graph area (measured in blocks).
   WORD SamplsPrBlock;        // The nuber of samples to be drawn for each block.
   BYTE NumGraphInPlot;       // The number of graphs in plot window.
   BYTE CurrGraph;            // The graph that are currently beeing drawn.
   WORD BlockToDraw;          // The block position in the graph for the next block.
   LONG MaxVertScale;         // The maximum value of the y-scale.
   DOUBLE VertScaleFactor;    // Samples are multiplied with this value to get the sample height in pixels.
   LONG AutoMaxScale;         // Automatically calculated scale.
   LONG SampleMax;            // The largest sample value found during the initial calculation.
   LONG SampleMin;            // The smallest sample found during the initial calculation.
   HFONT hFont;               // Handle to font used to draw texts and numbers in the window.
   HBITMAP hBitMap;           // Handle to the arrow bitmap used in the window.
   DEVMNGR_INTERF_STRUCT DS;  // Copy of device manager's interface structure.
   SYSTEMTIME GraphTime;      // Timestamps for first block in each graphline starting from top.
   BOOL ReadyToPaint;         // Painting will not be done until this becomes TRUE;
   BOOL bInitialCalc;         // TRUE while the initial calculations are in progress.
   DOUBLE SampleAverage;      // Calculated sample average.
   WORD SampleCount;          // Used in the initial calculations.
   DOUBLE DeviceOffset;       // Calculated offset of data coming from the device.
   MON_WND_CNF WndConfig;     // Configuration data for the window.
   HDC MemDC;                 // Handle to memory device context.
   WORD MemDCWidth;           // The width of the bitmap in pixels.
   WORD MemDCHeight;          // The height of the bitmap in pixels.
   INT YPosLast;
};



//
// Prototype declarations.
//
LONG CalculateScale2(LONG);
LRESULT CALLBACK MonitorWndProc(HWND, UINT, WPARAM, LPARAM);
BOOL CALLBACK SetMonPropDlgProc(HWND, UINT, WPARAM, LPARAM);
WORD ReduceSamples(LONG*, LONG*, WORD, BYTE);
WORD GetSizeOfReducedBlck(WORD, BYTE);
VOID SetWindowTitle(MON_WND_DATA*, HWND);

POINT points[2]  = {{0,0},{0,0}};

/***************************************************************/
/* The original LineTo function does not exist in PocketPC     */
/***************************************************************/  
BOOL LineTo(
  HDC hdc,    // device context handle
  int _x,  // x-coordinate of ending point
  int _y   // y-coordinate of ending point
)
{

	points[1].x = _x;
	points[1].y = _y;
	Polyline(hdc, points, 2); 
	points[0].x = points[1].x;
	points[0].y = points[1].y;
	return TRUE;
}

/***************************************************************/
/* The original MoveTo function does not exist in PocketPC     */
/***************************************************************/
BOOL MoveTo(
  HDC hdc,    // NOT used. (device context handle)
  int _x,  // x-coordinate of ending point
  int _y   // y-coordinate of ending point
)
{
	points[0].x = _x;
	points[0].y = _y;
	return TRUE;
}

/*************************************************************************/
/* Monitor window thread. Handles one channel.                            */
/**************************************************************************/
DWORD WINAPI MonitorThrd(PVOID StartupInfo)
{
   MSG msg;
   HDC hdc;
   RECT rect;
   HMENU hSysMenu;
   HBITMAP hbit;
   TCHAR Message[100], UniqueProcName[65], str2[3];
   TCHAR ChDir[MAX_PATH+1], WindowFile[MAX_PATH+1];
   DWORD WaitStatus;
   HANDLE hEvent;
   HGDIOBJ OldBrush;
   HWND hWndMonitor;
   MSG_PARAM MsgParams;
   STARTINFO_MONITOR SI;
   MON_WND_DATA MD;
   BOOL bStatus;


#ifdef WINCE
   // Make sure struct members of the SYSTEMTIME struct are 0
   ZeroMemory((PVOID) &MD.GraphTime, sizeof(SYSTEMTIME));
#endif

   //
   // Copy thread startup-information to a local buffer called SI.
   //
   CopyMemory((PVOID)&SI, StartupInfo, sizeof(STARTINFO_MONITOR));


   //
   // Copy the thread-id of the channel-thread into the MD structure.
   //
   MD.ChThreadID=SI.ChThreadID;

   //
   // Create unique processname for this process.
   //
   wcscpy(UniqueProcName, TEXT("Channel"));
   swprintf(str2, TEXT("%02u"), SI.LogicalChannelNr);
   wcscat(UniqueProcName, str2);
   wcscat(UniqueProcName, TEXT(" Monitor"));
   wcscat(MD.UniqueProcName, UniqueProcName);

   //
   // Copy device-manager's interface structure from
   // the channel-thread to local a buffer called DS.
   // This buffer is located in the MD structure.
   hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
   int res  = PostThreadMessage(SI.ChThreadID, TM_GET_INTERFACE, (WPARAM)hEvent, (LPARAM)&MsgParams);

#ifdef WINCE
   if (0 == res)
   {
	   return 0;
   }
   WaitStatus=WaitForSingleObject(hEvent, 3000);
#else
   WaitStatus=WaitForSingleObject(hEvent, 1000);
#endif
   CopyMemory((PVOID)&MD.DS, MsgParams.vPointer, sizeof(DEVMNGR_INTERF_STRUCT));
   CloseHandle(hEvent);


   //
   // Register this process in the process database.
   //
   RegisterProcess(UniqueProcName, 3, GetCurrentThreadId());


   //
   // This process is dependant on the channel process.
   // Therefore a dependency must be registered for this process.
   //
   RegisterDependency(GetCurrentThreadId(), SI.ChThreadID);


   //
   // Get the get the Station code and
   // Component code for this channel.
   //
   hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
   PostThreadMessage(SI.ChThreadID, TM_GET_STATION, (WPARAM)hEvent, (LPARAM)&MsgParams);
   WaitForSingleObject(hEvent, INFINITE);
   wcscpy(MD.Station, MsgParams.cStatus);
   PostThreadMessage(SI.ChThreadID, TM_GET_COMPONENT, (WPARAM)hEvent, (LPARAM)&MsgParams);
   WaitForSingleObject(hEvent, INFINITE);
   wcscpy(MD.Component, MsgParams.cStatus);
   CloseHandle(hEvent);


   //
   // Get and save pointer to the channel-thread's buffer in the MD structure.
   //
   hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
   PostThreadMessage(SI.ChThreadID, TM_GET_BUFFER, (WPARAM)hEvent, (LPARAM)&MsgParams);
   WaitForSingleObject(hEvent, INFINITE);
   MD.pChBuffer=MsgParams.vPointer;
   CloseHandle(hEvent);


   //
   // Create the font used in the window.
   //
#ifdef WINCE

	LOGFONT lf;
	lf.lfHeight = 12;
	lf.lfWidth = 0; 
	lf.lfEscapement = 0; 
	lf.lfOrientation = 0; 
	lf.lfWeight = FW_DONTCARE; 
	lf.lfItalic = 0; 
	lf.lfUnderline = 0; 
	lf.lfStrikeOut = 0; 
	lf.lfCharSet = DEFAULT_CHARSET ; 
	lf.lfOutPrecision  = OUT_DEFAULT_PRECIS; 
	lf.lfClipPrecision = CLIP_DEFAULT_PRECIS; 
	lf.lfQuality = DRAFT_QUALITY; 
	lf.lfPitchAndFamily = FIXED_PITCH | FF_DONTCARE; 
	//lf.lfFaceName[LF_FACESIZE]  = TEXT("Terminal"); 

	MD.hFont = CreateFontIndirect(&lf);
#else
   MD.hFont=CreateFont(-8, 0, 0, 0, FW_DONTCARE, 0, 0, 0, DEFAULT_CHARSET,
                       OUT_CHARACTER_PRECIS, CLIP_DEFAULT_PRECIS,
                       DRAFT_QUALITY, FIXED_PITCH | FF_DONTCARE, TEXT("Terminal"));
#endif

   //
   // Load A bitmap used in the window.
   //
   MD.hBitMap=LoadBitmap(hInst, MAKEINTRESOURCE(IDB_MONARROW));


   //
   // Calculate the minimun allowed width of the client area.
   // The window must be long enough to hold at least the graph
   // for one whole block of data, and the texts above the graph.
   //
#ifdef WINCE
   MD.ClientAreaMinWidth=51+1+5+MD.DS.SamplesPrBlock+10;
   if (MD.ClientAreaMinWidth < 392) MD.ClientAreaMinWidth=392;
#else
   MD.ClientAreaMinWidth=51+1+5+MD.DS.SamplesPrBlock+10;
   if (MD.ClientAreaMinWidth < 392) MD.ClientAreaMinWidth=392;
#endif

   //
   // Calculate and save the minimum tracking size of the MDI child window.
   //
#ifdef WINCE

	MD.MinTrackSize.x = 240;
	MD.MinTrackSize.y = 280;

#else
   MD.MinTrackSize.x=MD.ClientAreaMinWidth+GetSystemMetrics(SM_CYSIZEFRAME)*2;
   MD.MinTrackSize.y=CLIENT_HEIGHT_MIN+GetSystemMetrics(SM_CYCAPTION)+GetSystemMetrics(SM_CXSIZEFRAME)*2;
#endif

   //
   // Initialize some variables in the MD structure.
   //
   MD.SampleAverage=0.0;
   MD.SampleCount=0;
   MD.bInitialCalc=TRUE;
   MD.ReadyToPaint=FALSE;
   MD.WndConfig.AutoScaling=TRUE;
   MD.WndConfig.UserScale=100;
   MD.WndConfig.SmplReduction=FALSE;
	MD.WndConfig.OneSmplPrBlock=FALSE;
   MD.WndConfig.Reduction=50;
   MD.ClntHeight=0;
   MD.ClntWidth=0;


   //
   // Open 'Monitor.wnd' file and read window-placement/properties.
   //
   wcscpy(ChDir, ChannelDir);
   StripBackslashFromPath(ChDir); wcscat(ChDir, TEXT("\\Channel"));
   swprintf(str2, TEXT("%02u"), SI.LogicalChannelNr); wcscat(ChDir, str2);
   wcscpy(WindowFile, ChDir); wcscat(WindowFile, TEXT("\\Monitor.wnd"));
   if (ExistFile(ChDir, TEXT("Monitor.wnd"))) bStatus=ReadFileToMemory(WindowFile, (PVOID)&MD.WndConfig, (DWORD)sizeof(MON_WND_CNF));


   //
   // Calculate number of samples in each block.
   //
	MD.SamplsPrBlock=MD.DS.SampleRate;
   if (MD.WndConfig.SmplReduction) {
      MD.SamplsPrBlock=GetSizeOfReducedBlck(MD.DS.SampleRate, MD.WndConfig.Reduction);
		if (MD.WndConfig.OneSmplPrBlock) MD.SamplsPrBlock=1;
   }


   //
   // Allocate memory for a sample buffer used when sample reduction is enabled.
   //
   MD.WndConfig.SmplBuff=(long*)LocalAlloc(LPTR, (SIZE_T)(MD.DS.SampleRate*4));
   if (!MD.WndConfig.SmplBuff) {
      MD.WndConfig.SmplReduction=FALSE;

      // Update program log.
      wcscpy(Message, UniqueProcName);
      wcscat(Message, TEXT(":"));
      wcscat(Message, TEXT(" Unable to allocate memory. Sample reduction are disabled."));
      WriteSystemLog(Message, 2);
}


   //
   // Create the MDI child window.
   //
   GetClientRect(hWndFrame, &rect); // Get the client rectangle.
   if (rect.right<MD.ClientAreaMinWidth) rect.right=MD.ClientAreaMinWidth;
#ifdef WINCE
	hWndMonitor=CreateWindow(TEXT("SeislogMonitorClass"),		// Pointer to registered class name.
									 UniqueProcName,						// Pointer to window name.
									 WS_CHILD |WS_VISIBLE | WS_BORDER,	// Window style.
									 0,										// Horizontal position of window.
									 50,										// Vertical position of window.
									 240, //rect.right,							// Width of window. 240
									 218, //MD.MinTrackSize.y,					// Height of window. 240
									 SI.hParent,								// Handle to parent window
									 NULL,									// Handle to a Menu
									 hInst,									// Handle to application instance.
									 NULL										// Application-defined value.
									 );

#else
   hWndMonitor=CreateMDIWindow(TEXT("SeislogMonitorClass"),  // Pointer to registered class name.
                               UniqueProcName,         // Pointer to window name.
                               0,                      // Window style.
                               0,                      // Horizontal position of window.
                               0,                      // Vertical position of window.
                               rect.right,             // Width of window.
                               MD.MinTrackSize.y,      // Height of window.
                               hWndMDIClient,          // Handle to parent window (MDI client).
                               hInst,                  // Handle to application instance.
                               NULL);                  // Application-defined value.
#endif

   //
   // Acossiate with this window a pointer the the
   // window-functions static variables.
   //
   SetWindowLong(hWndMonitor, GWL_USERDATA, (LONG)(MON_WND_DATA*)&MD);

   //
   // Set window-placement.
   //
#ifndef WINCE
   if (bStatus) SetWindowPlacement(hWndMonitor, &MD.WndConfig.WndPlacement);
#endif

   // Always turn AutoOpen on when starting the monitor window.
   // AutoOpen will be turned off if user closes the window manually.
   MD.WndConfig.AutoOpen=TRUE;


   //
   // Add 'Properties' menu to the system menu so that
   // user can bring up the 'Monitor Properties' dialog box.
   //
   hSysMenu=GetSystemMenu(hWndMonitor, FALSE);
   AppendMenu(hSysMenu, MF_SEPARATOR, 0, NULL);
   AppendMenu(hSysMenu, MF_STRING, SC_PROPERTIES, TEXT("Properties ..."));


   //
   // Create a compatible memory device context. The graph will
   // be drawn in this context, and then copied to the screen.
   //
   MD.MemDCWidth=GetSystemMetrics(SM_CXSCREEN);  // Get the horizontal screen resolution.
   MD.MemDCHeight=GetSystemMetrics(SM_CYSCREEN); // Get the vertical screen resolution.
   hdc=GetDC(hWndMonitor);
   MD.MemDC=CreateCompatibleDC(hdc);
   hbit=CreateCompatibleBitmap(hdc, MD.MemDCWidth, MD.MemDCHeight);
   SelectObject(MD.MemDC, hbit);
   ReleaseDC(hWndMonitor, hdc);


   //
   // Make the MemDC background white.
   //
   OldBrush=SelectObject(MD.MemDC, GetStockObject(WHITE_BRUSH));
   PatBlt(MD.MemDC, 0, 0, MD.MemDCWidth, MD.MemDCHeight, PATCOPY);
   SelectObject(MD.MemDC, OldBrush);


   //
   // If running in debug mode, write a message to the log.
   //
   if (DebugLevel) {
      wcscpy(Message, UniqueProcName);
      wcscat(Message, TEXT(": Thread started."));
      WriteSystemLog(Message, 1);
   }


   // Start displaying data.
   PostMessage(hWndMonitor, WM_STARTUP, 0 ,0);


   //
   // Enter the thread message loop. The thread will now start to process messages.
   //
   while (GetMessage(&msg, (HWND)NULL, 0, 0)) {

      // If debuglevel is 2 then log all message recieved.
      if (DebugLevel>1) {
         TCHAR str10[11];
         wcscpy(Message, UniqueProcName);
         wcscat(Message, TEXT(": Recieved message "));
         _itow(msg.message, str10, 10);
         wcscat(Message, str10);
         WriteSystemLog(Message, 1);
      }

      TranslateMessage(&msg);
      DispatchMessage(&msg);

      // Handle the TM_EXIT message.
      if (msg.message==TM_EXIT)
		  SendMessage(hWndMonitor, WM_CLOSE, 0, 0);

#ifdef WINCE
	  if (msg.message==TM_MONITOR_PROPERTIES)
		  SendMessage(hWndMonitor, WM_MONITOR_PROPERTIES, 0, 0);
#endif

   }


   //
   // Save current window-placement to config file.
   //
   WriteFileFromMemory(WindowFile, (PVOID)&MD.WndConfig, (DWORD)sizeof(MON_WND_CNF));


   //
   // Delete allocated GDI objects.
   //
   DeleteDC(MD.MemDC);
   DeleteObject(MD.hFont);
   DeleteObject(MD.hBitMap);


   //
   // Free allocated memory.
   //
   if (MD.WndConfig.SmplBuff) LocalFree(MD.WndConfig.SmplBuff);


   //
   // Unregister this process from the process database.
   //
   UnregisterProcess(GetCurrentThreadId());


   //
   // If running in debug mode, write a message to the log.
   //
   if (DebugLevel) {
      wcscpy(Message, UniqueProcName);
      wcscat(Message, TEXT(": Thread stopped."));
      WriteSystemLog(Message, 1);
   }


   //
   // Exit from thread.
   //
   ExitThread(0);
   return 0L;

}







/*********************************************************************************/
/* Monitor Window Procedure                                                      */
/*********************************************************************************/
LRESULT CALLBACK MonitorWndProc(HWND hWnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
   HDC            hdc;
   RECT           rect;
   HPEN           hPen, hPenOld, hPenBlack, hPenRed;
   PAINTSTRUCT    ps;
   BYTE           OldNumGraphInPlot;
   TCHAR           TimeStr[13], str10[11], str11[12], str32[33];
   INT            xpos, ypos, index, BlockStartX;
   WORD           OldClntWidth, OldClntHeight;
   WORD           Index, OldGraphWidthInBlocks;
   MON_WND_DATA   *S;
   HANDLE         hEvent;
   MSG_PARAM      MsgParams;
   SUBSCRIBE      SubscrData;
   BLOCK_HEADER   *pBlockHdr;
   LONG           *pBlockData, *pSample, DO;
#ifdef WINCE
	RECT				textRect;
#else
	MINMAXINFO     *pMMI;
#endif

   // In a MDI child window function, the following messages MUST be
   // handled by DefMDIChildProc, whether the child handles them
   // or not:
   //
   //    WM_CHILDACTIVATE, WM_GETMINMAXINFO, WM_MENUCHAR,
   //    WM_MOVE, WM_SETFOCUS, WM_SIZE, WM_SYSCOMMAND.
   //

   // Get pointer to this function's static variables.
   S=(MON_WND_DATA*)GetWindowLong(hWnd, GWL_USERDATA);
#ifdef WINCE
	if (S==NULL) return DefWindowProc(hWnd, Message, wParam, lParam);
#else
   if (S==NULL) return DefMDIChildProc(hWnd, Message, wParam, lParam);
#endif

   switch (Message) {
		case WM_CREATE:
			SetForegroundWindow(hWnd);
			break;
      case WM_STARTUP:
      // Get the size of client area.
      GetClientRect(hWnd, &rect);
      S->ClntWidth=rect.right+1;    // Width of client area.
      S->ClntHeight=rect.bottom+1;  // Height of client area.

      // Calculate how many blocks there are room for in the graph.
#ifdef WINCE
		S->GraphWidthInBlocks=(S->ClntWidth-36)/S->SamplsPrBlock;
#else		
		S->GraphWidthInBlocks=(S->ClntWidth-67)/S->SamplsPrBlock;
#endif
      // Calculate the graphwidth in pixels.
      S->GraphWidthInPixels=S->GraphWidthInBlocks*S->SamplsPrBlock;

      // Calculate how many graphs we have room for in the window.
      S->NumGraphInPlot=(S->ClntHeight-40)/GSV;

      // Start drawing in block-position 0 (the first block) in the graph window.
      S->BlockToDraw=0;

      // Start drawing in the first graph.
      S->CurrGraph=0;

      // Set Window title.
      SetWindowTitle(S, hWnd);

      // We now need to subcribe to the TM_BUFFERUPDATED messages from the channel.
      SubscrData.uiMessage=TM_BUFFERUPDATED;
      SubscrData.idThread=0;
      SubscrData.hWnd=hWnd;
      MsgParams.vPointer=(PVOID)&SubscrData;
      hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
      PostThreadMessage(S->ChThreadID, TM_SUBSCRIBE, (WPARAM)hEvent, (LPARAM)&MsgParams);
      WaitForSingleObject(hEvent, INFINITE);
      CloseHandle(hEvent);
      return 0;



      case TM_BUFFERUPDATED:
      //
      // A new block of data is ready in the channel's ringbuffer.
      //

      // Get a pointer to the block-header and the block-samples.
      pBlockHdr=(BLOCK_HEADER*)((PBYTE)S->pChBuffer+((DWORD)lParam*S->DS.BlockSize));
      pBlockData=(long*) ((byte*)pBlockHdr+sizeof(BLOCK_HEADER));

      //
      // Calculate the offset in the data (device offset).
      // Also calculate an automatic scale factor.
      //
      if (S->bInitialCalc) {
         //
         // Calculate the average of 500+ samples
         // to find the offset of the device.
         //
         pSample=pBlockData;
         for (Index=0;Index<S->DS.SamplesPrBlock;Index++) {
            if (!S->SampleCount) {
               // Since this is the first sample, we
               // set the average equal to this sample.
               S->SampleAverage=(DOUBLE)*pSample;
               S->SampleMax=*pSample;
               S->SampleMin=*pSample;
            } else {
               // Calculate this sample's contribution to the average.
               S->SampleAverage=S->SampleAverage+(((DOUBLE)*pSample-S->SampleAverage)/(S->SampleCount+1));

               // Also save the largest and smallest sample found.
               // Theese values are used in the autoscale calculation.
               if (*pSample>S->SampleMax) S->SampleMax=*pSample;
               if (*pSample<S->SampleMin) S->SampleMin=*pSample;
            }

            // Move on to the next sample.
            S->SampleCount++;
            pSample++;
         }
         if (S->SampleCount>=500) {
            // We are finished calculating the sample average.
            // Calculate the offset of the device. The offset
            // is a rounded value of the sample average.
            S->DeviceOffset=S->SampleAverage;

            // Calculate the number on the top and bottom
            // of the y-axis. This value is selected so that
            // the largest sample found during the average
            // calculations, will occupy approx. one third
            // of the available y-scale.
            S->SampleMax=S->SampleMax-(LONG)S->DeviceOffset;
            S->SampleMin=S->SampleMin-(LONG)S->DeviceOffset;
            S->MaxVertScale=3*S->SampleMax;
            if (abs(S->SampleMin)>S->SampleMax) S->MaxVertScale=3*abs(S->SampleMin);
            if (S->MaxVertScale<=10) S->MaxVertScale=10;

            // The number we have calculated is probably
            // not very suited to go on the scale, so we
            // need to find a nice round number close to it.
            S->AutoMaxScale=CalculateScale2(S->MaxVertScale);

            // Calculate the factor wich all samples are
            // multiplied with to get the hight of the
            // samples on the screen (in pixels).
            S->VertScaleFactor=40.0/(FLOAT)S->MaxVertScale;


            //
            // Handle the graph scaling.
            //
            if (S->WndConfig.AutoScaling) {
               // Auto scaling is enabled.
               S->MaxVertScale=S->AutoMaxScale;
            } else {
               // User has selected a scale factor.
               S->MaxVertScale=S->WndConfig.UserScale;
            }
            S->VertScaleFactor=40.0/(FLOAT)S->MaxVertScale;

            // We are now finished with the initial calculations.
            S->bInitialCalc=FALSE;
            S->ReadyToPaint=TRUE;
            InvalidateRect(hWnd, NULL, TRUE);
            UpdateWindow(hWnd);  // Draw the client area.
         }
      } else {

         // Save the time for this block.
         S->GraphTime=pBlockHdr->TimeTag;

         // Handle sample reduction.
         if (S->WndConfig.SmplReduction) {
				if (S->WndConfig.OneSmplPrBlock) {
					// Use only one sample per block.
					S->WndConfig.SmplBuff[0]=*pBlockData;
				} else {
	            // Remove a certain percentage of the pixels.
					ReduceSamples(pBlockData, S->WndConfig.SmplBuff, S->DS.SampleRate, S->WndConfig.Reduction);
				}
         }

         // ********************************************************************
         // Draw the graph in the memory device-context and then copy to screen.
         // ********************************************************************
         // Calculate first x-position for this
         // block in the memory device context.
         BlockStartX=S->SamplsPrBlock*S->BlockToDraw;

         // Clear the area where the new block are to be drawn.
         SelectObject(S->MemDC, GetStockObject(WHITE_BRUSH));
         if (S->NumGraphInPlot==1) {
            PatBlt(S->MemDC, BlockStartX, 0, S->SamplsPrBlock+6, S->ClntHeight, PATCOPY);
         } else {
            if (!S->CurrGraph) {
               // Clear the top graph in a multigraph window.
               PatBlt(S->MemDC, BlockStartX, 0, S->SamplsPrBlock+6, 50+GSV/2, PATCOPY);
            } else if (S->CurrGraph==S->NumGraphInPlot-1) {
               // Clear the bottom graph in a multigraph window.
               ypos=50+(S->CurrGraph*GSV)-GSV/2;
               PatBlt(S->MemDC, BlockStartX, ypos, S->SamplsPrBlock+6, S->ClntHeight-ypos, PATCOPY);
            } else {
               // Clear one of the graphs in the middle of a multigraph window.
               ypos=50+(S->CurrGraph*GSV)-(GSV/2);
               PatBlt(S->MemDC, BlockStartX, ypos, S->SamplsPrBlock+6, GSV, PATCOPY);
            }
         }

         // Draw the graph for this block.
         hPen=CreatePen(PS_SOLID, 0, RGB(0,128,0)); // Create a green pen.
         hPenOld=(HPEN)SelectObject(S->MemDC, (HGDIOBJ)hPen);

         // Round the deviceoffset to the nearest whole value.
         DO=(LONG)DblRound(S->DeviceOffset);

         if (!S->BlockToDraw) {
            // Draw the first block in the window.
            xpos=0;
            pSample=pBlockData;
            if (S->WndConfig.SmplReduction) pSample=S->WndConfig.SmplBuff;
            ypos=50+(S->CurrGraph*GSV)-(INT)DblRound((DOUBLE)(*pSample-DO)*S->VertScaleFactor);
#ifdef WINCE
				MoveTo(S->MemDC, xpos, ypos);
#else
				MoveToEx(S->MemDC, xpos, ypos, NULL);
#endif
            for (index=2;index<=S->SamplsPrBlock;index++) {
               pSample++;
               xpos++;
               ypos=50+(S->CurrGraph*GSV)-(INT)DblRound((DOUBLE)(*pSample-DO)*S->VertScaleFactor);
               LineTo(S->MemDC, xpos, ypos);
            }
            // Save the last point in this block.
            S->YPosLast=ypos;
         } else {
            // Draw a block that are not in the first position.
            xpos=BlockStartX-1; // Start from the last point in the prev. block.
            pSample=pBlockData;
            if (S->WndConfig.SmplReduction) pSample=S->WndConfig.SmplBuff;
#ifdef WINCE
            MoveTo(S->MemDC, xpos, S->YPosLast);
#else
				MoveToEx(S->MemDC, xpos, S->YPosLast, NULL);
#endif
         for (index=1;index<=S->SamplsPrBlock;index++) {
               xpos++;
               ypos=50+(S->CurrGraph*GSV)-(INT)DblRound((DOUBLE)(*pSample-DO)*S->VertScaleFactor);
               LineTo(S->MemDC, xpos, ypos);
               pSample++;
            }
            // Save the last point in this block.
            S->YPosLast=ypos;
         }
         DeleteObject(SelectObject(S->MemDC, hPenOld));

         // Copy the graph from the memoryDC to the window.
         hdc=GetDC(hWnd);
#ifdef WINCE
		 BitBlt(hdc, 26, 39, S->ClntWidth, S->ClntHeight-19, S->MemDC, 0, 0, SRCCOPY);	
#else
         BitBlt(hdc, 56, 19, S->ClntWidth-41, S->ClntHeight-19, S->MemDC, 0, 0, SRCCOPY);
#endif
		 // Redraw the scale numbers
         swprintf(str10, TEXT("%d"), S->MaxVertScale);
         if (S->MaxVertScale>999999) {
            // Use 'E' notation to save space.
            swprintf(str10, TEXT("%.2lE"), (DOUBLE)S->MaxVertScale);
            StrDelete(str10, wcslen(str10)-2);  // Remove a zero from the exponent.
            StrDelete(str10, wcslen(str10)-2);  // Remove a zero from the exponent.
            StrDelete(str10, wcslen(str10)-2);  // Remove the + sign from the exponent.
         }

         // Draw the block-time above the graph.
         CreateTimeString(&S->GraphTime, TimeStr);
			SelectObject(hdc, S->hFont);
#ifdef WINCE
			
			//Draw scale numbers
			textRect.top = 40;
			textRect.left = 27;
			textRect.bottom = 56;
			textRect.right = 200;
			DrawText(hdc, str10, wcslen(str10), &textRect, DT_LEFT|DT_BOTTOM);
			
			wcscpy(str11, TEXT("-")); wcscat(str11, str10);

			textRect.top = 120;
			textRect.left = 24;
			textRect.bottom = 135;
			textRect.right = 200;
			DrawText(hdc, str11, wcslen(str11), &textRect, DT_LEFT|DT_BOTTOM);

			//Draw block-time
			textRect.top = 3; 
			textRect.left = 120; 
			textRect.bottom = 15; 
			textRect.right = 240;
			DrawText(hdc, TimeStr, wcslen(TimeStr), &textRect, DT_LEFT | DT_BOTTOM);
#else
        
			SetTextAlign(hdc, TA_LEFT|TA_BASELINE);
         TextOut(hdc, 185, 13, TimeStr, wcslen(TimeStr));
#endif
         // Release the DC.
         ReleaseDC(hWnd, hdc);

         // Find out which block and graph to draw next,
         S->BlockToDraw++;
         if (S->BlockToDraw>=S->GraphWidthInBlocks) {
            S->BlockToDraw=0;
            S->CurrGraph++;
            if (S->CurrGraph>=S->NumGraphInPlot) S->CurrGraph=0;
         }
      }
      return 0;


#ifndef WINCE
      case WM_GETMINMAXINFO:
      //
      // Set the minimum size of this window.
      //
      pMMI=(LPMINMAXINFO)lParam;
      pMMI->ptMinTrackSize=S->MinTrackSize;
      return 0;
#endif

      case WM_SIZE:
      // Save the old size of client area.
      OldClntWidth=S->ClntWidth;
      OldClntHeight=S->ClntHeight;
      OldNumGraphInPlot=S->NumGraphInPlot;
      OldGraphWidthInBlocks=S->GraphWidthInBlocks;

      // Save the new size of client area.
      S->ClntWidth=LOWORD(lParam);  // width of client area
      S->ClntHeight=HIWORD(lParam); // height of client area

      // Calculate how many blocks there are room
      // for in the graph after this window resize.
#ifdef WINCE
      S->GraphWidthInBlocks=(S->ClntWidth-36)/S->SamplsPrBlock;
#else
		S->GraphWidthInBlocks=(S->ClntWidth-67)/S->SamplsPrBlock;
#endif
      // Recalculate the new graphwidth in pixels.
      S->GraphWidthInPixels=S->GraphWidthInBlocks*S->SamplsPrBlock;

      // Recalculate how many graphs there is room for.
      S->NumGraphInPlot=(S->ClntHeight-40)/GSV;
      if (!S->NumGraphInPlot) S->NumGraphInPlot=1;

      // Handle reduction in number of graphs or blocks.
      if (S->NumGraphInPlot<OldNumGraphInPlot || S->GraphWidthInBlocks<OldGraphWidthInBlocks) {
         // Start from the top again.
         S->CurrGraph=0;
         S->BlockToDraw=0;

         // Clear the MemDC.
         SelectObject(S->MemDC, GetStockObject(WHITE_BRUSH));
         PatBlt(S->MemDC, 0, 0, S->MemDCWidth, S->MemDCHeight, PATCOPY);
      }

      // Redraw the client area.
      InvalidateRect(hWnd, NULL, TRUE);
      break;



      case WM_PAINT:
      //
      // Redraw the complete window.
      //
      hdc=BeginPaint(hWnd, &ps);

      if (S->ReadyToPaint) {
         // Save the current device context.
         SaveDC(hdc);

         // Create the pens needed for the drawing.
         hPenBlack=CreatePen(PS_SOLID, 0, RGB(0,0,0)); // Create a black pen.
         hPenRed=CreatePen(PS_SOLID, 0, RGB(128,0,0)); // Create a red pen.

         // Select the font to be used in the window.
         SelectObject(hdc, S->hFont);

         // Draw the Station and Component texts above the graph.
#ifndef WINCE
         SetTextAlign(hdc, TA_LEFT|TA_BASELINE);
#endif
         wcscpy(str32, TEXT("Stat:"));
         wcscat(str32, S->Station);
#ifdef WINCE
			textRect.top = 3; 
			textRect.left = 3;
			textRect.bottom = 15; 
			textRect.right = 65;
			DrawText(hdc, str32, wcslen(str32), &textRect, DT_LEFT|DT_BOTTOM);
#else
         TextOut(hdc, 51, 13, str32, wcslen(str32));
#endif
         wcscpy(str32, TEXT("Comp:"));
         wcscat(str32, S->Component);
#ifdef WINCE
			textRect.left = 65;
			textRect.right = 110;
			DrawText(hdc, str32, wcslen(str32), &textRect, DT_LEFT|DT_BOTTOM);
#else			
         TextOut(hdc, 121, 13, str32, wcslen(str32));
#endif

         // Draw the block-time above the graph.
         CreateTimeString(&S->GraphTime, TimeStr);
			SelectObject(hdc, S->hFont);
						
#ifdef WINCE
			textRect.top = 3; 
			textRect.left = 120; 
			textRect.bottom = 15; 
			textRect.right = 240;
			DrawText(hdc, TimeStr, wcslen(TimeStr), &textRect, DT_LEFT|DT_BOTTOM);
#else
         SetTextAlign(hdc, TA_LEFT|TA_BASELINE);
         TextOut(hdc, 185, 13, TimeStr, wcslen(TimeStr));
#endif
         // Draw the one-second length marker with text.
			// If one sample per block is turned on, then
			// we draw a 10 second length marker instead.
			
#ifdef WINCE
			if (S->WndConfig.SmplReduction && S->WndConfig.OneSmplPrBlock) {
				MoveTo(hdc, 26, 29);	
				LineTo(hdc, 26+29, 29);
				MoveTo(hdc, 26, 26);	
				LineTo(hdc, 26, 33);   
				MoveTo(hdc, 26+29, 26);
				LineTo(hdc, 26+29, 33);

				textRect.top = 19;
				textRect.left = 9;
				textRect.bottom = 33;
				textRect.right = 50;
				DrawText(hdc,TEXT("30s"), 3, &textRect, DT_LEFT|DT_BOTTOM);
				
			} else {
				
				MoveTo(hdc, 26, 29);
				LineTo(hdc, 26+S->SamplsPrBlock-1, 29);
				MoveTo(hdc, 26, 26);
				LineTo(hdc, 26, 33);
				MoveTo(hdc, 26+S->SamplsPrBlock-1, 26);
				LineTo(hdc, 26+S->SamplsPrBlock-1, 33);
				
				textRect.top = 19; //272+S->SamplsPrBlock+2;
				textRect.left = 13;
				textRect.bottom = 33; //272+32 + 40;
				textRect.right = 50; //500;
				DrawText(hdc,TEXT("1s"),2,&textRect,DT_LEFT|DT_BOTTOM);
				/*
				SetTextAlign(hdc, TA_LEFT|TA_BASELINE);
				TextOut(hdc, 272+S->SamplsPrBlock+2, 13, TEXT("1s"), 2);*/
			}
#else				
			if (S->WndConfig.SmplReduction && S->WndConfig.OneSmplPrBlock) {
				MoveToEx(hdc, 272, 9, NULL);
				LineTo(hdc, 272+29, 9);
				MoveToEx(hdc, 272, 6, NULL);
				LineTo(hdc, 272, 13);
				MoveToEx(hdc, 272+29, 6, NULL);
				LineTo(hdc, 272+29, 13);
				SetTextAlign(hdc, TA_LEFT|TA_BASELINE);
				TextOut(hdc, 272+32, 13, TEXT("30s"), 3);
			} else {
				MoveToEx(hdc, 272, 9, NULL);
				LineTo(hdc, 272+S->SamplsPrBlock-1, 9);
				MoveToEx(hdc, 272, 6, NULL);
				LineTo(hdc, 272, 13);
				MoveToEx(hdc, 272+S->SamplsPrBlock-1, 6, NULL);
				LineTo(hdc, 272+S->SamplsPrBlock-1, 13);
				SetTextAlign(hdc, TA_LEFT|TA_BASELINE);
				TextOut(hdc, 272+S->SamplsPrBlock+2, 13, TEXT("1s"), 2);
			}
#endif
         // Draw the scale numbers
         swprintf(str10, TEXT("%d"), S->MaxVertScale);
         if (S->MaxVertScale>999999) {
            // Use 'E' notation to save space.
            swprintf(str10, TEXT("%.2lE"), (DOUBLE)S->MaxVertScale);
            StrDelete(str10, wcslen(str10)-2);  // Remove a zero from the exponent.
            StrDelete(str10, wcslen(str10)-2);  // Remove a zero from the exponent.
            StrDelete(str10, wcslen(str10)-2);  // Remove the + sign from the exponent.
         }
#ifdef WINCE
			textRect.top = 40;
			textRect.left = 25;
			textRect.bottom = 56;
			textRect.right = 200;
			DrawText(hdc, str10, wcslen(str10), &textRect, DT_LEFT|DT_BOTTOM);
						
			wcscpy(str11, TEXT("-")); wcscat(str11, str10);

			textRect.top = 113;
			DrawText(hdc, str11, wcslen(str11), &textRect, DT_LEFT|DT_BOTTOM);

#else
         SetTextAlign(hdc, TA_RIGHT|TA_BASELINE);
         TextOut(hdc, 46, 33, str10, wcslen(str10));
         TextOut(hdc, 46, 73, TEXT("0"), 1);
         wcscpy(str11, TEXT("-")); wcscat(str11, str10);
         TextOut(hdc, 46, 113, str11, wcslen(str11));
#endif

         // Draw the vertical line in front of the graph.
         // Also draw the 3 small scale marks.
#ifdef WINCE
			MoveTo(hdc, 21, 49);
         LineTo(hdc, 21, 129);
         MoveTo(hdc, 21, 49);
         LineTo(hdc, 18, 49);
         MoveTo(hdc, 21, 89);
         LineTo(hdc, 18, 89);
         MoveTo(hdc, 21, 129);
         LineTo(hdc, 18, 129);
#else
			MoveToEx(hdc, 51, 29, NULL);
         LineTo(hdc, 51, 109);
         MoveToEx(hdc, 51, 29, NULL);
         LineTo(hdc, 48, 29);
         MoveToEx(hdc, 51, 69, NULL);
         LineTo(hdc, 48, 69);
         MoveToEx(hdc, 51, 109, NULL);
         LineTo(hdc, 48, 109);
#endif
         // Draw the graph by copying from the memory DC.
#ifdef WINCE
			BitBlt(hdc, 26, 39, S->ClntWidth-41, S->ClntHeight-19, S->MemDC, 0, 0, SRCCOPY); //16
#else
			BitBlt(hdc, 56, 19, S->ClntWidth-41, S->ClntHeight-19, S->MemDC, 0, 0, SRCCOPY);
#endif
         // Restore device context.
         RestoreDC(hdc, -1);

         // Delete pen objects.
         DeleteObject((HGDIOBJ)hPenRed);
         DeleteObject((HGDIOBJ)hPenBlack);
      }

      // The redraw has finished.
      EndPaint(hWnd, &ps);
      return 0;

#ifndef	WINCE
      case WM_CHILDACTIVATE:
      // This window is being activated.
      break;
#endif

      case WM_SYSCOMMAND:
      case WM_COMMAND:
      //
      // The WM_SYSCOMMAND message is sendt from the system menu.
      // The WM_COMMAND is sendt from the main menu if MDI child is maximized.
      //
      switch(wParam & 0xFFF0) {
         case SC_CLOSE:
         // The user closes this window manually.
         // The window should NOT be automatically
         // opened the next time the system is started.
         S->WndConfig.AutoOpen=FALSE;
         break;

         case SC_PROPERTIES:
         // User want to set the monitor properties, bring up the 'Monitor Properties' dialog.
         S->WndConfig.hWnd=hWnd;
         DialogBoxParam(hInst, MAKEINTRESOURCE(IDD_MONITOR_PROPERTIES), hWnd, (DLGPROC)SetMonPropDlgProc, (LPARAM)&S->WndConfig);
         break;
      }
      break;

#ifdef WINCE 
      case WM_MONITOR_PROPERTIES:
         // User want to set the monitor properties, bring up the 'Monitor Properties' dialog.
         S->WndConfig.hWnd=hWnd;
         DialogBoxParam(hInst, MAKEINTRESOURCE(IDD_MONITOR_PROPERTIES), hWnd, (DLGPROC)SetMonPropDlgProc, (LPARAM)&S->WndConfig);
         break;
#endif

      case TM_APPLY_CONFIGURATION:
      // Handle configuration changes.
      if (S->WndConfig.AutoScaling) {
         // Auto scaling is enabled.
         S->MaxVertScale=S->AutoMaxScale;
      } else {
         // User has selected a scale factor.
         S->MaxVertScale=S->WndConfig.UserScale;
      }

      S->VertScaleFactor=40.0/(FLOAT)S->MaxVertScale;

      if (S->WndConfig.SmplReduction) {
         // Sample reduction is enabled.
         S->SamplsPrBlock=GetSizeOfReducedBlck(S->DS.SampleRate, S->WndConfig.Reduction);
			if (S->WndConfig.OneSmplPrBlock) S->SamplsPrBlock=1;
      } else {
         // Sample reduction is disabled.
         S->SamplsPrBlock=S->DS.SampleRate;
      }

      // Set Window title.
      SetWindowTitle(S, hWnd);

      // Recalculate and redraw the client area.
      S->CurrGraph=0; S->BlockToDraw=0;
#ifdef WINCE
      S->GraphWidthInBlocks=(S->ClntWidth-36)/S->SamplsPrBlock;
#else
		S->GraphWidthInBlocks=(S->ClntWidth-67)/S->SamplsPrBlock;
#endif
      S->GraphWidthInPixels=S->GraphWidthInBlocks*S->SamplsPrBlock;
      SelectObject(S->MemDC, GetStockObject(WHITE_BRUSH));
      PatBlt(S->MemDC, 0, 0, S->MemDCWidth, S->MemDCHeight, PATCOPY);
      InvalidateRect(hWnd, NULL, TRUE);
      break;


      case WM_CLOSE:
      //
      // This message is either sendt from the thread message loop,
      // or it is sendt from the MDIClient window as a response to
      // a press on the X button in the upper right window corner,
      // or the 'Close' option on the system menu.

      // Unsubscribe to the TM_BUFFERUPDATED message.
      SubscrData.uiMessage=TM_BUFFERUPDATED;
      SubscrData.idThread=0;
      SubscrData.hWnd=hWnd;
      MsgParams.vPointer=(PVOID)&SubscrData;
      hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
      PostThreadMessage(S->ChThreadID, TM_UNSUBSCRIBE, (WPARAM)hEvent, (LPARAM)&MsgParams);
      WaitForSingleObject(hEvent, INFINITE);
      CloseHandle(hEvent);

      // Save the current window size/position.
#ifndef WINCE
      S->WndConfig.WndPlacement.length=(UINT)sizeof(WINDOWPLACEMENT);
      GetWindowPlacement(hWnd, &S->WndConfig.WndPlacement);
#endif
      // Destroy the MDI child window
#ifdef WINCE
		SendMessage(hWndMDIClient, WM_DESTROY, (WPARAM)hWnd,0);
#else
      SendMessage(hWndMDIClient, WM_MDIDESTROY, (WPARAM)hWnd, 0);
#endif

      // End the thread loop.
      PostQuitMessage(0);
      return 0;
   }

   // Pass unprocessed messages to the 'DefMDIChildProc' function.
   // NOTE: Some messages must be passed on to 'DefMDIChildProc' even if they are processed.
#ifdef WINCE
   return DefWindowProc(hWnd, Message, wParam, lParam);
#else
	return DefMDIChildProc(hWnd, Message, wParam, lParam);
#endif
}



/*******************************************************************/
/*   Calculates a nice round number for the top of the Y-cale.     */
/*******************************************************************/
LONG CalculateScale2(LONG MaxScale)
{
   INT Mag;
   LONG Step, RetValue;

   // Check input parameter.
   if (MaxScale<=0) return 0;

   // Start by finding the magnitude of the
   // incoming number. Decimals are discarded.
   Mag=max((INT)log10((DOUBLE)MaxScale), 1);
   Step=(LONG)pow(10.0, (DOUBLE)Mag)/5;

   // Going in 'step' steps from zero and upwards, find
   // the first number larger or equal to MaxScale.
   RetValue=0;
   while (RetValue<MaxScale) RetValue=RetValue+Step;

   return RetValue;
}








/*******************************************************************/
/*   Dialog box procedure for the 'Monitor Properties' dialog.     */
/*******************************************************************/
BOOL CALLBACK SetMonPropDlgProc(HWND hWndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{

#ifdef WINCE
   SHINITDLGINFO shidi; 
#endif

   TCHAR str15[16], MessageText[200], Index;
   BYTE Reductions[9]={10,20,25,33,50,66,75,80,90};
   HWND hwndDlgItem;
   DWORD dwIndex;
   MON_WND_CNF *SI;
   LONG Scale;

   switch(message)  {
      case WM_INITDIALOG:
#ifdef WINCE
	  shidi.dwMask = SHIDIM_FLAGS;
	  shidi.dwFlags = SHIDIF_DONEBUTTON | SHIDIF_SIPDOWN | SHIDIF_SIZEDLGFULLSCREEN;
	  shidi.hDlg = hWndDlg;
	  SHInitDialog(&shidi);
#endif
      // Save pointer to return structure.
      SetWindowLong(hWndDlg, GWL_USERDATA, (LONG)lParam);

      // Create a pointer to the startup structure.
      SI=(MON_WND_CNF*)lParam;

      // Initialize the 'Scale' fields.
      SetDlgItemText(hWndDlg, IDC_EDIT1, _ltow(SI->UserScale, str15, 10));
      if (SI->AutoScaling) {
         EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT1), FALSE);
         CheckDlgButton(hWndDlg, IDC_CHECK1, BST_CHECKED);
      } else {
         CheckDlgButton(hWndDlg, IDC_CHECK1, BST_UNCHECKED);
      }

      // Initialize the 'Sample Reduction' fields.
      hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO1);
      for (Index=0;Index<=8;Index++) {
         _itow(Reductions[Index], str15, 10);
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCSTR)str15);
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, (DWORD)Reductions[Index]);
         if (SI->Reduction==Reductions[Index]) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }

		if (SI->OneSmplPrBlock) {
			CheckDlgButton(hWndDlg, IDC_CHECK3, BST_CHECKED);
			EnableWindow(GetDlgItem(hWndDlg, IDC_COMBO1), FALSE);
		}

		if (SI->SmplReduction) {
			CheckDlgButton(hWndDlg, IDC_CHECK2, BST_CHECKED);
		} else {
         EnableWindow(GetDlgItem(hWndDlg, IDC_COMBO1), FALSE);
			EnableWindow(GetDlgItem(hWndDlg, IDC_CHECK3), FALSE);
		}

      // If memory allocation of sample buffer failed,
      // then disable the sample reduction controls.
      if (!SI->SmplBuff) {
         EnableWindow(GetDlgItem(hWndDlg, IDC_CHECK2), FALSE);
         EnableWindow(GetDlgItem(hWndDlg, IDC_COMBO1), FALSE);
			EnableWindow(GetDlgItem(hWndDlg, IDC_CHECK3), FALSE);
      }
      return TRUE;

      case WM_COMMAND:
      switch(LOWORD(wParam)) {
         case IDC_CHECK1:
         switch (HIWORD(wParam)) {
            case BN_CLICKED:
            if (IsDlgButtonChecked(hWndDlg, IDC_CHECK1)==BST_CHECKED) {
               // Disable the edit field.
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT1), FALSE);
            } else {
               // Enable the edit field.
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT1), TRUE);
               SetFocus(GetDlgItem(hWndDlg, IDC_EDIT1));
               SendDlgItemMessage(hWndDlg, IDC_EDIT1, EM_SETSEL, 0, -1);
            }
            break;
         }
         break;

         case IDC_CHECK2:
         switch (HIWORD(wParam)) {
            case BN_CLICKED:
            if (IsDlgButtonChecked(hWndDlg, IDC_CHECK2)==BST_UNCHECKED) {
               EnableWindow(GetDlgItem(hWndDlg, IDC_COMBO1), FALSE);
					EnableWindow(GetDlgItem(hWndDlg, IDC_CHECK3), FALSE);
            } else {
               EnableWindow(GetDlgItem(hWndDlg, IDC_COMBO1), TRUE);
					EnableWindow(GetDlgItem(hWndDlg, IDC_CHECK3), TRUE);
					if (IsDlgButtonChecked(hWndDlg, IDC_CHECK3)==BST_CHECKED) {
						EnableWindow(GetDlgItem(hWndDlg, IDC_COMBO1), FALSE);
					} else {
						SetFocus(GetDlgItem(hWndDlg, IDC_COMBO1));
					}
            }
            break;
         }
         break;

         case IDC_CHECK3:
         switch (HIWORD(wParam)) {
            case BN_CLICKED:
            if (IsDlgButtonChecked(hWndDlg, IDC_CHECK3)==BST_UNCHECKED) {
               EnableWindow(GetDlgItem(hWndDlg, IDC_COMBO1), TRUE);
               SetFocus(GetDlgItem(hWndDlg, IDC_COMBO1));
            } else {
               EnableWindow(GetDlgItem(hWndDlg, IDC_COMBO1), FALSE);
            }
            break;
         }
         break;

#ifdef WINCE
		case IDOK:
#else
		case ID_APPLY:
#endif
         //
         // This is the 'Apply' button.
         //

         // Get saved pointer to returndata structure.
         SI=(MON_WND_CNF*)GetWindowLong(hWndDlg, GWL_USERDATA);

         // Read the status of the first check box.
         SI->AutoScaling=(IsDlgButtonChecked(hWndDlg, IDC_CHECK1)==BST_CHECKED);

         // Read and check the scale value if autoscaling is off.
         if (!SI->AutoScaling) {
            GetDlgItemText(hWndDlg, IDC_EDIT1, str15, 15);
            Scale=_wtol(str15);
            if (Scale<5) {
               wcscpy(MessageText, TEXT("The value entered is invalid.\n"));
               wcscat(MessageText, TEXT("Please enter a positive integer\n"));
               wcscat(MessageText, TEXT("greater then or equal to 5."));
               MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);

               // Move the input focus to this field.
               SetFocus(GetDlgItem(hWndDlg, IDC_EDIT1));
               SendDlgItemMessage(hWndDlg, IDC_EDIT1, EM_SETSEL, 0, -1);
               break;
            } else {
               SI->UserScale=Scale;
            }
         }

         // Read the status of the second check box.
         SI->SmplReduction=(IsDlgButtonChecked(hWndDlg, IDC_CHECK2)==BST_CHECKED);

         // Read and check the sample reduction
         // value if sample reduction is on.
         if (SI->SmplReduction) {
            hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO1);
            dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
            SI->Reduction=(BYTE)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);
				SI->OneSmplPrBlock=(IsDlgButtonChecked(hWndDlg, IDC_CHECK3)==BST_CHECKED);
         }

         // Apply the changes.
         SendMessage(SI->hWnd, TM_APPLY_CONFIGURATION, 0, 0);

#ifndef WINCE
		 // In PocketPC versjon we also close after applying changes
         break;
         case ID_CLOSE:
#endif
         // Close the dialog box
         EndDialog(hWndDlg , 0);
         break;

      }
      return FALSE;
   }
   return FALSE;
}


WORD GetSizeOfReducedBlck(WORD SampleRate, BYTE Reduction)
{
   WORD B,R;
   BYTE FB,RB;

   switch (Reduction) {
      case 10: FB=10; RB=9; break;
      case 20: FB=5;  RB=4; break;
      case 25: FB=4;  RB=3; break;
      case 33: FB=3;  RB=2; break;
      case 50: FB=2;  RB=1; break;
      case 66: FB=3;  RB=1; break;
      case 75: FB=4;  RB=1; break;
      case 80: FB=5;  RB=1; break;
      case 90: FB=10; RB=1; break;
   }

   B=(SampleRate/FB)*RB;
   R=SampleRate-((SampleRate/FB)*FB);
   if (Reduction>50 && R>0) R=1;
   return B+R;
}



WORD ReduceSamples(LONG* InBuff, LONG* OutBuff, WORD InBuffSize, BYTE Reduction)
{
   BYTE RC,CC, NumCopied, NumRemoved;
   WORD InIndex, OutIndex, TotalNumCopied;
   BOOL Copy;

   switch (Reduction) {
      case 10: RC=1; CC=9; break;
      case 20: RC=1; CC=4; break;
      case 25: RC=1; CC=3; break;
      case 33: RC=1; CC=2; break;
      case 50: RC=1; CC=1; break;
      case 66: RC=2; CC=1; break;
      case 75: RC=3; CC=1; break;
      case 80: RC=4; CC=1; break;
      case 90: RC=9; CC=1; break;
   }

   Copy=TRUE;
   OutIndex=0;
   TotalNumCopied=0;
   NumCopied=NumRemoved=0;
   for (InIndex=0;InIndex<InBuffSize;InIndex++) {
      if (Copy) {
         OutBuff[OutIndex]=InBuff[InIndex];
         OutIndex++;
         NumCopied++;
         TotalNumCopied++;
         if (NumCopied==CC) {
            Copy=FALSE;
            NumCopied=0;
         }
      } else {
         NumRemoved++;
         if (NumRemoved==RC) {
            Copy=TRUE;
            NumRemoved=0;
         }
      }
   }

   return TotalNumCopied;
}







VOID SetWindowTitle(MON_WND_DATA* S, HWND hWnd)
{
   TCHAR str10[11], WindowText[128];

	wcscpy(WindowText, S->UniqueProcName);

   if (S->WndConfig.SmplReduction) {
		if (S->WndConfig.OneSmplPrBlock) {
			wcscat(WindowText, TEXT(" (Sample reduction: Reduced to one sample per block)"));
			SetWindowText(hWnd, WindowText);
		} else {
			wcscpy(WindowText, S->UniqueProcName);
			_itow(S->WndConfig.Reduction, str10, 10);
			wcscat(WindowText, TEXT(" (Sample reduction: "));
			wcscat(WindowText, str10);
			wcscat(WindowText, TEXT("%)"));
			SetWindowText(hWnd, WindowText);
		}
   } else {
      // Sample reduction is disabled.
      SetWindowText(hWnd, S->UniqueProcName);
   }
}

