// PS2400.cpp

#include "stdafx.h"
#include <aygshell.h>

//#define STRICT


#include <windows.h>
#include <stdio.h>
#include "seislog.h"
#include "library.h"
#include "msglist.h"
#include "resource.h"
#include "process.h"
#include "eventlog.h"


#define WINCE

#ifdef WINCE
   typedef SYSTEMTIME* PSYSTEMTIME;
#endif

//
// This structure is used to pass startup- 
// information to the serial-reader thread.
//
struct STARTINFO_DEVREADER { 
   DWORD ThreadID;            // Thread-ID for parent thread.
   HANDLE hComm;              // Handle of communications device 
   TCHAR CommSpeed[6];         // Communication speed.
   DOUBLE TimeCorrection;     // Time correction to apply (in seconds).
   BOOL UsePCClock;           // If TRUE then time tagging is based on the PC clock.
   BOOL SyncPCClock;          // If TRUE then device reader thread will syncronize the PC clock.
   BYTE SyncStatus;           // The current status of the time syncronization thread.
   HANDLE hSyncStatusMutex;   // Handle to a mutex object used to control access to the SyncStatus variable in this structure.
   BYTE BytesPrSample;        // Bytes per sample from device.
   BOOL *SerialRead;          // The serial-reader thread will exit if this becomes FALSE.
   DEVMNGR_INTERF_STRUCT *DS; // Pointer to the exported structure.
   DWORD* BlockNr;            // Pointer to variable who are to receive block numbers.
};


//
// This structure is read from the binary device config file.
//
struct DEVICE_CONFIG {
   WORD DeviceID;          // An identifier for this particular model.
   BYTE NumChannels;       // Number of channels. Support 3 or 6.
   TCHAR PortName[6];       // COM1, COM2 ... -> COM16
   TCHAR CommSpeed[7];      // Communication speed. (9600->115200 baud).
   BYTE SampleRate;        // Can be 25,50,75,100,125,150,200 or 250.   
   BYTE BytesPrSample;     // Can be 1,2,3 or 4;
   DOUBLE TimeCorrection;  // Time correction (seconds).
   BOOL UsePCClock;        // If TRUE then time tagging is based on the PC clock.
   BOOL SyncPCClock;       // If TRUE then device driver will syncronize the system clock.
};


//
// Global variables.
//
extern HWND hWndFrame;
extern BYTE DebugLevel;
extern BOOL StartGPSSyncThread;
extern DWORD SysClkSyncThreadID;


//
// Function prototyping.
//
DWORD WINAPI ReaderThreadPS2400(PVOID);
BOOL CALLBACK ConfigDlgProcPS2400(HWND, UINT, WPARAM, LPARAM);
BOOL GetTimeFromGPRMCSentence(PTCHAR, PSYSTEMTIME);
BOOL PS2400BytesOK(PBYTE, DWORD);
DWORD GetPS2400PacketSequenceNumber(PBYTE);
DWORD GetPS2400GPSBlockNumber(PBYTE);



/* *********************************************************************** */
/*  Manager thread for device Earth Data PS2400. DeviceID=105.             */
/* *********************************************************************** */
DWORD WINAPI PS2400MngrThrd(PVOID StartupInfo) 
{
   MSG msg;
   INT Status;
   BYTE SyncStatus, FixStatus;
   TCHAR ConfigFile[MAX_PATH+1], ChNr, UniqueDeviceName[65], Message[100];
   TCHAR DeviceResources[256], str10[11];
   WORD BlockSize, NumBlocksPrChannel, SampleRate;
   BOOL SerialRead, MemAlloc, ConfigRead;
   HANDLE hReaderThread, hComm;
   STARTINFO_DEVMANAGER SI;
   STARTINFO_DEVREADER StartInfo;
   DEVICE_CONFIG DeviceConf, DevConfDlg;
   DEVMNGR_INTERF_STRUCT DS;
   DWORD ParentThreadID, CurrentThreadID, ReaderThreadID;
   DWORD ChannelSize, status, BlockNr;
   MSG_PARAM ID, OD;
   MsgList NotifyList;

   
   //
   // Perform thread initialization.
   //
   FixStatus=0;
   SyncStatus=1;                                     // So far we have no fix.   
   hReaderThread=NULL;                               // The serial-reader thread is not running.
   CopyMemory((PVOID)&SI, StartupInfo, sizeof(SI));  // Copy startup information to local buffer.
   ParentThreadID=SI.ThreadID;                       // ThreadID for parent thread.
   CurrentThreadID=GetCurrentThreadId();             // ThreadID for this thread.
   

   //
   // Handle device configuration.
   //
   // Read configuration file.
   ConfigRead=FALSE;
   wcscpy(ConfigFile, SI.DeviceDir); 
   wcscat(ConfigFile, TEXT("\\device.cnf"));
   if (ExistFile(SI.DeviceDir, TEXT("device.cnf"))) {
      // Read device config from config file.
      ConfigRead=ReadFileToMemory(ConfigFile, (PVOID)&DeviceConf, (DWORD)sizeof(DeviceConf));
   }
   if (!ConfigRead) {
      // A valid config file could not be read.
      // Set up a default digitizer configuration.
      DeviceConf.DeviceID=105;               // PS2400 uses device id 105.
      DeviceConf.NumChannels=3;              // 3 or 6 channel configuration supported.
      wcscpy(DeviceConf.PortName, TEXT("COM1"));   // Assume digitizer on COM1.
      wcscpy(DeviceConf.CommSpeed, TEXT("19200")); // Assume comm speed of 19200 baud.
      DeviceConf.SampleRate=75;              // Assume 75 hz sample rate.
      DeviceConf.BytesPrSample=3;            // Assume 3 bytes pr sample.
      DeviceConf.TimeCorrection=0.0;         // Default time correction (seconds).
      DeviceConf.UsePCClock=FALSE;           // Assume 'use device time' (Device has GPS).
      DeviceConf.SyncPCClock=FALSE;          // Assume 'do not syncronize the system clock'.

      // Write settings to config file.
      WriteFileFromMemory(ConfigFile, (PVOID)&DeviceConf, (DWORD)sizeof(DeviceConf));
   }


   // 
   // Create a unique device instance name.
   //
   wcscpy(UniqueDeviceName, TEXT("PS2400 at "));
   wcscat(UniqueDeviceName, DeviceConf.PortName);

   //
   // Register this process in the process database.
   //
   if (!SI.ConfigOnly) RegisterProcess(UniqueDeviceName, 1, CurrentThreadID);

   //
   // Create device resource string.
   // 
   wcscpy(DeviceResources, TEXT("Serial ports: "));
   wcscat(DeviceResources, DeviceConf.PortName);

   //
   // Handle time syncronization stuff.
   //
   StartInfo.SyncPCClock=FALSE;
   if (!SI.ConfigOnly && DeviceConf.SyncPCClock) {
      // Device is configured to syncronize the system clock.
      // Check if any other thread is already doing this.
      if (SysClkSyncThreadID) {
         // Turn off time syncronication. Clock is 
         // already beeing syncronized by another thread.
         DeviceConf.SyncPCClock=FALSE;
         // Write warning in the system log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Cannot syncronize system clock. Clock already syncronized."));
         WriteSystemLog(Message, 2);
      } else {
         // System clock is not already syncronized.
         // This thread will attempt to syncronize it.
         StartInfo.SyncPCClock=TRUE;
         SysClkSyncThreadID=CurrentThreadID;
         // Write text on status bar.
         DrawStatusBarText(SyncStatus, UniqueDeviceName);
         // Write message in the system log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Syncronizing system clock."));
         WriteSystemLog(Message, 1);
      }
   }
  
   
   //
   // Calculate sample-rate, blocksize, channelsize and number of blocks pr. channel.
   //
   SampleRate=(WORD)DeviceConf.SampleRate;
   BlockSize=(WORD)(sizeof(BLOCK_HEADER)+(SampleRate*4));
   NumBlocksPrChannel=60;
   ChannelSize=NumBlocksPrChannel*BlockSize;

   // Open the serial port.
   if (!SI.ConfigOnly) {

#ifdef WINCE
	  // All ports in Windows CE must end with ':'
	  TCHAR CEPortName[7];
	  wcscpy(CEPortName, DeviceConf.PortName);
	  wcscat(CEPortName, TEXT(":"));
      hComm=CreateFile(CEPortName, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, 0);
#else
      hComm=CreateFile(DeviceConf.PortName, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, 0);
#endif

      if (hComm==INVALID_HANDLE_VALUE) {
         // Update program log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Unable to open communication port "));
         wcscat(Message, DeviceConf.PortName);
         wcscat(Message, TEXT("."));
         WriteSystemLog(Message, 3);
      }
   }

   
   //
   // Initialize the interface structure.
   //
   // Insert manufacturer name and model.
   wcscpy(DS.Manufacturer, TEXT("Earth Data Ltd"));
   wcscpy(DS.Model, TEXT("PS2400"));

   // Insert samplerate.
   DS.SampleRate=SampleRate;

   // Insert number of samples in each block.
   DS.SamplesPrBlock=SampleRate;

   // Insert number of channels
   DS.NumChannels=DeviceConf.NumChannels; 

   // Insert block size, block header included.
   DS.BlockSize=BlockSize;

   // Insert block size measured in milliseconds.
   DS.BlockSizeInMS=1000;

   // Insert number of blocks pr channel.
   DS.NumBlocksPrChannel=NumBlocksPrChannel;

   // Make sure all pointers to channel data is NULL.
   for (ChNr=0;ChNr<MAX_CHANNELS_PR_DEVICE;ChNr++) DS.ChannelData[ChNr]=(PVOID)NULL;
   
   
   //
   // Allocate memory for the channel buffers, and set up the pointers to the buffers.
   //
   MemAlloc=TRUE;
   if ((DS.ChannelData[0]=(PVOID)LocalAlloc(LPTR, (UINT)ChannelSize))==NULL) MemAlloc=FALSE;
   if ((DS.ChannelData[1]=(PVOID)LocalAlloc(LPTR, (UINT)ChannelSize))==NULL) MemAlloc=FALSE;
   if ((DS.ChannelData[2]=(PVOID)LocalAlloc(LPTR, (UINT)ChannelSize))==NULL) MemAlloc=FALSE;
   if (DS.NumChannels==6)  {
	   if ((DS.ChannelData[3]=(PVOID)LocalAlloc(LPTR, (UINT)ChannelSize))==NULL) MemAlloc=FALSE;
	   if ((DS.ChannelData[4]=(PVOID)LocalAlloc(LPTR, (UINT)ChannelSize))==NULL) MemAlloc=FALSE;
	   if ((DS.ChannelData[5]=(PVOID)LocalAlloc(LPTR, (UINT)ChannelSize))==NULL) MemAlloc=FALSE;
   }
   // Check that memory allocation succeded.
   if (!MemAlloc) {
         // Update program log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(":"));
         wcscat(Message, TEXT(" Could not allocate memory."));
         WriteSystemLog(Message, 3);
   }
   

   //
   // If running in debug mode, write a message to the log.
   //
   if (DebugLevel) {
      wcscpy(Message, UniqueDeviceName);
      wcscat(Message, TEXT(": Thread started."));
      WriteSystemLog(Message, 1);
   }
   

   //
   // Signal that the thread is started.
   //
   SetEvent(SI.StartEvent);


   //
   // Enter the thread message loop. The thread will now start to process messages.
   //
   while (GetMessage(&msg, (HWND)NULL, 0, 0)) 
   { 
   
      // If debuglevel is 2 then log all messages recieved.
      if (DebugLevel>1) {
         TCHAR str10[11];
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Recieved message "));
         _itow(msg.message, str10, 10);
         wcscat(Message, str10);
         WriteSystemLog(Message, 1);
      }

      // Save message parameters.
      ZeroMemory((PVOID)&ID, (DWORD)sizeof(ID));
      if (msg.lParam) ID=*((MSG_PARAM*)msg.lParam);
      
      // Initialize return-message parameter to default values.
      ZeroMemory((PVOID)&OD, (DWORD)sizeof(OD));
      OD.bStatus=TRUE;
      OD.idThread=CurrentThreadID;


      // Process message.
      switch (msg.message) {
         case TM_START:                            
         //
         // This message is sendt from a higher level 
         // thread to start reading the serial port.
         //
         if (hReaderThread==NULL && hComm!=INVALID_HANDLE_VALUE && MemAlloc && !SI.ConfigOnly) {
            if (SysClkSyncThreadID && SysClkSyncThreadID!=CurrentThreadID) {
               // The system clock is syncronized by some other thread.
               // Subscribe to the TM_TIMESYNC_STATUS message from thread.
               Subscribe(TM_TIMESYNC_STATUS, SysClkSyncThreadID, CurrentThreadID, (HWND)NULL);

               // Get the current syncronization status.
               StartInfo.SyncStatus=GetCurrentTimeSyncStatus(SysClkSyncThreadID);
            } else {
               // The system clock is not syncronized and
               // there is no other syncronization possibilities.
               StartInfo.SyncStatus=0;
               // Create a mutex used to access the 'StartInfo.SyncStatus' variable.
               StartInfo.hSyncStatusMutex=CreateMutex(NULL, FALSE, NULL);
            }

            // Set up startup information for the serial reader thread routine.
            SerialRead=TRUE;
            StartInfo.ThreadID=CurrentThreadID;
            StartInfo.hComm=hComm;
            wcscpy(StartInfo.CommSpeed, DeviceConf.CommSpeed);
            StartInfo.TimeCorrection=DeviceConf.TimeCorrection;
            StartInfo.UsePCClock=DeviceConf.UsePCClock;
            StartInfo.hSyncStatusMutex=CreateMutex(NULL, FALSE, NULL); 
            StartInfo.BytesPrSample=DeviceConf.BytesPrSample;
            StartInfo.SerialRead=&SerialRead;
            StartInfo.DS=&DS;
            StartInfo.BlockNr=&BlockNr;
            
            // Start the thread.
            hReaderThread=CreateThread(NULL, 0, ReaderThreadPS2400, (VOID*)&StartInfo, CREATE_SUSPENDED, &ReaderThreadID);
            SetThreadPriority(hReaderThread, THREAD_PRIORITY_ABOVE_NORMAL);            
            ResumeThread(hReaderThread); 
         }

         // Set return values. 
         if (!hReaderThread) {
            OD.bStatus=FALSE;
            OD.wStatus=1;
         }
         break;


         
         case TM_STOP:
         //
         // This message is sendt from a higher level thread to stop
         // reading the serial port. Returns the default return values.
         //
         if (hReaderThread) {
            // Make the serial thread loop exit, thereby stopping the thread.
            SerialRead=FALSE;

            // Wait up to 2 seconds for the thread to stop.
            status=WaitForSingleObject(hReaderThread, 2000);
            if (status==WAIT_TIMEOUT) TerminateThread(hReaderThread, 0);

            // Unsubscribe to the TM_TIMESYNC_STATUS message as needed.
            if (SysClkSyncThreadID && SysClkSyncThreadID!=CurrentThreadID) {
               UnSubscribe(TM_TIMESYNC_STATUS, SysClkSyncThreadID, CurrentThreadID, NULL);
               CloseHandle(StartInfo.hSyncStatusMutex); // Delete mutex object.
            }

            // The thread is now stopped, invalidate the thread handle.
				CloseHandle(hReaderThread);
            hReaderThread=NULL;
         }
         break;


         case TM_CONFIGURE:            
         //
         // This message is sendt from a higher level thread
         // when it wants to configure the digitizer settings.
         //
         if (hReaderThread==NULL) {
            // Let user work on a copy of the current device settings.
            DevConfDlg=DeviceConf;

            // Display the dialog box.
            Status=DialogBoxParam(SI.hInstance, MAKEINTRESOURCE(IDD_CNF_PS2400), NULL, (DLGPROC)ConfigDlgProcPS2400, (LPARAM)&DevConfDlg);   

            // Save settings if requested.
            if (Status==1) {
               // Save the new configuration.
               DeviceConf=DevConfDlg;

               // Write configuration to disk.
               WriteFileFromMemory(ConfigFile, (PVOID)&DeviceConf, (DWORD)sizeof(DEVICE_CONFIG));

               // Create a unique device instance name.
               wcscpy(UniqueDeviceName, TEXT("PS2400 at "));
               wcscat(UniqueDeviceName, DeviceConf.PortName);
            }

            // Set return values. 
            if (Status==0) {
               OD.bStatus=FALSE;
               OD.wStatus=100;
            }
         }
         break;

         case TM_SHOW_CONFIG:
            //
            // When this message is received, dump all configuration
            // parameters that is not given in the device resource 
            // string to the system log.
            //
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(" Configuration: Communication speed="));
            wcscat(Message, DeviceConf.CommSpeed);
            WriteSystemLog(Message, 1);
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(" Configuration: Nr of bytes pr sample="));
            swprintf(str10,TEXT("%d"), DeviceConf.BytesPrSample);
            wcscat(Message, str10);
            WriteSystemLog(Message, 1);
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(" Configuration: Time correction="));
            swprintf(str10,TEXT("%.3f"), DeviceConf.TimeCorrection);
            wcscat(Message, str10);
            WriteSystemLog(Message, 1);
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(" Configuration: Syncronize system clock="));
            if (DeviceConf.SyncPCClock) wcscat(Message, TEXT("TRUE"));
            if (!DeviceConf.SyncPCClock) wcscat(Message, TEXT("FALSE"));
            WriteSystemLog(Message, 1);
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(" Configuration: Use system clock for time tagging="));
            if (DeviceConf.UsePCClock) wcscat(Message, TEXT("TRUE"));
            if (!DeviceConf.UsePCClock) wcscat(Message, TEXT("FALSE"));
            WriteSystemLog(Message, 1);
         break;

         case TM_GET_INTERFACE:
         //
         // This message is used to get a pointer to the interface structure.
         //
         OD.vPointer=(PVOID)&DS;
         break;   


         case TM_GET_UNIQUE_DEVICE_NAME:
         //
         // This message is used to get a string, 
         // identifying this unique device instance.
         //
         wcscpy(OD.cStatus, UniqueDeviceName);
         break;   


         case TM_GET_RESOURCE_STRING:
         //
         // This message is used to get a string, 
         // identifying the system resources used by this device.
         //
         wcscpy(OD.cStatus, DeviceResources);
         break;   


         case TM_SUBSCRIBE:
         //
         // This message is sendt from a window or thread
         // Who wants to be informed about an event.
         // This message must be accompanied by a MSG_PARAM structure.
         // The MSG_PARAM structure must have a pointer to a valid
         // SUBSCRIBE structure.
         //
         //
         // Add window/thread to notify list.
         if (ID.vPointer) if (!NotifyList.Subscribe((SUBSCRIBE*)ID.vPointer)) {
            OD.bStatus=FALSE;
            OD.wStatus=3;
         }
         break;   


         case TM_UNSUBSCRIBE:
         //
         // This message is sendt from a window or thread who
         // wants to remove a notification from the notify list.
         // This message must be accompanied by a MSG_PARAM structure.
         // The MSG_PARAM structure must have a pointer to a valid
         // SUBSCRIBE structure.
         //
         //
         // Remove notification from list.
         if (ID.vPointer) NotifyList.UnSubscribe((SUBSCRIBE*)ID.vPointer);
         
         break;   

         

         case TM_EXIT:                 
         //
         // This is a request for the thread to end.
         // Close down all processes that are depandent on this thread.
         // The 'StopDependantThreads' function will return immediatly,
         // and will send a 'TM_DEPTHREADS_STOPPED' message back when
         // all dependant threads have been stopped.
         StopDependantThreads();
         break;



         case TM_DEPTHREADS_STOPPED:
         //
         // This message indicates that all dependent threads have been stopped.
         // The thread will cleanup and exit when this message is recieved.         
         //

         // First stop the device-reader thread.
         if (hReaderThread) {
            SerialRead=FALSE;                                            // Signal the reader thread.
            status=WaitForSingleObject(hReaderThread, 2000);             // Wait for the thread to stop.
            if (status==WAIT_TIMEOUT) TerminateThread(hReaderThread, 0); // If thread wont stop, kill it.
            hReaderThread=NULL;
         }

         if (!SI.ConfigOnly) {
            // Unregister this process from the process database.
            UnregisterProcess(CurrentThreadID);

            // Close the communication port.
            CloseHandle(hComm);

            // Handle time syncronization stuff.
            if (DeviceConf.SyncPCClock && StartInfo.SyncPCClock) {
               // Clear the SysClkSyncThreadID variable.
               SysClkSyncThreadID=0;
               // Clear the status bar.
               PostMessage(hWndFrame, TM_WRITE_STATUSBAR, 0, (LPARAM)TEXT(""));
            }

         }

         // Free memory buffers.
         if (DS.ChannelData[0]) LocalFree((HLOCAL)DS.ChannelData[0]); 
         if (DS.ChannelData[1]) LocalFree((HLOCAL)DS.ChannelData[1]); 
         if (DS.ChannelData[2]) LocalFree((HLOCAL)DS.ChannelData[2]); 
		 if (DS.NumChannels==6)  {
			 if (DS.ChannelData[3]) LocalFree((HLOCAL)DS.ChannelData[3]); 
			 if (DS.ChannelData[4]) LocalFree((HLOCAL)DS.ChannelData[4]); 
			 if (DS.ChannelData[5]) LocalFree((HLOCAL)DS.ChannelData[5]); 
		 }
         // End the message loop.
         ClearMessageQueue(); // Delete all messages in message queue.
         PostQuitMessage(0);  // Post WM_QUIT message.   
         break;
   
         
         //
         // Handle messages from device reader thread.
         //
         case TM_BUFFERUPDATED:  // A new block of data has been inserted into buffer.
         // Send Notification to all subscribed threads/windows.
         // The notofication is accompanied with this thread's thread-ID, and the blocknr of the updated block.
         NotifyList.Notify((UINT)TM_BUFFERUPDATED, (WPARAM)CurrentThreadID, (LPARAM)BlockNr);
         break;


         case TM_TIMESYNC_STATUS:
         //
         // This message is sendt from a time syncronization
         // thread (other then this thread or its child) to inform
         // about a change in the syncronization status. This thread
         // subscribe to this message from the time syncronization thread.
         WaitForSingleObject(StartInfo.hSyncStatusMutex, INFINITE); 
         StartInfo.SyncStatus=(BYTE)msg.wParam;
         ReleaseMutex(StartInfo.hSyncStatusMutex);
         break;
         
         
         case TM_GET_TIMESYNC_STATUS:
         //
         // This message is sendt from a window or thread who
         // wants to know the status of the GPS time syncronisation.
         // This message must be accompanied by a MSG_PARAM structure.
         //
         //
         // Returns the operational status of the thread.
         // Values are:
         //    0 :System clock is beeing syncronized, no error.
         //    1 :System clock is NOT beeing syncronized, but no error (fix lost).
         //    2 :System clock is NOT beeing syncronized, communication error.
         //
         OD.wStatus=(WORD)SyncStatus;
         break;


         case TM_DEVICEREADER_NODATAINPUT:   
         //
         // The device reader has lost data input.
         //
         // Send Notification to all subscribed threads/windows.
         NotifyList.Notify((UINT)TM_DEVICEREADER_NODATAINPUT, WPARAM(CurrentThreadID), 0);

         // Write a message to the log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Not receiving data."));
         WriteSystemLog(Message, 2);
         break;


         case TM_DEVICEREADER_DATAINPUTOK:   
         //
         // The device reader has regained data input.
         //

         // Send Notification to all subscribed threads/windows.
         NotifyList.Notify((UINT)TM_DEVICEREADER_DATAINPUTOK, WPARAM(CurrentThreadID), 0);

         // Write a message to the log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Receiving data."));
         WriteSystemLog(Message, 1);
         break;


         case TM_DEVICEREADER_BADBLOCK:   
         // A bad block of data was received from the device.
         // Send Notification to all subscribed threads/windows.
         // The notofication is accompanied with this thread's thread-ID, and the blocknr of the bad block.
         NotifyList.Notify((UINT)TM_DEVICEREADER_BADBLOCK, (WPARAM)CurrentThreadID, (LPARAM)BlockNr);

         // Write a message to the log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Received a bad block from device."));
         WriteSystemLog(Message, 2);
         break;

         
         case TM_DEVICEREADER_NOFIX:   
         //
         // The GPS reader-thread indicates that fix has been lost.
         //
         FixStatus=0;
         SyncStatus=1;
            
         // Send Notification to all subscribed threads/windows.
         NotifyList.Notify((UINT)TM_TIMESYNC_STATUS, (WPARAM)SyncStatus, 0);
         
         // Update status bar text.
         if (DeviceConf.SyncPCClock) DrawStatusBarText(SyncStatus, UniqueDeviceName);

         // Write a message to the log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Satellite fix lost, using system clock for time tagging."));
         WriteSystemLog(Message, 2);
         break;


         case TM_DEVICEREADER_FIXOK:   
         //
         // The GPS reader-thread indicates that fix has been regained.
         //
         FixStatus=1;
         SyncStatus=0;

         // Send Notification to all subscribed threads/windows.
         NotifyList.Notify((UINT)TM_TIMESYNC_STATUS, (WPARAM)SyncStatus, 0);

         // Update status bar text.
         if (DeviceConf.SyncPCClock) DrawStatusBarText(SyncStatus, UniqueDeviceName);        
         
         // Write a message to the log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Satellite fix OK."));
         WriteSystemLog(Message, 1);
         break;
      }
      
      // If this is requested by the thread who sent the message
      // then return status values, and release the caller thread.
      if (((MSG_PARAM*)msg.lParam)!=NULL && msg.wParam!=NULL) {
         *(MSG_PARAM*)msg.lParam=OD;
         SetEvent((HANDLE)msg.wParam);
      }
   } 


   // Free up memeory allocated in the MsgList class object.
   NotifyList.~MsgList();

   // If running in debug mode, write a message to the log.
   if (DebugLevel) {
      wcscpy(Message, UniqueDeviceName);
      wcscat(Message, TEXT(": Thread stopped."));
      WriteSystemLog(Message, 1);
   }

   // Exit this thread
   ExitThread(0);
   return 0L;
   
}




/* *********************************************************************** */
/*  Serial port reader procedure for the Earth Data PS2400 device.         */
/* *********************************************************************** */
DWORD WINAPI ReaderThreadPS2400(PVOID StartupStruct)
{
   DCB dcb;
#ifndef WINCE
   TCHAR DevCntrlStr[50];
#endif
   BYTE BytesPrSample, Buffer[3215], *bPtr, TimeQuality;
   BYTE StatusGPS, FixStatus, OldFixStatus, TimeSource;
   WORD BlockSize, NumChannels, NumBlocksPrChannel;
   WORD SampleNr, SampleRate;
   DWORD BytesRead, BlockNr, OffsetCurrentSample;
   DWORD BytesToRead, OffsetToBlockData, SeqNr, GPSBlockNr;
   DOUBLE TransferTime;
   SYSTEMTIME ReadTime, PacketTime, TagTime, SysTime;
   VOID *DataBlocksCh1, *DataBlocksCh2, *DataBlocksCh3; 
   VOID *DataBlocksCh4=NULL;
   VOID *DataBlocksCh5=NULL;
   VOID *DataBlocksCh6=NULL; 
   COMMTIMEOUTS ct={30, 0, 1200, 0, 0};
   STARTINFO_DEVREADER *SI;
   BLOCK_HEADER DBH;
   long int LIValue;
   BOOL BytesOK, DataLost, ReadStatus;
   BYTE Step;

   // Initialize som variables.
   BlockNr=0;         // Start filling data into first block. This variable is zero based.
   FixStatus=0;       // So far we have no satelite fix.
   OldFixStatus=0;    // We have no previous fix status.
   DataLost=TRUE;     // At this point we have no data input.

   // Copy values from startup structure.
   SI=(STARTINFO_DEVREADER*)StartupStruct;         // Use pointer of correct type.
   NumChannels=SI->DS->NumChannels;                // Number of channels digitizer is using.
   BytesPrSample=SI->BytesPrSample;                // Number of bytes pr sample.
   SampleRate=SI->DS->SampleRate;                  // Sample rate.
   NumBlocksPrChannel=SI->DS->NumBlocksPrChannel;  // Number of 1 sec. blocks in buffer.
   BlockSize=SI->DS->BlockSize;                    // Size of 1 sec. block, header included
   DataBlocksCh1=SI->DS->ChannelData[0];           // Pointer to first block channel 1.
   DataBlocksCh2=SI->DS->ChannelData[1];           // Pointer to first block channel 2.
   DataBlocksCh3=SI->DS->ChannelData[2];           // Pointer to first block channel 3.
   if (NumChannels==6) {
	   DataBlocksCh4=SI->DS->ChannelData[3];           // Pointer to first block channel 4.
	   DataBlocksCh5=SI->DS->ChannelData[4];           // Pointer to first block channel 5.
	   DataBlocksCh6=SI->DS->ChannelData[5];           // Pointer to first block channel 6.
   }
   // Find number of bytes to read from device.
   BytesToRead=192+(8+NumChannels*SampleRate*BytesPrSample)+12;

   // Calculate the the transfer time on the serial port.
   if (!wcscmp(SI->CommSpeed, TEXT("9600"))) TransferTime=(DOUBLE)(BytesToRead)/960.0;
   if (!wcscmp(SI->CommSpeed, TEXT("19200"))) TransferTime=(DOUBLE)(BytesToRead)/1920.0;
   if (!wcscmp(SI->CommSpeed, TEXT("38400"))) TransferTime=(DOUBLE)(BytesToRead)/3840.0;
   if (!wcscmp(SI->CommSpeed, TEXT("57600"))) TransferTime=(DOUBLE)(BytesToRead)/5760.0;
   if (!wcscmp(SI->CommSpeed, TEXT("115200"))) TransferTime=(DOUBLE)(BytesToRead)/11520.0;

   //Initialize the communications parameters. 
   SetupComm(SI->hComm, BytesToRead*2, 0);   // Input buffer will hold 2 seconds of data.
   GetCommState(SI->hComm, &dcb);


#ifdef WINCE
   dcb.BaudRate     = _wtoi(SI->CommSpeed);
   dcb.fParity      = TRUE;
   dcb.Parity       = NOPARITY;
   dcb.StopBits     = ONESTOPBIT;
   dcb.ByteSize     = 8;

   dcb.fInX         = FALSE;
   dcb.fOutX        = FALSE;
   dcb.fOutxDsrFlow = FALSE;
   dcb.fOutxCtsFlow = FALSE;  
   dcb.fDtrControl  = DTR_CONTROL_ENABLE;
   dcb.fRtsControl  = RTS_CONTROL_ENABLE;
#else
   wcscpy(DevCntrlStr, "baud=");
   wcscat(DevCntrlStr, SI->CommSpeed);
   wcscat(DevCntrlStr, " parity=n data=8 stop=1");
   BuildCommDCB(DevCntrlStr, &dcb);
#endif

   dcb.fAbortOnError=FALSE; 
   SetCommState(SI->hComm, &dcb);            // Set serial port mode. No flow control is used.
	SetCommTimeouts(SI->hComm, &ct);          // Set communication timeouts.
 

   
   // Start reading the serial port.
   PurgeComm(SI->hComm, PURGE_RXCLEAR); // Clear the input buffer.
   while(*SI->SerialRead) {
      
      // Try reading BytesToRead bytes into buffer. Also save the time of the read.
      ReadStatus=ReadFile(SI->hComm, (LPVOID)Buffer, BytesToRead, &BytesRead, NULL);
      GetSystemTime(&ReadTime); // This is UTC time.

      // Process incoming bytes.
      if (BytesRead==BytesToRead) {  // Correct number of bytes was read.
      
         // Check if bytes seems to be ok.
         BytesOK=PS2400BytesOK(Buffer, BytesToRead);

         if (BytesOK) {
            // Get the packet sequence number. This number starts at 3 when the device is powerd up.
            SeqNr=GetPS2400PacketSequenceNumber(Buffer);

            // Get the GPS block number. This is the packet nr for when the GPS was last locked.
            GPSBlockNr=GetPS2400GPSBlockNumber(Buffer);

            // Find the fix status of the GPS.
            if (GPSBlockNr==SeqNr) {
               // GPS is locked. $GPRMC string has correct time. 
               StatusGPS=2;
            } else {
               // Time from $GPRMC string can NOT be used.
               if (GPSBlockNr!=0xffffffff) StatusGPS=1;  // GPS is currently NOT locked.
               if (GPSBlockNr==0xffffffff) StatusGPS=0;  // GPS has never had lock, or is NOT fitted.
            }

            // Get the the GPS time from packet if possible.
            if (StatusGPS==2) GetTimeFromGPRMCSentence((PTCHAR)Buffer+120, &PacketTime);

            // Syncronize the system clock if requested and possible.
            if (SI->SyncPCClock && StatusGPS==2) {
               SysTime=PacketTime;
               AddToTime(&SysTime, 1.0+TransferTime);
               if (SI->TimeCorrection>0.0) AddToTime(&SysTime, SI->TimeCorrection);
               if (SI->TimeCorrection<0.0) SubstractFromTime(&SysTime, -SI->TimeCorrection);
               SetSystemTime(&SysTime);
            }

            // Find which time source should be used for timetagging the data.
            if (SI->UsePCClock) TimeSource=0;                  // Timetag from system clock.
            if (!SI->UsePCClock) TimeSource=1;                 // Timetag primarily from GPS, but
            if (TimeSource==1 && StatusGPS!=2) TimeSource=0;   // use system clock if GPS currently has no fix.   

            // Get time used for tagging the data block.
            if (TimeSource==1) {
               // Use the GPS time that comes with the packet.
               TagTime=PacketTime;
            } else {
               // Packet has no valid GPS time, use system clock.
               TagTime=ReadTime; 
               SubstractFromTime(&TagTime, 1.0+TransferTime);
               if (SI->TimeCorrection>0.0) AddToTime(&TagTime, SI->TimeCorrection);
               if (SI->TimeCorrection<0.0) SubstractFromTime(&TagTime, -SI->TimeCorrection);
            }

            // Find the timequality factor for this block.
            if (TimeSource==0) {
               // The time is taken from the system clock.
               if (SysClkSyncThreadID) {
                  // The system clock is syncronized. 
                  if (SI->SyncPCClock) {
                     // This thread is syncronizing the system clock.
                     // The system clock is NOT syncronized by any other thread.
                     if (StatusGPS==0) TimeQuality=1; // We have never had fix, or GPS is NOT fitted. 
                     if (StatusGPS==1) TimeQuality=1; // We have no satellite fix.
                     if (StatusGPS==2) TimeQuality=2; // We have satellite fix.
                  } else {
                     // Another thread or is syncronizing the system clock.
                     // We will have to get the syncstatus from the syncronizing thread.
                     WaitForSingleObject(SI->hSyncStatusMutex, INFINITE); 
                     if (SI->SyncStatus==0) TimeQuality=2;
                     if (SI->SyncStatus==1) TimeQuality=1;
                     if (SI->SyncStatus==2) TimeQuality=3;
                     ReleaseMutex(SI->hSyncStatusMutex);
                  }
               } else {
                  // The system clock is NOT syncronized. 
                  TimeQuality=0; 
               }
            } else {
               // The time is taken from the PS2400 GPS.
               TimeQuality=2; 
            }
         }


         // Update block headers.
         DBH.BlockID=BlockNr;
         DBH.TimeTag=TagTime;
         DBH.TimeQuality=TimeQuality;
         DBH.NullData=!BytesOK;
         DBH.DataLost=DataLost;
         DBH.EventDetected=FALSE;
         DBH.EventTime=0;
         bPtr=((BYTE*)DataBlocksCh1)+(BlockSize*BlockNr);
         CopyMemory((VOID*)bPtr, (VOID*)&DBH, (DWORD)sizeof(BLOCK_HEADER));
         bPtr=((BYTE*)DataBlocksCh2)+(BlockSize*BlockNr);
         CopyMemory((VOID*)bPtr, (VOID*)&DBH, (DWORD)sizeof(BLOCK_HEADER));
         bPtr=((BYTE*)DataBlocksCh3)+(BlockSize*BlockNr);
         CopyMemory((VOID*)bPtr, (VOID*)&DBH, (DWORD)sizeof(BLOCK_HEADER));
		 if (NumChannels == 6)  {
			 bPtr=((BYTE*)DataBlocksCh4)+(BlockSize*BlockNr);
			 CopyMemory((VOID*)bPtr, (VOID*)&DBH, (DWORD)sizeof(BLOCK_HEADER));
			 bPtr=((BYTE*)DataBlocksCh5)+(BlockSize*BlockNr);
			 CopyMemory((VOID*)bPtr, (VOID*)&DBH, (DWORD)sizeof(BLOCK_HEADER));
			 bPtr=((BYTE*)DataBlocksCh6)+(BlockSize*BlockNr);
			 CopyMemory((VOID*)bPtr, (VOID*)&DBH, (DWORD)sizeof(BLOCK_HEADER));
		 }

         // Calculate offset from start of data buffer, into data part of currrent block.
         OffsetToBlockData=(BlockNr*BlockSize)+sizeof(BLOCK_HEADER);

         // Insert sample values into block.
         if (BytesOK) {
            OffsetCurrentSample=OffsetToBlockData;
            for (SampleNr=0;SampleNr<SampleRate;SampleNr++) {
               if (BytesPrSample==3) {
                  Step=NumChannels*3;   
                  // Process sample pointed to by 'SampleNr' for channel 1.
                  bPtr=((BYTE*)DataBlocksCh1)+OffsetCurrentSample;
                  LIValue=CreateLongIntFrom3Bytes(Buffer[202+SampleNr*Step], Buffer[201+SampleNr*Step], Buffer[200+SampleNr*Step]);
                  *((long int*)bPtr)=LIValue;

                  // Process sample pointed to by 'SampleNr' for channel 2.
                  bPtr=((BYTE*)DataBlocksCh2)+OffsetCurrentSample;
                  LIValue=CreateLongIntFrom3Bytes(Buffer[205+SampleNr*Step], Buffer[204+SampleNr*Step], Buffer[203+SampleNr*Step]);
                  *((long int*)bPtr)=LIValue;

                  // Process sample pointed to by 'SampleNr' for channel 3.
                  bPtr=((BYTE*)DataBlocksCh3)+OffsetCurrentSample;
                  LIValue=CreateLongIntFrom3Bytes(Buffer[208+SampleNr*Step], Buffer[207+SampleNr*Step], Buffer[206+SampleNr*Step]);
                  *((long int*)bPtr)=LIValue;
				 
				  if (NumChannels==6)  { // options are 3 or 6
				      // Process sample pointed to by 'SampleNr' for channel 4.
					  bPtr=((BYTE*)DataBlocksCh4)+OffsetCurrentSample;
					  LIValue=CreateLongIntFrom3Bytes(Buffer[211+SampleNr*Step], Buffer[210+SampleNr*Step], Buffer[209+SampleNr*Step]);
					  *((long int*)bPtr)=LIValue;
					  
					  // Process sample pointed to by 'SampleNr' for channel 5.
					  bPtr=((BYTE*)DataBlocksCh5)+OffsetCurrentSample;
					  LIValue=CreateLongIntFrom3Bytes(Buffer[214+SampleNr*Step], Buffer[213+SampleNr*Step], Buffer[212+SampleNr*Step]);
					  *((long int*)bPtr)=LIValue;
					  
					  // Process sample pointed to by 'SampleNr' for channel 6.
					  bPtr=((BYTE*)DataBlocksCh6)+OffsetCurrentSample;
					  LIValue=CreateLongIntFrom3Bytes(Buffer[217+SampleNr*Step], Buffer[216+SampleNr*Step], Buffer[215+SampleNr*Step]);
					  *((long int*)bPtr)=LIValue;
				  }

               } else { 
                  // There are 4 bytes pr sample.
                  Step=NumChannels*4;
                  // Process sample pointed to by 'SampleNr' for channel 1.
                  bPtr=((BYTE*)DataBlocksCh1)+OffsetCurrentSample;
                  LIValue=CreateLongIntFrom4Bytes(Buffer[203+SampleNr*Step], Buffer[202+SampleNr*Step], Buffer[201+SampleNr*Step], Buffer[200+SampleNr*Step]);
                  *((long int*)bPtr)=LIValue;

                  // Process sample pointed to by 'SampleNr' for channel 2.
                  bPtr=((BYTE*)DataBlocksCh2)+OffsetCurrentSample;
                  LIValue=CreateLongIntFrom4Bytes(Buffer[207+SampleNr*Step], Buffer[206+SampleNr*Step], Buffer[205+SampleNr*Step], Buffer[204+SampleNr*Step]);
                  *((long int*)bPtr)=LIValue;

                  // Process sample pointed to by 'SampleNr' for channel 3.
                  bPtr=((BYTE*)DataBlocksCh3)+OffsetCurrentSample;
                  LIValue=CreateLongIntFrom4Bytes(Buffer[211+SampleNr*Step], Buffer[210+SampleNr*Step], Buffer[209+SampleNr*Step], Buffer[208+SampleNr*Step]);
                  *((long int*)bPtr)=LIValue;

				  if (NumChannels == 6)  {
					  // Process sample pointed to by 'SampleNr' for channel 4.
					  bPtr=((BYTE*)DataBlocksCh1)+OffsetCurrentSample;
					  LIValue=CreateLongIntFrom4Bytes(Buffer[215+SampleNr*Step], Buffer[214+SampleNr*Step], Buffer[213+SampleNr*Step], Buffer[212+SampleNr*Step]);
					  *((long int*)bPtr)=LIValue;
					  
					  // Process sample pointed to by 'SampleNr' for channel 5.
					  bPtr=((BYTE*)DataBlocksCh2)+OffsetCurrentSample;
					  LIValue=CreateLongIntFrom4Bytes(Buffer[219+SampleNr*Step], Buffer[218+SampleNr*Step], Buffer[217+SampleNr*Step], Buffer[216+SampleNr*Step]);
					  *((long int*)bPtr)=LIValue;
					  
					  // Process sample pointed to by 'SampleNr' for channel 6.
					  bPtr=((BYTE*)DataBlocksCh3)+OffsetCurrentSample;
					  LIValue=CreateLongIntFrom4Bytes(Buffer[223+SampleNr*Step], Buffer[222+SampleNr*Step], Buffer[221+SampleNr*Step], Buffer[220+SampleNr*Step]);
					  *((long int*)bPtr)=LIValue;
				  }
               }

               // Move to next sample.
               OffsetCurrentSample=OffsetCurrentSample+4;
            }
         
         } else {
            // Correct number of bytes was read, but they don't
            // seem to be OK. Fill the block with nulldata.
            ZeroMemory((PVOID)((PBYTE)DataBlocksCh1+OffsetToBlockData), SampleRate*4);
            ZeroMemory((PVOID)((PBYTE)DataBlocksCh2+OffsetToBlockData), SampleRate*4);
            ZeroMemory((PVOID)((PBYTE)DataBlocksCh3+OffsetToBlockData), SampleRate*4);
			if (NumChannels==6)  {
				ZeroMemory((PVOID)((PBYTE)DataBlocksCh4+OffsetToBlockData), SampleRate*4);
				ZeroMemory((PVOID)((PBYTE)DataBlocksCh5+OffsetToBlockData), SampleRate*4);
				ZeroMemory((PVOID)((PBYTE)DataBlocksCh6+OffsetToBlockData), SampleRate*4);
			}

            // Inform parent thread.
            PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_BADBLOCK, 0, 0); 
         }
   
         // If data stream has just been restored,then inform parent thread.
         if (DataLost) {
            DataLost=FALSE;
            PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_DATAINPUTOK, 0, 0); 
         }

         // Return block nr for updated block.
         *SI->BlockNr=BlockNr;

         // Signal parent process that buffer is updated.
         PostThreadMessage(SI->ThreadID, TM_BUFFERUPDATED, 0, 0); 

         // Advance to the next block.
         BlockNr++; 
         if (BlockNr==NumBlocksPrChannel) BlockNr=0;

         // Handle fix status. If fix quality has changed, inform parent process.
         FixStatus=0; if (StatusGPS==2) FixStatus=1;
         if (FixStatus!=OldFixStatus) {
            OldFixStatus=FixStatus;
            if (FixStatus) PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_FIXOK, 0, 0);
            if (!FixStatus) PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_NOFIX, 0, 0);
         }

      
      } else {
         // Failed to read correct number of bytes.The digitizer has
         // been disconnected, or another comm error has occured.
         if (DataLost==FALSE) {
            DataLost=TRUE;

            // Signal parent process that .
            PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_NODATAINPUT, 0, 0); 
         }

         // Wait one second before trying a new read.
         if (!ReadStatus) Sleep(1000);
      }
   }
   
   ExitThread(0);
   return 0L;
   
}




/*******************************************************************/
/*    Configuration dialog box procedure for PS2400.               */
/*******************************************************************/
BOOL CALLBACK ConfigDlgProcPS2400(HWND hWndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{

#ifdef WINCE
   SHINITDLGINFO shidi;
#endif

   HWND hwndDlgItem;
   DWORD dwIndex;
   INT Result;
   static DEVICE_CONFIG *DC;
   TCHAR PortNr, Str20[21], Message[100];
   TCHAR Ports[16][6]={TEXT("COM1"),TEXT("COM2"), TEXT("COM3"), TEXT("COM4"), TEXT("COM5"), TEXT("COM6"), TEXT("COM7"), TEXT("COM8")
                    , TEXT("COM9"),TEXT("COM10"),TEXT("COM11"),TEXT("COM12"),TEXT("COM13"),TEXT("COM14"),TEXT("COM15"),TEXT("COM16")};

   switch(message)  {
      case WM_INITDIALOG:

#ifdef WINCE
	 shidi.dwMask = SHIDIM_FLAGS;
	 shidi.dwFlags = SHIDIF_SIPDOWN | SHIDIF_SIZEDLGFULLSCREEN;
	 shidi.hDlg = hWndDlg;
	 SHInitDialog(&shidi);
#endif

      // Create pointer to the device config structure.
      DC=(DEVICE_CONFIG*)lParam;
   
      // Initialize the 'Device is connected to' combo box.
      hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO1);
      for (PortNr=0;PortNr<16;PortNr++) {
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)Ports[PortNr]);
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, PortNr);
         if (!wcscmp(DC->PortName, Ports[PortNr])) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }

      // Initialize the 'Communication speed' combo box.
      hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO2);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("9600"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 9600);
      if (!wcscmp(DC->CommSpeed, TEXT("9600"))) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("19200"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 19200);
      if (!wcscmp(DC->CommSpeed, TEXT("19200"))) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("38400"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 38400);
      if (!wcscmp(DC->CommSpeed, TEXT("38400"))) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("57600"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 57600);
      if (!wcscmp(DC->CommSpeed, TEXT("57600"))) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("115200"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 115200);
      if (!wcscmp(DC->CommSpeed, TEXT("115200"))) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);

	  // Initialize the 'number of channels' combo box.
      hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO3);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("3"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 3);
      if (DC->NumChannels==3) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("6"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 6);
      if (DC->NumChannels==6) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      
      // Initialize the 'sample rate' combo box.
      hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO4);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("25"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 25);
      if (DC->SampleRate==25) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("50"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 50);
      if (DC->SampleRate==50) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("75"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 75);
      if (DC->SampleRate==75) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("100"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 100);
      if (DC->SampleRate==100) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("125"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 125);
      if (DC->SampleRate==125) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("150"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 150);
      if (DC->SampleRate==150) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("200"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 200);
      if (DC->SampleRate==200) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("250"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 250);
      if (DC->SampleRate==250) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);

      // Initialize the 'Nr of bytes pr sample' combo box.
      hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO5);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("3"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 3);
      if (DC->BytesPrSample==3) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("4"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 4);
      if (DC->BytesPrSample==4) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);

      // Initialize the 'time correction (seconds)' edit box.
      swprintf(Str20, TEXT("%.3f"), DC->TimeCorrection);
      SetDlgItemText(hWndDlg, IDC_TIME_CORR, Str20);

      // Initialize the 'Syncronize system clock.' check box.
      CheckDlgButton(hWndDlg, IDC_SYNC_SYS_CLK, BST_UNCHECKED); 
      if (DC->SyncPCClock) CheckDlgButton(hWndDlg, IDC_CHECK1, BST_CHECKED); 

      // Initialize the 'Use system clock for time tagging.' check box.
      CheckDlgButton(hWndDlg, IDC_USE_SYS_CLK, BST_UNCHECKED); 
      if (DC->UsePCClock) CheckDlgButton(hWndDlg, IDC_CHECK2, BST_CHECKED); 


      // Bring window to front.

#ifdef WINCE
	  SetForegroundWindow(hWndDlg);
#else
      BringWindowToTop(hWndDlg);
#endif

      return TRUE;


      case WM_COMMAND:
      switch(wParam) {
         case IDOK:  // User has pressed the OK button.
         // Read current choice from the 'Device is connected to' combo box.
         hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO1);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         SendMessage(hwndDlgItem, CB_GETLBTEXT, dwIndex, (LPARAM)&DC->PortName);

         // Read current choice from the 'Communication speed' combo box.
         hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO2);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         SendMessage(hwndDlgItem, CB_GETLBTEXT, dwIndex, (LPARAM)&DC->CommSpeed);

		 // Read current choice from the 'Ch.' (number of channels) combo box
         hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO3);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         DC->NumChannels=(BYTE)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);


         // Read current choice from the 'Sample rate is' combo box.
         hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO4);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         DC->SampleRate=(BYTE)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

         // Read current choice from the 'Nr of bytes pr sample is' combo box.
         hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO5);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         DC->BytesPrSample=(BYTE)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

         // Read current choice from the time correction (seconds) edit box.
         GetDlgItemText(hWndDlg, IDC_TIME_CORR, Str20, 20);
         Result=swscanf(Str20, TEXT("%Lf"), &DC->TimeCorrection);
         if (Result==0 || Result==EOF) {
            // A string that cannot be converted to a float value is entered.
            wcscpy(Message, TEXT("The time correction value is missing or invalid.\n"));
            MessageBox(hWndDlg, Message, TEXT("Seislog message"), MB_OK|MB_ICONERROR);

            // Move the input focus to this field.
            SetFocus(GetDlgItem(hWndDlg, IDC_TIME_CORR));
            SendDlgItemMessage(hWndDlg, IDC_TIME_CORR, EM_SETSEL, 0, -1); 

            // Return without closing.
            return TRUE;            
         }

         // Read current choices from the 'Syncronize system clock.' check box.
         DC->SyncPCClock=(IsDlgButtonChecked(hWndDlg, IDC_SYNC_SYS_CLK)==BST_CHECKED); 

         // Read current choices from the 'Use system clock for time tagging.' check box.
         DC->UsePCClock=(IsDlgButtonChecked(hWndDlg, IDC_USE_SYS_CLK)==BST_CHECKED); 

         // Close the dialog box
         EndDialog(hWndDlg , 1);
         return TRUE;
         
         
         case IDCANCEL:  // User has pressed the Cancel button.
         // Do nothing, just exit.
         EndDialog(hWndDlg , 0);
         return TRUE;      
      }
      return FALSE;

   }
   return FALSE;
}



/*******************************************************************/
/*    Checks if the string from the device is OK.                  */
/*******************************************************************/
BOOL PS2400BytesOK(PBYTE Packet, DWORD PacketSize)
{
   WORD CheckSum, PacketCheckSum, Index;
   
   // Check that string starts with 'MOD\0'.
   if (strcmp("MOD", (PCHAR)Packet)) return FALSE;

   // Calculate checksum.
   CheckSum=0;
   for (Index=0;Index<PacketSize-2;Index++) CheckSum=CheckSum+*(Packet+Index);

   // Compare with checksum given in the block.
   PacketCheckSum=*(Packet+PacketSize-1)*256;
   PacketCheckSum=PacketCheckSum+*(Packet+PacketSize-2);
   if (CheckSum!=PacketCheckSum) return FALSE;

   // No problems detected, return TRUE.
   return TRUE;
}




/*********************************************************************/
/* Extracts the packet sequence number for this packet.              */
/*********************************************************************/
DWORD GetPS2400PacketSequenceNumber(PBYTE Packet)
{
   DWORD SeqNumber;

   SeqNumber=Packet[43];
   SeqNumber=SeqNumber << 8;
   SeqNumber=SeqNumber + Packet[42];
   SeqNumber=SeqNumber << 8;
   SeqNumber=SeqNumber + Packet[41];
   SeqNumber=SeqNumber << 8;
   SeqNumber=SeqNumber + Packet[40];

   return SeqNumber;
}



/*********************************************************************/
/* Extracts the GPS block number from this packet.                   */
/*********************************************************************/
DWORD GetPS2400GPSBlockNumber(PBYTE Packet)
{
   DWORD SeqNumber;

   SeqNumber=Packet[119];
   SeqNumber=SeqNumber << 8;
   SeqNumber=SeqNumber + Packet[118];
   SeqNumber=SeqNumber << 8;
   SeqNumber=SeqNumber + Packet[117];
   SeqNumber=SeqNumber << 8;
   SeqNumber=SeqNumber + Packet[116];

   return SeqNumber;
}
