//#define STRICT
//#include <windows.h>
#include "stdafx.h"
#include <stdio.h>
#include "seislog.h"

//
// Author: yvind Natvik 1998.
//
//
//
//
//

#define PROC_TAB_SIZE   MAX_NUM_OF_CHANNELS*2+20
#define DEPEND_TAB_SIZE MAX_NUM_OF_CHANNELS+20


struct PROCESS_INFO {
   CHAR ProcType;                       // Type of process: 0=No process, 1=DeviceManager, 2=Ringbuffer
                                        //                  3=Monitor, 4=MainMonitor, 5=Channel, 
                                        //                  6=WriteEvent, 7=EventDetection, 8=SecondTicker
										//                  9=Spectra					
   TCHAR ProcName[65];                   // Unique name of process.
   DWORD ThreadID;                      // ThreadID of process.
   BYTE NumDependencies;                // Number of other processes this process depend on.
   DWORD Dependecies[DEPEND_TAB_SIZE];  // ThreadID of dependency processes.
};



//
// Prototype declarations.
//
BYTE GetProcessByName(PROCESS_INFO*, BYTE, PTCHAR);
BYTE GetProcessByThreadID(PROCESS_INFO*, BYTE, DWORD);
VOID RemoveProcess(PROCESS_INFO*, BYTE, BYTE);
BOOL IsDependant(PROCESS_INFO*, BYTE, DWORD);
DWORD WINAPI StopDependantThrdsProc(PVOID); 


/**************************************************************************/
/* ProcessKeeper thread. Keeps track of all running processes.            */
/**************************************************************************/
DWORD WINAPI ProcKeeperThrd(PVOID StartupInfo) 
{

   MSG msg;
   BYTE NumRegProcesses, Index, NumDependencies;
   DWORD CurrentThreadID, *dwArray;
   MSG_PARAM OD, ID;
   PROCESS_INFO Processes[PROC_TAB_SIZE];


   //
   // Initialize some variables.
   //
   CurrentThreadID=GetCurrentThreadId();
   NumRegProcesses=0;

   // Clear/initialize the process database.
   for (Index=0;Index<PROC_TAB_SIZE;Index++) {
      Processes[Index].ProcType=0;
      wcscpy(Processes[Index].ProcName, TEXT(""));
      Processes[Index].ThreadID=0;
      Processes[Index].NumDependencies=0;
      ZeroMemory((PVOID)Processes[Index].Dependecies, DEPEND_TAB_SIZE*sizeof(DWORD));
   }
   

   // 
   // Enter the thread message loop. The thread will now start to process messages.
   //
   while (GetMessage(&msg, (HWND)NULL, 0, 0)) {
      
      // Save incoming message parameters in the 'ID' structure.
      ZeroMemory((PVOID)&ID, (DWORD)sizeof(MSG_PARAM));
      if (msg.lParam) ID=*((MSG_PARAM*)msg.lParam);
      
      // Initialize return-message parameters
      // in the OD structure to default values.
      ZeroMemory((PVOID)&OD, (DWORD)sizeof(MSG_PARAM));
      OD.bStatus=TRUE;
      OD.idThread=CurrentThreadID;

      // Process message.
      switch (msg.message) {

         case TM_INITIALIZE:                             
         //
         // This message is sendt from a higher level 
         // thread to initialize the process database.
         // The message takes no parameters.
         //
         // Initialize the process database.
         for (Index=0;Index<PROC_TAB_SIZE;Index++) {
            Processes[Index].ProcType=0;
            wcscpy(Processes[Index].ProcName, TEXT(""));
            Processes[Index].ThreadID=0;
            Processes[Index].NumDependencies=0;
            ZeroMemory((PVOID)Processes[Index].Dependecies, DEPEND_TAB_SIZE*sizeof(DWORD));
         }
         NumRegProcesses=0;
         break;

         
         case TM_REGISTER_PROCESS:                             
         //
         // This message must be sendt from all processes (threads) 
         // when they start. The message will cause the process to 
         // be registered in the database as a running prcess.
         // There are only 3 processes that should not need to register
         // in this database: This process, the main program and the 
         // system log process.
         // Parameters are supplied in the 'ID' MSG_PARAM structure
         // (that must accompany this message) as follows:
         //
         // ID.dwStatus - The process type (See definition of PROCESS_INFO).
         // ID.cStatus  - The unique name of the calling process.
         // ID.idThread - The process thread-id.
         // 
         Processes[NumRegProcesses].ProcType=(char)ID.dwStatus;
         wcscpy(Processes[NumRegProcesses].ProcName, ID.cStatus);
         Processes[NumRegProcesses].ThreadID=ID.idThread;
         NumRegProcesses++;   
         break;


         case TM_GET_NUM_REGISTERED:                              
         //
         // This message is sendt from a processes (threads) 
         // who wants to know how many processes are registered 
         // in the database.
         // Parameters are returned in the 'IO' MSG_PARAM structure
         // (that must accompany this message) as follows:
         //
         // OD.dwStatus - Number of registered processes.
         // 
         OD.dwStatus=(DWORD)NumRegProcesses;
         break;


         case TM_UNREGISTER_PROCESS:
         // This message must be sendt from all processes (threads) 
         // when they exit. The message will cause the process to 
         // be removed from the database of running processes.
         // Parameters are supplied in the 'ID' MSG_PARAM structure
         // (that must accompany this message) as follows:
         //
         // ID.idThread - The process thread-id.
         // 
         // Find the index of the calling process in the 'Processes' table.
         Index=GetProcessByThreadID(Processes, NumRegProcesses, ID.idThread);

         // Remove process from the process table.
         if (Index<255) {
            RemoveProcess(Processes, NumRegProcesses, Index);
            NumRegProcesses--;
         } else {
            OD.bStatus=FALSE;
         }
         break;



         case TM_SET_DEPENDENCY:                
         // This message must be sendt from all processes (threads) 
         // when they start if they are dependent on another process
         // in order to run. This message must be sendt after the
         // 'TM_REGISTER_PROCESS' message has been sendt.
         // Parameters are supplied in the 'ID' MSG_PARAM structure
         // (that must accompany this message) as follows:
         //
         // ID.idThread - The thread-id of the calling process.
         // ID.dwStatus - The thread-id of the dependancy process.
         //
         // Find the index of the calling process in the 'Processes' table.
         Index=GetProcessByThreadID(Processes, NumRegProcesses, ID.idThread);

         // Update the dependency table.
         if (Index<255) {
            NumDependencies=Processes[Index].NumDependencies;
            if (NumDependencies<DEPEND_TAB_SIZE) {
               Processes[Index].Dependecies[NumDependencies]=ID.dwStatus;
               Processes[Index].NumDependencies++;
            } else { OD.bStatus=FALSE; }
         } else { OD.bStatus=FALSE; }
         break;


         
         case TM_GET_DEPENDENCIES:
         // This message is sendt from a registered process (thread) 
         // who want to get a list of all processes that are dependent
         // on it(the calling process), in order to run. 
         // Parameters are supplied in the 'ID' MSG_PARAM structure
         // (that must accompany this message) as follows:
         //
         // ID.idThread - The thread-id of the calling process.
         // ID.vPointer - A pointer to an array of DWORD's used to
         //               return the list of dependant processes.
         // ID.wStatus -  The size of the supplied array.
         //
         // This thread will fill out the given array, and
         // return number of dependancies in the 'wStatus'
         // parameter. This parameter will be 0 if no dependent
         // processes are found. The 'bStatus' parameter will
         // always return 'TRUE', accept if the given list is
         // too small to hold all dependencies. In this case the
         // list will still be filled up with as many dependencies
         // there is room for, and 'wStatus' will return number of
         // actually returned dependencies in the list.
         //
         OD.wStatus=0;
         dwArray=(DWORD*)ID.vPointer;
         // Go through the process list.
         for (Index=0;Index<NumRegProcesses;Index++) {
            if (IsDependant(Processes, Index, ID.idThread)) {
               // The process pointed to by 'Index' is depandant.
               if (OD.wStatus<ID.wStatus) {
                  dwArray[OD.wStatus]=Processes[Index].ThreadID;
                  OD.wStatus++;
               } else {
                  OD.bStatus=FALSE;
               }
            }
         }
         break;
      
         
         case TM_GET_DEVICEMANAGERS:
         // This message can be used to get a list of all
         // device-manager threads currently registered
         // in the list of running processes.
         // Parameters are supplied in the 'ID' MSG_PARAM structure
         // (that must accompany this message) as follows:
         //
         // ID.vPointer - A pointer to an array of DWORD's used to
         //               return the list of device-manager processes.
         //
         // This thread will fill out the given array, and
         // return number of device-managers in the 'dwStatus'
         // parameter.
         // The 'bStatus' parameter will always return 'TRUE'.
         //
         OD.dwStatus=0;
         dwArray=(DWORD*)ID.vPointer;
         // Go through the process list.
         for (Index=0;Index<NumRegProcesses;Index++) {
            if (Processes[Index].ProcType==1) {
               // The process pointed to by 'Index' is a device-manager process.
               if (dwArray!=NULL) dwArray[OD.dwStatus]=Processes[Index].ThreadID;
               OD.dwStatus++;
            }
         }
         break;



         case TM_GET_CHANNELS:
         // This message can be used to get a list of all
         // channel threads currently registered in the 
         // list of running processes.
         // Parameters are supplied in the 'ID' MSG_PARAM structure
         // (that must accompany this message) as follows:
         //
         // ID.vPointer - A pointer to an array of DWORD's 
         //               used to return the list of processes.
         //
         // This thread will fill out the given array, and
         // return number of channels in the 'dwStatus' parameter.
         // The 'bStatus' parameter will always return 'TRUE'.
         //
         OD.dwStatus=0;
         dwArray=(DWORD*)ID.vPointer;
         // Go through the process list.
         for (Index=0;Index<NumRegProcesses;Index++) {
            if (Processes[Index].ProcType==5) {
               // The process pointed to by 'Index' is a channel process.
               if (dwArray!=NULL) dwArray[OD.dwStatus]=Processes[Index].ThreadID;
               OD.dwStatus++;
            }
         }
         break;



         case TM_GET_RINGBUFFERS:
         // This message can be used to get a list of all
         // ringbuffer threads currently registered
         // in the list of running processes.
         // Parameters are supplied in the 'ID' MSG_PARAM structure
         // (that must accompany this message) as follows:
         //
         // ID.vPointer - A pointer to an array of DWORD's 
         //               used to return the list of processes.
         //
         // This thread will fill out the given array, and
         // return number of ringbuffers in the 'dwStatus' parameter.
         // The 'bStatus' parameter will always return 'TRUE'.
         //
         OD.dwStatus=0;
         dwArray=(DWORD*)ID.vPointer;
         // Go through the process list.
         for (Index=0;Index<NumRegProcesses;Index++) {
            if (Processes[Index].ProcType==2) {
               // The process pointed to by 'Index' is a ringbuffer process.
               dwArray[OD.dwStatus]=Processes[Index].ThreadID;
               OD.dwStatus++;
            }
         }
         break;



         case TM_GET_EVENTDETECTIONS:
         // This message can be used to get a list of all
         // event detection threads currently registered
         // in the list of running processes.
         // Parameters are supplied in the 'ID' MSG_PARAM structure
         // (that must accompany this message) as follows:
         //
         // ID.vPointer - A pointer to an array of DWORD's 
         //               used to return the list of processes.
         //
         // This thread will fill out the given array, and
         // return number of ringbuffers in the 'dwStatus' parameter.
         // The 'bStatus' parameter will always return 'TRUE'.
         //
         OD.dwStatus=0;
         dwArray=(DWORD*)ID.vPointer;
         // Go through the process list.
         for (Index=0;Index<NumRegProcesses;Index++) {
            if (Processes[Index].ProcType==7) {
               // The process pointed to by 'Index' is a eventdetection process.
               dwArray[OD.dwStatus]=Processes[Index].ThreadID;
               OD.dwStatus++;
            }
         }
         break;



         case TM_GET_PROCESS_BY_NAME:
         // This message can be used find out if a certain
         // process is running or not. If found the process       
         // thread-id will be returned to the calling process.
         // Parameters are supplied in the 'ID' MSG_PARAM structure
         // (that must accompany this message) as follows:
         //
         // ID.cStatus - The name of the process to find.
         // 
         // OD.bStatus  - TRUE if process is found.
         // OD.dwStatus - Thread id of found process.
         Index=GetProcessByName(Processes, NumRegProcesses, ID.cStatus);   
         if (Index<255) {
            // The process was found.
            OD.dwStatus=Processes[Index].ThreadID;
         } else {
            // The process was NOT found.
            OD.bStatus=FALSE;
            OD.dwStatus=0;
         }
         break;





         case TM_EXIT:                 
         //
         // This is a request for the thread to end.
         //
         // Post QUIT message.
         PostQuitMessage(0); 
         break;
      }
      
      // Return status values, and release calling process.
      if (((MSG_PARAM*)msg.lParam)!=NULL && msg.wParam!=NULL) {
         *(MSG_PARAM*)msg.lParam=OD;
         SetEvent((HANDLE)msg.wParam);
      }
   } 

   ExitThread(0);
   return 0L;

}



/***********************************************************************************/
/* Returns an index into the 'processes' table to the process                      */
/* registered by the name given in the 'ProcName' parameter.                       */
/***********************************************************************************/
BYTE GetProcessByName(PROCESS_INFO *Processes, BYTE NumRegProcesses, PTCHAR ProcName)
{
   BYTE Index;

   // Search through the table for the process name.
   for (Index=0;Index<NumRegProcesses;Index++) {
      if (!_wcsicmp(Processes[Index].ProcName, ProcName)) return Index; 
   }

   // Return error value.
   return 255;
}



/**************************************************************************************/
/* Returns an index into the 'processes' table to the process                         */
/* registered with the thread id given by the 'ThreadID' parameter.                   */
/**************************************************************************************/
BYTE GetProcessByThreadID(PROCESS_INFO *Processes, BYTE NumRegProcesses, DWORD ThreadID)
{
   BYTE Index;

   // Search through the table for the process name.
   for (Index=0;Index<NumRegProcesses;Index++) {
      if (Processes[Index].ThreadID==ThreadID) return Index;   
   }

   // Return error value.
   return 255;
}



/*****************************************************************************/
/* Removes a registered process from the 'processes' table                   */
/*****************************************************************************/
VOID RemoveProcess(PROCESS_INFO *Processes, BYTE NumRegProcesses, BYTE Index)
{
   BYTE Index2;

   if ((Index+1)==NumRegProcesses) {
      // Remove the last process in the table.
      // Just clear the last table entry.
      Processes[Index].ProcType=0;
      wcscpy(Processes[Index].ProcName, TEXT(""));
      Processes[Index].ThreadID=0;
      Processes[Index].NumDependencies=0;
      ZeroMemory((PVOID)Processes[Index].Dependecies, DEPEND_TAB_SIZE*sizeof(DWORD));
   } else {
      // Remove any other process but the last.
      // This is done by moving the processes after
      // the 'process to be removed' one place up in the
      // table, and thereby overwriting the 'remove process'.
      for (Index2=Index+1;Index2<=(NumRegProcesses-1);Index2++) {
         Processes[Index2-1]=Processes[Index2];
      }

      // Then the the last table entry are cleared.
      Processes[NumRegProcesses-1].ProcType=0;
      wcscpy(Processes[NumRegProcesses-1].ProcName, TEXT(""));
      Processes[NumRegProcesses-1].ThreadID=0;
      Processes[NumRegProcesses-1].NumDependencies=0;
      ZeroMemory((PVOID)Processes[NumRegProcesses-1].Dependecies, DEPEND_TAB_SIZE*sizeof(DWORD));
   }

   return;
}



/*****************************************************************************/
/* Searches through a process's dependancy list for a match with             */
/* the process given in the 'ThreadID' parameter.                            */
/*****************************************************************************/
BOOL IsDependant(PROCESS_INFO *Processes, BYTE Index, DWORD ThreadID)
{
   BYTE Index2;

   for (Index2=0;Index2<Processes[Index].NumDependencies;Index2++) {
      if (Processes[Index].Dependecies[Index2]==ThreadID) return TRUE;
   }

   return FALSE;
}



/************************************************************************/
/* Stopps all threads that has registered a depndency of                */
/* the thread identified by the 'TreadID' parameter.                    */
/* A TM_DEPENDANT_THREADS_STOPPED is sent to the caller when all        */
/* threads are stopped. This function does NOT hang the calling thread. */
/************************************************************************/
VOID StopDependantThreads(VOID)
{
	HANDLE hThread;
   DWORD ThreadID, CallerThrdID;

   // Start the shutdown thread.
   CallerThrdID=GetCurrentThreadId();
   hThread=CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)StopDependantThrdsProc, (PVOID)CallerThrdID, 0, &ThreadID);
	CloseHandle(hThread);

   return;
}



/************************************************************************/
/* Send TM_EXIT message to all threads that has registered a dependency */
/* the thread identified by the 'StartupInfo' parameter.                */
/* A TM_DEPTHREADS_STOPPED is sent to the caller when all threads are   */
/* stopped.                                                             */
/************************************************************************/
DWORD WINAPI StopDependantThrdsProc(PVOID StartupInfo) 
{
   DWORD ThreadTable[200];
   BYTE NumDepThreads, Index;
   HANDLE hEvent;
   MSG_PARAM MsgParam;
   extern DWORD ProcKeeperThreadID;

   // Get a list of all dependant threads.
   MsgParam.idThread=(DWORD)StartupInfo;
   MsgParam.vPointer=(PVOID)&ThreadTable;
   MsgParam.wStatus=200;
   hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
   PostThreadMessage(ProcKeeperThreadID, TM_GET_DEPENDENCIES, (WPARAM)hEvent, (LPARAM)&MsgParam); 
   WaitForSingleObject(hEvent, INFINITE);
   NumDepThreads=(BYTE)MsgParam.wStatus;
   CloseHandle(hEvent);

   // Stop threads, one by one.
   for (Index=0;Index<NumDepThreads;Index++) {
      // Send stop signal to the thread.
      if (PostThreadMessage(ThreadTable[Index], TM_EXIT, 0, 0)==0) MessageBeep(-1); 

      // Wait for the thread to exit.
      while (PostThreadMessage(ThreadTable[Index], TM_DUMMY, 0, 0)) Sleep(200);
   }


   // Send TM_DEPENDANT_THREADS_STOPPED message back to caller.
   PostThreadMessage((DWORD)StartupInfo, TM_DEPTHREADS_STOPPED, 0, 0); 

   // Exit this thread.
   ExitThread(0);
   return 0;
}





/*********************************************************************/
/* Inserts a process entry in the running processes list.            */
/*********************************************************************/
VOID RegisterProcess(PTCHAR ProcName, TCHAR ProcType, DWORD ThreadID)
{
   HANDLE hEvent;
   MSG_PARAM MsgParams;
   extern DWORD ProcKeeperThreadID;

   MsgParams.dwStatus=(DWORD)ProcType;    // The type of process.        
   wcscpy(MsgParams.cStatus, ProcName);   // The name of this process.
   MsgParams.idThread=ThreadID;           // The process thread-id.
   hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
   PostThreadMessage(ProcKeeperThreadID, TM_REGISTER_PROCESS, (WPARAM)hEvent, (LPARAM)&MsgParams); 
   WaitForSingleObject(hEvent, INFINITE);
   CloseHandle(hEvent);

}


/*********************************************************************/
/* Removes a process entry from the running processes list           */
/*********************************************************************/
VOID UnregisterProcess(DWORD ThreadID)
{
   HANDLE hEvent;
   MSG_PARAM MsgParam;
   extern DWORD ProcKeeperThreadID;

   MsgParam.idThread=ThreadID;               
   hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
   PostThreadMessage(ProcKeeperThreadID, TM_UNREGISTER_PROCESS, (WPARAM)hEvent, (LPARAM)&MsgParam); 
   WaitForSingleObject(hEvent, INFINITE);
   CloseHandle(hEvent);

}


/*********************************************************************/
/* Register a process's dependency of another process.               */
/*********************************************************************/
VOID RegisterDependency(DWORD ThreadID, DWORD DepThreadID)
{
   HANDLE hEvent;
   MSG_PARAM MsgParams;
   extern DWORD ProcKeeperThreadID;

   MsgParams.idThread=ThreadID;        // The thread-id of the thread which registers the dependeny.
   MsgParams.dwStatus=DepThreadID;     // The thread-id of the thread the thread is dependant of.
   hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
   PostThreadMessage(ProcKeeperThreadID, TM_SET_DEPENDENCY, (WPARAM)hEvent, (LPARAM)&MsgParams); 
   WaitForSingleObject(hEvent, INFINITE); // Wait for opration to complete.
   CloseHandle(hEvent);

}




/************************************************************************/
/* Returns the thread-id of the process identified by name 'ProcName'.  */
/************************************************************************/
DWORD GetProcessThreadIDByName(TCHAR *ProcName)
{
   HANDLE hEvent;
   MSG_PARAM MsgParam;
   extern DWORD ProcKeeperThreadID;

   wcscpy(MsgParam.cStatus, ProcName);             
   hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
   PostThreadMessage(ProcKeeperThreadID, TM_GET_PROCESS_BY_NAME, (WPARAM)hEvent, (LPARAM)&MsgParam); 
   WaitForSingleObject(hEvent, INFINITE);
   CloseHandle(hEvent);

   return MsgParam.dwStatus;
}




/************************************************************************/
/* Returns TRUE if any processes are registered (running).              */
/************************************************************************/
BOOL IsSystemRunning(VOID)
{

   HANDLE hEvent;
   MSG_PARAM MsgParam;
   extern DWORD ProcKeeperThreadID;

   hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
   PostThreadMessage(ProcKeeperThreadID, TM_GET_NUM_REGISTERED, (WPARAM)hEvent, (LPARAM)&MsgParam); 
   WaitForSingleObject(hEvent, INFINITE);
   CloseHandle(hEvent);

   return (MsgParam.dwStatus>0);
}