// Sadc.cpp
// Converted to SeislogCE by Jose Aasheim Ojeda 26.2.2004

#include "stdafx.h"
#include <aygshell.h>
#include <windows.h>
#include <stdio.h>
#include "seislog.h"
#include "library.h"
#include "msglist.h"
#include "resource.h"
#include "process.h"
#include "eventlog.h"

#define WINCE

//
// This structure is read from the binary device config file.
//
struct DEVICE_CONFIG {
   WORD DeviceID;          // MUST be '510'.
   BYTE NumChannels;       // Can be 1 or 4.
   TCHAR PortName[6];       // COM1, COM2 ...
   TCHAR CommSpeed[6];	   // Communication speed.
   BYTE SampleRate;        // Can be 10, 50, 100 . 
   BOOL HighSmplRate;      // TRUE if device supports sample rate of 200hz.
   BYTE DataBits;          // Device data-resolution (16, 18 or 24).
   BOOL DeviceHasDT;       // TRUE if device transmits both date and time.
   BOOL UsePCClock;        // If TRUE then PC clock will be used to set the SADC clock.
   BOOL SyncPCClock;       // If TRUE then device driver will syncronize the system clock.
   DOUBLE TimeCorrection;  // Time correction (seconds).
};


//
// This structure is used to pass startup- 
// information to the serial-reader thread.
//
struct STARTINFO_DEVREADER { 
   HANDLE hComm;              // Handle of communications device.
   WORD FwVersion;            // Firmware version.
   TCHAR CommSpeed[6];	    	// Communication speed.
   DWORD ThreadID;            // Thread-ID for parent thread.
   BOOL *SerialRead;          // The serial-reader thread will exit if this becomes FALSE.
   DEVMNGR_INTERF_STRUCT *DS; // Pointer to the exported structure.
   DEVICE_CONFIG *DC;         // Device configuration structure.
   DWORD* BlockNr;            // Pointer to variable who are to receive block numbers.
   BYTE SyncStatus;           // The current time syncronization status.
   HANDLE hSyncStatusMutex;   // Handle to a mutex object used to control access to the
                              // SyncStatus variable in this structure.
   TCHAR UniqueDeviceName[65]; // The devicename for this instance of the SADC driver.
};




//
// Global variables.
//
extern BYTE DebugLevel;
extern HWND hWndFrame;
extern DWORD SysClkSyncThreadID;


//
// Function prototyping.
//
DWORD WINAPI ReaderThreadSADC(PVOID);
BOOL CALLBACK ConfigDlgProcSADC(HWND, UINT, WPARAM, LPARAM);
BOOL SADCLocateTime(  STARTINFO_DEVREADER *,PBYTE, PBYTE, DWORD, DWORD &);
DWORD SADCNext(PBYTE , PBYTE,  DWORD , DWORD, DWORD &, LONG &, BOOL);
BOOL GetSADCFirmwareRevision(HANDLE, PWORD);
VOID SADCSetup(STARTINFO_DEVREADER*, DEVICE_CONFIG*);
BOOL SADCStop(HANDLE);
BYTE SADCReadBlock(HANDLE, PBYTE, DWORD);
VOID SADCGetBlockTime(PBYTE, SYSTEMTIME*, PBOOL);
VOID SADCGetChSamples(PBYTE, BYTE, PLONG, WORD, BYTE, WORD);



/* ********************************************************************* */
/*  Manager thread for the SADC device. DeviceID=510.                    */
/* ********************************************************************* */
DWORD WINAPI SADCMngrThrd(PVOID StartupInfo) 
{
   MSG msg;
   int Status;
   DCB dcb;
   BYTE SyncStatus, FixStatus, Index;
   TCHAR ConfigFile[MAX_PATH+1], ChNr, UniqueDeviceName[65], Message[100];
   TCHAR DeviceResources[256], SerPort[10], str10[11], DevCntrlStr[50];
   WORD BlockSize, NumBlocksPrChannel, SampleRate, FwVersion, Speed;
   HANDLE hReaderThread, hComm;
   STARTINFO_DEVMANAGER SI;
   STARTINFO_DEVREADER StartInfo;
   DEVICE_CONFIG DeviceConf, DevConfDlg;
   DEVMNGR_INTERF_STRUCT DS;
   DWORD ParentThreadID, CurrentThreadID, ReaderThreadID;
   DWORD ChannelSize, status, BlockNr;
   MSG_PARAM ID, OD;
   MsgList NotifyList;
   BOOL SerialRead, MemAlloc, ConfigRead;
   
   
   //
   // Perform thread initialization.
   //
   FixStatus=0;
   SyncStatus=1;                                     // So far we have no fix.   
   hReaderThread=(HANDLE)NULL;                       // The serial-reader thread is not running.
   CopyMemory((PVOID)&SI, StartupInfo, sizeof(SI));  // Copy startup information to local buffer.
   ParentThreadID=SI.ThreadID;                       // ThreadID for parent thread.
   CurrentThreadID=GetCurrentThreadId();             // ThreadID for this thread.
   

   //
   // Handle device configuration.
   //
   // Read configuration file.
   ConfigRead=FALSE;
   wcscpy(ConfigFile, SI.DeviceDir); 
   wcscat(ConfigFile, TEXT("\\device.cnf"));
   if (ExistFile(SI.DeviceDir, TEXT("device.cnf"))) {
      // Read device config from config file.
      ConfigRead=ReadFileToMemory(ConfigFile, (PVOID)&DeviceConf, (DWORD)sizeof(DeviceConf));
   }
   if (!ConfigRead) {
      // A valid config file could not be read.
      // Set up a default digitizer configuration.
      DeviceConf.DeviceID=510;               // SADC uses device-id 500.
      DeviceConf.NumChannels=4;              // Assume 4 channel in use.
      wcscpy(DeviceConf.PortName, TEXT("COM1"));   // Assume digitizer on COM1.
      wcscpy(DeviceConf.CommSpeed, TEXT("38400")); // Assume comm speed of 38400 baud.
      DeviceConf.SampleRate=100;              // Assume 50hz sample rate.
      DeviceConf.HighSmplRate=TRUE;         // Assume 200hz sample rate capability.
      DeviceConf.DataBits=16;                // Assume 16 bit data resolution.
      DeviceConf.DeviceHasDT=TRUE;          // Assume device has date.
      DeviceConf.TimeCorrection=0.0;         // Time correction is off by default.
      DeviceConf.UsePCClock=FALSE;            // Use PC clock to set time on digitizer.
      DeviceConf.SyncPCClock=TRUE;          // Syncronize the system clock.

      // Write settings to config file.
      WriteFileFromMemory(ConfigFile, (PVOID)&DeviceConf, (DWORD)sizeof(DeviceConf));
   }

   // 
   // Create a unique device instance name.
   //
   wcscpy(UniqueDeviceName, TEXT("SADC at "));
   wcscat(UniqueDeviceName, DeviceConf.PortName);


   //
   // Register this process in the process database.
   //
   if (!SI.ConfigOnly) RegisterProcess(UniqueDeviceName, 1, CurrentThreadID);


   //
   // Create device resource string.
   // 
   wcscpy(DeviceResources, TEXT("Serial ports: "));
   wcscat(DeviceResources, DeviceConf.PortName);


   //
   // Handle time synchronization stuff.
   //
   if (DeviceConf.SyncPCClock && !SI.ConfigOnly) {
      // Device is configured to synchronize the system clock.
      // Check if any other thread is already doing this.
      if (SysClkSyncThreadID) {
         // Turn off time synchronication. Clock is 
         // already beeing synchronized by another thread.
         DeviceConf.SyncPCClock=FALSE;
         // Write warning in the system log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Cannot synchronize system clock. Clock already synchronized."));
         WriteSystemLog(Message, 2);
      } else {
         // System clock is not already synchronized.
         // This thread will attempt to synchronize it.
         SysClkSyncThreadID=CurrentThreadID;
         // Write text on status bar.
         DrawStatusBarText(SyncStatus, UniqueDeviceName);
         // Write message in the system log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Synchronizing system clock."));
         WriteSystemLog(Message, 1);
      }
   }


   //
   // Calculate sample-rate, blocksize, channelsize and number of blocks pr. channel.
   //
   SampleRate=(WORD)DeviceConf.SampleRate;
   BlockSize=(WORD)(sizeof(BLOCK_HEADER)+(SampleRate*4));
   NumBlocksPrChannel=60;
   ChannelSize=NumBlocksPrChannel*BlockSize;


   //
   // Open the serial port.
   //
   if (!SI.ConfigOnly) {
      #ifdef WINCE
      // All ports in Windows CE must end with ':'
      TCHAR CEPortName[7];
      wcscpy(CEPortName, DeviceConf.PortName);
      wcscat(CEPortName, TEXT(":"));
      hComm=CreateFile(CEPortName, GENERIC_READ|GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
      #else 
      wcscpy(SerPort, TEXT("\\\\.\\")); wcscat(SerPort, DeviceConf.PortName);
      hComm=CreateFile(SerPort, GENERIC_READ|GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
      #endif

      if (hComm==INVALID_HANDLE_VALUE) {
         // Update program log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Unable to open communication port "));
         wcscat(Message, DeviceConf.PortName);
         wcscat(Message, TEXT("."));
         WriteSystemLog(Message, 3);
      } else {
         //Initialize the communications parameters.
         SetupComm(hComm, 6600, 0);
         GetCommState(hComm, &dcb);
         #ifdef WINCE  
		 dcb.BaudRate  = _wtoi(DeviceConf.CommSpeed);
         dcb.Parity    = NOPARITY;
         dcb.StopBits  = ONESTOPBIT;
         dcb.ByteSize  = 8;
         #else
         wcscpy(DevCntrlStr, TEXT("baud="));
         wcscat(DevCntrlStr, DeviceConf.CommSpeed);
         wcscat(DevCntrlStr, TEXT(" parity=N data=8 stop=1"));
         BuildCommDCB(DevCntrlStr, &dcb);
         #endif
         dcb.fAbortOnError=FALSE;
         SetCommState(hComm, &dcb); // Set serial port mode. No flow control is used.
      }
   }

   
   //
   // Initialize the interface structure.
   //
   // Insert manufacturer name and model.
   wcscpy(DS.Manufacturer, TEXT("M.Mariotti"));
   wcscpy(DS.Model, TEXT("SADC"));

   // Insert samplerate.
   DS.SampleRate=SampleRate;

   // Insert number of samples in each block.
   DS.SamplesPrBlock=SampleRate;

   // Insert number of channels
   DS.NumChannels=DeviceConf.NumChannels;

   // Insert block size, block header included.
   DS.BlockSize=BlockSize;

   // Insert block size measured in milliseconds.
   DS.BlockSizeInMS=1000;

   // Insert number of blocks pr channel.
   DS.NumBlocksPrChannel=NumBlocksPrChannel;

   // Make sure all pointers to channel data is NULL.
   for (ChNr=0;ChNr<MAX_CHANNELS_PR_DEVICE;ChNr++) DS.ChannelData[ChNr]=(PVOID)NULL;
   
   
   //
   // Allocate memory for the channel buffers, and set up the pointers to the buffers.
   //
   MemAlloc=TRUE;
   for (Index=0;Index<DeviceConf.NumChannels;Index++) {
      DS.ChannelData[Index]=(PVOID)LocalAlloc(LPTR, (UINT)ChannelSize);
      if (DS.ChannelData[Index]==NULL) MemAlloc=FALSE;
   }

   // Check that memory allocation succeded.
   if (!MemAlloc) {
         // Update program log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(":"));
         wcscat(Message, TEXT(" Could not allocate memory."));
         WriteSystemLog(Message, 3);
   }

   
   //
   // Report the detected firmware revision if possible.
   //
   if (!SI.ConfigOnly) {
      SADCStop(hComm);
      if (GetSADCFirmwareRevision(hComm, &FwVersion)) {
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Firmware revision is '"));
         swprintf(str10,TEXT("%.*f "),2,(DOUBLE)(FwVersion/100.0));
         wcscat(Message, str10); wcscat(Message, TEXT("'."));
         WriteSystemLog(Message, 1);
         if (FwVersion<140) {
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(":"));
            wcscat(Message, TEXT(" This firmware revision is NOT supported by Seislog!"));
            WriteSystemLog(Message, 3);
         }
      }
   }   


   //
   // Check that the baudrate is high enough for the data stream.
   //
   if (!wcscmp(DeviceConf.CommSpeed, TEXT("14400"))) Speed=1440;
   if (!wcscmp(DeviceConf.CommSpeed, TEXT("38400"))) Speed=3840;
	if (DS.NumChannels*4*DS.SampleRate+6 > Speed) {
      wcscpy(Message, UniqueDeviceName);
      wcscat(Message, TEXT(": Samplerate to high for current baudrate. Reduce samplerate or number of channels. Operation halted!"));
      WriteSystemLog(Message, 3);
      MemAlloc=FALSE;
	}


   //
   // If running in debug mode, write a message to the log.
   //
   if (DebugLevel) {
      wcscpy(Message, UniqueDeviceName);
      wcscat(Message, TEXT(": Thread started."));
      WriteSystemLog(Message, 1);
   }
  

   //
   // Signal that the thread is started.
   //
   SetEvent(SI.StartEvent);


   //
   // Enter the thread message loop. The thread will now start to process messages.
   //
   while (GetMessage(&msg, (HWND)NULL, 0, 0)) 
   { 
   
      // If debuglevel is 2 then log all messages recieved.
      if (DebugLevel>1) {
         TCHAR str10[11];
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Recieved message "));
         _itow(msg.message, str10, 10);
         wcscat(Message, str10);
         WriteSystemLog(Message, 1);
      }

      // Save message parameters.
      ZeroMemory((PVOID)&ID, (DWORD)sizeof(ID));
      if (msg.lParam) ID=*((MSG_PARAM*)msg.lParam);
      
      // Initialize return-message parameter to default values.
      ZeroMemory((PVOID)&OD, (DWORD)sizeof(OD));
      OD.bStatus=TRUE;
      OD.idThread=CurrentThreadID;


      // Process message.
      switch (msg.message) {
         case TM_START:                            
         //
         // This message is sendt from a higher level 
         // thread to start reading the serial port.
         //
         if (hReaderThread==NULL && hComm!=INVALID_HANDLE_VALUE && MemAlloc && !SI.ConfigOnly) {
            if (SysClkSyncThreadID && SysClkSyncThreadID!=CurrentThreadID) {
               // The system clock is synchronized by some other thread.
               // Subscribe to the TM_TIMESYNC_STATUS message from thread.
               Subscribe(TM_TIMESYNC_STATUS, SysClkSyncThreadID, CurrentThreadID, (HWND)NULL);

               // Get the current synchronization status.
               StartInfo.SyncStatus=GetCurrentTimeSyncStatus(SysClkSyncThreadID);
            } else {
               // The system clock is not synchronized and
               // there is no other synchronization possibilities.
               StartInfo.SyncStatus=0;
               // Create a mutex used to access the 'StartInfo.SyncStatus' variable.
               StartInfo.hSyncStatusMutex=CreateMutex(NULL, FALSE, NULL);
            }

            // Set up startup information for the serial reader thread routine.
            SerialRead=TRUE;
            StartInfo.hComm=hComm;
            StartInfo.FwVersion=FwVersion;
				wcscpy(StartInfo.CommSpeed, DeviceConf.CommSpeed);
            StartInfo.ThreadID=CurrentThreadID;
            StartInfo.SerialRead=&SerialRead;
            StartInfo.DS=&DS;
            StartInfo.DC=&DeviceConf;
            StartInfo.BlockNr=&BlockNr;
            wcscpy(StartInfo.UniqueDeviceName, UniqueDeviceName);

            // Start the thread.
            hReaderThread=CreateThread(NULL, 0, ReaderThreadSADC, (VOID*)&StartInfo, CREATE_SUSPENDED, &ReaderThreadID);
            SetThreadPriority(hReaderThread, THREAD_PRIORITY_ABOVE_NORMAL);  // Set priority above normal.
            ResumeThread(hReaderThread);

         }

         // Set return values. 
         if (!hReaderThread) {
            OD.bStatus=FALSE;
            OD.wStatus=1;
         }
         break;


         
         case TM_STOP:
         //
         // This message is sendt from a higher level thread to stop
         // reading the serial port. Returns the default return values.
         //
         if (hReaderThread) {
            // Make the serial thread loop exit, thereby stopping the thread.
            SerialRead=FALSE;

            // Wait up to 2 seconds for the thread to stop.
            status=WaitForSingleObject(hReaderThread, 2000);
            if (status==WAIT_TIMEOUT) TerminateThread(hReaderThread, 0);

            // Unsubscribe to the TM_TIMESYNC_STATUS message as needed.
            if (SysClkSyncThreadID && SysClkSyncThreadID!=CurrentThreadID) {
               UnSubscribe(TM_TIMESYNC_STATUS, SysClkSyncThreadID, CurrentThreadID, NULL);
               CloseHandle(StartInfo.hSyncStatusMutex); // Delete mutex object.
            }

            // The thread is now stopped, invalidate the thread handle.
            CloseHandle(hReaderThread);
            hReaderThread=NULL;
         }
         break;


         case TM_CONFIGURE:            
         //
         // This message is sendt from a higher level thread
         // when it wants to configure the digitizer settings.
         //
         if (hReaderThread==NULL) {
            // Let user work on a copy of the current device settings.
            DevConfDlg=DeviceConf;

            // Display the dialog box.
            Status=DialogBoxParam(SI.hInstance, MAKEINTRESOURCE(IDD_CNF_SADC), NULL, (DLGPROC)ConfigDlgProcSADC, (LPARAM)&DevConfDlg);   

            // Save settings if requested.
            if (Status==1) {
               // Save the new configuration.
               DeviceConf=DevConfDlg;

               // Write configuration to disk.
               WriteFileFromMemory(ConfigFile, (PVOID)&DeviceConf, (DWORD)sizeof(DEVICE_CONFIG));

               // Recreate the unique name of this device instance.
			      wcscpy(UniqueDeviceName, TEXT("SADC at "));
			      wcscat(UniqueDeviceName, DeviceConf.PortName);
            }

            // Set return values. 
            if (Status==0) {
               OD.bStatus=FALSE;
               OD.wStatus=100;
            }
         }
         break;


         case TM_SHOW_CONFIG:
            //
            // When this message is received, dump all configuration
            // parameters that is not given in the device resource 
            // string to the system log.
            //
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(": Communication speed="));
            wcscat(Message, DeviceConf.CommSpeed);
			   wcscat(Message, TEXT(" baud"));
            WriteSystemLog(Message, 1);
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(": Device supports sample rates up to 200hz="));
            if (DeviceConf.HighSmplRate==TRUE) wcscat(Message, TEXT("TRUE"));
            if (DeviceConf.HighSmplRate==FALSE) wcscat(Message, TEXT("FALSE"));
            WriteSystemLog(Message, 1);
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(": Data resolution is "));
            if (DeviceConf.DataBits==16) wcscat(Message, TEXT("16"));
            if (DeviceConf.DataBits==18) wcscat(Message, TEXT("18"));
            if (DeviceConf.DataBits==24) wcscat(Message, TEXT("24"));
            wcscat(Message, TEXT(" bits pr sample."));
            WriteSystemLog(Message, 1);
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(": Device sends both date and time="));
            if (DeviceConf.DeviceHasDT==TRUE) wcscat(Message, TEXT("TRUE"));
            if (DeviceConf.DeviceHasDT==FALSE) wcscat(Message, TEXT("FALSE"));
            WriteSystemLog(Message, 1);
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(": Use system clock for timetagging="));
            if (DeviceConf.UsePCClock==TRUE) wcscat(Message, TEXT("TRUE"));
            if (DeviceConf.UsePCClock==FALSE) wcscat(Message, TEXT("FALSE"));
            WriteSystemLog(Message, 1);
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(": Synchronize system clock="));
            if (DeviceConf.SyncPCClock==TRUE) wcscat(Message, TEXT("TRUE"));
            if (DeviceConf.SyncPCClock==FALSE) wcscat(Message, TEXT("FALSE"));
            WriteSystemLog(Message, 1);
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(": Time Correction="));
            swprintf(str10,TEXT("%.3f"), DeviceConf.TimeCorrection);
            wcscat(Message, str10);
            WriteSystemLog(Message, 1);
         break;


         case TM_GET_INTERFACE:
         //
         // This message is used to get a pointer to the interface structure.
         //
         OD.vPointer=(PVOID)&DS;
         break;   


         case TM_GET_UNIQUE_DEVICE_NAME:
         //
         // This message is used to get a string, 
         // identifying this unique device instance.
         //
         wcscpy(OD.cStatus, UniqueDeviceName);
         break;   


         case TM_GET_RESOURCE_STRING:
         //
         // This message is used to get a string, 
         // identifying the system resources used by this device.
         //
         wcscpy(OD.cStatus, DeviceResources);
         break;   


         case TM_SUBSCRIBE:
         //
         // This message is sendt from a window or thread
         // Who wants to be informed about an event.
         // This message must be accompanied by a MSG_PARAM structure.
         // The MSG_PARAM structure must have a pointer to a valid
         // SUBSCRIBE structure.
         //
         //
         // Add window/thread to notify list.
         if (ID.vPointer) if (!NotifyList.Subscribe((SUBSCRIBE*)ID.vPointer)) {
            OD.bStatus=FALSE;
            OD.wStatus=3;
         }
         break;   


         case TM_UNSUBSCRIBE:
         //
         // This message is sendt from a window or thread who
         // wants to remove a notification from the notify list.
         // This message must be accompanied by a MSG_PARAM structure.
         // The MSG_PARAM structure must have a pointer to a valid
         // SUBSCRIBE structure.
         //
         //
         // Remove notification from list.
         if (ID.vPointer) NotifyList.UnSubscribe((SUBSCRIBE*)ID.vPointer);
         
         break;   



         case TM_EXIT:                 
         //
         // This is a request for the thread to end.
         // Close down all processes that are depandent on this thread.
         // The 'StopDependantThreads' function will return immediatly,
         // and will send a 'TM_DEPTHREADS_STOPPED' message back when
         // all dependant threads have been stopped.
         StopDependantThreads();
         break;



         case TM_DEPTHREADS_STOPPED:
         //
         // This message indicates that all dependent threads have been stopped.
         // The thread will cleanup and exit when this message is recieved.         
         //

         // First stop the device-reader thread.
         if (hReaderThread) {
            SerialRead=FALSE;                                            // Signal the reader thread.
            status=WaitForSingleObject(hReaderThread, 2000);             // Wait for the thread to stop.
            if (status==WAIT_TIMEOUT) TerminateThread(hReaderThread, 0); // If thread wont stop, kill it.
            hReaderThread=NULL;
         }

         if (!SI.ConfigOnly) {
            // Unregister this process from the process database.
            UnregisterProcess(CurrentThreadID);

            // Close the communication port.
            CloseHandle(hComm);

			//Handle time synchronization stuff.
			if (DeviceConf.SyncPCClock)  {
			  // Clear the SysClkSyncThreadID variable.
              SysClkSyncThreadID=0;
			  // Clear the status bar.
			  PostMessage(hWndFrame, TM_WRITE_STATUSBAR, 0, (LPARAM)"");
            }
         }

         // Free memory buffers.
         for (Index=0;Index<DeviceConf.NumChannels;Index++) {
            if (DS.ChannelData[Index]) LocalFree((HLOCAL)DS.ChannelData[Index]); 
         }

         // End the message loop.
         ClearMessageQueue(); // Delete all messages in message queue.
         PostQuitMessage(0);  // Post WM_QUIT message.   
         break;
   
         
         //
         // Handle messages from device reader thread.
         //
         case TM_BUFFERUPDATED:  // A new block of data has been inserted into buffer.
         // Send Notification to all subscribed threads/windows.
         // The notofication is accompanied with this thread's thread-ID, and the blocknr of the updated block.
         NotifyList.Notify((UINT)TM_BUFFERUPDATED, (WPARAM)CurrentThreadID, (LPARAM)BlockNr);
         break;


         case TM_TIMESYNC_STATUS:
         //
         // This message is sendt from a time syncronization
         // thread (other then this thread or its child) to inform
         // about a change in the syncronization status. This thread
         // subscribe to this message from the other thread.
         WaitForSingleObject(StartInfo.hSyncStatusMutex, INFINITE); 
         StartInfo.SyncStatus=(BYTE)msg.wParam;
         ReleaseMutex(StartInfo.hSyncStatusMutex);
         break;

         
         case TM_GET_TIMESYNC_STATUS:
         //
         // This message is sendt from a window or thread who
         // wants to know the status of the GPS time synchronization.
         // This message must be accompanied by a MSG_PARAM structure.
         //
         //
         // Returns the operational status of the thread.
         // Values are:
         //    0 :System clock is beeing synchronized, no error.
         //    1 :System clock is NOT beeing synchronized, but no error (fix lost).
         //    2 :System clock is NOT beeing synchronized, communication error.
         //
         OD.wStatus=(WORD)SyncStatus;
         break;

         
         case TM_DEVICEREADER_DATALOST:
         //
         // The device reader reports that one, or several
         // seconds of data has, or may have been, lost.
         //
         // Set the sync status.
         SyncStatus=2;

         // Send Notification to all subscribed threads/windows.
         NotifyList.Notify((UINT)TM_DEVICEREADER_DATALOST, WPARAM(CurrentThreadID), 0);

         // Write a message to the log.
         wcscpy(Message, UniqueDeviceName);
         if (msg.lParam==0) wcscat(Message, TEXT(": Data have been lost!"));
         if (msg.lParam==1) wcscat(Message, TEXT(": Data may have been lost!"));
         WriteSystemLog(Message, 2);
         break;


         case TM_DEVICEREADER_NODATAINPUT:   
         //
         // The device reader has lost data input.
         //
         // Send Notification to all subscribed threads/windows.
         NotifyList.Notify((UINT)TM_DEVICEREADER_NODATAINPUT, WPARAM(CurrentThreadID), 0);

         // Write a message to the log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Not receiving data."));
         WriteSystemLog(Message, 2);
         break;


         case TM_DEVICEREADER_DATAINPUTOK:   
         //
         // The device reader has regained data input.
         //
         // Set the sync status.
         SyncStatus=1;
            
         // Send Notification to all subscribed threads/windows.
         NotifyList.Notify((UINT)TM_DEVICEREADER_DATAINPUTOK, WPARAM(CurrentThreadID), 0);

         // Write a message to the log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Receiving data."));
         WriteSystemLog(Message, 1);
         break;

         case TM_DEVICEREADER_NOFIX:   
         //
         // The reader-thread indicates that sync has been lost.
         //
         FixStatus=0;
         SyncStatus=1;
            
         // Send Notification to all subscribed threads/windows.
         NotifyList.Notify((UINT)TM_TIMESYNC_STATUS, (WPARAM)SyncStatus, 0);
         
         // Update status bar text.
         if (DeviceConf.SyncPCClock) DrawStatusBarText(SyncStatus, UniqueDeviceName);

         // Write a message to the log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Satellite fix lost."));
         WriteSystemLog(Message, 2);
         break;

         case TM_DEVICEREADER_FIXOK:   
         //
         // The reader-thread indicates that sync has been regained.
         //
         FixStatus=1;
         SyncStatus=0;

         // Send Notification to all subscribed threads/windows.
         NotifyList.Notify((UINT)TM_TIMESYNC_STATUS, (WPARAM)SyncStatus, 0);

         // Update status bar text.
         if (DeviceConf.SyncPCClock) DrawStatusBarText(SyncStatus, UniqueDeviceName);        
         
         // Write a message to the log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Satellite fix OK."));
         WriteSystemLog(Message, 1);
         break;


      }
      
      // If this is requested by the thread who sent the message
      // then return status values, and release the caller thread.
      if (((MSG_PARAM*)msg.lParam)!=NULL && msg.wParam!=NULL) {
         *(MSG_PARAM*)msg.lParam=OD;
         SetEvent((HANDLE)msg.wParam);
      }
   } 


   // Free up memeory allocated in the MsgList class object.
   NotifyList.~MsgList();

   // If running in debug mode, write a message to the log.
   if (DebugLevel) {
      wcscpy(Message, UniqueDeviceName);
      wcscat(Message, TEXT(": Thread stopped."));
      WriteSystemLog(Message, 1);
   }

   // Exit this thread
   ExitThread(0);
   return 0L;
   
}




/* ******************************************************************** */
/*  Serial port reader procedure for the SADC.                          */
/* ******************************************************************** */
DWORD WINAPI ReaderThreadSADC(PVOID StartupStruct)
{
   INT Speed;
   VOID *DataBlocks; 
   BYTE *bPtr, TimeQuality, TimeSource;
   BYTE DataBlock[3210], ReadStatus, ChNr;
   WORD BlockSize, SampleNr;
   DWORD BlockNr, OffsetCurrentSample, OffsetToBlockData;
   LONG ChSamples[210];
   DOUBLE TransferTime, SecsSinceLastSync;
   DEVICE_CONFIG DC;
   STARTINFO_DEVREADER *SI;
   BLOCK_HEADER DBH;
   SYSTEMTIME DeviceTime, ReadTime, BlockTime, TimeLastSync;
   COMMTIMEOUTS CT={30, 0, 1200, 0, 0};
   BOOL DataLost, DataInputLost, FirstRead;
   BOOL DeviceSync, OldDeviceSync, InitialSync, Sync;
   
   // Copy values from startup structure.
   SI=(STARTINFO_DEVREADER*)StartupStruct;         // Use pointer of correct type.
   DC=*SI->DC;                                     // Copy device configuration.
   
   // Initialize some variables.
   BlockNr=0;           // Start filling data into first block.
   TimeSource=0;        // 0=System clock, 1=device time.
   DataLost=FALSE;      // No data has been lost.
   DataInputLost=TRUE;  // At this point we have no data input.
   InitialSync=FALSE;   // At this point we assume device has never been timesynced.
   DeviceSync=FALSE;    // At this point we assume no timesync.
   OldDeviceSync=FALSE;

	// Calculate size of incoming block.
	BlockSize=6+4*DC.SampleRate*DC.NumChannels;
   if (DC.DataBits==24) BlockSize=6+5*DC.SampleRate*DC.NumChannels;
   if (DC.DeviceHasDT) BlockSize=BlockSize+3;
    
	// Calculate transfertime for one second of data. Also find tranfer speed measuerd in chrs/sec.
   if (!wcscmp(SI->CommSpeed, TEXT("14400"))) { TransferTime=(DOUBLE)(BlockSize)/1440.0; Speed=1440; }
   if (!wcscmp(SI->CommSpeed, TEXT("38400"))) { TransferTime=(DOUBLE)(BlockSize)/3840.0; Speed=3840; }

   //Initialize the communications parameters.
   SetCommTimeouts(SI->hComm, &CT);     // Set communication timeouts..

   // Initialize/setup the SADC card. This function programs the samplerates.
   SADCSetup(SI, &DC);

   // Clear the input buffer.
   PurgeComm(SI->hComm, PURGE_RXCLEAR);
   
   //
   // Start reading the serial port.
   //
   FirstRead=TRUE;
   while(*SI->SerialRead) {
	   //
      // Read one second of data from device.
	   //
      // Return codes are: 0=We are in serial-sync, block returned.
      //                   1=We were out of sync (data probably lost), block returned.
      //                   2=The read timed out, no block returned.

      ReadStatus=SADCReadBlock(SI->hComm, DataBlock, BlockSize);
      if (FirstRead && ReadStatus==1) { FirstRead=FALSE; ReadStatus=0; }

      // Save the time when block was read.
      GetSystemTime(&ReadTime);

      // Handle read result.
      if (ReadStatus==0 || ReadStatus==1) {
         // We have successfully read a block(second) of data.
         // Flag a datalost situation if we have last data before this block.
         if (ReadStatus==1) DataLost=TRUE;

         // Get the time and DCF77 sync-status from block.
         if (DC.DeviceHasDT) SADCGetBlockTime(DataBlock, &DeviceTime, &Sync);

         // Check if device has been initially synced.
         if (DC.DeviceHasDT && !InitialSync) {
            if (DeviceTime.wMonth!=0) InitialSync=TRUE;
         }

         // Establish the currrent time-sync status.
         if (Sync) TimeLastSync=ReadTime;
         if (DC.DeviceHasDT && InitialSync) {
            if (!DeviceSync) {
               // No current sync, check for sync.
               if (Sync) DeviceSync=TRUE;
            } else {
               // We have sync, check if sync is lost.
               SecsSinceLastSync=TimeDiff(&TimeLastSync, &ReadTime);
               if (!Sync && SecsSinceLastSync>=600.0) DeviceSync=FALSE;
            }
         } else {
            DeviceSync=FALSE;
         }

         // Establish time source for timing of current block.
         if (DC.UsePCClock) {
            // Timetaging is to be done from the PC clock.
            TimeSource=0;
         } else {
            // Timetaging is to be done from the device.
            TimeSource=1;
            // If the device has not yet had an initial time-sync,
            // then we timetag from system clock instead.
            if (DeviceTime.wMonth==0) TimeSource=0;
         }

         // Do timetagging from PC or from device.
         if (TimeSource==0) {
            // Timetag from system clock. 
            BlockTime=ReadTime;
            SubstractFromTime(&BlockTime, 1.0+TransferTime);
            // Apply user specified time correction.
            if (DC.TimeCorrection>0.0) AddToTime(&BlockTime, DC.TimeCorrection);
            if (DC.TimeCorrection<0.0) SubstractFromTime(&BlockTime, -DC.TimeCorrection);
         } else {
            // Timetag from the device.
            BlockTime=DeviceTime;
            // Apply user-specified time correction.
            if (DC.TimeCorrection>0.0) AddToTime(&BlockTime, DC.TimeCorrection);
            if (DC.TimeCorrection<0.0) SubstractFromTime(&BlockTime, -DC.TimeCorrection);
         }

         // Sync PC clock if requested.
         if (DC.DeviceHasDT && DC.SyncPCClock && DeviceSync) {
            AddToTime(&DeviceTime, 1.0+TransferTime);
            SetSystemTime(&DeviceTime);
         }

         // Find the timequality factor for this block.
         if (TimeSource==0) {
            // The time is taken from the system clock.
            if (SysClkSyncThreadID) {
               // The system clock is synchronized. 
               if (DC.SyncPCClock) {
                  // This thread is synchronizing the system clock.
                  // The system clock is NOT synchronized by any other thread.
                  TimeQuality=1;                 // We have no satellite fix.
                  if (DeviceSync) TimeQuality=2; // We have satellite fix.
               } else {
                  // Another thread or is synchronizing the system clock.
                  // We will have to get the syncstatus from the synchronizing thread.
                  WaitForSingleObject(SI->hSyncStatusMutex, INFINITE); 
                  if (SI->SyncStatus==0) TimeQuality=2;
                  if (SI->SyncStatus==1) TimeQuality=1;
                  if (SI->SyncStatus==2) TimeQuality=3;
                  ReleaseMutex(SI->hSyncStatusMutex);
               }
            } else {
               // The system clock is NOT synchronized. 
               TimeQuality=0; 
            }
         } else {
            // The time is taken from the device.
            TimeQuality=1;                 // We have no satellite fix.
            if (DeviceSync) TimeQuality=2; // We have satellite fix.
         }

         // Deliver data to parent thread.
         for (ChNr=0;ChNr<DC.NumChannels;ChNr++) {
            //
            // Process the channel pointed to by 'ChNr'.
            //
            // Make pointer to current block in parent-thread's buffer.
            DataBlocks=SI->DS->ChannelData[ChNr];
            bPtr=((BYTE*)DataBlocks)+(SI->DS->BlockSize*BlockNr);

            // Create and copy the block header.
            DBH.BlockID=BlockNr;
            DBH.TimeTag=BlockTime;
            DBH.TimeQuality=TimeQuality; 
            DBH.NullData=0;
            DBH.DataLost=DataLost;
            DBH.EventDetected=FALSE;
            DBH.EventTime=0;
            CopyMemory((VOID*)bPtr, (VOID*)&DBH, (DWORD)sizeof(BLOCK_HEADER));

            // Extract the samples for this channel.
            SADCGetChSamples(DataBlock, ChNr, ChSamples, BlockSize, DC.DataBits, DC.SampleRate);
            
            // Copy samples for this channel.
            OffsetToBlockData=(BlockNr*SI->DS->BlockSize)+sizeof(BLOCK_HEADER);
            OffsetCurrentSample=OffsetToBlockData;
            for (SampleNr=0;SampleNr<DC.SampleRate;SampleNr++) {
               // Process sample pointed to by 'SampleNr'.
               bPtr=((BYTE*)DataBlocks)+OffsetCurrentSample;
               *((LONG*)bPtr)=ChSamples[SampleNr];
               OffsetCurrentSample=OffsetCurrentSample+4;
            }
         }

         // If data stream has just been restored,then inform parent thread.
         if (DataInputLost) {
            DataInputLost=FALSE;
            PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_DATAINPUTOK, 0, 0); 
         }

         // If data has been lost,then inform parent thread.
         if (DataLost) {
            PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_DATALOST, 0, 0);
            DataLost=FALSE;
         }
         
         // Return block nr for updated block.
         *SI->BlockNr=BlockNr;

         // Signal parent process that buffer is updated.
         PostThreadMessage(SI->ThreadID, TM_BUFFERUPDATED, 0, 0); 

         // Advance to the next block.
         BlockNr++; 
         if (BlockNr==SI->DS->NumBlocksPrChannel) BlockNr=0;

         // Handle sync status. If sync status has changed, inform parent process.
         if (DeviceSync!=OldDeviceSync) {
            OldDeviceSync=DeviceSync;
            if (DeviceSync) PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_FIXOK, 0, 0);
            if (!DeviceSync) PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_NOFIX, 0, 0);
         }

      } else {
         // Failed to read a block from device.The digitizer has
         // been disconnected, or another comm error has occured.
         if (!DataInputLost) {
            DataInputLost=TRUE;
            // Signal parent process that we have no data input.
            PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_NODATAINPUT, 0, 0);
            // Assume no more sync.
            InitialSync=DeviceSync=FALSE;
            // Try to initialize/setup the SADC card again.
            SADCSetup(SI, &DC);
         }
         // Wait 100ms before trying a new read.
         if (!ReadStatus) Sleep(100);
      }
   }
   
   ExitThread(0);
   return 0L;
   
}





/*****************************************************************/
/*    Configuration dialog box procedure for SADC                */
/*****************************************************************/
BOOL CALLBACK ConfigDlgProcSADC(HWND hWndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
   HWND hwndDlgItem;
   DWORD dwIndex;
   INT Result;
   static DEVICE_CONFIG *DC;
   TCHAR PortNr, Str20[21], Message[100];
   TCHAR Ports[16][6]={ TEXT("COM1"),TEXT("COM2"), TEXT("COM3"), TEXT("COM4"), TEXT("COM5"), TEXT("COM6"), TEXT("COM7"), TEXT("COM8")
                    , TEXT("COM9"),TEXT("COM10"),TEXT("COM11"),TEXT("COM12"),TEXT("COM13"),TEXT("COM14"),TEXT("COM15"),TEXT("COM16")};
   SHINITDLGINFO shidi; // New in PocketPC version

   switch(message)  {
      case WM_INITDIALOG:
      // Create a Done button and size it. New in PocketPC version.
 	  shidi.dwMask = SHIDIM_FLAGS;
	  shidi.dwFlags = SHIDIF_SIPDOWN | SHIDIF_SIZEDLGFULLSCREEN;
	  shidi.hDlg = hWndDlg;
	  SHInitDialog(&shidi);
   
	  // Create pointer to the device config structure.
      DC=(DEVICE_CONFIG*)lParam;

      // Initialize the 'Number of channels' combo box.
      hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO1);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCSTR)"1");
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 1);
      if (DC->NumChannels==1) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCSTR)"3");
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 3);
      if (DC->NumChannels==3) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCSTR)"4");
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 4);
      if (DC->NumChannels==4) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);

      // Initialize the 'Device is connected to' combo box.
      hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO2);
      for (PortNr=0;PortNr<16;PortNr++) {
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCSTR)Ports[PortNr]);
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, PortNr);
         if (!wcscmp(DC->PortName, Ports[PortNr])) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }

      // Initialize the 'Comm speed' combo box.
      hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO3);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCSTR) TEXT("14400"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 14400);
      if (!wcscmp(DC->CommSpeed, TEXT("14400"))) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCSTR) TEXT("38400"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 38400);
      if (!wcscmp(DC->CommSpeed, TEXT("38400"))) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);

      // Initialize the 'sample rate' combo box.
      hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO4);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCSTR) TEXT("50"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 50);
      if (DC->SampleRate==50) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCSTR) TEXT("100"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 100);
      if (DC->SampleRate==100) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCSTR) TEXT("200"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 200);
      if (DC->SampleRate==200) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);

      // Initialize the 'Device supports sample rates up to 200hz' check box.
      CheckDlgButton(hWndDlg, IDC_CHECK1, BST_UNCHECKED);
      if (DC->HighSmplRate) CheckDlgButton(hWndDlg, IDC_CHECK1, BST_CHECKED);

      // Initialize the 'Device resolution' radio buttons.
      if (DC->DataBits==16) CheckRadioButton(hWndDlg, IDC_RADIO1, IDC_RADIO3,IDC_RADIO1);
      if (DC->DataBits==18) CheckRadioButton(hWndDlg, IDC_RADIO1, IDC_RADIO3,IDC_RADIO2);
      if (DC->DataBits==24) CheckRadioButton(hWndDlg, IDC_RADIO1, IDC_RADIO3,IDC_RADIO3);

      // Initialize the 'Device sends both date and time' check box.
      CheckDlgButton(hWndDlg, IDC_CHECK3, BST_UNCHECKED);
      if (DC->DeviceHasDT) CheckDlgButton(hWndDlg, IDC_CHECK3, BST_CHECKED);

      // Initialize the 'Use system clock for timetagging' check box.
      CheckDlgButton(hWndDlg, IDC_CHECK4, BST_UNCHECKED);
      if (DC->UsePCClock) CheckDlgButton(hWndDlg, IDC_CHECK4, BST_CHECKED);
      if (!DC->DeviceHasDT) EnableWindow(GetDlgItem(hWndDlg, IDC_CHECK4), FALSE);

      // Initialize the 'Synchronize system clock.' check box.
      CheckDlgButton(hWndDlg, IDC_CHECK5, BST_UNCHECKED);
      if (DC->SyncPCClock) CheckDlgButton(hWndDlg, IDC_CHECK5, BST_CHECKED);
      if (!DC->DeviceHasDT) EnableWindow(GetDlgItem(hWndDlg, IDC_CHECK5), FALSE);

      // Initialize the 'time correction' edit box.
      swprintf(Str20, TEXT("%.3f"), DC->TimeCorrection);
      SetDlgItemText(hWndDlg, IDC_TIME_CORR, Str20);

	  #ifdef WINCE
	   SetForegroundWindow(hWndDlg);
      #else
        // Bring window to front.
        BringWindowToTop(hWndDlg);
      #endif

      return TRUE;


      case WM_COMMAND:
      switch(LOWORD(wParam)) {
         case IDC_RADIO1:
         // This is the '16 bits' radiobox.
         switch (HIWORD(wParam)) { 
            case BN_CLICKED:
            CheckRadioButton(hWndDlg, IDC_RADIO1, IDC_RADIO3,IDC_RADIO1);
         }
         break;

         case IDC_RADIO2:
         // This is the '18 bits' radiobox.
         switch (HIWORD(wParam)) { 
            case BN_CLICKED:
            CheckRadioButton(hWndDlg, IDC_RADIO1, IDC_RADIO3,IDC_RADIO2);
         }
         break;

         case IDC_RADIO3:
         // This is the '24 bits' radiobox.
         switch (HIWORD(wParam)) { 
            case BN_CLICKED:
            CheckRadioButton(hWndDlg, IDC_RADIO1, IDC_RADIO3,IDC_RADIO3);
         }
         break;

         case IDC_CHECK3:
         // This is the 'Device sends both date and time' checkbox.
         switch (HIWORD(wParam)) { 
            case BN_CLICKED:
            if (IsDlgButtonChecked(hWndDlg, IDC_CHECK3)==BST_CHECKED) {
               CheckDlgButton(hWndDlg, IDC_CHECK4, BST_UNCHECKED);
               EnableWindow(GetDlgItem(hWndDlg, IDC_CHECK4), TRUE);
               EnableWindow(GetDlgItem(hWndDlg, IDC_CHECK5), TRUE);
            } else {
               CheckDlgButton(hWndDlg, IDC_CHECK4, BST_CHECKED);
               CheckDlgButton(hWndDlg, IDC_CHECK5, BST_UNCHECKED);
               EnableWindow(GetDlgItem(hWndDlg, IDC_CHECK4), FALSE);
               EnableWindow(GetDlgItem(hWndDlg, IDC_CHECK5), FALSE);
            }
         }
         break;

         case IDC_COMBO4:
         // This is the sample rate combobox.
         switch (HIWORD(wParam)) { 
            case LBN_SELCHANGE:
            hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO4);
            dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
            if (SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0)==200) {
               CheckDlgButton(hWndDlg, IDC_CHECK1, BST_CHECKED);
            }
         }
         break;

         case IDOK:  // User has pressed the OK button.
         // Read current choice from the 'Number of channels' combo box.
         hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO1);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
		   DC->NumChannels=(BYTE)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

         // Read current choice from the 'Device is connected to' combo box.
         hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO2);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         SendMessage(hwndDlgItem, CB_GETLBTEXT, dwIndex, (LPARAM)&DC->PortName);

         // Read current choice from the 'Comm speed' combo box.
         hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO3);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         SendMessage(hwndDlgItem, CB_GETLBTEXT, dwIndex, (LPARAM)&DC->CommSpeed);
   
         // Read current choice from the sample rate combo box.
         hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO4);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         DC->SampleRate=(BYTE)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

         // Read current choice from the 'Device supports sample rates up to 200hz' check box.
         DC->HighSmplRate=(IsDlgButtonChecked(hWndDlg, IDC_CHECK1)==BST_CHECKED); 

         // Read current choice from the 'Device resolution' radio boxes.
         if (IsDlgButtonChecked(hWndDlg, IDC_RADIO1)==BST_CHECKED) DC->DataBits=16;
         if (IsDlgButtonChecked(hWndDlg, IDC_RADIO2)==BST_CHECKED) DC->DataBits=18;
         if (IsDlgButtonChecked(hWndDlg, IDC_RADIO3)==BST_CHECKED) DC->DataBits=24;
         
         // Read current choices from the 'Device sends both date and time' check box.
         DC->DeviceHasDT=(IsDlgButtonChecked(hWndDlg, IDC_CHECK3)==BST_CHECKED); 

         // Read current choices from the 'Use system clock for timetagging' check box.
         DC->UsePCClock=(IsDlgButtonChecked(hWndDlg, IDC_CHECK4)==BST_CHECKED); 

         // Read current choices from the 'Synchronize system clock' check box.
         DC->SyncPCClock=(IsDlgButtonChecked(hWndDlg, IDC_CHECK5)==BST_CHECKED); 

         // Read current choices from the time-correction edit box.
         GetDlgItemText(hWndDlg, IDC_TIME_CORR, Str20, 20);
         Result=swscanf(Str20, TEXT("%Lf"), &DC->TimeCorrection);
         if (Result==0 || Result==EOF) {
            // A string that cannot be converted to a float value is entered.
            wcscpy(Message, TEXT("The time correction value is missing or invalid.\n"));
            MessageBox(hWndDlg, Message, TEXT("Seislog message"), MB_OK|MB_ICONERROR);

            // Move the input focus to this field.
            SetFocus(GetDlgItem(hWndDlg, IDC_TIME_CORR));
            SendDlgItemMessage(hWndDlg, IDC_TIME_CORR, EM_SETSEL, 0, -1); 
  
            // Return without closing.
            return TRUE;            
         }

         // Check that max 3 channels are chosen with 24 bit data resolution.
         if (DC->DataBits==24 && DC->NumChannels==4) {
            wcscpy(Message, TEXT("24-bit devices do not support 4 channels!\n"));
            MessageBox(hWndDlg, Message, TEXT("Seislog message"), MB_OK|MB_ICONERROR);

            // Move the input focus to this field.
            SetFocus(GetDlgItem(hWndDlg, IDC_COMBO1));
  
            // Return without closing.
            return TRUE;            
         }

         // Close the dialog box
         EndDialog(hWndDlg , 1);
         return TRUE;
         
         
         case IDCANCEL:  // User has pressed the Cancel button.
         // Do nothing, just exit.
         EndDialog(hWndDlg , 0);
         return TRUE;      
      }
      return FALSE;

   }
   return FALSE;
}




/*******************************************************************/
/* Returns all samples (for one channel) from a data block.        */
/*******************************************************************/
VOID SADCGetChSamples(PBYTE DataBlock, BYTE ChNr, PLONG ChSamples,
                        WORD BlockSize, BYTE DataBits, WORD SmplRate)
{
   WORD Index;
   LONG lValue;
   ULONG ulValue;
   BYTE IdByte, NumSamples, *BlockPtr; 
   BYTE lbyte,mbyte,hbyte,xbyte;

   // Set correct channel.
   IdByte=0x82;
   if (ChNr==1) IdByte=0x83;
   if (ChNr==2) IdByte=0x84;
   if (ChNr==3) IdByte=0x85;

   // Search trough the block and extract all
   // samples for this channel. Samples will
   // be returned in the ChSamples array.
   NumSamples=0;
   BlockPtr=DataBlock;
   for (Index=0;Index<BlockSize;Index++) {
      if (DataBlock[Index]==IdByte) {
         // Extract one sample
         if (DataBits==24) {
            // 24 bit data from device.
            lbyte=DataBlock[Index+1];
            mbyte=DataBlock[Index+2];
            hbyte=DataBlock[Index+3];
            xbyte=DataBlock[Index+4];
            if ((xbyte&1)==1) lbyte=lbyte+128;
            if ((xbyte&2)==2) mbyte=mbyte+128;
            ulValue=hbyte*65536+mbyte*256+lbyte;
            lValue=ulValue;
            if ((xbyte&4)==4) lValue=ulValue-8388608;
         } 
         if (DataBits==18) {
            // 18 bit data from device.
            lbyte=DataBlock[Index+1];
            mbyte=DataBlock[Index+2]; 
            hbyte=0;
            xbyte=DataBlock[Index+3];
            if ((xbyte&1)==1) lbyte=lbyte+128;
            if ((xbyte&2)==2) mbyte=mbyte+128;
            if ((xbyte&4)==4) hbyte=hbyte+1;
            ulValue=hbyte*65536+mbyte*256+lbyte;
            lValue=ulValue;
            if ((xbyte&8)==8) lValue=ulValue-131072;
         } 
         if (DataBits==16) {
            // 16 bit data from device.
            lbyte=DataBlock[Index+1];
            hbyte=DataBlock[Index+2]; 
            xbyte=DataBlock[Index+3];
            if ((xbyte&1)==1) lbyte=lbyte+128;
            ulValue=hbyte*256+lbyte;
            lValue=ulValue;
            if ((xbyte&2)==2) lValue=ulValue-32768;
         }
         ChSamples[NumSamples]=lValue;
         NumSamples++;
      }
   }

   // Check if we have found a 'SmplRate' number of samples. If not, then
   // replace the missing sample(s) with a value equal to the last sample.
   // This will only be done if there is an error in the block casusing
   // the packet's id-byte to change.
   if (NumSamples<SmplRate) {
      for (Index=NumSamples;Index<SmplRate;Index++) ChSamples[Index]=lValue;
   }

}





/*******************************************************************/
/* Read and return time from device datablock.                     */
/*******************************************************************/
VOID SADCGetBlockTime(PBYTE DataBlock,  SYSTEMTIME* BlockTime, BOOL* Sync)
{
   BYTE Extra;

   // First get correct date.
   BlockTime->wYear=2000+(WORD)DataBlock[1];
   BlockTime->wMonth=(WORD)DataBlock[2];
   BlockTime->wDay=(WORD)DataBlock[3];

   // Then get time from block.
   BlockTime->wDayOfWeek=0;
   BlockTime->wMilliseconds=0;
   BlockTime->wSecond=(WORD)DataBlock[4];
   BlockTime->wMinute=(WORD)DataBlock[5];
   BlockTime->wHour=(WORD)DataBlock[6];

   // Get DCF77 sync status.
   Extra=DataBlock[7];
   if (Extra&32) { *Sync=TRUE; } else { *Sync=FALSE; }
   
}





/*******************************************************************/
/* Read and return one data block from the device.                 */
/* Block is guarantied to start with a 0x81 byte.                  */
/* Return codes are: 0=OK                                          */  
/*                   1=Out of sync (data probably lost).           */
/*                   2=The read timed out, to few bytes read.      */
/*******************************************************************/
BYTE SADCReadBlock(HANDLE hComm, PBYTE DataBlock, DWORD BlockSize)
{
   BYTE pByte, RetValue;
   BOOL ReadStatus;
   DWORD BytesRead;

   // Initialize the return value.
   RetValue=0;

   // First read one byte to ensure that we are in sync.
   ReadStatus=ReadFile(hComm, (PVOID)&pByte, 1, &BytesRead, NULL);
   if (BytesRead!=1) return 2;

   // If we are out of sync, search for start of second.
   // We will read one byte repeateadly until the 0x81
   // byte is found.
   if (pByte!=0x81) {
      // Get in sync with device.
      do {
         ReadStatus=ReadFile(hComm, (PVOID)&pByte, 1, &BytesRead, NULL);
         if (BytesRead!=1) return 2;
      } while (pByte!=0x81);

      // Returnvalue must indicate that data has been lost.
      RetValue=1;
   }

   // Read and return the rest of the second.
   DataBlock[0]=0x81;
   ReadStatus=ReadFile(hComm, (PVOID)(DataBlock+1), BlockSize-1, &BytesRead, NULL);
   if (BytesRead!=BlockSize-1) return 2;

   return RetValue;
}





/********************************************************************/
/* Stop datatransmission from SADC card.                            */
/********************************************************************/
BOOL SADCStop(HANDLE hComm)
{
   BYTE CmdStr[6];
   BOOL WriteStatus;
   DWORD BytesWritten;
 	COMMTIMEOUTS CurrentCt, Ct={0, 0, 0, 0, 200};

   // Get and save current communication timeouts.
	GetCommTimeouts(hComm, &CurrentCt);	
   
   // Set new communication timeouts.
	SetCommTimeouts(hComm, &Ct);

   // Stop datatransmission.
   ZeroMemory((PVOID)CmdStr, 6); CmdStr[0]=0x84;
   WriteStatus=WriteFile(hComm, CmdStr, 6, &BytesWritten, NULL);
   
   // Restore old communication timeouts.
	SetCommTimeouts(hComm, &CurrentCt);

   return TRUE;
}





/********************************************************************/
/* Get the SADC firmware revision.                                  */
/********************************************************************/
BOOL GetSADCFirmwareRevision(HANDLE hComm, PWORD Version)
{
   BYTE CmdStr[6], Index, TryCount;
   DWORD BytesRead, BytesWritten;
   TCHAR str3[4], *pTCHAR, FirmwareRev[30];
   BOOL WriteStatus,ReadStatus, FwFound;
 	COMMTIMEOUTS CurrentCt, Ct={0, 0, 200, 0, 200};

   // Get and save current communication timeouts.
	GetCommTimeouts(hComm, &CurrentCt);	
   
   // Set new communication timeouts.
	SetCommTimeouts(hComm, &Ct);

   // Create the 'get firmware' command string.
   ZeroMemory((PVOID)CmdStr, 6); CmdStr[0]=0x81;

   // Try up to 5 times to get the firmware version.
   TryCount=0; FwFound=FALSE;
   while (!FwFound && TryCount<6) {
      PurgeComm(hComm, PURGE_RXCLEAR);
      WriteStatus=WriteFile(hComm, CmdStr, 6, &BytesWritten, NULL);
      ReadStatus=ReadFile(hComm, FirmwareRev, 30 , &BytesRead, NULL);
      if (BytesRead) {
         pTCHAR=NULL;
         for (Index=0;Index<BytesRead;Index++) {
            if (FirmwareRev[Index]=='V') pTCHAR=FirmwareRev+Index;
         }
         if (pTCHAR!=NULL) { 
            wcsncpy(str3, pTCHAR+1, 3); str3[3]=0;
            *Version=_wtoi(str3); 
            FwFound=TRUE; 
         }
      }
      TryCount++;
   }

   // Restore communication timeouts.
	SetCommTimeouts(hComm, &CurrentCt);

   if (FwFound) return TRUE;
   return FALSE;
}




/********************************************************************/
/* Set up the SADC channel sample rates.                            */
/********************************************************************/
VOID SADCSetup(STARTINFO_DEVREADER *SI, DEVICE_CONFIG *DC)
{
   BOOL WriteStatus;
   DWORD BytesWritten;
   BYTE CmdStr[6], spsx;
 	COMMTIMEOUTS CurrentCt, Ct={0, 0, 0, 0, 200};

   // Get and save current communication timeouts.
	GetCommTimeouts(SI->hComm, &CurrentCt);	
   
   // Set new communication timeouts.
	SetCommTimeouts(SI->hComm, &Ct);

  	// Set clock compensation.
 	Sleep(50);
   CmdStr[0]=0x85;
   CmdStr[1]=CmdStr[2]=CmdStr[3]=CmdStr[4]=0xFF;
   CmdStr[5]=0;
   WriteStatus=WriteFile(SI->hComm, CmdStr, 6, &BytesWritten, NULL);

   // Set GMT correction.
 	Sleep(50);
   CmdStr[0]=0x82;
   CmdStr[1]=CmdStr[2]=CmdStr[3]=CmdStr[4]=CmdStr[5]=0;
   WriteStatus=WriteFile(SI->hComm, CmdStr, 6, &BytesWritten, NULL);

  	// Set channel SPS rates.
 	Sleep(50);
   spsx=100/DC->SampleRate; 
   if (DC->HighSmplRate) spsx=200/DC->SampleRate;
   CmdStr[0]=0x84; CmdStr[1]=spsx;
   CmdStr[2]=0; if (DC->NumChannels>=2) CmdStr[2]=spsx;
   CmdStr[3]=0; if (DC->NumChannels>=3) CmdStr[3]=spsx;
   CmdStr[4]=0; if (DC->NumChannels==4) CmdStr[4]=spsx;
   CmdStr[5]=0;
   WriteStatus=WriteFile(SI->hComm, CmdStr, 6, &BytesWritten, NULL);

   // Restore communication timeouts.
   SetCommTimeouts(SI->hComm, &CurrentCt);
}
