#include "stdafx.h"
#include "Server.h"
#include <Winsock.h>
#include <aygshell.h>
#include "resource.h"
#include "Seislog.h"
#include "Process.h"
#include "MainMon.h"
#include "Channel.h"
#include "Eventlog.H"

// Global variables for this file
TCHAR       szEventFileDir[MAX_PATH+1];  
TCHAR       szSFileDir[MAX_PATH+1];
HWND	    hWnd = NULL;
SOCKET      g_commSock;
int         g_nPort;
SOCKADDR_IN g_saClient;
HANDLE      hOutFile;

// Global variables from Seislog
extern DWORD  ServerThreadID;
extern BOOL   StartGPSThread;
extern BOOL   GPSThreadRunning;
extern HANDLE hSysClkSyncThread;        
extern DWORD  SysClkSyncThreadID;

// Types and global variables from MainMon

extern BYTE      NumRunningChannels;

struct CHAN_PROP {
   CHANNEL_INFO_EX ChInfo;
   CHANNEL_STATUS ChStat;
   BUFF_PROP BuffProp;
   SYSTEMTIME TimeTag;
   signed long Sample;
   TCHAR RBFile[7];
   BYTE InLEDStatus;
   BYTE InLEDTimerCnt;  
   BYTE DiskLEDStatus;
   BYTE DiskLEDTimerCnt;   
   BYTE TrigLEDStatus;
   BYTE SyncLEDStatus;
   INT VertPos;
   DWORD CHThreadID;
   DWORD RBThreadID;
};

extern CHAN_PROP ChData[MAX_NUM_OF_CHANNELS];


// Forward declarations of functions in other files
VOID StartSystem(VOID); 
VOID StopSystem(BOOL); 

DWORD WINAPI SysClkSyncThrd(PVOID);

void GetStatusInfo()
{
	char szAck[] = ACKNOWLEDGE;
	BYTE nNumChannels;

	//Send number of channels we have info on

	if (IsSystemRunning())
	{
		nNumChannels = NumRunningChannels;	
	}
	else
	{
		nNumChannels = 0;
	}

	
	// Send filesize
	send(g_commSock, (char*)&nNumChannels, sizeof(nNumChannels), 0);

	// Receive acknowledge
	recv(g_commSock, szAck, strlen(szAck), 0);

	if (nNumChannels > 0)
	{
		// Allocate temporary buffer for status info
		StatusInfo* si = new StatusInfo[nNumChannels];

		// Copy status info for all channels

		for (int i = 0; i < nNumChannels; i++)
		{
			// FROM_CHANNEL_INFO_EX

			WideCharToMultiByte(CP_ACP, WC_COMPOSITECHECK,
			  ChData[i].ChInfo.ChannelName, -1, si[i].ChannelName, 10, NULL, NULL);

			WideCharToMultiByte(CP_ACP, WC_COMPOSITECHECK,
			  ChData[i].ChInfo.StationCode, -1, si[i].StationCode, 5+1, NULL, NULL);

			WideCharToMultiByte(CP_ACP, WC_COMPOSITECHECK,
			  ChData[i].ChInfo.Component, -1, si[i].Component, 4+1, NULL, NULL);

			si[i].LogToDisk = ChData[i].ChInfo.LogToDisk;
			si[i].UseTrigging = ChData[i].ChInfo.UseTrigging;

			si[i].STA = ChData[i].ChStat.STA;
			si[i].LTA = ChData[i].ChStat.LTA;

			// FROM_CHAN_PROP

			TCHAR strTimeTag[24];

			CreateDateAndTimeString(strTimeTag, &ChData[i].TimeTag);

			WideCharToMultiByte(CP_ACP, WC_COMPOSITECHECK,
			  strTimeTag, -1, si[i].TimeTag, 24, NULL, NULL);

			si[i].Sample =  ChData[i].Sample;
			si[i].InLEDStatus =  ChData[i].InLEDStatus;
			si[i].DiskLEDStatus =  ChData[i].DiskLEDStatus;
			si[i].TrigLEDStatus =  ChData[i].TrigLEDStatus;
			si[i].SyncLEDStatus =  ChData[i].SyncLEDStatus;
		}

		// Send all the channel info

		
		//WriteSystemLog(TEXT("Trying to send"), 1);
		
		int sent = send(g_commSock, (char*)si, nNumChannels*sizeof(StatusInfo), 0);
		
		//int tried = nNumChannels*sizeof(StatusInfo);
		

		int s = sizeof(StatusInfo);

		/*
		TCHAR msg[205];
		wsprintf(msg, TEXT("Sent: %d - tried to send: %d - sizeofstat: %d"), sent, tried, s);
		WriteSystemLog(msg, 1);
		*/

		delete[] si;

	}
		

}


//===============================================================================
// ReadRegistrySettings - Reads from registry the base directory for the S-files and
//   directory for the waveform files.

BOOL ReadRegistrySettings()
{
	HKEY  hKey;
	DWORD DataSize;
	LONG  RegStatus;
	BOOL  bRegSettingsOK;

	bRegSettingsOK = TRUE;

	// Read registry settings. 
	
	RegStatus=RegOpenKeyEx(HKEY_LOCAL_MACHINE, TEXT("Software\\IFG\\Seislog"),
		                   0, KEY_READ, &hKey);

	if (ERROR_SUCCESS == RegStatus)
	{
		// Get the event file directory
		DataSize=sizeof(szEventFileDir);

		RegStatus=RegQueryValueEx(hKey, TEXT("EventFileDir"),
								  NULL, NULL, (PBYTE)szEventFileDir, &DataSize);

		if (ERROR_SUCCESS != RegStatus)
		{
			bRegSettingsOK = FALSE;
		}

		// Get the S-file directory
		DataSize=sizeof(szSFileDir);

		RegStatus=RegQueryValueEx(hKey, TEXT("SFileDir"),
								  NULL, NULL, (PBYTE)szSFileDir, &DataSize);

		if (ERROR_SUCCESS != RegStatus)
		{
			bRegSettingsOK = FALSE;
		}
	}
	else
	{
		bRegSettingsOK=FALSE;
	}


	return bRegSettingsOK;

}


//===============================================================================
// SearchDir4SFiles - Searches the specified directory for S-files
//


void AddFile2List(TCHAR* szDirectory, TCHAR* szFileName)
{
	DWORD dwBytesWritten;

	char szSFileDirA[10],
		 szFileNameA[32];

	// Remove the leading filename

	TCHAR szSeisanDir[9];

	szSeisanDir[0] = 0; // Empty string

	if (wcslen(szSFileDir) != wcslen(szDirectory))
	{
		int strLen = wcslen(szSFileDir) + 1;
		wcscpy(szSeisanDir, szDirectory+strLen);	

	}


	WideCharToMultiByte(CP_ACP, WC_COMPOSITECHECK, szSeisanDir, -1/*wcslen(szSeisanDir)*/, szSFileDirA, 9, NULL, NULL);
	WideCharToMultiByte(CP_ACP, WC_COMPOSITECHECK, szFileName, -1/* wcslen(szFileName)*/, szFileNameA, 32, NULL, NULL);


	EventInfo ei;

	memset(&ei, 0, sizeof(ei));
	
	strcpy(ei.szSFileDir, szSFileDirA);
	strcpy(ei.szSFileName, szFileNameA);

	// Get name of the Wave-file


	TCHAR szFullFileName[MAX_PATH];

	wcscpy(szFullFileName, szDirectory);
	wcscat(szFullFileName, TEXT("\\"));
	wcscat(szFullFileName, szFileName);


	HANDLE hFile = CreateFile(szFullFileName, GENERIC_READ,
  						      0, NULL, OPEN_EXISTING, 0, 0);

	char pBuffer[80*2];

	//char szWaveFileName[29+1];
	//szWaveFileName[29] = 0;

	DWORD dwNumBytesRead;

	ReadFile(hFile, pBuffer, 80*2, &dwNumBytesRead, 0);

	strncpy(ei.szWavFileName, pBuffer+83, 29);

	ei.szWavFileName[29] = 0;

	WriteFile(hOutFile, &ei, sizeof(ei), &dwBytesWritten, 0);


	CloseHandle(hFile);
}


void SearchDir4SFiles(TCHAR* szDirectory)
{
	WIN32_FIND_DATA fileData;    // Used to store search result 
	HANDLE hFile;			     // Handle to file found
	TCHAR szSearchString[MAX_PATH];

	// Create search string

	wcscpy(szSearchString, szDirectory);
	wcscat(szSearchString, TEXT("\\*L.S*"));

	// Find possibly first occurrence of file

	if ((hFile = FindFirstFile(szSearchString, &fileData)) == INVALID_HANDLE_VALUE)
	{
		return;
	}
	// TODO: Add first file to list
	AddFile2List(szDirectory, fileData.cFileName);
	
	while (FindNextFile(hFile, &fileData) != 0)
	{
		// TODO: Add the other files to list
		AddFile2List(szDirectory, fileData.cFileName);
	}
}

//===============================================================================
// Search4Months - Searches the specified year-directory for and any month-
//   directories for S-files
//

void Search4Months(TCHAR* szSearchPath)
{
	WIN32_FIND_DATA fileData;    // Used to store search result 
	HANDLE hFile;			     // Handle to file found
	TCHAR szSearchString[MAX_PATH];
	TCHAR szPath[MAX_PATH];

	// Create search string

	wcscpy(szSearchString, szSearchPath);
	wcscat(szSearchString, TEXT("\\??"));

	// Process first year subdirectory

	if ((hFile = FindFirstFile(szSearchString, &fileData)) == INVALID_HANDLE_VALUE)
	{
		return;
	}

	// Read next line as: "if file is a directory"
	if (fileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
	{
		wcscpy(szPath, szSearchPath);
		wcscat(szPath, TEXT("\\"));
		wcscat(szPath, fileData.cFileName); // Concat directory name
		SearchDir4SFiles(szPath);
	}

	while (FindNextFile(hFile, &fileData) != 0)
	{
		// Read next line as: "if file is a directory"
		if (fileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
		{
			wcscpy(szPath, szSearchPath);
			wcscat(szPath, TEXT("\\"));
			wcscat(szPath, fileData.cFileName); // Concat directory name
			SearchDir4SFiles(szPath);
		}
	}


}

//===============================================================================
// ReadSFiles - Searches all directories and creates a list of S-files
//



void ReadSFiles()
{
	hOutFile = CreateFile(TEXT("\\filelist.bin"), GENERIC_WRITE,
						  0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);


	WIN32_FIND_DATA fileData;       // Used to store search result 
	HANDLE hFile;                    // Handle to file found
	TCHAR szSearchString[MAX_PATH]; // Search string
	TCHAR szPath[MAX_PATH];

	// First we search the base directory for simulated S-files

	SearchDir4SFiles(szSFileDir);

	// Then we search subdirectories for triggered events

	wcscpy(szSearchString, szSFileDir);
	wcscat(szSearchString, TEXT("\\????"));

	// Process first year subdirectory

	if ((hFile = FindFirstFile(szSearchString, &fileData)) == INVALID_HANDLE_VALUE)
	{
		return;
	}

	// Read next line as: "if file is a directory"
	if (fileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
	{
		wcscpy(szPath, szSFileDir);
		wcscat(szPath, TEXT("\\"));
		wcscat(szPath, fileData.cFileName); // Concat directory name
		Search4Months(szPath);
	}


	while (FindNextFile(hFile, &fileData) != 0)
	{
		// Read next line as: "if file is a directory"
		if (fileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
		{
			wcscpy(szPath, szSFileDir);
			wcscat(szPath, TEXT("\\"));
			wcscat(szPath, fileData.cFileName); // Concat directory name
			Search4Months(szPath);
		}
	}
	CloseHandle(hOutFile);
}

//===============================================================================
// ServerThread - Monitors for connection, connects and notifies window
// when connection occurs
//

ULONG ServerThread(PVOID pArg)
{

	SOCKET       listenSock;
	SOCKADDR_IN  sa;
	int          iError;
	WSADATA      WSAData;

	memset(&listenSock, 0, sizeof(SOCKET));
	memset(&sa, 0, sizeof(sockaddr_in));

	sa.sin_family = AF_INET;
	sa.sin_port   = htons(g_nPort);

	PHOSTENT hostinfo;

	char name[255];


	char *ip;

	// Find the first IP address of the PocketPC

	if( gethostname ( name, sizeof(name)) == 0)
	{
		if((hostinfo = gethostbyname(name)) != NULL)
		{
			ip = inet_ntoa(*(struct in_addr *)hostinfo->h_addr_list[0]);
		}
	}

	sa.sin_addr.S_un.S_addr = inet_addr(ip);

	if (iError = WSAStartup (MAKEWORD(2,0), &WSAData))
	{
		TCHAR szMessage[MAX_MSG_STRING];
		wsprintf(szMessage, TEXT("Startup error #%i."), iError);
		MessageBox(hWnd, szMessage, TEXT("Error"), MB_OK);
		return iError;
	}




	// Create listening socket

	listenSock = socket(AF_INET, SOCK_STREAM, 0);

	// Bind socket
	iError = bind(listenSock, (SOCKADDR *) &sa, sizeof(SOCKADDR_IN));

	if (iError != 0)
	{
		TCHAR szMessage[MAX_MSG_STRING];
		wsprintf(szMessage, TEXT("Bind error #%i."), iError);
		MessageBox(hWnd, szMessage, TEXT("Error"), MB_OK);
		return iError;
	}




	// Listen
	 iError = listen(listenSock, 1);

	if (iError != 0)
	{
		TCHAR szMessage[MAX_MSG_STRING];
		wsprintf(szMessage, TEXT("Listen error #%i."), iError);
		MessageBox(hWnd, szMessage, TEXT("Error"), MB_OK);
		return iError;
	}

	int size = sizeof(SOCKADDR_IN);

	while (true)
	{		
		// Wait
		g_commSock = accept(listenSock, (SOCKADDR *) &g_saClient, &size);

		TCHAR szMessage[100], szPort[6], szAddr[16];

		MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, inet_ntoa(g_saClient.sin_addr), -1,
			szAddr, 16);
		
		wsprintf(szPort, TEXT("%d"), g_saClient.sin_port);

		wcscpy(szMessage, TEXT("Connection from "));
		wcscat(szMessage, szAddr);
		wcscat(szMessage, TEXT(" on port "));
		wcscat(szMessage, szPort);
		wcscat(szMessage, TEXT(" accepted"));
		WriteSystemLog(szMessage, 1);

		ServeClient();

		wcscpy(szMessage, TEXT("Connection from "));
		wcscat(szMessage, szAddr);
		wcscat(szMessage, TEXT(" on port "));
		wcscat(szMessage, szPort);
		wcscat(szMessage, TEXT(" closed"));
		WriteSystemLog(szMessage, 1);

	}

	WSACleanup();

	return 0;
}

//===============================================================================
// ServeClient - 
//

void ServeClient()
{
	// Allocate memory for ascii command
	char buffer[CMD_STRING_LENGTH];

	// Allocate memory for unicode command string
	TCHAR szCmdString[CMD_STRING_LENGTH];


	while (true)
	{
		// Get command string
		recv(g_commSock, buffer, CMD_STRING_LENGTH, 0);

		MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, buffer, CMD_STRING_LENGTH,
			szCmdString, CMD_STRING_LENGTH);

		TCHAR szCommand[CMD_LENGTH+1];
		wcsncpy(szCommand, szCmdString, CMD_LENGTH);
		szCommand[CMD_LENGTH] = 0;

		if (wcscmp(TEXT(CMD_GET_STATUS), szCommand) == 0)
		{
			GetStatusInfo();
		}
		if (wcscmp(TEXT(CMD_START_LOGGING), szCommand) == 0)
		{

			// Start system:

			 if (!IsSystemRunning())
			 {
				// Show the 'wait' cursor.
				SetCursor(LoadCursor(NULL, IDC_WAIT));    

				// Start the GPS system-clock syncronization thread if needed.
				if (StartGPSThread && !GPSThreadRunning)
				{
				   GPSThreadRunning=TRUE;
				   hSysClkSyncThread=CreateThread(NULL, 0, SysClkSyncThrd, NULL, 0, &SysClkSyncThreadID);
				}

				// Start the system.
				StartSystem();
        
				// Play a sound to indicate that system is started.
				MessageBeep(MB_ICONINFORMATION);

				// Show the normal 'arrow' cursor.
				SetCursor(LoadCursor(NULL, IDC_ARROW));   
			 }
			 else
			 {
				// System is already running -- just ignore
				//wcscpy(MessageText, TEXT("The system is already running."));
				//MessageBox(hWnd, MessageText, TEXT("Seislog"), MB_OK|MB_ICONERROR);
			 }
			send(g_commSock, ACKNOWLEDGE, strlen(ACKNOWLEDGE), 0);

		}

		if (wcscmp(TEXT(CMD_STOP_LOGGING), szCommand) == 0)
		{
			
			// Stop system:

			 //
			 if (IsSystemRunning())
			 {
				// Stop system without asking

				// Show the 'wait' cursor.
				SetCursor(LoadCursor(NULL, IDC_WAIT));    

				// Stop the system.
				StopSystem(FALSE);

				// Stop the GPS system-clock syncronization thread if needed.
				if (!StartGPSThread && GPSThreadRunning)
				{
					GPSThreadRunning=FALSE;
					PostThreadMessage(SysClkSyncThreadID, TM_EXIT, 0, 0);
					WaitForSingleObject(hSysClkSyncThread , INFINITE);
					SysClkSyncThreadID=0;
				}

				// Play a sound to indicate that system is stopped.
				MessageBeep(MB_ICONINFORMATION);

				// Show the normal 'arrow' cursor.
				SetCursor(LoadCursor(NULL, IDC_ARROW));   

			 }
			 else
			 {
				 // System is already stopped -- just ignore
				//wcscpy(MessageText, TEXT("The system is not running."));
				//MessageBox(hWnd, MessageText, TEXT("Seislog"), MB_OK|MB_ICONERROR);
			 }
			send(g_commSock, ACKNOWLEDGE, strlen(ACKNOWLEDGE), 0);
		}

		if (wcscmp(TEXT(CMD_QUIT), szCommand) == 0)
		{
			closesocket(g_commSock);
			g_commSock = 0;
			//DestroyWindow(hWnd);
			return;
		}
		
		if (wcscmp(TEXT(CMD_GET_S_FILE), szCommand) == 0)
		{
			// We are about to send a S-file
			TCHAR szFullFileName[MAX_PATH];
			wcscpy(szFullFileName, szSFileDir);
			wcscat(szFullFileName, TEXT("\\"));
			int strLen = wcslen(szFullFileName);
			wcsncpy(szFullFileName+strLen, szCmdString + CMD_LENGTH + 1, MAX_SFILENAME+1);
			SendFile(szFullFileName);
		}

		if (wcscmp(TEXT(CMD_GET_W_FILE), szCommand) == 0)
		{
			// We are about to send a waveform-file
			TCHAR szFullFileName[MAX_PATH];
			wcscpy(szFullFileName, szEventFileDir);
			wcscat(szFullFileName, TEXT("\\"));
			int strLen = wcslen(szFullFileName);
			wcsncpy(szFullFileName+strLen, szCmdString + CMD_LENGTH + 1, 29+1);
			SendFile(szFullFileName);
		}

		if (wcscmp(TEXT(CMD_GET_FILE_LIST), szCommand) == 0)
		{
			// Generate list file
			ReadSFiles();
			SendFile(TEXT("\\filelist.bin"));
		}

		if (wcscmp(TEXT(CMD_DEL_S_FILE), szCommand) == 0)
		{
			// We are about to delete a S-file
			TCHAR szFullFileName[MAX_PATH];
			wcscpy(szFullFileName, szSFileDir);
			wcscat(szFullFileName, TEXT("\\"));
			int strLen = wcslen(szFullFileName);
			wcsncpy(szFullFileName+strLen, szCmdString + CMD_LENGTH + 1, MAX_SFILENAME+1);
			
			DeleteFile(szFullFileName);			

			send(g_commSock, ACKNOWLEDGE, strlen(ACKNOWLEDGE), 0);
		}

		if (wcscmp(TEXT(CMD_DEL_W_FILE), szCommand) == 0)
		{
			// We are about to delete a waveform-file
			TCHAR szFullFileName[MAX_PATH];
			wcscpy(szFullFileName, szEventFileDir);
			wcscat(szFullFileName, TEXT("\\"));
			int strLen = wcslen(szFullFileName);
			wcsncpy(szFullFileName+strLen, szCmdString + CMD_LENGTH + 1, 29+1);
			
			DeleteFile(szFullFileName);			

			send(g_commSock, ACKNOWLEDGE, strlen(ACKNOWLEDGE), 0);
		}


	}
}


void SendFile(TCHAR* szFullFileName)
{
		// Open the file
		HANDLE hFile = CreateFile(szFullFileName, GENERIC_READ,
								  0, NULL, OPEN_EXISTING, 0, 0);

		INT nCnt, nFileSize;
		nFileSize = GetFileSize(hFile, NULL);
		char   szAck[] = ACKNOWLEDGE;
		char*  pBuffer;
		pBuffer = new char[BUFFER_SIZE];


		// Send filesize
		send(g_commSock, (char*)&nFileSize, sizeof(nFileSize), 0);
			
		// Start sending
		while (nFileSize > 0)
		{
			nCnt = min(BUFFER_SIZE, nFileSize);
			DWORD dwBytesRead;
			// Write block to file
			ReadFile(hFile, pBuffer, nCnt, &dwBytesRead, 0);
			// Send block
			send(g_commSock, pBuffer, nCnt, 0);

			// Wait for acknowledge from client
			recv(g_commSock, szAck, strlen(szAck), 0);
			nFileSize -= BUFFER_SIZE;
		}
		delete[] pBuffer;
		CloseHandle(hFile);

}

LRESULT CALLBACK ServeDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	SHINITDLGINFO shidi;

	switch (message)
	{
		case WM_INITDIALOG:
			// Create a Done button and size it.  
			shidi.dwMask = SHIDIM_FLAGS;
			 shidi.dwFlags = SHIDIF_DONEBUTTON | SHIDIF_SIPDOWN | SHIDIF_SIZEDLGFULLSCREEN;
			shidi.hDlg = hDlg;
			SHInitDialog(&shidi);

			// Read in registry settings
			ReadRegistrySettings();

			SetDlgItemText(hDlg, IDC_TCP_PORT, TEXT("80"));
			return TRUE; 

		case WM_COMMAND:

			if (LOWORD(wParam) == IDCANCEL)
			{
				EndDialog(hDlg, LOWORD(wParam));
			}
			if (LOWORD(wParam) == IDOK)
			{
				TCHAR szPort[6];
				
				GetDlgItemText(hDlg, IDC_TCP_PORT, szPort, 6);

				int nPort = _wtoi(szPort);

				if ((nPort <= 0) || (nPort >= 65353))
				{
					// Illegal port number
					MessageBox(hDlg, TEXT("Illegal port number"),  TEXT("Error"), MB_OK);
					return FALSE;
				}
				
				g_nPort = nPort;

				// Create thread
				CreateThread(NULL, 0, ServerThread, 0, 0,  &ServerThreadID);

				EndDialog(hDlg, LOWORD(wParam));
				return TRUE;
			}
			break;
	}
    return FALSE;
}


