// * Spectra.cpp
// **************

//#define STRICT

#include "stdafx.h"
#define CLIENT_HEIGHT_MIN 137
#define GRAPH_AREA_HEIGHT 101
#define GSV 60                 // Vertical space between graph lines.
#define SC_PROPERTIES 0xE000
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "seislog.h"
#include "library.h"
#include "resource.h"
#include "spectra.h"
#include "devices.h"
#include "channel.h"
#include "eventlog.h"
#include "process.h"

#include <Aygshell.h>

#define WINCE
//
// Author: Jos Aasheim Ojeda 2004.
//
//
//
//
extern HINSTANCE hInst;
extern HWND hWndFrame;
extern HWND hWndMDIClient;
extern TCHAR ChannelDir[];
extern DEVICE_INFO Devices[];
extern CHANNEL_INFO Channels[];
extern DWORD ProcKeeperThreadID;
extern BYTE DebugLevel;



//
// This structure is used to provide static
// storage for the window procedure.
//
struct MON_WND_DATA {
   DWORD ChThreadID;          // ThreadID of the channel thread used by this window.
   TCHAR Station[5+1];         // This channel's station code.
   TCHAR Component[4+1];       // This channel's component.
   TCHAR UniqueProcName[65];   // The name of this process.
   VOID *pChBuffer;           // Pointer to channel's buffer.
   WORD ClntHeight;           // Height of client area.
   WORD ClntWidth;            // Width of client area.
   WORD ClientAreaMinWidth;   // The minimum allowed width of the client area of the window.
   POINT MinTrackSize;        // The minimum trackingsize for the window.
   WORD GraphWidthInPixels;   // The width of the graph area (measured in pixels).
   WORD GraphWidthInBlocks;   // The width of the graph area (measured in blocks).
   WORD SamplsPrBlock;        // The nuber of samples to be drawn for each block.
   BYTE NumGraphInPlot;       // The number of graphs in plot window.
   BYTE CurrGraph;            // The graph that are currently beeing drawn.
   WORD BlockToDraw;          // The block position in the graph for the next block.
   LONG MaxVertScale;         // The maximum value of the y-scale.
   DOUBLE VertScaleFactor;    // Samples are multiplied with this value to get the sample height in pixels.
   LONG AutoMaxScale;         // Automatically calculated scale.
   LONG SampleMax;            // The largest sample value found during the initial calculation.
   LONG SampleMin;            // The smallest sample found during the initial calculation.
   HFONT hFont;               // Handle to font used to draw texts and numbers in the window.
   HBITMAP hBitMap;           // Handle to the arrow bitmap used in the window.
   DEVMNGR_INTERF_STRUCT DS;  // Copy of device manager's interface structure. Info about Digitizer. (struct def. in Seislog.h)
   SYSTEMTIME GraphTime;      // Timestamps for first block in each graphline starting from top.
   BOOL ReadyToPaint;         // Painting will not be done until this becomes TRUE;
   BOOL bInitialCalc;         // TRUE while the initial calculations are in progress.
   DOUBLE SampleAverage;      // Calculated sample average.
   WORD SampleCount;          // Used in the initial calculations.
   DOUBLE DeviceOffset;       // Calculated offset of data coming from the device.
   MON_WND_CNF WndConfig;     // Configuration data for the window.
   HDC MemDC;                 // Handle to memory device context.
   WORD MemDCWidth;           // The width of the bitmap in pixels.
   WORD MemDCHeight;          // The height of the bitmap in pixels.
   INT YPosLast;
   float Bits[1024];
   float FFTdata[1024];
   float FFTx[512];
   float FFTy[512];

};



//
// Prototype declarations.
//
void four1(float data[], unsigned long nn, int isign);
LONG CalculateScale22(LONG);
LRESULT CALLBACK SpectraWndProc(HWND, UINT, WPARAM, LPARAM);
BOOL CALLBACK SpectraPropDlgProc(HWND, UINT, WPARAM, LPARAM);
WORD ReduceSamples2(LONG*, LONG*, WORD, BYTE);
WORD GetSizeOfReducedBlck2(WORD, BYTE);
VOID SetWindowTitle2(MON_WND_DATA*, HWND);

extern POINT points[2]; // = {{0,0},{0,0}};

/***************************************************************/
/* The original LineTo function does not exist in PocketPC     */
/***************************************************************/  
extern BOOL LineTo(
  HDC hdc,    // device context handle
  int _x,  // x-coordinate of ending point
  int _y   // y-coordinate of ending point
);/*
{

	points[1].x = _x;
	points[1].y = _y;
	Polyline(hdc, points, 2); 
	points[0].x = points[1].x;
	points[0].y = points[1].y;
	return TRUE;
}*/

/***************************************************************/
/* The original MoveTo function does not exist in PocketPC     */
/***************************************************************/
extern BOOL MoveTo(
  HDC hdc,    // NOT used. (device context handle)
  int _x,  // x-coordinate of ending point
  int _y   // y-coordinate of ending point
);/*
{
	points[0].x = _x;
	points[0].y = _y;
	return TRUE;
}*/

/*************************************************************************/
/* Monitor window thread. Handles one channel.                            */
/**************************************************************************/
DWORD WINAPI SpectraThrd(PVOID StartupInfo)
{
   MSG msg;
   HDC hdc;
   HMENU hSysMenu;
   HBITMAP hbit;
   TCHAR Message[100], UniqueProcName[65], str2[3];
   TCHAR ChDir[MAX_PATH+1], WindowFile[MAX_PATH+1];
   DWORD WaitStatus;
   HANDLE hEvent;
   HGDIOBJ OldBrush;
   HWND hWndMonitor;
   MSG_PARAM MsgParams;
   STARTINFO_MONITOR SI;
   MON_WND_DATA MD;
   BOOL bStatus;


#ifdef WINCE
   // Make sure struct members of the SYSTEMTIME struct are 0
   ZeroMemory((PVOID) &MD.GraphTime, sizeof(SYSTEMTIME));
#endif

   //
   // Copy thread startup-information to a local buffer called SI.
   //
   CopyMemory((PVOID)&SI, StartupInfo, sizeof(STARTINFO_MONITOR));


   //
   // Copy the thread-id of the channel-thread into the MD structure.
   //
   MD.ChThreadID=SI.ChThreadID;

   //
   // Create unique processname for this process.
   //
   wcscpy(UniqueProcName, TEXT("Channel"));
   swprintf(str2, TEXT("%02u"), SI.LogicalChannelNr);
   wcscat(UniqueProcName, str2);
   wcscat(UniqueProcName, TEXT(" Spectra"));
   wcscat(MD.UniqueProcName, UniqueProcName);

   //
   // Copy device-manager's interface structure from
   // the channel-thread to local a buffer called DS.
   // This buffer is located in the MD structure.
   hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
   int res  = PostThreadMessage(SI.ChThreadID, TM_GET_INTERFACE, (WPARAM)hEvent, (LPARAM)&MsgParams);

#ifdef WINCE
   if (0 == res)
   {
	   return 0;
   }
   WaitStatus=WaitForSingleObject(hEvent, 3000);
#else
   WaitStatus=WaitForSingleObject(hEvent, 1000);
#endif
   CopyMemory((PVOID)&MD.DS, MsgParams.vPointer, sizeof(DEVMNGR_INTERF_STRUCT));
   CloseHandle(hEvent);


   //
   // Register this process in the process database.
   // process type 9 is spectra
   RegisterProcess(UniqueProcName, 9, GetCurrentThreadId());


   //
   // This process is dependant on the channel process.
   // Therefore a dependency must be registered for this process.
   //
   RegisterDependency(GetCurrentThreadId(), SI.ChThreadID); //monitor dependent on channel thread


   //
   // Get the get the Station code and
   // Component code for this channel.
   //
   hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
   PostThreadMessage(SI.ChThreadID, TM_GET_STATION, (WPARAM)hEvent, (LPARAM)&MsgParams);
   WaitForSingleObject(hEvent, INFINITE);
   wcscpy(MD.Station, MsgParams.cStatus);
   PostThreadMessage(SI.ChThreadID, TM_GET_COMPONENT, (WPARAM)hEvent, (LPARAM)&MsgParams);
   WaitForSingleObject(hEvent, INFINITE);
   wcscpy(MD.Component, MsgParams.cStatus);
   CloseHandle(hEvent);


   //
   // Get and save pointer to the channel-thread's buffer in the MD structure.
   //
   hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
   PostThreadMessage(SI.ChThreadID, TM_GET_BUFFER, (WPARAM)hEvent, (LPARAM)&MsgParams);
   WaitForSingleObject(hEvent, INFINITE);
   MD.pChBuffer=MsgParams.vPointer;
   CloseHandle(hEvent);


   //
   // Create the font used in the window.
   //
#ifdef WINCE

	LOGFONT lf;
	lf.lfHeight = 12;
	lf.lfWidth = 0; 
	lf.lfEscapement = 0; 
	lf.lfOrientation = 0; 
	lf.lfWeight = FW_DONTCARE; 
	lf.lfItalic = 0; 
	lf.lfUnderline = 0; 
	lf.lfStrikeOut = 0; 
	lf.lfCharSet = DEFAULT_CHARSET ; 
	lf.lfOutPrecision  = OUT_DEFAULT_PRECIS; 
	lf.lfClipPrecision = CLIP_DEFAULT_PRECIS; 
	lf.lfQuality = DRAFT_QUALITY; 
	lf.lfPitchAndFamily = FIXED_PITCH | FF_DONTCARE; 
	//lf.lfFaceName[LF_FACESIZE]  = TEXT("Terminal"); 

	MD.hFont = CreateFontIndirect(&lf);
#else
   MD.hFont=CreateFont(-8, 0, 0, 0, FW_DONTCARE, 0, 0, 0, DEFAULT_CHARSET,
                       OUT_CHARACTER_PRECIS, CLIP_DEFAULT_PRECIS,
                       DRAFT_QUALITY, FIXED_PITCH | FF_DONTCARE, TEXT("Terminal"));
#endif

   //
   // Load A bitmap used in the window.
   //
   MD.hBitMap=LoadBitmap(hInst, MAKEINTRESOURCE(IDB_AXES));


   //
   // Calculate the minimun allowed width of the client area.
   // The window must be long enough to hold at least the graph
   // for one whole block of data, and the texts above the graph.
   //
#ifdef WINCE
   MD.ClientAreaMinWidth=51+1+5+MD.DS.SamplesPrBlock+10;
   if (MD.ClientAreaMinWidth < 392) MD.ClientAreaMinWidth=392;
#else
   MD.ClientAreaMinWidth=51+1+5+MD.DS.SamplesPrBlock+10;
   if (MD.ClientAreaMinWidth < 392) MD.ClientAreaMinWidth=392;
#endif

   //
   // Calculate and save the minimum tracking size of the MDI child window.
   //
#ifdef WINCE

	MD.MinTrackSize.x = 240;
	MD.MinTrackSize.y = 280;

#else
   MD.MinTrackSize.x=+GetSystemMetrics(SM_CYSIZEFRAME)*2;
   MD.MinTrackSize.y=CLIENT_HEIGHT_MIN+GetSystemMetrics(SM_CYCAPTION)+GetSystemMetrics(SM_CXSIZEFRAME)*2;
#endif

   //
   // Initialize some variables in the MD structure.
   //
   MD.SampleAverage=0.0;
   MD.SampleCount=0;
   MD.bInitialCalc=TRUE;
   MD.ReadyToPaint=FALSE;
   MD.WndConfig.AutoScaling=TRUE;
   MD.WndConfig.UserScale=100;
   MD.WndConfig.SmplReduction=FALSE;
	MD.WndConfig.OneSmplPrBlock=FALSE;
   MD.WndConfig.Reduction=50;
   MD.ClntHeight=0;
   MD.ClntWidth=0;


   //
   // Open 'Monitor.wnd' file and read window-placement/properties.
   //
   wcscpy(ChDir, ChannelDir);
   StripBackslashFromPath(ChDir); wcscat(ChDir, TEXT("\\Channel"));
   swprintf(str2, TEXT("%02u"), SI.LogicalChannelNr); wcscat(ChDir, str2);
   wcscpy(WindowFile, ChDir); wcscat(WindowFile, TEXT("\\Monitor.wnd"));
   if (ExistFile(ChDir, TEXT("Monitor.wnd"))) bStatus=ReadFileToMemory(WindowFile, (PVOID)&MD.WndConfig, (DWORD)sizeof(MON_WND_CNF));


   //
   // Calculate number of samples in each block.
   //
	MD.SamplsPrBlock=MD.DS.SampleRate;
   if (MD.WndConfig.SmplReduction) {
      MD.SamplsPrBlock=GetSizeOfReducedBlck2(MD.DS.SampleRate, MD.WndConfig.Reduction);
		if (MD.WndConfig.OneSmplPrBlock) MD.SamplsPrBlock=1;
   }


   //
   // Allocate memory for a sample buffer used when sample reduction is enabled.
   //
   MD.WndConfig.SmplBuff=(long*)LocalAlloc(LPTR, (SIZE_T)(MD.DS.SampleRate*4));
   if (!MD.WndConfig.SmplBuff) {
      MD.WndConfig.SmplReduction=FALSE;

      // Update program log.
      wcscpy(Message, UniqueProcName);
      wcscat(Message, TEXT(":"));
      wcscat(Message, TEXT(" Unable to allocate memory. Sample reduction are disabled."));
      WriteSystemLog(Message, 2);
}


   //
   // Create the MDI child window.
   //
   hWndMonitor=CreateWindow(TEXT("SeislogSpectraClass"),		// Pointer to registered class name.
									 UniqueProcName,						// Pointer to window name.
									 WS_CHILD |WS_VISIBLE | WS_BORDER,	// Window style.
									 0,										// Horizontal position of window.
									 50,									// Vertical position of window.
									 240, 			          	 			// Width of window. 240
									 218,            			     		// Height of window. 240
									 SI.hParent,							// Handle to parent window
									 NULL,									// Handle to a Menu
									 hInst,									// Handle to application instance.
									 NULL									// Application-defined value.
									 );



   //
   // Acossiate with this window a pointer to the
   // window-functions static variables.
   //
   SetWindowLong(hWndMonitor, GWL_USERDATA, (LONG)(MON_WND_DATA*)&MD);

   //
   // Set window-placement.
   //
#ifndef WINCE
   if (bStatus) SetWindowPlacement(hWndMonitor, &MD.WndConfig.WndPlacement);
#endif

   // Always turn AutoOpen on when starting the monitor window.
   // AutoOpen will be turned off if user closes the window manually.
   MD.WndConfig.AutoOpen=TRUE;


   //
   // Add 'Properties' menu to the system menu so that
   // user can bring up the 'Monitor Properties' dialog box.
   //
   hSysMenu=GetSystemMenu(hWndMonitor, FALSE);
   AppendMenu(hSysMenu, MF_SEPARATOR, 0, NULL);
   AppendMenu(hSysMenu, MF_STRING, SC_PROPERTIES, TEXT("Properties ..."));


   //
   // Create a compatible memory device context. The graph will
   // be drawn in this context, and then copied to the screen.
   //
   MD.MemDCWidth=GetSystemMetrics(SM_CXSCREEN);  // Get the horizontal screen resolution.
   MD.MemDCHeight=GetSystemMetrics(SM_CYSCREEN); // Get the vertical screen resolution.
   hdc=GetDC(hWndMonitor);
   MD.MemDC=CreateCompatibleDC(hdc);
   hbit=CreateCompatibleBitmap(hdc, MD.MemDCWidth, MD.MemDCHeight);
   SelectObject(MD.MemDC, hbit);
   ReleaseDC(hWndMonitor, hdc);


   //
   // Make the MemDC background white.
   //
   OldBrush=SelectObject(MD.MemDC, GetStockObject(WHITE_BRUSH));
   PatBlt(MD.MemDC, 0, 0, MD.MemDCWidth, MD.MemDCHeight, PATCOPY);
   SelectObject(MD.MemDC, OldBrush);


   //
   // If running in debug mode, write a message to the log.
   //
   if (DebugLevel) {
      wcscpy(Message, UniqueProcName);
      wcscat(Message, TEXT(": Thread started."));
      WriteSystemLog(Message, 1);
   }


   // Start displaying data.
   PostMessage(hWndMonitor, WM_STARTUP, 0 ,0);


   //
   // Enter the thread message loop. The thread will now start to process messages.
   //
   while (GetMessage(&msg, (HWND)NULL, 0, 0)) {

      // If debuglevel is 2 then log all message recieved.
      if (DebugLevel>1) {
         TCHAR str10[11];
         wcscpy(Message, UniqueProcName);
         wcscat(Message, TEXT(": Recieved message "));
         _itow(msg.message, str10, 10);
         wcscat(Message, str10);
         WriteSystemLog(Message, 1);
      }

      TranslateMessage(&msg);
      DispatchMessage(&msg);

      // Handle the TM_EXIT message.
      if (msg.message==TM_EXIT)
		  SendMessage(hWndMonitor, WM_CLOSE, 0, 0);

#ifdef WINCE
	  if (msg.message==TM_SPECTRA_PROPERTIES)
		  SendMessage(hWndMonitor, WM_SPECTRA_PROPERTIES, 0, 0);
#endif

   }


   //
   // Save current window-placement to config file.
   //
   WriteFileFromMemory(WindowFile, (PVOID)&MD.WndConfig, (DWORD)sizeof(MON_WND_CNF));


   //
   // Delete allocated GDI objects.
   //
   DeleteDC(MD.MemDC);
   DeleteObject(MD.hFont);
   DeleteObject(MD.hBitMap);


   //
   // Free allocated memory.
   //
   if (MD.WndConfig.SmplBuff) LocalFree(MD.WndConfig.SmplBuff);


   //
   // Unregister this process from the process database.
   //
   UnregisterProcess(GetCurrentThreadId());


   //
   // If running in debug mode, write a message to the log.
   //
   if (DebugLevel) {
      wcscpy(Message, UniqueProcName);
      wcscat(Message, TEXT(": Thread stopped."));
      WriteSystemLog(Message, 1);
   }


   //
   // Exit from thread.
   //
   ExitThread(0);
   return 0L;

}







/*********************************************************************************/
/* Monitor Window Procedure                                                      */
/*********************************************************************************/
LRESULT CALLBACK SpectraWndProc(HWND hWnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
   HDC            hdc;
   RECT           rect;
   HPEN           hPen, hPenOld, hPenBlack, hPenRed;
   PAINTSTRUCT    ps;
   BYTE           OldNumGraphInPlot;
   TCHAR          TimeStr[13], str10[11], str11[12], str32[33];
   INT            xpos, ypos, BlockStartX;
   WORD           OldClntWidth, OldClntHeight;
   WORD           Index, OldGraphWidthInBlocks;
   MON_WND_DATA   *S;
   HANDLE         hEvent;
   MSG_PARAM      MsgParams;
   SUBSCRIBE      SubscrData;
   BLOCK_HEADER   *pBlockHdr;
   LONG           *pBlockData, *pSample, DO;
   RECT			  textRect;
   int            StartSample, EndSample, Offset;


   // In a MDI child window function, the following messages MUST be
   // handled by DefMDIChildProc, whether the child handles them
   // or not:
   //
   //    WM_CHILDACTIVATE, WM_GETMINMAXINFO, WM_MENUCHAR,
   //    WM_MOVE, WM_SETFOCUS, WM_SIZE, WM_SYSCOMMAND.
   //

   // Get pointer to this function's static variables.
   S=(MON_WND_DATA*)GetWindowLong(hWnd, GWL_USERDATA);
   if (S==NULL) return DefWindowProc(hWnd, Message, wParam, lParam);

   switch (Message) {

	  case WM_CREATE:
			SetForegroundWindow(hWnd);
			break;

      case WM_STARTUP:
      // Get the size of client area.
      GetClientRect(hWnd, &rect);
      S->ClntWidth=+1;    // Width of client area.
      S->ClntHeight=rect.bottom+1;  // Height of client area.

      // Set Window title.
      SetWindowTitle2(S, hWnd);

      // We now need to subcribe to the TM_BUFFERUPDATED messages from the channel.
      SubscrData.uiMessage=TM_BUFFERUPDATED;
      SubscrData.idThread=0;
      SubscrData.hWnd=hWnd;
      MsgParams.vPointer=(PVOID)&SubscrData;
      hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
      PostThreadMessage(S->ChThreadID, TM_SUBSCRIBE, (WPARAM)hEvent, (LPARAM)&MsgParams);
      WaitForSingleObject(hEvent, INFINITE);
      CloseHandle(hEvent);
      return 0;



      case TM_BUFFERUPDATED:
      //
      // A new block of data is ready in the channel's ringbuffer.
      //

      // Get a pointer to the block-header and the block-samples.
      pBlockHdr=(BLOCK_HEADER*)((PBYTE)S->pChBuffer+((DWORD)lParam*S->DS.BlockSize));
      pBlockData=(long*)((byte*)pBlockHdr+sizeof(BLOCK_HEADER));
	  pSample=pBlockData;
     
	  if (S->bInitialCalc)  {
		  StartSample = S->SampleCount; 
		  EndSample = StartSample+S->DS.SamplesPrBlock;
		  //int Reminder = 512 Mod S->DS.SamplesPrBlock;
		  
		  for (Index=StartSample; Index<EndSample; Index++) {
			  S->Bits[Index*2]=(float)*pSample;
			  S->Bits[Index*2+1]=0; //complex part
			  if (S->Bits[Index*2] > S->SampleMax)  {
				  S->SampleMax = S->Bits[Index*2];
			  }
			  // Move on to the next sample.
			  S->SampleCount++; //starts on 0
			  if (S->SampleCount >= 512) break; 
			  pSample++;
		  }
		  if (EndSample>=512)  {
			  Offset = EndSample - 511;
			  for (Index=0; Index<512-Offset; Index++)  {
			    S->Bits[Index*2]=S->Bits[(Index+Offset)*2];
			  }
			  for (Index=512-Offset; Index<512; Index++)  {
                S->Bits[Index*2]=(float)*pSample;
				S->SampleCount++; //starts on 0
			    pSample++;
			  }
		  }


	  }
	  else  { // 
		 // pSample = pSample-511+S->DS.SamplesPrBlock; //skal vere 511
		  S->ReadyToPaint = true;
		  InvalidateRect(hWnd, NULL, TRUE);
	      UpdateWindow(hWnd);  // Draw the client area.
		  // move array element SamplesPrBlock number backwards
		  for (Index=0; Index<512-(S->DS.SamplesPrBlock); Index++)  {
			  S->Bits[Index*2] = S->Bits[(Index+S->DS.SamplesPrBlock)*2];
		  }
          // insert new data from block at end of the array
		  for (Index=512-(S->DS.SamplesPrBlock); Index<512; Index++)  {
			  S->Bits[Index*2] = (float)*pSample;
			  pSample++;
			 /* if (S->Bits[Index*2] > S->SampleMax)  {
				  S->SampleMax = S->Bits[Index*2];
			  }*/
		  }
		  // Save the time for this block.
          S->GraphTime=pBlockHdr->TimeTag;
	  }
	  
      //
      // Calculate the spectra of 512 samples (FFT)
      //
	  if ((S->SampleCount>=512) || (!S->bInitialCalc)) { // 512 real samples starting on 0
		  S->SampleCount = 0;
		  // We are now finished with the initial calculations.
		  S->bInitialCalc=FALSE;
		  S->ReadyToPaint=TRUE;
		  S->SampleMax = 0;
		  // calculate FFT of FFTdata
		  for (Index=0; Index<1024; Index++)  {
		    S->FFTdata[Index] = S->Bits[Index];
		  }
		  four1(S->FFTdata,512,1);
		  for (Index=0;Index<512;Index++) {
			  S->FFTy[Index] = 20*log10(sqrt( pow(S->FFTdata[2*Index],2) + pow(S->FFTdata[2*Index+1],2) )); 
			  S->FFTx[Index] = (Index/1024)*S->DS.SamplesPrBlock;
			  if (S->FFTy[Index] > S->SampleMax)  {
				  S->SampleMax = S->FFTy[Index];
			  }
		  }
		  S->MaxVertScale=S->SampleMax;
		  
		  // The number we have calculated is probably
		  // not very suited to go on the scale, so we
		  // need to find a nice round number close to it.
		  //S->AutoMaxScale=CalculateScale22(S->MaxVertScale);
		  
		  // Calculate the factor wich all samples are
		  // multiplied with to get the hight of the
		  // samples on the screen (in pixels) before max 160 pixel (both ways) .
		  S->VertScaleFactor=160/(FLOAT)S->SampleMax;
		  
	  
	  
	     // ********************************************************************
    	 // Draw the graph in the memory device-context and then copy to screen.
	     // ********************************************************************
	     // Calculate first x-position for this
	     // block in the memory device context.
	     //BlockStartX=S->SamplsPrBlock*S->BlockToDraw;
	     BlockStartX=30;

         // Clear the area where the new block are to be drawn.
         SelectObject(S->MemDC, GetStockObject(WHITE_BRUSH));

         PatBlt(S->MemDC, BlockStartX, 20, 240-BlockStartX, S->ClntHeight-60, PATCOPY);
     

         // Draw the graph for this block.
         hPen=CreatePen(PS_SOLID, 0, RGB(0,128,0)); // Create a green pen.
         hPenOld=(HPEN)SelectObject(S->MemDC, (HGDIOBJ)hPen);
		 
         // Round the deviceoffset to the nearest whole value.
         DO=(LONG)DblRound(S->DeviceOffset);
		 
		 xpos=0;
		 pSample=pBlockData;
		 if (S->WndConfig.SmplReduction) pSample=S->WndConfig.SmplBuff;
		 ypos=160-(int)(DOUBLE) S->FFTy[0]*(S->VertScaleFactor); //280
		// if (ypos>160) ypos=160;
		// if (ypos<30) ypos=30;

		 MoveTo(S->MemDC, xpos, ypos);

		 for (Index=1;Index<170;Index++) {
			 pSample++;
			 xpos++;
			 ypos=160-(INT)DblRound((DOUBLE)(S->FFTy[Index])*S->VertScaleFactor);
			 //if (ypos>160) ypos=160;
		     //if (ypos<30) ypos=30;
			 LineTo(S->MemDC, xpos, ypos);
		 }
		 
         DeleteObject(SelectObject(S->MemDC, hPenOld));

         // Copy the graph from the memoryDC to the window.
         hdc=GetDC(hWnd);
		 BitBlt(hdc, BlockStartX, 20, 240-BlockStartX/*S->ClntWidth*/, S->ClntHeight-60, S->MemDC, 0, 0, SRCCOPY);	

			// Redraw the scale numbers
         swprintf(str10, TEXT("%d"), S->MaxVertScale);
         if (S->MaxVertScale>999999) {
            // Use 'E' notation to save space.
            swprintf(str10, TEXT("%.2lE"), (DOUBLE)S->MaxVertScale);
            StrDelete(str10, wcslen(str10)-2);  // Remove a zero from the exponent.
            StrDelete(str10, wcslen(str10)-2);  // Remove a zero from the exponent.
            StrDelete(str10, wcslen(str10)-2);  // Remove the + sign from the exponent.
         }

         // Draw the block-time above the graph.
         CreateTimeString(&S->GraphTime, TimeStr);
		 SelectObject(hdc, S->hFont);

		 //Draw scale numbers
		 textRect.top = 15;
		 textRect.left = 30;
		 textRect.bottom = 25;
		 textRect.right = 45;
		 DrawText(hdc, str10, wcslen(str10), &textRect, DT_LEFT|DT_TOP);
		 
		/* wcscpy(str11, TEXT("-")); wcscat(str11, str10);
		 
		 textRect.top = 180;
		 textRect.left = 24;
		 textRect.bottom = 190;
		 textRect.right = 200;
		 DrawText(hdc, str11, wcslen(str11), &textRect, DT_LEFT|DT_BOTTOM);*/
		 
		 //Draw block-time
		 textRect.top = 3; 
		 textRect.left = 120; 
		 textRect.bottom = 15; 
		 textRect.right = 240;
		 DrawText(hdc, TimeStr, wcslen(TimeStr), &textRect, DT_LEFT | DT_BOTTOM);

         // Release the DC.
         ReleaseDC(hWnd, hdc);

      } //if samplecount >= 512
	 
      return 0;



      case WM_SIZE:
      // Save the old size of client area.
      OldClntWidth=S->ClntWidth;
      OldClntHeight=S->ClntHeight;
      OldNumGraphInPlot=S->NumGraphInPlot;
      OldGraphWidthInBlocks=S->GraphWidthInBlocks;

      // Save the new size of client area.
      S->ClntWidth=LOWORD(lParam);  // width of client area
      S->ClntHeight=HIWORD(lParam); // height of client area

      // Calculate how many blocks there are room
      // for in the graph after this window resize.
      S->GraphWidthInBlocks=(S->ClntWidth-36)/S->SamplsPrBlock;

      // Recalculate the new graphwidth in pixels.
      S->GraphWidthInPixels=S->GraphWidthInBlocks*S->SamplsPrBlock;

      // Recalculate how many graphs there is room for.
      S->NumGraphInPlot=(S->ClntHeight-40)/GSV;
      if (!S->NumGraphInPlot) S->NumGraphInPlot=1;

      // Handle reduction in number of graphs or blocks.
      if (S->NumGraphInPlot<OldNumGraphInPlot || S->GraphWidthInBlocks<OldGraphWidthInBlocks) {
         // Start from the top again.
         S->CurrGraph=0;
         S->BlockToDraw=0;

         // Clear the MemDC.
         SelectObject(S->MemDC, GetStockObject(WHITE_BRUSH));
         PatBlt(S->MemDC, 0, 0, S->MemDCWidth, S->MemDCHeight, PATCOPY);
      }

      // Redraw the client area.
      InvalidateRect(hWnd, NULL, TRUE);
      break;



      case WM_PAINT:
      //
      // Redraw the complete window.
      //
      hdc=BeginPaint(hWnd, &ps);

      if (S->ReadyToPaint) {
		  // Save the current device context.
		  SaveDC(hdc);
		  
		  // Create the pens needed for the drawing.
		  hPenBlack=CreatePen(PS_SOLID, 0, RGB(0,0,0)); // Create a black pen.
		  hPenRed=CreatePen(PS_SOLID, 0, RGB(128,0,0)); // Create a red pen.
		  
		  // Select the font to be used in the window.
		  SelectObject(hdc, S->hFont);
		  
		  // Draw the Station and Component texts above the graph.
		  
		  wcscpy(str32, TEXT("Stat:"));
		  wcscat(str32, S->Station);
		  
		  textRect.top = 3; 
		  textRect.left = 3;
		  textRect.bottom = 15; 
		  textRect.right = 65;
		  DrawText(hdc, str32, wcslen(str32), &textRect, DT_LEFT|DT_BOTTOM);
		  
		  
		  wcscpy(str32, TEXT("Comp:"));
		  wcscat(str32, S->Component);
		  textRect.left = 65;
		  textRect.right = 110;
		  DrawText(hdc, str32, wcslen(str32), &textRect, DT_LEFT|DT_BOTTOM);
		  
		  // Draw the block-time above the graph.
		  CreateTimeString(&S->GraphTime, TimeStr);
		  SelectObject(hdc, S->hFont);
		  
		  textRect.top = 3; 
		  textRect.left = 120; 
		  textRect.bottom = 15; 
		  textRect.right = 240;
		  DrawText(hdc, TimeStr, wcslen(TimeStr), &textRect, DT_LEFT|DT_BOTTOM);
		  
		  // Draw the horisontal axes.
		  MoveTo(hdc, 30, 190);
		  LineTo(hdc, 210, 190);
		  MoveTo(hdc, 30, 187);
		  LineTo(hdc, 30, 193);
		  MoveTo(hdc, 90, 187);
		  LineTo(hdc, 90, 193);
		  MoveTo(hdc, 150, 187);
		  LineTo(hdc, 150, 193);
		  MoveTo(hdc, 210, 187);
		  LineTo(hdc, 210, 193);
		  
		  textRect.top = 190; 
		  textRect.left = 25;
		  textRect.bottom = 210; 
		  textRect.right = 46; 
		  DrawText(hdc,TEXT("0 Hz"),4,&textRect,DT_LEFT|DT_BOTTOM);
		  
		  textRect.top = 190; 
		  textRect.left = 85;
		  textRect.bottom = 210; 
		  textRect.right = 105; 
		  swprintf(str10, TEXT("%.1f Hz"), (DOUBLE)S->SamplsPrBlock/18);
		  DrawText(hdc,str10,wcslen(str10),&textRect,DT_LEFT|DT_BOTTOM);
		  
		  textRect.top = 190; 
		  textRect.left = 195;
		  textRect.bottom = 210; 
		  textRect.right = 220; 
		  swprintf(str10, TEXT("%.1f Hz"), (DOUBLE)S->SamplsPrBlock/6);
		  DrawText(hdc,str10,wcslen(str10),&textRect,DT_LEFT|DT_BOTTOM);
		  
		  // Draw the vertical line in front of the graph.
		  // Also draw the 3 small scale marks.
		  MoveTo(hdc, 25, 20);
		  LineTo(hdc, 25, 180);
		  MoveTo(hdc, 25, 20);
          LineTo(hdc, 22, 20);
		  MoveTo(hdc, 25, 60);
		  LineTo(hdc, 22, 60);
          MoveTo(hdc, 25, 100);
		  LineTo(hdc, 22, 100);
          MoveTo(hdc, 25, 140);
		  LineTo(hdc, 22, 140);
		  MoveTo(hdc, 25, 180);
		  LineTo(hdc, 23, 180);
		  
		  textRect.top = 15;
		  textRect.left = 2;
		  textRect.bottom = 25;
		  textRect.right = 19;
		  DrawText(hdc, TEXT("0dB"), 3, &textRect, DT_LEFT|DT_TOP);
		  
		  textRect.top = 175;
		  textRect.left = 2;
		  textRect.bottom = 185;
		  textRect.right = 19;
		  
          swprintf(str10, TEXT("%d"), S->SampleMax);
          wcscpy(str11, TEXT("-")); wcscat(str11, str10);
		  DrawText(hdc, str11, wcslen(str11), &textRect, DT_LEFT|DT_TOP);
		  
		  textRect.top = 55;
		  textRect.left = 2;
		  textRect.bottom = 65;
		  textRect.right = 19;
		  swprintf(str10, TEXT("%d"), S->SampleMax/4);
          wcscpy(str11, TEXT("-")); wcscat(str11, str10);
		  DrawText(hdc, str11, wcslen(str11), &textRect, DT_LEFT|DT_BOTTOM);
		  
		  // Draw the graph by copying from the memory DC.
		  BitBlt(hdc, 30, 20, S->ClntWidth-20, S->ClntHeight-50, S->MemDC, 0, 0, SRCCOPY); //16
		  
		  // Restore device context.
		  RestoreDC(hdc, -1);
		  
		  // Delete pen objects.
		  DeleteObject((HGDIOBJ)hPenRed);
		  DeleteObject((HGDIOBJ)hPenBlack);
      } // if bReadyToPaint
      else  {
		  textRect.top = 50; 
		  textRect.left = 30;
		  textRect.bottom = 70; 
		  textRect.right = 200; 
		  DrawText(hdc,TEXT("Building FFT."),12,&textRect,DT_LEFT|DT_BOTTOM);
	  }

      // The redraw has finished.
      EndPaint(hWnd, &ps);
      return 0;

#ifndef	WINCE
      case WM_CHILDACTIVATE:
      // This window is being activated.
      break;
#endif

      case WM_SYSCOMMAND:
      case WM_COMMAND:
      //
      // The WM_SYSCOMMAND message is sendt from the system menu.
      // The WM_COMMAND is sendt from the main menu if MDI child is maximized.
      //
      switch(wParam & 0xFFF0) {
         case SC_CLOSE:
         // The user closes this window manually.
         // The window should NOT be automatically
         // opened the next time the system is started.
         S->WndConfig.AutoOpen=FALSE;
         break;
         case SC_PROPERTIES:
         // User want to set the spectra properties, bring up the 'Spectra Properties' dialog.
         S->WndConfig.hWnd=hWnd;
         DialogBoxParam(hInst, MAKEINTRESOURCE(IDD_SPECTRA_PROPERTIES), hWnd, (DLGPROC)SpectraPropDlgProc, (LPARAM)&S->WndConfig);
         break;
      }
      break;

#ifdef WINCE 
      case WM_SPECTRA_PROPERTIES:
         // User want to set the monitor properties, bring up the 'Monitor Properties' dialog.
         S->WndConfig.hWnd=hWnd;
         DialogBoxParam(hInst, MAKEINTRESOURCE(IDD_SPECTRA_PROPERTIES), hWnd, (DLGPROC)SpectraPropDlgProc, (LPARAM)&S->WndConfig);
         break;
#endif

      case TM_APPLY_CONFIGURATION:
      // Handle configuration changes.
      if (S->WndConfig.AutoScaling) {
         // Auto scaling is enabled.
         S->MaxVertScale=S->AutoMaxScale;
      } else {
         // User has selected a scale factor.
         S->MaxVertScale=S->WndConfig.UserScale;
      }

      S->VertScaleFactor=40.0/(FLOAT)S->MaxVertScale;

      if (S->WndConfig.SmplReduction) {
         // Sample reduction is enabled.
         S->SamplsPrBlock=GetSizeOfReducedBlck2(S->DS.SampleRate, S->WndConfig.Reduction);
			if (S->WndConfig.OneSmplPrBlock) S->SamplsPrBlock=1;
      } else {
         // Sample reduction is disabled.
         S->SamplsPrBlock=S->DS.SampleRate;
      }

      // Set Window title.
      SetWindowTitle2(S, hWnd);

      // Recalculate and redraw the client area.
      S->CurrGraph=0; S->BlockToDraw=0;
#ifdef WINCE
      S->GraphWidthInBlocks=(S->ClntWidth-36)/S->SamplsPrBlock;
#else
		S->GraphWidthInBlocks=(S->ClntWidth-67)/S->SamplsPrBlock;
#endif
      S->GraphWidthInPixels=S->GraphWidthInBlocks*S->SamplsPrBlock;
      SelectObject(S->MemDC, GetStockObject(WHITE_BRUSH));
      PatBlt(S->MemDC, 0, 0, S->MemDCWidth, S->MemDCHeight, PATCOPY);
      InvalidateRect(hWnd, NULL, TRUE);
      break;


      case WM_CLOSE:
      //
      // This message is either sendt from the thread message loop,
      // or it is sendt from the MDIClient window as a response to
      // a press on the X button in the upper right window corner,
      // or the 'Close' option on the system menu.

      // Unsubscribe to the TM_BUFFERUPDATED message.
      SubscrData.uiMessage=TM_BUFFERUPDATED;
      SubscrData.idThread=0;
      SubscrData.hWnd=hWnd;
      MsgParams.vPointer=(PVOID)&SubscrData;
      hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
      PostThreadMessage(S->ChThreadID, TM_UNSUBSCRIBE, (WPARAM)hEvent, (LPARAM)&MsgParams);
      WaitForSingleObject(hEvent, INFINITE);
      CloseHandle(hEvent);

      // Save the current window size/position.
#ifndef WINCE
      S->WndConfig.WndPlacement.length=(UINT)sizeof(WINDOWPLACEMENT);
      GetWindowPlacement(hWnd, &S->WndConfig.WndPlacement);
#endif
      // Destroy the MDI child window
#ifdef WINCE
		SendMessage(hWndMDIClient, WM_DESTROY, (WPARAM)hWnd,0);
#else
      SendMessage(hWndMDIClient, WM_MDIDESTROY, (WPARAM)hWnd, 0);
#endif

      // End the thread loop.
      PostQuitMessage(0);
      return 0;
   }

   // Pass unprocessed messages to the 'DefMDIChildProc' function.
   // NOTE: Some messages must be passed on to 'DefMDIChildProc' even if they are processed.
#ifdef WINCE
   return DefWindowProc(hWnd, Message, wParam, lParam);
#else
	return DefMDIChildProc(hWnd, Message, wParam, lParam);
#endif
}



/*******************************************************************/
/*   Calculates a nice round number for the top of the Y-cale.     */
/*******************************************************************/
LONG CalculateScale22(LONG MaxScale)
{
   INT Mag;
   LONG Step, RetValue;

   // Check input parameter.
   if (MaxScale<=0) return 0;

   // Start by finding the magnitude of the
   // incoming number. Decimals are discarded.
   Mag=max((INT)log10((DOUBLE)MaxScale), 1);
   Step=(LONG)pow(10.0, (DOUBLE)Mag)/5;

   // Going in 'step' steps from zero and upwards, find
   // the first number larger or equal to MaxScale.
   RetValue=0;
   while (RetValue<MaxScale) RetValue=RetValue+Step;

   return RetValue;
}








/*******************************************************************/
/*   Dialog box procedure for the 'Monitor Properties' dialog.     */
/*******************************************************************/
BOOL CALLBACK SpectraPropDlgProc(HWND hWndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{

#ifdef WINCE
   SHINITDLGINFO shidi; 
#endif

   TCHAR str15[16], MessageText[200], Index;
   BYTE Reductions[9]={10,20,25,33,50,66,75,80,90};
   HWND hwndDlgItem;
   DWORD dwIndex;
   MON_WND_CNF *SI;
   LONG Scale;

   switch(message)  {
      case WM_INITDIALOG:
#ifdef WINCE
	  shidi.dwMask = SHIDIM_FLAGS;
	  shidi.dwFlags = SHIDIF_DONEBUTTON | SHIDIF_SIPDOWN | SHIDIF_SIZEDLGFULLSCREEN;
	  shidi.hDlg = hWndDlg;
	  SHInitDialog(&shidi);
#endif
      // Save pointer to return structure.
      SetWindowLong(hWndDlg, GWL_USERDATA, (LONG)lParam);

      // Create a pointer to the startup structure.
      SI=(MON_WND_CNF*)lParam;

      // Initialize the 'Scale' fields.
      SetDlgItemText(hWndDlg, IDC_EDIT1, _ltow(SI->UserScale, str15, 10));
      if (SI->AutoScaling) {
         EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT1), FALSE);
         CheckDlgButton(hWndDlg, IDC_CHECK1, BST_CHECKED);
      } else {
         CheckDlgButton(hWndDlg, IDC_CHECK1, BST_UNCHECKED);
      }

      // Initialize the 'Sample Reduction' fields.
      hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO1);
      for (Index=0;Index<=8;Index++) {
         _itow(Reductions[Index], str15, 10);
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCSTR)str15);
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, (DWORD)Reductions[Index]);
         if (SI->Reduction==Reductions[Index]) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }

		if (SI->OneSmplPrBlock) {
			CheckDlgButton(hWndDlg, IDC_CHECK3, BST_CHECKED);
			EnableWindow(GetDlgItem(hWndDlg, IDC_COMBO1), FALSE);
		}

		if (SI->SmplReduction) {
			CheckDlgButton(hWndDlg, IDC_CHECK2, BST_CHECKED);
		} else {
         EnableWindow(GetDlgItem(hWndDlg, IDC_COMBO1), FALSE);
			EnableWindow(GetDlgItem(hWndDlg, IDC_CHECK3), FALSE);
		}

      // If memory allocation of sample buffer failed,
      // then disable the sample reduction controls.
      if (!SI->SmplBuff) {
         EnableWindow(GetDlgItem(hWndDlg, IDC_CHECK2), FALSE);
         EnableWindow(GetDlgItem(hWndDlg, IDC_COMBO1), FALSE);
			EnableWindow(GetDlgItem(hWndDlg, IDC_CHECK3), FALSE);
      }
      return TRUE;

      case WM_COMMAND:
      switch(LOWORD(wParam)) {
         case IDC_CHECK1:
         switch (HIWORD(wParam)) {
            case BN_CLICKED:
            if (IsDlgButtonChecked(hWndDlg, IDC_CHECK1)==BST_CHECKED) {
               // Disable the edit field.
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT1), FALSE);
            } else {
               // Enable the edit field.
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT1), TRUE);
               SetFocus(GetDlgItem(hWndDlg, IDC_EDIT1));
               SendDlgItemMessage(hWndDlg, IDC_EDIT1, EM_SETSEL, 0, -1);
            }
            break;
         }
         break;

         case IDC_CHECK2:
         switch (HIWORD(wParam)) {
            case BN_CLICKED:
            if (IsDlgButtonChecked(hWndDlg, IDC_CHECK2)==BST_UNCHECKED) {
               EnableWindow(GetDlgItem(hWndDlg, IDC_COMBO1), FALSE);
					EnableWindow(GetDlgItem(hWndDlg, IDC_CHECK3), FALSE);
            } else {
               EnableWindow(GetDlgItem(hWndDlg, IDC_COMBO1), TRUE);
					EnableWindow(GetDlgItem(hWndDlg, IDC_CHECK3), TRUE);
					if (IsDlgButtonChecked(hWndDlg, IDC_CHECK3)==BST_CHECKED) {
						EnableWindow(GetDlgItem(hWndDlg, IDC_COMBO1), FALSE);
					} else {
						SetFocus(GetDlgItem(hWndDlg, IDC_COMBO1));
					}
            }
            break;
         }
         break;

         case IDC_CHECK3:
         switch (HIWORD(wParam)) {
            case BN_CLICKED:
            if (IsDlgButtonChecked(hWndDlg, IDC_CHECK3)==BST_UNCHECKED) {
               EnableWindow(GetDlgItem(hWndDlg, IDC_COMBO1), TRUE);
               SetFocus(GetDlgItem(hWndDlg, IDC_COMBO1));
            } else {
               EnableWindow(GetDlgItem(hWndDlg, IDC_COMBO1), FALSE);
            }
            break;
         }
         break;

		 case IDOK:
	    // Close the dialog box
        EndDialog(hWndDlg , 0);
		break;
			
		 case IDCANCEL:
		// Close the dialog box
	    EndDialog(hWndDlg , 0);
		break;
		
      }
      return FALSE;
   }
   return FALSE;
}


WORD GetSizeOfReducedBlck2(WORD SampleRate, BYTE Reduction)
{
   WORD B,R;
   BYTE FB,RB;

   switch (Reduction) {
      case 10: FB=10; RB=9; break;
      case 20: FB=5;  RB=4; break;
      case 25: FB=4;  RB=3; break;
      case 33: FB=3;  RB=2; break;
      case 50: FB=2;  RB=1; break;
      case 66: FB=3;  RB=1; break;
      case 75: FB=4;  RB=1; break;
      case 80: FB=5;  RB=1; break;
      case 90: FB=10; RB=1; break;
   }

   B=(SampleRate/FB)*RB;
   R=SampleRate-((SampleRate/FB)*FB);
   if (Reduction>50 && R>0) R=1;
   return B+R;
}



WORD ReduceSamples2(LONG* InBuff, LONG* OutBuff, WORD InBuffSize, BYTE Reduction)
{
   BYTE RC,CC, NumCopied, NumRemoved;
   WORD InIndex, OutIndex, TotalNumCopied;
   BOOL Copy;

   switch (Reduction) {
      case 10: RC=1; CC=9; break;
      case 20: RC=1; CC=4; break;
      case 25: RC=1; CC=3; break;
      case 33: RC=1; CC=2; break;
      case 50: RC=1; CC=1; break;
      case 66: RC=2; CC=1; break;
      case 75: RC=3; CC=1; break;
      case 80: RC=4; CC=1; break;
      case 90: RC=9; CC=1; break;
   }

   Copy=TRUE;
   OutIndex=0;
   TotalNumCopied=0;
   NumCopied=NumRemoved=0;
   for (InIndex=0;InIndex<InBuffSize;InIndex++) {
      if (Copy) {
         OutBuff[OutIndex]=InBuff[InIndex];
         OutIndex++;
         NumCopied++;
         TotalNumCopied++;
         if (NumCopied==CC) {
            Copy=FALSE;
            NumCopied=0;
         }
      } else {
         NumRemoved++;
         if (NumRemoved==RC) {
            Copy=TRUE;
            NumRemoved=0;
         }
      }
   }

   return TotalNumCopied;
}



VOID SetWindowTitle2(MON_WND_DATA* S, HWND hWnd)
{
   TCHAR str10[11], WindowText[128];

	wcscpy(WindowText, S->UniqueProcName);

   if (S->WndConfig.SmplReduction) {
		if (S->WndConfig.OneSmplPrBlock) {
			wcscat(WindowText, TEXT(" (Sample reduction: Reduced to one sample per block)"));
			SetWindowText(hWnd, WindowText);
		} else {
			wcscpy(WindowText, S->UniqueProcName);
			_itow(S->WndConfig.Reduction, str10, 10);
			wcscat(WindowText, TEXT(" (Sample reduction: "));
			wcscat(WindowText, str10);
			wcscat(WindowText, TEXT("%)"));
			SetWindowText(hWnd, WindowText);
		}
   } else {
      // Sample reduction is disabled.
      SetWindowText(hWnd, S->UniqueProcName);
   }

}

// * FFT Fast Fourier Transform in 1D from Numerical recipies
// ***********************************************************

#define SWAP(a,b) tempr=(a);(a)=(b);(b)=tempr
   
void four1(float data[], unsigned long nn, int isign)
{
	// data[] input and output dataset, nn number of samples (complex pair), Size of data[] is 2*nn 
	// isign: sign of i in exponential. Fourier = 1, inverse Fourier = -1 
	unsigned long n,mmax,m,j,istep,i;
	double wtemp,wr,wpr,wpi,wi,theta;
	float tempr,tempi;
   
	n=nn << 1;
	j=1;
	for (i=1;i<n;i+=2) {
		if (j > i) {
			SWAP(data[j],data[i]);
			SWAP(data[j+1],data[i+1]);
		}
		m=n >> 1;
		while (m >= 2 && j > m) {
			j -= m;
			m >>= 1;
		}
		j += m;
	}
	mmax=2;
	while (n > mmax) {
		istep=mmax << 1;
		theta=isign*(6.28318530717959/mmax);
		wtemp=sin(0.5*theta);
		wpr = -2.0*wtemp*wtemp;
		wpi=sin(theta);
		wr=1.0;
		wi=0.0;

		for (m=1;m<mmax;m+=2) {
			for (i=m;i<=n;i+=istep) {
				j=i+mmax;
				tempr=wr*data[j]-wi*data[j+1];
				tempi=wr*data[j+1]+wi*data[j];
				data[j]=data[i]-tempr;
				data[j+1]=data[i+1]-tempi;
				data[i] += tempr;
				data[i+1] += tempi;
			}
			wr=(wtemp=wr)*wpr-wi*wpi+wr;
			wi=wi*wpr+wtemp*wpi+wi;
		}
		mmax=istep;
	}
}
#undef SWAP



