#include "stdafx.h"
#define WINCE
//#define STRICT
#include <windows.h>
#include "seislog.h"
#include "eventlog.h"
#include "devices.h"
#include "channel.h"
#include "trigger.h"
#include "mainmon.h"
#include "library.h"

//
// Theese are global variables used by functions in this file
//
extern HWND hWndFrame;
extern DWORD SysClkSyncThreadID;
extern DWORD ProcKeeperThreadID;
extern DWORD MainMonThreadID;
extern DWORD SysClkSyncThreadID;
extern HANDLE hTickerThread;
extern DWORD TickerThreadID;
extern CHAR NumRegisteredDevices;
extern BYTE NumRegisteredChannels;
extern BOOL GPSSyncThreadRunning;
extern BOOL StartGPSSyncThread;




//
// Prototype definitions of thread functions.
//
DWORD WINAPI StartSystemThrdProc(PVOID);
DWORD WINAPI StopSystemThrdProc(PVOID);
DWORD WINAPI MainMonThrdProc(PVOID);
DWORD WINAPI TickerThrdProc(PVOID);
DWORD WINAPI SysClkSyncThrd(PVOID);
VOID StopSystem(BOOL); 

/************************************************************************/
/* Starts the system.                                                   */
/************************************************************************/
VOID StartSystem(VOID) 
{
   HANDLE hThread;
	DWORD ThreadID;

   if (NumRegisteredDevices && NumRegisteredChannels) {
      // Start the startup thread.
      hThread=CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)StartSystemThrdProc, (PVOID)NULL, 0, &ThreadID);
		CloseHandle(hThread);
   } else {
      // Write an error in the program log.
      if (!NumRegisteredDevices) {
         WriteSystemLog(TEXT("Seislog: Unable to start system. No devices have been configured."), 3);
      } else {
         WriteSystemLog(TEXT("Seislog: Unable to start system. No channels have been configured."), 3);
      }
   }
   return;
}


/************************************************************************/
/* Reads and load information from disk about all registered processes. */
/* Then starts all processes in correct order.                          */          
/************************************************************************/
DWORD WINAPI StartSystemThrdProc(PVOID StartupInfo)
{

   HANDLE hThread;
	STARTINFO_MAINMONITOR SIMM;

#ifdef WINCE
	DisableCommandBar();
#else
	DisableMenu(hWndFrame);
#endif

   // Initialize the process table handled by the process-keeper thread.
   PostThreadMessage(ProcKeeperThreadID, TM_INITIALIZE, 0, 0); 

   // Start the one-second ticker thread.
   hTickerThread=CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)TickerThrdProc, (PVOID)NULL, 0, &TickerThreadID);

   // Register all devices into the 'Devices' array.
   //
   // Note: All exisiting directories in the 'DeviceDir'
   //       directory, is assumed to represent a device if
   //       the directory contains a 'device.config' file.
   RegisterDevices();

   // Register all channels into the 'Channels' array.
   //
   // Note: All exisiting directories in the 'ChannelDir'
   //       directory, is assumed to represent a channel if
   //       the directory contains a 'channel.config' file.
   RegisterChannels();

   // Register all triggersets into the 'Triggers' array.
   //
   // Note: All files in the trigger directory with 
   //       file extension .trg is considered to be a valid
   //       triggerset configuration file.
   RegisterTriggerSets();

   // Start all device-manager threads. 
   //
   // NOTE: The thread does not attempt to read from
   //       the device unitl a TM_START message is 
   //       sendt to the thread.
   LoadDevices();

   // Start all the registered channels.
   //
   // NOTE: The channel threads are started but 
   //       they are not sendt the TM_START message 
   //       at this point.
   LoadChannels();
   
   // Check that at least one channel
   // thread was successfully loaded.
   // Channel threads may fail to load
   // if they are configured to use a
   // non-existant device.
   if (!GetRunningChannels(NULL)) {
      // All channels failed to start.
      //
      // Write a note in the program log.
      WriteSystemLog(TEXT("Seislog: Cannot load any channels, unable to start system."), 3);

      // Stop the system.
      StopSystem(FALSE);

   } else {
      // At least one channel started.
      // Continue the startup process.
      //
      //
      // Start all triggerset threads.
      //
      // NOTE: The triggerset threads are started but 
      //       they are not sendt the TM_START message 
      //       at this point.
      LoadTriggers();

      // Send TM_START message to all channel threads.
      StartChannels();

      // Send TM_START message to all device-manager threads.
      StartDevices();

      // Send TM_START message to all trigger threads.
      StartTriggers();
   
      // Start the Main monitor window.
      SIMM.StartEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
      hThread=CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)MainMonThrdProc, (PVOID)&SIMM, 0, &MainMonThreadID);
      WaitForSingleObject(SIMM.StartEvent, INFINITE); // Wait for the thread to start.
      CloseHandle(SIMM.StartEvent);                   // Delete the event object.
		CloseHandle(hThread);

      // Write a note in the program log.
      WriteSystemLog(TEXT("Seislog: System started."), 1);

      // If no thread is syncronizing the system clock,
      // then update the status bar to reflect this.
      if (!SysClkSyncThreadID) {
         PostMessage(hWndFrame, TM_WRITE_STATUSBAR, 0, (LPARAM)TEXT("System clock is NOT syncronized."));
      }

   }

   // Enable the program menu.
#ifdef WINCE
   EnableCommandBar();
#else
   EnableMenu(hWndFrame); 
#endif
   // Exit the thread.
   ExitThread(0);

   return 0;
}









/************************************************************************/
/* Stops the system.                                                    */
/* If the SndCloseMsg is TRUE, then a WM_CLOSE message will be sendt    */
/* to the frame window when all processes have been stopped.            */
/* This function does NOT hang the calling thread.                      */
/************************************************************************/
VOID StopSystem(BOOL SndCloseMsg) 
{
   HANDLE hThread;
   DWORD ThreadID;

   // Start the shutdown thread.
   hThread=CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)StopSystemThrdProc, (PVOID)SndCloseMsg, 0, &ThreadID);
	CloseHandle(hThread);

   return;
}


/************************************************************************/
/* Stops all running proceses in correct order.                         */
/************************************************************************/
DWORD WINAPI StopSystemThrdProc(PVOID StartupInfo)
{
   CHAR NumDevices, Index;
   HANDLE hEvent;
   MSG_PARAM MsgParam;
   DWORD DeviceManagers[MAX_NUM_OF_DEVICES];

   // Disable the program menu.
#ifdef WINCE
   DisableCommandBar(); 
#else
   DisableMenu(hWndFrame); 
#endif

   // Stop the Main monitor thread if it is running.
   wcscpy(MsgParam.cStatus, TEXT("Activity Monitor"));
   hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
   PostThreadMessage(ProcKeeperThreadID, TM_GET_PROCESS_BY_NAME, (WPARAM)hEvent, (LPARAM)&MsgParam); 
   WaitForSingleObject(hEvent, INFINITE);
   if (MsgParam.bStatus) {
	   PostThreadMessage(MsgParam.dwStatus, TM_EXIT, 0, 0);    
	   while (PostThreadMessage(MsgParam.dwStatus, TM_DUMMY, 0, 0)) Sleep(100);
   }
	CloseHandle(hEvent);
   

   //
   // Stopping all the device managers, will cause the whole system to stop.
   //
   //
   // Get a list of all device-managers running.
   MsgParam.vPointer=(PVOID)&DeviceManagers;
   hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
   PostThreadMessage(ProcKeeperThreadID, TM_GET_DEVICEMANAGERS, (WPARAM)hEvent, (LPARAM)&MsgParam); 
   WaitForSingleObject(hEvent, INFINITE);
   NumDevices=(char)MsgParam.dwStatus;
   CloseHandle(hEvent);

   // Stop device-manager threads, one by one.
   for (Index=0;Index<NumDevices;Index++) {
      // Send stop signal to the thread.
      PostThreadMessage(DeviceManagers[Index], TM_EXIT, 0, 0); 

      // Wait for the thread to exit.
      while (PostThreadMessage(DeviceManagers[Index], TM_DUMMY, 0, 0)) Sleep(250);
   }

   // The ticker thread has to be stopped manually.
   PostThreadMessage(TickerThreadID, TM_EXIT, 0, 0);

   // Clear the processkeeper database.
   PostThreadMessage(ProcKeeperThreadID, TM_INITIALIZE, 0, 0); 

   // Write a note in the program log that the system has been stopped.
   WriteSystemLog(TEXT("Seislog: System stopped."), 1);

   // Send WM_CLOSE message if requested.
   if ((BOOL)StartupInfo) PostMessage(hWndFrame, WM_CLOSE, 0, 0);

   // Enable the program menu.
#ifdef WINCE   
   EnableCommandBar(); 
#else
   EnableMenu(hWndFrame); 
#endif
   // Exit the thread.
   ExitThread(0);

   return 0;
}




