#include "stdafx.h"

//#define STRICT
#include <windows.h>
#include "resource.h"
#include "seislog.h"
#include "library.h"
#include "msglist.h"
#include "eventlog.h"

extern BYTE DebugLevel;


/***************************************************/
/* Ticker thread. Sends a TM_TICK message to all   */
/* registered threads/windows once every second.   */
/***************************************************/
DWORD WINAPI TickerThrdProc(PVOID StartupInfo) 
{
   MSG msg;
   TCHAR Message[100], str10[11];
   WORD OldSecond;
   DWORD CurrentThreadID;
   SUBSCRIBE Subscribe;
   SYSTEMTIME Time;
   MsgList *NotifyList;
   BOOL Finished;


   // Initialize some variables.
	OldSecond=0;
   Finished=FALSE;
   CurrentThreadID=GetCurrentThreadId();

   // Create an instance of the MsgList class.
   NotifyList = new MsgList;

   // If running in debug mode, write a startup message to the log.
   if (DebugLevel) {
      wcscat(Message, TEXT("SecondTicker: Thread started."));
      WriteSystemLog(Message, 1);
   }

   
   //
   // This is the thread's main loop.
   //
   while(!Finished) {
      // If a new second has arrived, send out a TM_TICKER message.
      GetLocalTime(&Time);
      if (Time.wSecond!=OldSecond) {
         // A new second has arrived, notify all registered threads/windows.
         NotifyList->Notify((UINT)TM_TICKER, (WPARAM)CurrentThreadID, 0);
         // Save the new second.
         OldSecond=Time.wSecond;
      }
      
      
      // Handle messages from thread's message queue.
      while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) {
         // If debuglevel is 2 then log all messages recieved.
         if (DebugLevel>1) {
            wcscpy(Message, TEXT("SecondTicker: Recieved message "));
            _itow(msg.message, str10, 10);
            wcscat(Message, str10);
            WriteSystemLog(Message, 1);
         }

         switch (msg.message) {
         case TM_EXIT:
            // The thread is requested to end.
            // This will end the thread loop.
            Finished=TRUE;
            break;

         case TM_SUBSCRIBE:
            // A process or window subscibes to the TM_TICKER message.
            ZeroMemory((PVOID)&Subscribe, (DWORD)sizeof(SUBSCRIBE));
            Subscribe.uiMessage=TM_TICKER;
            if (msg.lParam) Subscribe.idThread=(DWORD)msg.lParam;
            if (msg.wParam) Subscribe.hWnd=(HWND)msg.wParam;
            NotifyList->Subscribe(&Subscribe);
            break;   

         case TM_UNSUBSCRIBE:
            // A process or window unsubscribes to the TM_TICKER message.
            ZeroMemory((PVOID)&Subscribe, (DWORD)sizeof(SUBSCRIBE));
            Subscribe.uiMessage=TM_TICKER;
            if (msg.lParam) Subscribe.idThread=(DWORD)msg.lParam;
            if (msg.wParam) Subscribe.hWnd=(HWND)msg.wParam;
            NotifyList->UnSubscribe(&Subscribe);
            break;   
         }
      }

      // Suspend the execution of this thread 25 milliseconds.
      // This is done to reduce the CPU load the thread generates.
      Sleep(25);
   }



   //
   // The thread's exit code starts here.
   //

   // Free the allocated instance of the MsgList class.
   delete NotifyList;

   // If running in debug mode, write a stop message to the log.
   if (DebugLevel) {
      wcscpy(Message, TEXT("SecondTicker: Thread stopped."));
      WriteSystemLog(Message, 1);
   }

   // Exit the thread.
   ExitThread(0);
   return 0;
}



