/* ************************************************** */
/* Implementation of the unformpcbuffer class.        */
/* ************************************************** */

#include "stdafx.h"

//#define STRICT
#include <windows.h>
#include "FifoBuffer.h"
#include "unformpcfile.h"


UnformPcFile::UnformPcFile(WORD MaxBytesPrWrite) {

   // Initiate some variables.
   FileIsOpened=FALSE;
   FilePointer=0;
   NumLeftovers=0;

   // Allocate memory for an internal buffer.
   pLocalBuffer=(PBYTE)LocalAlloc(LMEM_FIXED, (UINT)(MaxBytesPrWrite+128));
}


UnformPcFile::~UnformPcFile() {

   // Free allocated memory.
   LocalFree((HLOCAL)pLocalBuffer);
}



BOOL UnformPcFile::Open(LPCTSTR FileName) {
   DWORD BytesWritten;

   // If a file is already open, close it.
   if (FileIsOpened) {
      Close();
      FileIsOpened=FALSE;
      FilePointer=0;
   }

   // Open existing file.
   FileHandle=CreateFile(FileName,           // pointer to name of the file 
                     GENERIC_WRITE,          // access (read-write) mode 
                     0,                      // share mode 
                     NULL,                   // pointer to security attributes 
                     OPEN_EXISTING,          // how to create 
                     FILE_ATTRIBUTE_NORMAL,  // file attributes 
                     NULL);                  // handle to file with attributes to copy 
   
   // Check that the file has been sucessfully opened.
   if (FileHandle==INVALID_HANDLE_VALUE) return FALSE;

   // The file is now opened.
   FileIsOpened=TRUE;

   // Insert 1 byte ('K') to indicate start of file.
   WriteFile(FileHandle, (VOID*)"K", (DWORD)1, (DWORD*)&BytesWritten, (OVERLAPPED*)(VOID*)NULL);
   if (BytesWritten!=1) return FALSE;
   
   // Update the filepointer variable.
   FilePointer=1;
   
   return TRUE;
}


BOOL UnformPcFile::Write(DWORD Count, BYTE BlockSize, BYTE *Source, DWORD *BytesActuallyWritten) {
//
// Write 'Count' number of bytes to the file. Bytes must
// be written in blocks of size 'BlockSize'. Bytes left over
// are not written now, but placed in the leftover buffer to
// be written netxt time this function is called.
// Returns the number of bytes actually written (Count-leftoverbytes).
//
   BYTE  str1[1], *bPtr, OldLeftovers;
   DWORD BytesInBuffer, BytesWritten, NumBlocks, Index;

   // Set value of 'BytesAcuallyWritten' to zero.
   if (BytesActuallyWritten!=NULL) *BytesActuallyWritten=0;

   // Check that a file has been opened.
   if (!FileIsOpened) return FALSE;

   // Check value of 'BlockSize'.
   if (BlockSize==0 || BlockSize>128) BlockSize=128;

   //
   // Copy bytes to a local buffer.
   //
   bPtr=pLocalBuffer;
   BytesInBuffer=0;

   // First copy leftover bytes from Leftover buffer.
   if (NumLeftovers) {
      CopyMemory((PVOID)bPtr, (PVOID)LeftOverBytes, NumLeftovers);
      bPtr=bPtr+NumLeftovers;
      BytesInBuffer=NumLeftovers;
      OldLeftovers=NumLeftovers;
      NumLeftovers=0;
   }

   // Then copy the rest of the bytes.
   CopyMemory((PVOID)bPtr, (PVOID)Source, Count);
   BytesInBuffer=BytesInBuffer+Count;

   // Write data to file. First calculate the number of blocks.
   NumBlocks=BytesInBuffer/BlockSize;

   // Write blocks of size 'BlockSize'.
   if (NumBlocks) {
      str1[0]=BlockSize;
      bPtr=pLocalBuffer;
      for (Index=1;Index<=NumBlocks;Index++) {
         // Write block header byte.
         WriteFile(FileHandle, (VOID*)str1, (DWORD)1, &BytesWritten, (OVERLAPPED*)(VOID*)NULL);
         if (BytesWritten!=1) return FALSE;

         // Write 'BlockSize' bytes.
         WriteFile(FileHandle, (VOID*)bPtr, (DWORD)BlockSize, &BytesWritten, (OVERLAPPED*)(VOID*)NULL);
         if (BytesWritten!=BlockSize) return FALSE;

         // Move the data pointer.
         bPtr=bPtr+BlockSize;

         // Write block trailer.
         WriteFile(FileHandle, (VOID*)str1, (DWORD)1, &BytesWritten, (OVERLAPPED*)(VOID*)NULL);
         if (BytesWritten!=1) return FALSE;

         // Update the filepointer variable.
         FilePointer=FilePointer+BlockSize+2;
      }
   }

   // Save leftover bytes in the Leftover buffer.
   NumLeftovers=(BYTE)(BytesInBuffer-(NumBlocks*BlockSize));
   CopyMemory((PVOID)LeftOverBytes, (PVOID)bPtr, NumLeftovers);
      
   // Return number of bytes actually written.
   if (BytesActuallyWritten!=NULL) *BytesActuallyWritten=(DWORD)(NumBlocks*BlockSize);

   return TRUE;
}



BOOL UnformPcFile::WriteLeftovers(DWORD *Bytes) {
//
// Write all bytes in leftoverbuffer,
// and returns number of bytes written.
//
   BYTE  str1[1];
   DWORD BytesWritten, RetValue;

   // Check that a file has been opened.
   if (!FileIsOpened) return FALSE;

   // Save return value
   RetValue=NumLeftovers;

   // Write leftover bytes.
   if (NumLeftovers) {
      str1[0]=NumLeftovers;
      
      // Write block header byte.
      WriteFile(FileHandle, (VOID*)str1, (DWORD)1, &BytesWritten, (OVERLAPPED*)(VOID*)NULL);
      if (BytesWritten!=1) return FALSE;

      // Write leftover bytes.
      WriteFile(FileHandle, (VOID*)LeftOverBytes, (DWORD)NumLeftovers, &BytesWritten, (OVERLAPPED*)(VOID*)NULL);
      if (BytesWritten!=NumLeftovers) return FALSE;

      // Write block trailer.
      WriteFile(FileHandle, (VOID*)str1, (DWORD)1, &BytesWritten, (OVERLAPPED*)(VOID*)NULL);
      if (BytesWritten!=1) return FALSE;

      // Update the filepointer variable.
      FilePointer=FilePointer+NumLeftovers+2;

      // Update the 'NumLeftovers' variable.
      NumLeftovers=0;
   }

   // Return number of bytes written.
   if (Bytes!=NULL) *Bytes=RetValue;

   return TRUE;
}



BOOL UnformPcFile::Seek(LONG FilePosition) {

   DWORD PointerLow; 
   
   // Check that a file has been opened.
   if (!FileIsOpened) return FALSE;

   // Set filepointer to new position.
   PointerLow=SetFilePointer(FileHandle, FilePosition, 0, FILE_BEGIN); 
   if (PointerLow == 0xFFFFFFFF) return FALSE;
   FilePointer=PointerLow;
   
   return TRUE;
}


VOID UnformPcFile::Close() {

   // Close the file.
   if (FileIsOpened) CloseHandle(FileHandle);

   // Clean up.
   FilePointer=0;
   FileIsOpened=FALSE;

}


DWORD UnformPcFile::GetFilePointer() {
   return FilePointer;
}


VOID UnformPcFile::Flush() {
   FlushFileBuffers(FileHandle);
}