#include "stdafx.h"

//#define STRICT
#include <windows.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#define MAXPOL 400
#define ABS(a) ((a) >= (0) ? (a) : -(a))


typedef struct {
   double r, i;
} __complex;



/*                           */    
/* Local function prototypes */
/*                           */             
int buroots(__complex *p, TCHAR *rtype, double *dcvalue, int *nsects ,int iord);
int lptbp(__complex *p, __complex *z, TCHAR *rtype, double *dcvalue, double *fl, double *fh, int *nsects, double sn[], double sd[]);
int bilin2(int *nsects, double *sn, double *sd);
double warp(double *f, double *ts);
__complex cpowi(__complex a, int n);
__complex csqrt(__complex z);
__complex conjg(__complex z);
__complex cmul(__complex a, __complex b);
__complex cdiv(__complex a, __complex b);



/*  Subroutine to design IIR digital filters from analog prototypes.       */
/*  Input Arguments:                                                       */
/*  ----------------                                                       */
/*    IORD                Filter order (10 MAXIMUM)                        */
/*    FL                  Low-frequency cutoff                             */
/*    FH                  High-frequency cutoff                            */
/*    TS                  Sampling interval (in seconds)                   */
/*                                                                         */
/*  Output Arguments:                                                      */
/*  -----------------                                                      */
/*    SN                  Array containing numerator coefficients of       */
/*                        second-order sections packed head-to-tail.       */
/*    SD                  Array containing denominator coefficients        */
/*                        of second-order sections packed head-to-tail.    */
/*                                                                         */
void design(int iord, double fl, double fh, double ts, double sn[], double sd[], int *nsects)
{
   double r__1, fhw, flw, dcvalue, C_B12;
   __complex p[10], z[10];
   TCHAR stype[3*10]=TEXT("");

   /*  Analog prototype selection */
   buroots(p, stype, &dcvalue, nsects, iord);

   /*  Analog mapping selection */
   C_B12=2.0;
   r__1 = fl * ts / 2;
   flw = warp(&r__1, &C_B12);
   r__1 = fh * ts / 2;
   fhw = warp(&r__1, &C_B12);
   lptbp(p, z, stype, &dcvalue, &flw, &fhw, nsects, sn, sd);

   /*  Bilinear analog to digital transformation */
   bilin2(nsects, &sn[1], &sd[1]);
}




/*  Subroutine to apply an iir filter to a data sequence.            */
/*  The filter is assumed to be stored as second order sections.     */
/*  Filtering is in-place.                                           */
/*                                                                   */
/*  Input Arguments:                                                 */
/*  ----------------                                                 */
/*    data                           Array containing data           */
/*    nsamps                         Number of data samples          */
/*                                                                   */
/*  Output Arguments:                                                */
/*  -----------------                                                */
/*    data                          Data array (same as input)       */
/*                                                                   */
void apply(double data[], int nsamps, int nsects, double sn[], double sd[],
                    double sx1[], double sx2[], double sy1[], double sy2[])
{
   int  jptr, i, j;
   double b0, b1, b2, a1, a2, x1, x2, y1, y2, output;

   --data;
   jptr = 1;
   for (j = 1; j <= nsects; ++j) {
      x1 = sx1[j];
      x2 = sx2[j];
      y1 = sy1[j];
      y2 = sy2[j];
      b0 = sn[jptr];
      b1 = sn[jptr + 1];
      b2 = sn[jptr + 2];
      a1 = sd[jptr + 1];
      a2 = sd[jptr + 2];
      for (i=1; i<=nsamps; ++i) {
         output = b0 * data[i] + b1 * x1 + b2 * x2;
         output -= a1 * y1 + a2 * y2;
         y2 = y1;
         y1 = output;
         x2 = x1;
         x1 = data[i];
         data[i] = output;
      }
      jptr += 3;
      sx1[j]=x1;
      sx2[j]=x2;
      sy1[j]=y1;
      sy2[j]=y2;
   }
}





/* BUROOTS -- SUBROUTINE TO COMPUTE BUTTERWORTH POLES FOR            */
/* NORMALIZED LOWPASS FILTER                                         */
/* LAST MODIFIED:  SEPTEMBER 7, 1990                                 */
/*  OUTPUT ARGUMENTS:                                                */
/*  -----------------                                                */
/*      P              complex ARRAY CONTAINING POLES                */
/*                       CONTAINS ONLY ONE FROM EACH                 */
/*                       complex CONJUGATE PAIR, AND                 */
/*                       ALL REAL POLES                              */
/*      RTYPE          CHARACTER ARRAY INDICATING 2ND ORDER SECTION  */
/*                       TYPE:                                       */
/*                         (SP)  SINGLE REAL POLE                    */
/*                         (CP)  complex CONJUGATE POLE PAIR         */
/*                         (CPZ) complex CONJUGATE POLE-ZERO PAIRS   */
/*      DCVALUE        MAGNITUDE OF FILTER AT ZERO FREQUENCY         */
/*  INPUT ARGUMENTS:                                                 */
/*  ----------------                                                 */
/*      IORD           DESIRED FILTER ORDER                          */
/*                                                                   */
int buroots(__complex *p, TCHAR *rtype, double *dcvalue, int *nsects, int iord)
{
   /* System generated locals */
   int i__1, i__2;
   double d__1, d__2;
   __complex q__1;

   /* Local variables */
   int half, k;
   double angle, pi;

   /* Parameter adjustments */
   rtype -= 3;
   --p;

   /* Function Body */
   pi = 3.14159265;
   half = iord/2;

   /* TEST FOR ODD ORDER, AND ADD POLE AT -1 */
   *nsects = 0;
   if (half << 1 < iord) {
      p[1].r = -1, p[1].i = 0;
      wcsncpy(rtype + 3, TEXT("SP"), 2);
      *nsects = 1;
   }
   i__1 = half;
   for (k = 1; k <= i__1; ++k) {
      angle = pi * ((double)((k<<1)-1) / (double)(iord<<1) + 0.5);
      ++(*nsects);
      i__2 = *nsects;
      d__1 = cos(angle);
      d__2 = sin(angle);
      q__1.r = d__1, q__1.i = d__2;
      p[i__2].r = q__1.r, p[i__2].i = q__1.i;
      wcsncpy(rtype + *nsects * 3, TEXT("CP"), 2);
   }
   *dcvalue = 1;

   return 1;
}








/* Subroutine to convert an prototype lowpass filter to a bandpass filter via */
/*    the analog polynomial transformation.  The lowpass filter is            */
/*   described in terms of its poles and zeros (as input to this routine).    */
/*    The output consists of the parameters for second order sections.        */
/*  Input Arguments:                                                          */
/*  ----------------                                                          */
/*    P                       Array containing poles                          */
/*    Z                       Array containing zeros                          */
/*    RTYPE                   Character array containing type information     */
/*                              (SP) single real pole  or                     */
/*                              (CP) complex conjugate pole pair  or          */
/*                              (CPZ) complex conjugate pole/zero pairs       */
/*    DCVALUE                 Zero frequency value of filter                  */
/*    FL                      Low-frequency cutoff                            */
/*    FH                      High-frequency cutoff                           */
/*  Output Arguments:                                                         */
/*  -----------------                                                         */
/*    SN                      Numerator polynomials for second order          */
/*                              sections.                                     */
/*    SD                      Denominator polynomials for second order        */
/*                              sections.                                     */
/*                              This subroutine doubles the number of         */
/*                              sections.                                     */
/*                                                                            */   
int lptbp(__complex *p, __complex *z, TCHAR *rtype, double *dcvalue, double *fl,
          double *fh, int *nsects, double sn[], double sd[])
{
   int i__1, i__2, i__3, i__4, i__5, i__6, i__7, iptr, i, n;
   double d__1, a, b, scale, pi, twopi;
   __complex q__1, q__2, q__3, q__4, q__5, q__6, q__7, q__8, q__9, q__10;
   __complex h, s, ctemp, p1, p2, z1, z2;

   /* Parameter adjustments */
   rtype -= 3;
   --z;
   --p;

   /* Function Body */
   pi = 3.14159265;
   twopi = pi * 2.0;
   a = twopi * twopi * *fl * *fh;
   b = twopi * (*fh - *fl);
   n = *nsects;
   *nsects = 0;
   iptr = 1;
   i__1 = n;
   for (i = 1; i <= i__1; ++i) {
      if (wcsncmp(rtype + i * 3, TEXT("CPZ"), 3) == 0) {
         i__2 = i;
         q__3.r = b * z[i__2].r, q__3.i = b * z[i__2].i;
         q__2 = cpowi(q__3, 2);
         d__1 = a * 4.0;
         q__1.r = q__2.r - d__1, q__1.i = q__2.i;
         ctemp.r = q__1.r, ctemp.i = q__1.i;
         q__1 = csqrt(ctemp);
         ctemp.r = q__1.r, ctemp.i = q__1.i;
         i__2 = i;
         q__3.r = b * z[i__2].r, q__3.i = b * z[i__2].i;
         q__2.r = q__3.r + ctemp.r, q__2.i = q__3.i + ctemp.i;
         q__1.r = q__2.r * 0.5, q__1.i = q__2.i * 0.5;
         z1.r = q__1.r, z1.i = q__1.i;
         i__2 = i;
         q__3.r = b * z[i__2].r, q__3.i = b * z[i__2].i;
         q__2.r = q__3.r - ctemp.r, q__2.i = q__3.i - ctemp.i;
         q__1.r = q__2.r * 0.5, q__1.i = q__2.i * 0.5;
         z2.r = q__1.r, z2.i = q__1.i;
         i__2 = i;
         q__3.r = b * p[i__2].r, q__3.i = b * p[i__2].i;
         q__2 = cpowi(q__3, 2);
         d__1 = a * 4.0;
         q__1.r = q__2.r - d__1, q__1.i = q__2.i;
         ctemp.r = q__1.r, ctemp.i = q__1.i;
         q__1 = csqrt(ctemp);
         ctemp.r = q__1.r, ctemp.i = q__1.i;
         i__2 = i;
         q__3.r = b * p[i__2].r, q__3.i = b * p[i__2].i;
         q__2.r = q__3.r + ctemp.r, q__2.i = q__3.i + ctemp.i;
         q__1.r = q__2.r * 0.5, q__1.i = q__2.i * 0.5;
         p1.r = q__1.r, p1.i = q__1.i;
         i__2 = i;
         q__3.r = b * p[i__2].r, q__3.i = b * p[i__2].i;
         q__2.r = q__3.r - ctemp.r, q__2.i = q__3.i - ctemp.i;
         q__1.r = q__2.r * 0.5, q__1.i = q__2.i * 0.5;
         p2.r = q__1.r, p2.i = q__1.i;
         q__2 = conjg(z1);
         q__1.r = z1.r * q__2.r - z1.i * q__2.i, q__1.i = z1.r * q__2.i + z1.i * q__2.r;
         sn[iptr] = q__1.r;
         sn[iptr + 1] = z1.r * -2.0;
         sn[iptr + 2] = 1.0;
         q__2 = conjg(p1);
         q__1.r = p1.r * q__2.r - p1.i * q__2.i, q__1.i = p1.r * q__2.i + p1.i * q__2.r;
         sd[iptr] = q__1.r;
         sd[iptr + 1] = p1.r * -2.0;
         sd[iptr + 2] = 1.0;
         iptr += 3;
         q__2 = conjg(z2);
         q__1.r = z2.r * q__2.r - z2.i * q__2.i, q__1.i = z2.r * q__2.i + z2.i * q__2.r;
         sn[iptr] = q__1.r;
         sn[iptr + 1] = z2.r * -2.0;
         sn[iptr + 2] = 1.0;
         q__2 = conjg(p2);
         q__1.r = p2.r * q__2.r - p2.i * q__2.i, q__1.i = p2.r * q__2.i + p2.i * q__2.r;
         sd[iptr] = q__1.r;
         sd[iptr + 1] = p2.r * -2.0;
         sd[iptr + 2] = 1.0;
         iptr += 3;
         *nsects += 2;
      } else if (wcsncmp(rtype + i * 3, TEXT("CP"), 2) == 0) {
         i__2 = i;
         q__3.r = b * p[i__2].r, q__3.i = b * p[i__2].i;
            q__2 = cpowi(q__3, 2);
         d__1 = a * 4.0;
         q__1.r = q__2.r - d__1, q__1.i = q__2.i;
         ctemp.r = q__1.r, ctemp.i = q__1.i;
         q__1 = csqrt(ctemp);
         ctemp.r = q__1.r, ctemp.i = q__1.i;
         i__2 = i;
         q__3.r = b * p[i__2].r, q__3.i = b * p[i__2].i;
         q__2.r = q__3.r + ctemp.r, q__2.i = q__3.i + ctemp.i;
         q__1.r = q__2.r * 0.5, q__1.i = q__2.i * 0.5;
         p1.r = q__1.r, p1.i = q__1.i;
         i__2 = i;
         q__3.r = b * p[i__2].r, q__3.i = b * p[i__2].i;
         q__2.r = q__3.r - ctemp.r, q__2.i = q__3.i - ctemp.i;
         q__1.r = q__2.r * 0.5, q__1.i = q__2.i * 0.5;
         p2.r = q__1.r, p2.i = q__1.i;
         sn[iptr] = 0.0;
         sn[iptr + 1] = b;
         sn[iptr + 2] = 0.0;
         q__2 = conjg(p1);
         q__1.r = p1.r * q__2.r - p1.i * q__2.i, q__1.i = p1.r * q__2.i + p1.i * q__2.r;
         sd[iptr] = q__1.r;
         sd[iptr + 1] = p1.r * -2.0;
         sd[iptr + 2] = 1.0;
         iptr += 3;
         sn[iptr] = 0.0;
         sn[iptr + 1] = b;
         sn[iptr + 2] = 0.0;
         q__2 = conjg(p2);
         q__1.r = p2.r * q__2.r - p2.i * q__2.i, q__1.i = p2.r * q__2.i + p2.i * q__2.r;
         sd[iptr] = q__1.r;
         sd[iptr + 1] = p2.r * -2.0;
         sd[iptr + 2] = 1.0;
         iptr += 3;
         *nsects += 2;
      } else if (wcsncmp(rtype + i * 3, TEXT("SP"), 2) == 0) {
         sn[iptr] = 0.0;
         sn[iptr + 1] = b;
         sn[iptr + 2] = 0.0;
         sd[iptr] = a;
         i__2 = i;
         sd[iptr + 1] = -b * p[i__2].r;
         sd[iptr + 2] = 1.0;
         iptr += 3;
         ++(*nsects);
      }
   }
   /* Scaling - use fact that the bandpass filter amplitude at sqrt( omega_l **/
   /*            equals the amplitude of the lowpass prototype at d.c. */
   d__1 = sqrt(a);
   q__1.r = 0.0, q__1.i = d__1;
   s.r = q__1.r, s.i = q__1.i;
   h.r = 1.0, h.i = 0.0;

   iptr = 1;
   i__1 = *nsects;
   for (i = 1; i <= i__1; ++i) {
      i__2 = iptr + 2;
      q__6.r = sn[i__2] * s.r, q__6.i = sn[i__2] * s.i;
      i__3 = iptr + 1;
      q__5.r = q__6.r + sn[i__3], q__5.i = q__6.i;
      q__4.r = q__5.r * s.r - q__5.i * s.i, q__4.i = q__5.r * s.i + q__5.i * s.r;
      i__4 = iptr;
      q__3.r = q__4.r + sn[i__4], q__3.i = q__4.i;
      q__2.r = h.r * q__3.r - h.i * q__3.i, q__2.i = h.r * q__3.i + h.i * q__3.r;
      i__5 = iptr + 2;
      q__10.r = sd[i__5] * s.r, q__10.i = sd[i__5] * s.i;
      i__6 = iptr + 1;
      q__9.r = q__10.r + sd[i__6], q__9.i = q__10.i;
      q__8.r = q__9.r * s.r - q__9.i * s.i, q__8.i = q__9.r * s.i + q__9.i * s.r;
      i__7 = iptr;
      q__7.r = q__8.r + sd[i__7], q__7.i = q__8.i;
      q__1 = cdiv(q__2, q__7);
      h.r = q__1.r, h.i = q__1.i;
      iptr += 3;
   }
   q__2.r = *dcvalue, q__2.i = 0.0;
   d__1 = h.r;
   q__5 = conjg(h);
   q__4.r = d__1 * q__5.r, q__4.i = d__1 * q__5.i;
   q__3 = csqrt(q__4);
   q__1 = cdiv(q__2, q__3);
   scale = q__1.r;
   sn[1] *= scale;
   sn[2] *= scale;
   sn[3] *= scale;

   return 1;
}







/* Transforms an analog filter to a digital filter via the bilinear transformation  */
/*   Assumes both are stored as second order sections.  The transformation is       */
/*    done in-place.                                                                */
/*  Input Arguments:                                                                */
/*  ----------------                                                                */
/*   SN                   Array containing numerator polynomial coefficients for    */
/*                           second order sections.  Packed head-to-tail.           */
/*   SD                   Array containing denominator polynomial coefficients      */
/*                           second order sections.  Packed head-to-tail.           */
/*                                                                                  */
int bilin2(int *nsects, double sn[], double sd[])
{
   int i__1, iptr, i;
   double scale, a0, a1, a2;

   /* Parameter adjustments */
   --sd;
   --sn;

   /* Function Body */
   iptr = 1;
   i__1 = *nsects;
   for (i = 1; i <= i__1; ++i) {
      a0 = sd[iptr];
      a1 = sd[iptr + 1];
      a2 = sd[iptr + 2];
      scale = a2 + a1 + a0;
      sd[iptr] = 1.0;
      sd[iptr + 1] = (a0 - a2) * 2.0 / scale;
      sd[iptr + 2] = (a2 - a1 + a0) / scale;
      a0 = sn[iptr];
      a1 = sn[iptr + 1];
      a2 = sn[iptr + 2];
      sn[iptr] = (a2 + a1 + a0) / scale;
      sn[iptr + 1] = (a0 - a2) * 2.0 / scale;
      sn[iptr + 2] = (a2 - a1 + a0) / scale;
      iptr += 3;
   }

   return 1;
}




/* WARP -- FUNCTION, APPLIES TANGENT FREQUENCY WARPING TO COMPENSATE */
/*         FOR BILINEAR ANALOG -> DIGITAL TRANSFORMATION             */
/* ARGUMENTS:                                                        */
/* ----------                                                        */
/*      F       ORIGINAL DESIGN FREQUENCY SPECIFICATION (HERTZ)      */
/*      TS      SAMPLING INTERVAL (SECONDS)                          */
/*  LAST MODIFIED:  SEPTEMBER 20, 1990                               */
/*                                                                   */
double warp(double *f, double *ts)
{
   double ret_val, angle, twopi;

   twopi = 6.2831853;
   angle = twopi * *f * *ts / 2.0;
   ret_val = tan(angle) * 2.0 / *ts;
   ret_val /= twopi;
   return ret_val;
}





__complex cpowi(__complex a, int n)
{
   int i;
   __complex c;

   c.i = 0;
   c.r = 1;
   for (i = 1; i <= n; ++i) c = cmul(c, a);

   return c;
}


__complex csqrt(__complex z)
{   
   __complex c;
   double x,y,w,r;
   if ((z.r == 0.0) && (z.i == 0.0)) {
      c.r=0.0;
      c.i=0.0;
      return c;
   } else {
      x=fabs(z.r);
      y=fabs(z.i);
      if (x >= y) {
         r=y/x;
         w=sqrt(x)*sqrt(0.5*(1.0+sqrt(1.0+r*r)));
      } else {
         r=x/y;
         w=sqrt(y)*sqrt(0.5*(r+sqrt(1.0+r*r)));
      }
      if (z.r >= 0.0) {
         c.r=w;
         c.i=z.i/(2.0*w);
      } else {
         c.i=(z.i >= 0) ? w : -w;
         c.r=z.i/(2.0*c.i);
      }
      return c;
   }

   return c;
}


__complex conjg(__complex z)
{   
   __complex c;
   c.r=z.r;
   c.i = -z.i;
   return c;
}


__complex cmul(__complex a, __complex b)
{
   __complex c;
   c.r = a.r * b.r - a.i * b.i;
   c.i = a.i * b.r + a.r * b.i;

   return c;
}


__complex cdiv(__complex a, __complex b)
{   
   __complex c;
   double r,den;
   if (fabs(b.r) >= fabs(b.i)) {
      r=b.i/b.r;
      den=b.r+r*b.i;
      c.r=(a.r+r*a.i)/den;
      c.i=(a.i-r*a.r)/den;
   } else {
      r=b.r/b.i;
      den=b.i+r*b.r;
      c.r=(a.r*r+a.i)/den;
      c.i=(a.i*r-a.r)/den;
   }
   return c;
}
