// Sadc10.cpp

/* ----------------------------------------------------- */
/*                      IMPORTANT                        */
/* ----------------------------------------------------- */
/* This driver was written with a SADC10 Version 1.4 A/D Card.
   This A/D card send's serial comm's data at a fixed rate.
   This fixed rate comprises of packets of data plus a time
   packet spread over a second of time.
   
   This format is different than sending a "block" packet of
   one seconds worth of data in a short time. Thus special
   methods were used to code in this driver. Some of the code
   can be upgraded at a later date. 
   As always the code can be improved.

   July 12 / 2001
/* ----------------------------------------------------- */

#include "stdafx.h"
#include <aygshell.h>
//#define STRICT
#include <windows.h>
#include <stdio.h>
#include "seislog.h"
#include "library.h"
#include "msglist.h"
#include "resource.h"
#include "process.h"
#include "eventlog.h"
//#include <time.h>
#include <math.h>

#define WINCE

#ifdef WINCE
    // The type pointer to SYSTEMTIME is not defined 
    // in Windows CE so we define it here.
	typedef SYSTEMTIME* PSYSTEMTIME;
#endif

//
// This structure is used to pass startup- 
// information to the serial-reader thread.
//
struct STARTINFO_DEVREADER { 
   HANDLE hComm;              // Handle of communications device 
   TCHAR CommSpeed[7];		   // Communication speed.
   DWORD ThreadID;            // Thread-ID for parent thread.
   BOOL *SerialRead;          // The serial-reader thread will exit if this becomes FALSE.
   DEVMNGR_INTERF_STRUCT *DS; // Pointer to the exported structure.
   DWORD* BlockNr;            // Pointer to variable who are to receive block numbers.
   DOUBLE TimeCorrection;     // Time correction (seconds).
   BOOL UsePCClock;           // If TRUE then time tagging is based on the PC clock.
   BOOL SyncPCClock;          // If TRUE then device reader thread will syncronize the PC clock.
   BYTE SyncStatus;           // The current time syncronization status.
   HANDLE hSyncStatusMutex;   // Handle to a mutex object used to control access to the
                              // SyncStatus variable in this structure.
};


//
// This structure is read from the binary device config file.
//
struct DEVICE_CONFIG {
   WORD DeviceID;          // MUST be '510'.
   BYTE NumChannels;       // Can be 1 or 4.
   TCHAR PortName[6];       // COM1, COM2 ...
   TCHAR CommSpeed[7];	   // Communication speed.
   BYTE SampleRate;        // Can be 10, 50, 100 . 
   DOUBLE TimeCorrection;  // Time correction (seconds).
   BOOL UsePCClock;        // If TRUE then time tagging is based on the PC clock.
   BOOL SyncPCClock;       // If TRUE then device driver will syncronize the system clock.
};


//
// This sructure represents the data block from the device SADC10.
//
struct DEVICE_BLOCK {

	BYTE Samples[1636];        // Data 
  
};

//
// This sructure represents the time Packet from the device SADC10.
// Some of these fields are not used in this version.
//
struct DEVICE_BLOCK_TIME {

	BYTE hour;        // Hour (Byte 0)
   BYTE min;         // Min
	BYTE sec;         // Sec
	BYTE DCF77;       // DCF77 Status (Byte 3)
	BYTE Line1;       // External Input Bit on A/D Card
	BYTE Line2;       // External Input Bit on A/D Card
	BYTE Det;         // Time Packet Detected = 0x01. No Detection = 0x00.   (Byte 6)
	BYTE L1_Hold;     // Hold Line One Data
	BYTE L2_Hold;     // Hold Line Two Data
	BYTE Part;        // Part of Time Read Flag (Byte 9)
};


//
// This sructure represents the extra info from the cark. Such as OK data.
//
struct DEVICE_BLOCK_EXTRA {

	BOOL OK;        // Hour (Byte 0)
};


//
// Global variables.
//
extern BYTE DebugLevel;
extern BOOL StartGPSThread;
extern HWND hWndFrame;
extern DWORD SysClkSyncThreadID;

//
// Function prototyping.
//
DWORD WINAPI ReaderThreadSADC10(PVOID);
BOOL CALLBACK ConfigDlgProcSADC10(HWND, UINT, WPARAM, LPARAM);
BOOL SADC10LocateTime(  STARTINFO_DEVREADER *,PBYTE, PBYTE, DWORD, DWORD &);
DWORD  SADC10Next(PBYTE , PBYTE,  DWORD , DWORD, DWORD &, LONG &);
void SADC10Startup(STARTINFO_DEVREADER *, WORD , WORD , int,  int &  );
void SADC10Line2(PBYTE, BOOL &);



/* *********************************************************************** */
/*  Manager thread for the SADC10 device. DeviceID=510.                    */
/* *********************************************************************** */
DWORD WINAPI SADC10MngrThrd(PVOID StartupInfo) 
{
   MSG msg;
   int Status;
   BYTE SyncStatus, FixStatus, Index;
   TCHAR ConfigFile[MAX_PATH+1], ChNr, UniqueDeviceName[65], Message[100];
   TCHAR DeviceResources[256], SerPort[10], str10[11];
   WORD BlockSize, NumBlocksPrChannel, SampleRate;
   HANDLE hReaderThread, hComm;
   STARTINFO_DEVMANAGER SI;
   STARTINFO_DEVREADER StartInfo;
   DEVICE_CONFIG DeviceConf, DevConfDlg;
   DEVMNGR_INTERF_STRUCT DS;
   DWORD ParentThreadID, CurrentThreadID, ReaderThreadID;
   DWORD ChannelSize, status, BlockNr;
   MSG_PARAM ID, OD;
   MsgList NotifyList;
   BOOL SerialRead, MemAlloc, ConfigRead;
   

   
   //
   // Perform thread initialization.
   //
   FixStatus=0;
   SyncStatus=2;
   hReaderThread=(HANDLE)NULL;                       // The serial-reader thread is not running.
   CopyMemory((PVOID)&SI, StartupInfo, sizeof(SI));  // Copy startup information to local buffer.
   ParentThreadID=SI.ThreadID;                       // ThreadID for parent thread.
   CurrentThreadID=GetCurrentThreadId();             // ThreadID for this thread.
   

   //
   // Handle device configuration.
   //
   // Read configuration file.
   ConfigRead=FALSE;
   wcscpy(ConfigFile, SI.DeviceDir); 
   wcscat(ConfigFile, TEXT("\\device.cnf"));
   if (ExistFile(SI.DeviceDir, TEXT("device.cnf"))) {
      // Read device config from config file.
      ConfigRead=ReadFileToMemory(ConfigFile, (PVOID)&DeviceConf, (DWORD)sizeof(DeviceConf));
   }
   if (!ConfigRead) {
      // A valid config file could not be read.
      // Set up a default digitizer configuration.
      DeviceConf.DeviceID=510;               // SADC10 uses device-id 500.
      DeviceConf.NumChannels=1;              // Assume model SADC10.
      wcscpy(DeviceConf.PortName, TEXT("COM1"));   // Assume digitizer on COM1.
      wcscpy(DeviceConf.CommSpeed, TEXT("14400")); // Assume comm speed of 9600 baud.
      DeviceConf.SampleRate=50;              // Assume 100hz sample rate.  
      DeviceConf.TimeCorrection=0.0;         // Time correction is off by default.
      DeviceConf.UsePCClock=TRUE;            // Use device time.
      DeviceConf.SyncPCClock=FALSE;          // Syncronize the system clock.

      // Write settings to config file.
      WriteFileFromMemory(ConfigFile, (PVOID)&DeviceConf, (DWORD)sizeof(DeviceConf));
   }

   // 
   // Create a unique device instance name.
   //
   wcscpy(UniqueDeviceName, TEXT("SADC10 at "));
   wcscat(UniqueDeviceName, DeviceConf.PortName);


   //
   // Register this process in the process database.
   //
   if (!SI.ConfigOnly) RegisterProcess(UniqueDeviceName, 1, CurrentThreadID);


   //
   // Create device resource string.
   // 
   wcscpy(DeviceResources, TEXT("Serial ports: "));
   wcscat(DeviceResources, DeviceConf.PortName);


   //
   // Handle time syncronization stuff.
   //
   StartInfo.SyncPCClock=FALSE;
   if (!SI.ConfigOnly && DeviceConf.SyncPCClock) {
      // Device is configured to syncronize the system clock.
      // Check if any other thread is already doing this.
      if (SysClkSyncThreadID) {
         // Turn off time syncronication. Clock is 
         // already beeing syncronized by another thread.
         DeviceConf.SyncPCClock=FALSE;
         // Write warning in the system log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Cannot syncronize system clock. Clock already syncronized."));
         WriteSystemLog(Message, 2);
      } else {
         // System clock is not already syncronized.
         // This thread will attempt to syncronize it.
         StartInfo.SyncPCClock=TRUE;
         SysClkSyncThreadID=CurrentThreadID;
         // Write text on status bar.
         DrawStatusBarText(SyncStatus, UniqueDeviceName);
         // Write message in the system log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Syncronizing system clock."));
         WriteSystemLog(Message, 1);
      }
   }


   //
   // Calculate sample-rate, blocksize, channelsize and number of blocks pr. channel.
   //
   SampleRate=(WORD)DeviceConf.SampleRate;
   BlockSize=(WORD)(sizeof(BLOCK_HEADER)+(SampleRate*4));
   NumBlocksPrChannel=60;
   ChannelSize=NumBlocksPrChannel*BlockSize;


   //
   // Open the serial port.
   //
   if (!SI.ConfigOnly) {
#ifdef WINCE
	  // All ports in Windows CE must end with ':'
	  TCHAR CEPortName[7];
	  wcscpy(CEPortName, DeviceConf.PortName);
	  wcscat(CEPortName, TEXT(":"));
	  hComm=CreateFile(CEPortName, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, NULL);
#else
	  wcscpy(SerPort, TEXT("\\\\.\\")); wcscat(SerPort, DeviceConf.PortName);
      hComm=CreateFile(SerPort, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
#endif
      if (hComm==INVALID_HANDLE_VALUE) {
         // Update program log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Unable to open communication port "));
         wcscat(Message, DeviceConf.PortName);
         wcscat(Message, TEXT("."));
         WriteSystemLog(Message, 3);
      }
   }

   
   //
   // Initialize the interface structure.
   //
   // Insert manufacturer name and model.
   wcscpy(DS.Manufacturer, TEXT("M.Mariotti"));
   wcscpy(DS.Model, TEXT("SADC10"));

   // Insert samplerate.
   DS.SampleRate=SampleRate;

   // Insert number of samples in each block.
   DS.SamplesPrBlock=SampleRate;

   // Insert number of channels
   DS.NumChannels=DeviceConf.NumChannels;

   // Insert block size, block header included.
   DS.BlockSize=BlockSize;

   // Insert block size measured in milliseconds.
   DS.BlockSizeInMS=1000;

   // Insert number of blocks pr channel.
   DS.NumBlocksPrChannel=NumBlocksPrChannel;

   // Make sure all pointers to channel data is NULL.
   for (ChNr=0;ChNr<MAX_CHANNELS_PR_DEVICE;ChNr++) DS.ChannelData[ChNr]=(PVOID)NULL;
   
   
   //
   // Allocate memory for the channel buffers, and set up the pointers to the buffers.
   //
   MemAlloc=TRUE;
   for (Index=0;Index<DeviceConf.NumChannels;Index++) {
      DS.ChannelData[Index]=(PVOID)LocalAlloc(LPTR, (UINT)ChannelSize);
      if (DS.ChannelData[Index]==NULL) MemAlloc=FALSE;
   }

   // Check that memory allocation succeded.
   if (!MemAlloc) {
         // Update program log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(":"));
         wcscat(Message, TEXT(" Could not allocate memory."));
         WriteSystemLog(Message, 3);
   }
   

   //
   // If running in debug mode, write a message to the log.
   //
   if (DebugLevel) {
      wcscpy(Message, UniqueDeviceName);
      wcscat(Message, TEXT(": Thread started."));
      WriteSystemLog(Message, 1);
   }
  

   //
   // Signal that the thread is started.
   //
   SetEvent(SI.StartEvent);


   //
   // Enter the thread message loop. The thread will now start to process messages.
   //
   while (GetMessage(&msg, (HWND)NULL, 0, 0)) 
   { 
   
      // If debuglevel is 2 then log all messages recieved.
      if (DebugLevel>1) {
         TCHAR str10[11];
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Recieved message "));
         _itow(msg.message, str10, 10);
         wcscat(Message, str10);
         WriteSystemLog(Message, 1);
      }

      // Save message parameters.
      ZeroMemory((PVOID)&ID, (DWORD)sizeof(ID));
      if (msg.lParam) ID=*((MSG_PARAM*)msg.lParam);
      
      // Initialize return-message parameter to default values.
      ZeroMemory((PVOID)&OD, (DWORD)sizeof(OD));
      OD.bStatus=TRUE;
      OD.idThread=CurrentThreadID;


      // Process message.
      switch (msg.message) {
         case TM_START:                            
         //
         // This message is sendt from a higher level 
         // thread to start reading the serial port.
         //
         if (hReaderThread==NULL && hComm!=INVALID_HANDLE_VALUE && MemAlloc && !SI.ConfigOnly) {
            if (SysClkSyncThreadID && SysClkSyncThreadID!=CurrentThreadID) {
               // The system clock is syncronized by some other thread.
               // Subscribe to the TM_TIMESYNC_STATUS message from thread.
               Subscribe(TM_TIMESYNC_STATUS, SysClkSyncThreadID, CurrentThreadID, (HWND)NULL);

               // Get the current syncronization status.
               StartInfo.SyncStatus=GetCurrentTimeSyncStatus(SysClkSyncThreadID);
            } else {
               // The system clock is not syncronized and
               // there is no other syncronization possibilities.
               StartInfo.SyncStatus=0;
               // Create a mutex used to access the 'StartInfo.SyncStatus' variable.
               StartInfo.hSyncStatusMutex=CreateMutex(NULL, FALSE, NULL);
            }

            // Set up startup information for the serial reader thread routine.
            SerialRead=TRUE;
            StartInfo.hComm=hComm;
				wcscpy(StartInfo.CommSpeed, DeviceConf.CommSpeed);
            StartInfo.ThreadID=CurrentThreadID;
            StartInfo.SerialRead=&SerialRead;
            StartInfo.DS=&DS;
            StartInfo.BlockNr=&BlockNr;
            StartInfo.TimeCorrection=DeviceConf.TimeCorrection;
            StartInfo.UsePCClock=DeviceConf.UsePCClock;

            // Start the thread.
            hReaderThread=CreateThread(NULL, 0, ReaderThreadSADC10, (VOID*)&StartInfo, CREATE_SUSPENDED, &ReaderThreadID);
            SetThreadPriority(hReaderThread, THREAD_PRIORITY_ABOVE_NORMAL);  // Set priority above normal.
            ResumeThread(hReaderThread); 
         }

         // Set return values. 
         if (!hReaderThread) {
            OD.bStatus=FALSE;
            OD.wStatus=1;
         }
         break;


         
         case TM_STOP:
         //
         // This message is sendt from a higher level thread to stop
         // reading the serial port. Returns the default return values.
         //
         if (hReaderThread) {
            // Make the serial thread loop exit, thereby stopping the thread.
            SerialRead=FALSE;

            // Wait up to 2 seconds for the thread to stop.
            status=WaitForSingleObject(hReaderThread, 2000);
            if (status==WAIT_TIMEOUT) TerminateThread(hReaderThread, 0);

            // Unsubscribe to the TM_TIMESYNC_STATUS message as needed.
            if (SysClkSyncThreadID && SysClkSyncThreadID!=CurrentThreadID) {
               UnSubscribe(TM_TIMESYNC_STATUS, SysClkSyncThreadID, CurrentThreadID, NULL);
               CloseHandle(StartInfo.hSyncStatusMutex); // Delete mutex object.
            }

            // The thread is now stopped, invalidate the thread handle.
            hReaderThread=NULL;
         }
         break;


         case TM_CONFIGURE:            
         //
         // This message is sendt from a higher level thread
         // when it wants to configure the digitizer settings.
         //
         if (hReaderThread==NULL) {
            // Let user work on a copy of the current device settings.
            DevConfDlg=DeviceConf;

            // Display the dialog box.
            Status=DialogBoxParam(SI.hInstance, MAKEINTRESOURCE(IDD_CNF_SADC10), NULL, (DLGPROC)ConfigDlgProcSADC10, (LPARAM)&DevConfDlg);   

            // Save settings if requested.
            if (Status==1) {
               // Save the new configuration.
               DeviceConf=DevConfDlg;

               // Write configuration to disk.
               WriteFileFromMemory(ConfigFile, (PVOID)&DeviceConf, (DWORD)sizeof(DEVICE_CONFIG));

               // Recreate the unique name of this device instance.
			   wcscpy(UniqueDeviceName, TEXT("SADC10 at "));
			   wcscat(UniqueDeviceName, DeviceConf.PortName);
            }

            // Set return values. 
            if (Status==0) {
               OD.bStatus=FALSE;
               OD.wStatus=100;
            }
         }
         break;


         case TM_SHOW_CONFIG:
            //
            // When this message is received, dump all configuration
            // parameters that is not given in the device resource 
            // string to the system log.
            //
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(": Time Correction="));
            swprintf(str10,TEXT("%.3f"), DeviceConf.TimeCorrection);
            wcscat(Message, str10);
            WriteSystemLog(Message, 1);
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(": Syncronize system clock="));
            if (DeviceConf.SyncPCClock) wcscat(Message, TEXT("TRUE"));
            if (!DeviceConf.SyncPCClock) wcscat(Message, TEXT("FALSE"));
            WriteSystemLog(Message, 1);
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(": Communication speed="));
            wcscat(Message, DeviceConf.CommSpeed);
			wcscat(Message, TEXT(" baud"));
            WriteSystemLog(Message, 1);
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(": Use system clock for time tagging="));
            if (DeviceConf.UsePCClock) wcscat(Message, TEXT("TRUE"));
            if (!DeviceConf.UsePCClock) wcscat(Message, TEXT("FALSE"));
            WriteSystemLog(Message, 1);
         break;


         case TM_GET_INTERFACE:
         //
         // This message is used to get a pointer to the interface structure.
         //
         OD.vPointer=(PVOID)&DS;
         break;   


         case TM_GET_UNIQUE_DEVICE_NAME:
         //
         // This message is used to get a string, 
         // identifying this unique device instance.
         //
         wcscpy(OD.cStatus, UniqueDeviceName);
         break;   


         case TM_GET_RESOURCE_STRING:
         //
         // This message is used to get a string, 
         // identifying the system resources used by this device.
         //
         wcscpy(OD.cStatus, DeviceResources);
         break;   


         case TM_SUBSCRIBE:
         //
         // This message is sendt from a window or thread
         // Who wants to be informed about an event.
         // This message must be accompanied by a MSG_PARAM structure.
         // The MSG_PARAM structure must have a pointer to a valid
         // SUBSCRIBE structure.
         //
         //
         // Add window/thread to notify list.
         if (ID.vPointer) if (!NotifyList.Subscribe((SUBSCRIBE*)ID.vPointer)) {
            OD.bStatus=FALSE;
            OD.wStatus=3;
         }
         break;   


         case TM_UNSUBSCRIBE:
         //
         // This message is sendt from a window or thread who
         // wants to remove a notification from the notify list.
         // This message must be accompanied by a MSG_PARAM structure.
         // The MSG_PARAM structure must have a pointer to a valid
         // SUBSCRIBE structure.
         //
         //
         // Remove notification from list.
         if (ID.vPointer) NotifyList.UnSubscribe((SUBSCRIBE*)ID.vPointer);
         
         break;   



         case TM_EXIT:                 
         //
         // This is a request for the thread to end.
         // Close down all processes that are depandent on this thread.
         // The 'StopDependantThreads' function will return immediatly,
         // and will send a 'TM_DEPTHREADS_STOPPED' message back when
         // all dependant threads have been stopped.
         StopDependantThreads();
         break;



         case TM_DEPTHREADS_STOPPED:
         //
         // This message indicates that all dependent threads have been stopped.
         // The thread will cleanup and exit when this message is recieved.         
         //

		 // Next if statement Added 04.11.2003. Differs from PC version.


         // First stop the device-reader thread.
         if (hReaderThread) {
            SerialRead=FALSE;                                            // Signal the reader thread.
            status=WaitForSingleObject(hReaderThread, 2000);             // Wait for the thread to stop.
            if (status==WAIT_TIMEOUT) TerminateThread(hReaderThread, 0); // If thread wont stop, kill it.
            hReaderThread=NULL;
         }

         if (!SI.ConfigOnly) {
            // Unregister this process from the process database.
            UnregisterProcess(CurrentThreadID);

            // Close the communication port.
            CloseHandle(hComm);

            // Handle time syncronization stuff.
            if (DeviceConf.SyncPCClock && StartInfo.SyncPCClock) {
               // Clear the SysClkSyncThreadID variable.
               SysClkSyncThreadID=0;
               // Clear the status bar.
               PostMessage(hWndFrame, TM_WRITE_STATUSBAR, 0, (LPARAM)TEXT(""));
            }
         }

         // Free memory buffers.
         for (Index=0;Index<DeviceConf.NumChannels;Index++) {
            if (DS.ChannelData[Index]) LocalFree((HLOCAL)DS.ChannelData[Index]); 
         }

         // End the message loop.
         ClearMessageQueue(); // Delete all messages in message queue.
         PostQuitMessage(0);  // Post WM_QUIT message.   
         break;
   
         
         //
         // Handle messages from device reader thread.
         //
         case TM_BUFFERUPDATED:  // A new block of data has been inserted into buffer.
         // Send Notification to all subscribed threads/windows.
         // The notofication is accompanied with this thread's thread-ID, and the blocknr of the updated block.
         NotifyList.Notify((UINT)TM_BUFFERUPDATED, (WPARAM)CurrentThreadID, (LPARAM)BlockNr);
         break;


         case TM_TIMESYNC_STATUS:
         //
         // This message is sendt from a time syncronization
         // thread (other then this thread or its child) to inform
         // about a change in the syncronization status. This thread
         // subscribe to this message from the other thread.
         WaitForSingleObject(StartInfo.hSyncStatusMutex, INFINITE); 
         StartInfo.SyncStatus=(BYTE)msg.wParam;
         ReleaseMutex(StartInfo.hSyncStatusMutex);
         break;
         
         
         case TM_GET_TIMESYNC_STATUS:
         //
         // This message is sendt from a window or thread who
         // wants to know the status of the GPS time syncronisation.
         // This message must be accompanied by a MSG_PARAM structure.
         //
         //
         // Returns the operational status of the thread.
         // Values are:
         //    0 :System clock is beeing syncronized, no error.
         //    1 :System clock is NOT beeing syncronized, but no error (fix lost).
         //    2 :System clock is NOT beeing syncronized, communication error.
         //
         OD.wStatus=(WORD)SyncStatus;
         break;


         case TM_DEVICEREADER_NODATAINPUT:   
         //
         // The device reader has lost data input.
         //
         // Send Notification to all subscribed threads/windows.
         NotifyList.Notify((UINT)TM_DEVICEREADER_NODATAINPUT, WPARAM(CurrentThreadID), 0);

         // Handle GPS stuff.
         if (DeviceConf.SyncPCClock) {
            SyncStatus=2;
            NotifyList.Notify((UINT)TM_TIMESYNC_STATUS, (WPARAM)SyncStatus, 0);
            DrawStatusBarText(SyncStatus, UniqueDeviceName);
         }

         // Write a message to the log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Not receiving data."));
         WriteSystemLog(Message, 2);
         break;


         case TM_DEVICEREADER_DATAINPUTOK:   
         //
         // The device reader has regained data input.
         //
         // Send Notification to all subscribed threads/windows.
         NotifyList.Notify((UINT)TM_DEVICEREADER_DATAINPUTOK, WPARAM(CurrentThreadID), 0);

         // Handle GPS stuff.
         if (DeviceConf.SyncPCClock) {
            SyncStatus=0;
            if (!FixStatus) SyncStatus=1;
            NotifyList.Notify((UINT)TM_TIMESYNC_STATUS, (WPARAM)SyncStatus, 0);
            DrawStatusBarText(SyncStatus, UniqueDeviceName);
         }

         // Write a message to the log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Receiving data."));
         WriteSystemLog(Message, 1);
         break;


         case TM_DEVICEREADER_BADBLOCK:   
         //
         // A bad block of data was received from the device.
         //
         // Send Notification to all subscibed threads/windows.
         // The notification is accompanied with this thread's thread-ID, and the blocknr of the bad block.
         NotifyList.Notify((UINT)TM_DEVICEREADER_BADBLOCK, (WPARAM)CurrentThreadID, (LPARAM)BlockNr);

         // Write a message to the log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Received a bad block from device."));
         WriteSystemLog(Message, 2);
         break;


         case TM_DEVICEREADER_NOFIX:   
         //
         // The GPS reader-thread indicates that fix has been lost.
         //
		 // Handle GPS stuff.
		 if (DeviceConf.SyncPCClock) {
			FixStatus=0;
			if (!SyncStatus) SyncStatus=1;
			NotifyList.Notify((UINT)TM_TIMESYNC_STATUS, (WPARAM)SyncStatus, 0);
	        DrawStatusBarText(SyncStatus, UniqueDeviceName);
		 }

         // Write a message to the log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": A/D Card Time fix lost."));
         WriteSystemLog(Message, 2);
         break;


         case TM_DEVICEREADER_FIXOK:   
         //
         // The GPS reader-thread indicates that fix has been regained.
         //
		 // Handle GPS stuff.
		 if (DeviceConf.SyncPCClock) {
		 	FixStatus=1;
			if (SyncStatus==1) SyncStatus=0;
			NotifyList.Notify((UINT)TM_TIMESYNC_STATUS, (WPARAM)SyncStatus, 0);
            DrawStatusBarText(SyncStatus, UniqueDeviceName);
		 }
         
         // Write a message to the log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": A/D Card Time fix OK."));
         WriteSystemLog(Message, 1);
         break;
      }
      
      // If this is requested by the thread who sent the message
      // then return status values, and release the caller thread.
      if (((MSG_PARAM*)msg.lParam)!=NULL && msg.wParam!=NULL) {
         *(MSG_PARAM*)msg.lParam=OD;
         SetEvent((HANDLE)msg.wParam);
      }
   } 


   // Free up memeory allocated in the MsgList class object.
   NotifyList.~MsgList();

   // If running in debug mode, write a message to the log.
   if (DebugLevel) {
      wcscpy(Message, UniqueDeviceName);
      wcscat(Message, TEXT(": Thread stopped."));
      WriteSystemLog(Message, 1);
   }

   // Exit this thread
   ExitThread(0);
   return 0L;
   
}




/* *********************************************************************** */
/*  Serial port reader procedure for the SADC10                            */
/* *********************************************************************** */
DWORD WINAPI ReaderThreadSADC10(PVOID StartupStruct)
{

   VOID *DataBlocksCh1, *DataBlocksCh2, *DataBlocksCh3, *DataBlocksCh4; 
   BOOL FirstTime,Lines,BytesOK, DataLost, ReadStatus;
   TCHAR DevCntrlStr[50];
   BYTE *bPtr, TimeQuality, FixStatus, OldFixStatus, SyncStatus;  
   WORD BlockSize, NumChannels, NumBlocksPrChannel, SampleRate, SampleNr;
   DWORD BytesRead, BytesToRead, BlockNr, OffsetCurrentSample, OffsetToBlockData;
   DWORD WhichField,NextField,ch1,ToAverage,chtime,Min10;
   LONG LIValue,LValue[10];
   int FieldType,speed,Version;
   long tm;
   DOUBLE TransferTime;
   DCB dcb;
   STARTINFO_DEVREADER *SI;
   BLOCK_HEADER DBH;
   DEVICE_BLOCK DtaBlck;
   DEVICE_BLOCK_TIME  DataTime;
   DEVICE_BLOCK_EXTRA Card;
   extern BOOL StartGPSThread;
   struct tm *nt;
   SYSTEMTIME TimeTag, SysTime, BlockTime, ReadTime;
   COMMTIMEOUTS ct={0, 0, 1500, 0, 0};
   OVERLAPPED osWrite = {0};
  


   // Copy values from startup structure.
   SI=(STARTINFO_DEVREADER*)StartupStruct;         // Use pointer of correct type.
   NumChannels=SI->DS->NumChannels;                // Number of channels digitizer is using.
   SampleRate=SI->DS->SampleRate;                  // Sample rate.
   NumBlocksPrChannel=SI->DS->NumBlocksPrChannel;  // Number of 1 sec. blocks in buffer.
   BlockSize=SI->DS->BlockSize;                    // Size of 1 sec. block header included
   DataBlocksCh1=SI->DS->ChannelData[0];           // Pointer to first block channel 1.
   DataBlocksCh2=SI->DS->ChannelData[1];           // Pointer to first block channel 2.
   DataBlocksCh3=SI->DS->ChannelData[2];           // Pointer to first block channel 3.
   DataBlocksCh4=SI->DS->ChannelData[3];           // Pointer to first block channel 4.

   // Initialize some variables.
   FixStatus=0;       // So far we have no satelite fix.
   OldFixStatus=0;    // We have no previous fix status.
   BlockNr=0;         // Start filling data into first block.
   DataLost=TRUE;     // At this point we have no data input.

   /* -------------------- */
   /* SADC10 Specific Data */
   /* -------------------- */

   DataTime.Part = 0x00;  
   BytesOK       = TRUE; // Check for vaild time packet
   Card.OK       = TRUE; // Used when BytesOk error occurs
   Min10         = 0;    // Corrects for 10 minute data slip ! (in Bytes)
   Lines         = TRUE; // Display Cards two input line status
   FirstTime     = TRUE; // First Time Flag.
   DataTime.Det  = 0x00; // NO Time Packet detected
   ch1           = 4;    // Bytes for  one channel ch1, 
   chtime        = 6;    // Bytes in a Time sequence
	//

   // In Use. Bytes used for data Packets
   if ((NumChannels > 0) && (NumChannels < 5))
	   ToAverage = NumChannels * ch1;
   else {    
	   ToAverage = ch1;
	   NumChannels = 1; // The Never Happen error !
   }
   
	// Bytes Needed to be read
	BytesToRead = (ToAverage * SampleRate) + chtime ; 
    
	// Get the baud rate and convert to to an integer (speed)
    if (swscanf(SI->CommSpeed, TEXT("%d"), &speed) == 0) speed = 14400; // default if error
	if ((speed < 9600) || (speed > 38400)) speed = 14400;
    speed = speed / 10; // Baud / 10 gives bytes per second

	// Transfer Time to read Data in one second
    TransferTime=(DOUBLE)((BytesToRead)/(DOUBLE)speed);


   //
   //Initialize the communications parameters.
   //
   SetupComm(SI->hComm, BytesToRead*2, 0);   // Input buffer will hold 2 seconds of data.
   GetCommState(SI->hComm, &dcb);
#ifdef WINCE
   dcb.BaudRate     = _wtoi(SI->CommSpeed);
   dcb.Parity       = NOPARITY;
   dcb.StopBits     = ONESTOPBIT;
   dcb.ByteSize     = 8;
#else
   wcscpy(DevCntrlStr, TEXT("baud="));
   wcscat(DevCntrlStr, SI->CommSpeed);
   wcscat(DevCntrlStr, TEXT(" parity=n data=8 stop=1"));
   BuildCommDCB(DevCntrlStr, &dcb);
#endif
   dcb.fAbortOnError=FALSE;
   SetCommState(SI->hComm, &dcb);       // Set serial port mode. No flow control is used.
   SetCommTimeouts(SI->hComm, &ct);     // Set communication timeouts..
   PurgeComm(SI->hComm, PURGE_RXCLEAR); // Clear the input buffer.


   /* ---------------------------------------------------------------------- */
   /* Send Messages and setting up the SADC10 card via the Seial Comm's Port */
   /* ---------------------------------------------------------------------- */
    SADC10Startup(SI, NumChannels, SampleRate, speed, Version);
  
  
 
   /* ******************************************************** */
   /*                                                          */
   /*               Start reading the serial port.             */
   /*                                                          */
   /* ******************************************************** */
   while(*SI->SerialRead) {
	   //
       // Read Serial data and correct for any time Packet slip.
	   //
	   if (Min10 > 0) { // Get rid of those extra Bytes !
		  ReadStatus=ReadFile(SI->hComm, (PVOID)&DtaBlck, Min10, &BytesRead, NULL);		
		  Min10 = 0;
	   }
     
	 ReadStatus=ReadFile(SI->hComm, (PVOID)&DtaBlck, BytesToRead, &BytesRead, NULL);		

     
     GetSystemTime(&ReadTime); // Get current time/date. This is UTC time.

	 
	 /* ----------------------------------------------- */
     /* Sync the serial bit stream with the time packet */
	 /* TimePacket followed by data packets.            */
	 /* ----------------------------------------------- */

     if ((FirstTime) || (Card.OK == FALSE)) {
	  do {
		ReadStatus=ReadFile(SI->hComm, (PVOID)&DtaBlck, 1, &BytesRead, NULL);		
	  } while  ( DtaBlck.Samples[0] != 0x81); //(SADC10LocateTime((PBYTE)&DtaBlck,BytesRead) == false);

	  ReadStatus=ReadFile(SI->hComm, (PVOID)&DtaBlck, BytesToRead-1, &BytesRead, NULL);		
      ReadStatus=ReadFile(SI->hComm, (PVOID)&DtaBlck,BytesToRead, &BytesRead, NULL);		
      GetSystemTime(&ReadTime); // Get current time/date. This is UTC time.
      OffsetToBlockData=(BlockNr*BlockSize)+sizeof(BLOCK_HEADER);
	  FirstTime = FALSE;
	 }

  	  // 
	  // Make sure this is  SADC10 card is version 1.40 or greater.
	  // Make sure the Samples per second fits in the Cards sample rate.
	  //

	   // Force error on wrong version
	   if (Version < 140) BytesRead = 0; 

	   // Force error on Wrong Sample rate
	   if (((NumChannels *4)*SampleRate )+6 > speed) BytesRead = 0; 

	  /* ----------------------------------------------------- */
	  /*                      IMPORTANT                        */
	  /* ----------------------------------------------------- */
	  /* Make sure the right amount of data is read            */
	  /* ----------------------------------------------------- */

      if ((BytesRead == BytesToRead ) || ( Min10)){
       // 
	   // Make Sure The Data is Valid 
	   //
	   
	   BytesOK = SADC10LocateTime(SI, (PBYTE)&DtaBlck,(PBYTE)&DataTime,BytesRead, Min10);

         
	   // If Min10 > 0 Then an Time packet has slipped. This is in the SADC10
	   // card and occurs every 10 minutes.
       // Min10 comes from SADC10LocateTime
	   //
	   if (Min10 > 0)   BytesOK = TRUE;  // Force Read on Time slip
	   if (Min10 == -2) BytesOK = FALSE; // No Time Pactet detected.
    
       WhichField = 0; // Start With Zero index
 
 	  
	  /* --------------- */
      /* ALL OK So Far ? */
	  /* --------------- */

	  if (BytesOK == TRUE) { // The block from the digitizer seems to be OK.
			//
		    // Reset Card OK flag via a count down 
		    //
            Card.OK = TRUE;

			//
            // Write Line 2 state if it changes
			//
			SADC10Line2((PBYTE)&DataTime,Lines);

		    /* ****************************** */
		    /*  Get time from incoming block. */
		    /* ****************************** */
        //    time(&tm);  
        //    nt    = gmtime(&tm);    // read GMT System Time

            // Get current time/date. This is UTC time.
            GetSystemTime(&SysTime);

			//
			//
			// Set Date from System Time 
		  //BlockTime.wYear=(WORD)nt->tm_year + 1900;  
			BlockTime.wYear=(WORD)SysTime.wYear;  
            //BlockTime.wMonth=(WORD)nt->tm_mon + 1;
			  BlockTime.wMonth=(WORD)SysTime.wMonth;
            BlockTime.wDayOfWeek= 0;
            //BlockTime.wDay=(WORD)nt->tm_mday;
			BlockTime.wDay=(WORD)SysTime.wDay;
			// 
            // Time From A/D card
			//
            // Correct for Packet slip in time packet
            if (Min10 > 0) AddToTime(&BlockTime, 1.0 ); 
			else  {
				BlockTime.wHour  =(WORD)DtaBlck.Samples[3 ]; 
				BlockTime.wMinute=(WORD)DtaBlck.Samples[2 ];
				BlockTime.wSecond=(WORD)DtaBlck.Samples[1 ];
				BlockTime.wMilliseconds=(WORD)(0); // Boundary on second
			}
            
	        // Apply user specified time correction to block time.
            if (SI->TimeCorrection>0.0) AddToTime(&BlockTime, SI->TimeCorrection);
            if (SI->TimeCorrection<0.0) SubstractFromTime(&BlockTime, -SI->TimeCorrection);

			//
            // Determine SyncStatus 
			//
	        DataTime.Det = (DataTime.Det >> 1) & 0xFE; // 8 second countdown 
 			if (DataTime.Det) SyncStatus=0x03;
			else              SyncStatus=0x01;

			//
			// Get the GPS status from the block.
			//
            // Values are:
            // 0 - RTC error  								     
            // 1 - Real time clock only. (No GPS connected, or no fix since PowerON).
            // 2 - GPS has no satellite fix (we have had fix).
            // 3 - GPS has satellite fix.


            if (SI->SyncPCClock) {
               // The system time will be set equal to the
               // block time + transfertime + 1 second.
               SysTime=BlockTime;
               AddToTime(&SysTime, 1.0 ); 
               SetSystemTime(&SysTime);
            }

            //
			// Find the time to be inserted into the block header.
            // This is either the devicetime, or a corrected system time.
			//
            if (SI->UsePCClock) {
               // The system clock will be used for timetagging.
               // Use the current time, and compensate for
               // the transfer time + 1 second digitizer delay.
               TimeTag=ReadTime;                                
               SubstractFromTime(&TimeTag, 1.0);
					
			   // Find the time quality.
               if (SysClkSyncThreadID) {
                  // The clock is syncronized. We will have to 
                  // get the syncstatus from the parent thread.
                  WaitForSingleObject(SI->hSyncStatusMutex, INFINITE); 
                  SyncStatus=SI->SyncStatus;
                  if (SI->SyncStatus==0) TimeQuality=2;
                  if (SI->SyncStatus==1) TimeQuality=1;
                  if (SI->SyncStatus==2) TimeQuality=3;
                  ReleaseMutex(SI->hSyncStatusMutex);
               } else {
                  // The clock is NOT syncronized. 
                  TimeQuality=0;
               }
            } else {
               // The block time will be used for timetagging.
               TimeTag=BlockTime;
			   // Find the time quality.
               if (SyncStatus==0) { TimeQuality=3; FixStatus=0; }
               if (SyncStatus==1) { TimeQuality=1; FixStatus=0; }
               if (SyncStatus==2) { TimeQuality=1; FixStatus=0; }
               if (SyncStatus==3) { TimeQuality=2; FixStatus=1; }
            }
            
            //
            // Create a block header and insert in all channels.
			//
            DBH.BlockID=BlockNr;
            DBH.TimeTag=TimeTag;
            DBH.TimeQuality=TimeQuality;
            DBH.NullData=FALSE;
            DBH.DataLost=DataLost;
            DBH.EventDetected=FALSE;
            DBH.EventTime=0;
            bPtr=((BYTE*)DataBlocksCh1)+(BlockSize*BlockNr);
            CopyMemory((VOID*)bPtr, (VOID*)&DBH, (DWORD)sizeof(BLOCK_HEADER));
            if ((NumChannels==3) || (NumChannels==4)) {
               bPtr=((BYTE*)DataBlocksCh2)+(BlockSize*BlockNr);
               CopyMemory((VOID*)bPtr, (VOID*)&DBH, (DWORD)sizeof(BLOCK_HEADER));
               bPtr=((BYTE*)DataBlocksCh3)+(BlockSize*BlockNr);
               CopyMemory((VOID*)bPtr, (VOID*)&DBH, (DWORD)sizeof(BLOCK_HEADER));
               if (NumChannels==4) {
				   bPtr=((BYTE*)DataBlocksCh4)+(BlockSize*BlockNr);
                   CopyMemory((VOID*)bPtr, (VOID*)&DBH, (DWORD)sizeof(BLOCK_HEADER));
			   }
            }
   
			// Calculate offset from start of data buffer,
            // into data part of currrent block.
            OffsetToBlockData=(BlockNr*BlockSize)+sizeof(BLOCK_HEADER);

			/* ************************************************* */
            /* Insert samples from digitizer into current block. */
			/* ************************************************* */
            if (NumChannels==1) { // This code is for 1 channel digitizer.
               OffsetCurrentSample=OffsetToBlockData;
               for (SampleNr=0;SampleNr<SampleRate;SampleNr++) {
         
                // Read One Channel
				do
				{
                   FieldType = SADC10Next((PBYTE)&DtaBlck,(PBYTE)&DataTime,WhichField,BytesToRead,NextField,LIValue);
                   WhichField = NextField;
				} while (FieldType < 1) ;
	            LValue[FieldType] = LIValue;
 
				 // Save the Data values
                *((LONG*)(((BYTE*)DataBlocksCh1)+OffsetCurrentSample))=LValue[1];
               
			    // Next Sample offset
                OffsetCurrentSample=OffsetCurrentSample+4;
			   } // For SampleNr
			}    // If NumChannels = 1
            if ((NumChannels==3) || (NumChannels==4)) { // This code is for 3 or 4 channel digitizer.
               OffsetCurrentSample=OffsetToBlockData;
               for (SampleNr=0;SampleNr<SampleRate;SampleNr++) {
            	// This Array of data method compensates for lost data and 
				// time slip.
				//
			    // Read One Channel
				do
				{
                   FieldType = SADC10Next((PBYTE)&DtaBlck,(PBYTE)&DataTime,WhichField,BytesToRead,NextField,LIValue);
                   WhichField = NextField;
				} while (FieldType < 1) ;
	            LValue[FieldType] = LIValue;
                // Read the next Channel
	            do
				{
                   FieldType = SADC10Next((PBYTE)&DtaBlck,(PBYTE)&DataTime,WhichField,BytesToRead,NextField,LIValue);
                   WhichField = NextField;
                } while (FieldType < 1) ;
	            LValue[FieldType] = LIValue;
                // Read The Last of the 3 Channels
	            do
				{
                   FieldType = SADC10Next((PBYTE)&DtaBlck,(PBYTE)&DataTime,WhichField,BytesToRead,NextField,LIValue);
 	  			   WhichField = NextField;
				} while (FieldType < 1) ;
	             LValue[FieldType] = LIValue;
 
				 // There are 4 Channels
				 if (NumChannels == 4) {
					  do
					  {
						FieldType = SADC10Next((PBYTE)&DtaBlck,(PBYTE)&DataTime,WhichField,BytesToRead,NextField,LIValue);
 	  					WhichField = NextField;
						} while (FieldType < 1) ;
	             LValue[FieldType] = LIValue;
				 }

				 // Save the Data values
                *((LONG*)(((BYTE*)DataBlocksCh1)+OffsetCurrentSample))=LValue[1];
                *((LONG*)(((BYTE*)DataBlocksCh2)+OffsetCurrentSample))=LValue[2];
                *((LONG*)(((BYTE*)DataBlocksCh3)+OffsetCurrentSample))=LValue[3];
                if (NumChannels == 4) *((LONG*)(((BYTE*)DataBlocksCh4)+OffsetCurrentSample))=LValue[4];
			    
				// Next Sample offset
                OffsetCurrentSample=OffsetCurrentSample+4;
               } // For SampleNr
            }    // NumSamples = 3 or 4
         } // BytesOk = TRUE
		 else {
            //
            // The block from the digitizer does NOT seem to be OK.
            // Since this is probably a rear happening, we will just
            // fill this block with zeros and mark it with a zero flag
            // in the header. This is done so the channel thread can 
            // recognize it as a zero block, and replace the zeros 
            // with the currently calculated device offset value.
            //
            
			 Card.OK = FALSE;

            // Find the timequality factor for this block.
            if (!SI->SyncPCClock) {
               // Another thread or is syncronizing the system clock.
               // or the system clock is unsyncronized.
               if (SysClkSyncThreadID) {
                  // The clock is syncronized. We will have to 
                  // get the syncstatus from the device manager.
                  WaitForSingleObject(SI->hSyncStatusMutex, INFINITE); 
                  SyncStatus=SI->SyncStatus;
                  if (SI->SyncStatus==0) TimeQuality=2;
                  if (SI->SyncStatus==1) TimeQuality=1;
                  if (SI->SyncStatus==2) TimeQuality=3;
                  ReleaseMutex(SI->hSyncStatusMutex);
               } else {
                  // The clock is NOT syncronized. 
                  TimeQuality=0;
               }
            }

            // Find the time to be inserted into the block header.
            // Since this block is bad, the system clock must be 
            // used for timetagging.
            TimeTag=ReadTime;                           // Use the time when the block was read.
            SubstractFromTime(&TimeTag, TransferTime);  // Compensate for  + 1 sec. 
            // Apply additional user specified time correction.
            if (SI->TimeCorrection>0.0) AddToTime(&TimeTag, SI->TimeCorrection);
            if (SI->TimeCorrection<0.0) SubstractFromTime(&TimeTag, SI->TimeCorrection);

            // Create a block header and insert in all channels.
            DBH.BlockID=BlockNr;
            DBH.TimeTag=TimeTag;
            DBH.TimeQuality=TimeQuality;
            DBH.NullData=TRUE;
            DBH.DataLost=DataLost;
            DBH.EventDetected=FALSE;
            DBH.EventTime=0;
            bPtr=((BYTE*)DataBlocksCh1)+(BlockSize*BlockNr);
            CopyMemory((VOID*)bPtr, (VOID*)&DBH, (DWORD)sizeof(BLOCK_HEADER));
            if ((NumChannels==3) || (NumChannels==4)) {
               bPtr=((BYTE*)DataBlocksCh2)+(BlockSize*BlockNr);
               CopyMemory((VOID*)bPtr, (VOID*)&DBH, (DWORD)sizeof(BLOCK_HEADER));
               bPtr=((BYTE*)DataBlocksCh3)+(BlockSize*BlockNr);
               CopyMemory((VOID*)bPtr, (VOID*)&DBH, (DWORD)sizeof(BLOCK_HEADER));
			   if (NumChannels==4) {
                   bPtr=((BYTE*)DataBlocksCh4)+(BlockSize*BlockNr);
                   CopyMemory((VOID*)bPtr, (VOID*)&DBH, (DWORD)sizeof(BLOCK_HEADER));
			   }
            }

            // Insert null data into current block.
            if (NumChannels==1) ZeroMemory((PVOID)((PBYTE)DataBlocksCh1+OffsetToBlockData), SampleRate*4);
            if ((NumChannels==3) || (NumChannels == 4)) {
               ZeroMemory((PVOID)((PBYTE)DataBlocksCh1+OffsetToBlockData), SampleRate*4);
               ZeroMemory((PVOID)((PBYTE)DataBlocksCh2+OffsetToBlockData), SampleRate*4);
               ZeroMemory((PVOID)((PBYTE)DataBlocksCh3+OffsetToBlockData), SampleRate*4);
			   if (NumChannels==4) ZeroMemory((PVOID)((PBYTE)DataBlocksCh4+OffsetToBlockData), SampleRate*4);
            }

            // Inform parent thread that a bad block was received.
            PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_BADBLOCK, 0, 0);
         }


         // Return block nr for updated block.
         *SI->BlockNr=BlockNr;

         // Signal parent process that buffer is updated.
         PostThreadMessage(SI->ThreadID, TM_BUFFERUPDATED, 0, 0); 

         // Advance to the next block.
         BlockNr++; 
         if (BlockNr==NumBlocksPrChannel) BlockNr=0;

         // If fix quality has changed, inform parent process.
         if (FixStatus!=OldFixStatus) {
            OldFixStatus=FixStatus;
            if (FixStatus) PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_FIXOK, 0, 0);
            if (!FixStatus) PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_NOFIX, 0, 0);
         }

         // If data stream has just been restored,then inform parent thread.
         if (DataLost) {
            DataLost=FALSE;
            PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_DATAINPUTOK, 0, 0); 
         }

      } else {
         // Failed to read bytes.The digitizer has been
         // disconnected, or an other comm error has occured.
         if (DataLost==FALSE) {
            // Signal parent process that we have no data input.
            PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_NODATAINPUT, 0, 0); 
            DataLost=TRUE;
         }

         // Wait 50ms before trying a new read.
         if (!ReadStatus) Sleep(50);
      }
   }
   
   ExitThread(0);
   return 0L;
   
}


/*******************************************************************/
/*    Configuration dialog box procedure for SADC10                */
/*******************************************************************/
BOOL CALLBACK ConfigDlgProcSADC10(HWND hWndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
   SHINITDLGINFO shidi; // New in PocketPC version
   HWND hwndDlgItem;
   DWORD dwIndex;
   INT Result;
   static DEVICE_CONFIG *DC;
   TCHAR PortNr, Str20[21], Message[100];
   TCHAR Ports[16][6]={TEXT("COM1"),TEXT("COM2"), TEXT("COM3"), TEXT("COM4"), TEXT("COM5"), TEXT("COM6"), TEXT("COM7"), TEXT("COM8")
                    , TEXT("COM9"),TEXT("COM10"),TEXT("COM11"),TEXT("COM12"),TEXT("COM13"),TEXT("COM14"),TEXT("COM15"),TEXT("COM16")};

   switch(message)  {
      case WM_INITDIALOG:

      // Create a Done button and size it. New in PocketPC version.
 	  shidi.dwMask = SHIDIM_FLAGS;
	  shidi.dwFlags = SHIDIF_SIPDOWN | SHIDIF_SIZEDLGFULLSCREEN;
	  shidi.hDlg = hWndDlg;
	  SHInitDialog(&shidi);
      
	  // Create pointer to the device config structure.
      DC=(DEVICE_CONFIG*)lParam;

      // Initialize the 'Number of channels' combo box.
      hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO1);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCSTR) TEXT("1") );
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 1);
      if (DC->NumChannels==1) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCSTR) TEXT("3") );
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 3);
      if (DC->NumChannels==3) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCSTR) TEXT("4") );
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 4);
      if (DC->NumChannels==4) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);

      // Initialize the 'Device is connected to' combo box.
      hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO2);
      for (PortNr=0;PortNr<16;PortNr++) {
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCSTR)Ports[PortNr]);
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, PortNr);
         if (!wcscmp(DC->PortName, Ports[PortNr])) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }

      // Initialize the 'Comm speed' combo box.
      hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO3);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCSTR)TEXT("9600") );
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 9600);
      if (!wcscmp(DC->CommSpeed, TEXT("9600"))) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);

      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCSTR)TEXT("14400") );
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 14400);
      if (!wcscmp(DC->CommSpeed, TEXT("14400"))) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);


      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCSTR)TEXT("19200") );
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 19200);
      if (!wcscmp(DC->CommSpeed, TEXT("19200"))) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);

      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCSTR)TEXT("38400") );
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 38400);
      if (!wcscmp(DC->CommSpeed, TEXT("38400"))) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);


      // Initialize the 'sample rate' combo box.
      hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO4);

      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCSTR)TEXT("5") );
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 5);
      if (DC->SampleRate==5) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);

      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCSTR)TEXT("10") );
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 10);
      if (DC->SampleRate==10) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);

      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCSTR)TEXT("20") );
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 20);
      if (DC->SampleRate==20) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);

	  dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCSTR)TEXT("25") );
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 25);
      if (DC->SampleRate==25) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);

      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCSTR)TEXT("50") );
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 50);
      if (DC->SampleRate==50) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);

      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCSTR)TEXT("100") );
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 100);
      if (DC->SampleRate==100) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);

      // Initialize the 'time correction' edit box.
      swprintf(Str20, TEXT("%.3f"), DC->TimeCorrection);
      SetDlgItemText(hWndDlg, IDC_TIME_CORR, Str20);

      // Initialize the 'Syncronize system clock.' check box.
      CheckDlgButton(hWndDlg, IDC_CHECK1, BST_UNCHECKED); 
      if (DC->SyncPCClock) CheckDlgButton(hWndDlg, IDC_CHECK1, BST_CHECKED); 

      // Initialize the 'Use system clock for time tagging.' check box.
      CheckDlgButton(hWndDlg, IDC_CHECK2, BST_UNCHECKED); 
      if (DC->UsePCClock) CheckDlgButton(hWndDlg, IDC_CHECK2, BST_CHECKED); 
      
      // Bring window to front.
      //BringWindowToTop(hWndDlg);
	  SetForegroundWindow(hWndDlg);

      return TRUE;


      case WM_COMMAND:
      switch(wParam) {

		   case IDC_CHECK1:
         // This is the 'Syncronize system clock.' checkbox.
         switch (HIWORD(wParam)) { 
            case BN_CLICKED:
				// If checked, uncheck the 'Use system clock for time tagging.' check box.
            if (IsDlgButtonChecked(hWndDlg, IDC_CHECK1)==BST_CHECKED) 
					CheckDlgButton(hWndDlg, IDC_CHECK2, BST_UNCHECKED);
         }
         break;


			case IDC_CHECK2:
         // This is the 'Use system clock for time tagging.' checkbox.
         switch (HIWORD(wParam)) { 
            case BN_CLICKED:
				// If checked, uncheck the 'Syncronize system clock.' check box.
            if (IsDlgButtonChecked(hWndDlg, IDC_CHECK2)==BST_CHECKED) 
					CheckDlgButton(hWndDlg, IDC_CHECK1, BST_UNCHECKED); 
         }
         break;


         case IDOK:  // User has pressed the OK button.
         // Read current choice from the 'Number of channels' combo box.
         hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO1);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
		 DC->NumChannels=(BYTE)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

         // Read current choice from the 'Device is connected to' combo box.
         hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO2);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         SendMessage(hwndDlgItem, CB_GETLBTEXT, dwIndex, (LPARAM)&DC->PortName);

         // Read current choice from the 'Comm speed' combo box.
         hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO3);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         SendMessage(hwndDlgItem, CB_GETLBTEXT, dwIndex, (LPARAM)&DC->CommSpeed);
   
         // Read current choice from the sample rate combo box.
         hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO4);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         DC->SampleRate=(BYTE)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

         // Read current choices from the time-correction edit box.
         GetDlgItemText(hWndDlg, IDC_TIME_CORR, Str20, 20);
         Result=swscanf(Str20, TEXT("%Lf"), &DC->TimeCorrection);
         if (Result==0 || Result==EOF) {
            // A string that cannot be converted to a float value is entered.
            wcscpy(Message, TEXT("The time correction value is missing or invalid.\n"));
            MessageBox(hWndDlg, Message, TEXT("Seislog message"), MB_OK|MB_ICONERROR);

            // Move the input focus to this field.
            SetFocus(GetDlgItem(hWndDlg, IDC_TIME_CORR));
            SendDlgItemMessage(hWndDlg, IDC_TIME_CORR, EM_SETSEL, 0, -1); 
  
            // Return without closing.
            return TRUE;            
         }

         // Read current choices from the 'Syncronize system clock.' check box.
         DC->SyncPCClock=(IsDlgButtonChecked(hWndDlg, IDC_CHECK1)==BST_CHECKED); 

         // Read current choices from the 'Use system clock for time tagging.' check box.
         DC->UsePCClock=(IsDlgButtonChecked(hWndDlg, IDC_CHECK2)==BST_CHECKED); 
 
         // Close the dialog box
         EndDialog(hWndDlg , 1);
         return TRUE;
         
         
         case IDCANCEL:  // User has pressed the Cancel button.
         // Do nothing, just exit.
         EndDialog(hWndDlg , 0);
         return TRUE;      
      }
      return FALSE;

   }
   return FALSE;
}



/*******************************************************************/
/* Checks to see if the time packet is where it should be.         */
/* That is at the begining of the serial data.                     */
/*******************************************************************/

BOOL SADC10LocateTime(   STARTINFO_DEVREADER *SI,PBYTE Block, PBYTE Block_Time,DWORD Offset,  DWORD  &Min10)
{
   BOOL Ok;

   Ok = FALSE;       // Default
   Min10 = 0;       // Corretion for 10 minute data slip !!
 
  
   // Is the first block a valid time block?
   if (*(Block+ 0) == 0x81 ) {  
		 if (Offset > 4) 
		   if (*(Block + 5) > 0xFB )  Ok = TRUE;
   }
   else  { // Their is a stange effect that comes from the card every
	       // 10 minutes. It sends extra data info.
	       // The extra bytes are Byts per Data packet times NumChannels.
	       // This solution may work.


			   for (Min10=0;Min10 <  (Offset + 1);Min10++)
			     if (*(Block+ Min10) == 0x81 ) break;
			   
				 if (Min10 == Offset) {
					 Min10 = -1;                 // Overflow
					 // -2 = No Time Packet Deteteted
					 if (*(Block+ Min10) != 0x81 ) Min10 = -2; 
				 }

	           if (Min10 == 0) Min10 = -10;      // No flow

			   /*  // Used in Debugging.
			       TCHAR Message[100], StrInfo[84];
			       wcscpy(Message, TEXT("Slip:  "));
   	               swprintf(StrInfo,TEXT("%d ",(int)Min10);
		           wcscat(Message, StrInfo);
		           WriteSystemLog(Message, 3);
			    */
			
   }         // end of else

   return (Ok);
}


/**********************************************************************/
/* Finds the Next Valid Data Block and extracts data from it.         */
/* 0x81 = time, 0x82 = Channel 1, 0x83 = Channel 2, 0x84 = Channel 3  */
/* LIValue is the extracted data.                                     */
/* Returns (Type) 1 = Ch1 - > , 3 = Ch3. Returns a 8 if overflow and  */
/* 0 if not valid.                                                    */
/* Also if a time packet is detected it will return the values.       */
/* Also singals if a vaild data or time packet is found. (Card)       */
/* and if a part of a time packet is found.                           */
/* Some extra code on time packet recovery has been add for later     */
/* upgrades.                                                          */
/**********************************************************************/

DWORD SADC10Next(PBYTE Block, PBYTE Block_Time, DWORD Offset, DWORD BytesToRead, DWORD &NextOffset, LONG &LIValue)
{
    DWORD Type;
    short value;

    LIValue = 0;
	Type = 0;
	//
	// Stay within boundarys
	//
	if (Offset < 3)
	{
        NextOffset = Offset + 1;
		return(Type);
	}

	/* ------------------ */
    /* Packet information */
	/* ------------------ */
	//  Data Packet 
	//
	// *(Block+Offset - 3); // Byte data Marker
	//						// 0x81 = time, 0x82 = Channel 1 -> 0x85 = Channel 4
	// *(Block+Offset - 1); // Byte Data High
	// *(Block+Offset - 2); // Byte Data Low
	// *(Block+Offset);     // Extra Marker for High and Lower high order bits.
	//
	//  Time Packet 
	//
	// *(Block+Offset - 5); // Byte Data Marker
	// *(Block+Offset - 4); // Byte Second
	// *(Block+Offset - 3); // Byte Minutes
	// *(Block+Offset - 2); // Byte Hour
	// *(Block+Offset - 1); // DCF77 + Line 1 + Line 2
	// *(Block+Offset);     // End Marker


	// force an error if limits excceded
	if (*(Block+Offset - 3) > 0x85)
		*(Block+Offset - 3)  = 0x00; // Note Above Channel 4 masked out. Just in Case

    // Read Packet if Possible
	if (Offset < BytesToRead)
	{
	  if (*(Block+Offset) > 0xFB ) // End Marker 
	  {
		if ((*(Block+Offset - 3) > 0x81) && (Offset > 2)) 
		{  // Data Packets 0x82 -> 0x85. Channel 1 -> Channel 4
           value = (short)(*(Block+Offset - 1) | ((*(Block+Offset) & 0x02) << 6)); // High Byte
           value = value << 8;
           value = value + (short)(*(Block+Offset - 2) | ((*(Block+Offset) & 0x01) << 7)); 
           LIValue = (LONG) value;
		   NextOffset = Offset + 1; // Next Data Element
		   Type       = (DWORD) (*(Block+Offset - 3) - 0x81);
		   *(Block_Time + 9) = 0x00; // Reset Part time packet flag
		}
		else
		{  // Time Packet
		   if (Offset > 4)
		   { // Read Time Packet and check for boundarys. Watch for time slip in AVR clock
      		if (*(Block + Offset - 5) == 0x81)
			{    // Time Packet verified
		       NextOffset = Offset + 4;
			   Type = 0;
			   *(Block_Time + 0) = *(Block + Offset - 2); // hour
			   *(Block_Time + 1) = *(Block + Offset - 3); // Minutes
			   *(Block_Time + 2) = *(Block + Offset - 4); // Seconds
			   *(Block_Time + 3)  = (*(Block + Offset - 1) & 0x20 ) >> 5; // DCF77
			   *(Block_Time + 4)  = (*(Block + Offset - 1) & 0x08 ) >> 3; // Line 1
			   *(Block_Time + 5)  = (*(Block + Offset - 1) & 0x10 ) >> 4; // Line 2
			   *(Block_Time + 6)  = 0x80; // Time Packet Detected
			   *(Block_Time + 9)  = 0x00;  // Clear part read time packet flag.
			}  // End Verified
		   }   // Offset > 4
		   else
		   {    // Partial read Time Packet repair
		       NextOffset = Offset + 1;
			   Type = 0;

			   if (( Offset - 1) > -1 )  {
				   *(Block_Time + 3)  = (*(Block + Offset - 1) & 0x20 ) >> 5; // DCF77
				   *(Block_Time + 4)  = (*(Block + Offset - 1) & 0x08 ) >> 3; // Line 1
				   *(Block_Time + 5)  = (*(Block + Offset - 1) & 0x10 ) >> 4; // Line 2
				   *(Block_Time + 9)  = *(Block_Time + 9) & 0x0E; // Reset Line1 flag 1110
			   } 

			   if (( Offset - 2) > -1 )  {
				   *(Block_Time + 0) = *(Block + Offset - 2); // hour
				   *(Block_Time + 9) = *(Block_Time + 9) & 0x07; // Reset hour flag 0111
			   }
			   if (( Offset - 3) > -1 ) {
				   *(Block_Time + 1) = *(Block + Offset - 3); // Minutes
				   *(Block_Time + 9) = *(Block_Time + 9) & 0x0B;  // Reset Minutes Flag 1011
			   }
			   if (( Offset - 4) > -1 ) {
				   *(Block_Time + 2) = *(Block + Offset - 4); // Seconds
				   *(Block_Time + 9) = 0x00; // Reset Part time packet flag Reset ALL  0000
			       *(Block_Time + 6) = 0x80; // Time Packet Detected
			   }
		   }  // Offset < 4
		}  //  Time packet
	 }	
	 else 
	 {    // Between Data or Time (error)
		   Type       = 0;
		   NextOffset = Offset + 1;
          // Correct for partial read time Packet;
		   if (*(Block + Offset) == 0x81) 
		   {
			   if (( Offset + 1 ) < BytesToRead )  {
				   	NextOffset = Offset + 1;
				   *(Block_Time + 2) = *(Block + Offset + 1); // Second				   
     			   *(Block_Time + 9) = 0x0D; // Set above Seconds Flag 1101
			   }

			   if (( Offset + 2 ) < BytesToRead )  {
		           NextOffset = Offset + 2;
				   *(Block_Time + 1) = *(Block + Offset + 2); // Minutes
				   *(Block_Time + 9) = 0x09; // Set Above Minutes Flag 1001
			   }
			   if (( Offset + 3 ) < BytesToRead )  {
		           NextOffset = Offset + 3;
				   *(Block_Time + 0) = *(Block + Offset + 3);// Hour
				   *(Block_Time + 9) = 0x01; // Set Above Hours Flag 0001
			   }

			   if (( Offset + 4) < BytesToRead )  {
				   NextOffset = Offset + 4;
				   *(Block_Time + 3)  = (*(Block + Offset + 4) & 0x20 ) >> 5; // DCF77
				   *(Block_Time + 4)  = (*(Block + Offset + 4) & 0x08 ) >> 3; // Line 1
				   *(Block_Time + 5)  = (*(Block + Offset + 4) & 0x10 ) >> 4; // Line 2
				   *(Block_Time + 9)  = 0x00; // Reset Time Packet flag
			       *(Block_Time + 6)  = 0x80; // Time Packet Detected
			   } 
		   }
	 }
	}
    else 	
	{
		Type = 8;  // Overflow Condition
		NextOffset = Offset;
	}
	

    return (Type);
}
   

/**********************************************************************/
/* Setup's the SADC10 card and sends messages to the User             */
/**********************************************************************/

void SADC10Startup( STARTINFO_DEVREADER *SI, WORD  NumChannels, WORD  SampleRate , int speed, int  &Version )

{
   int i,j;
   BYTE setup[32];   // Holds data to setup The A/D card (Serial to card )
   BYTE Hold[64];    // Hold Initilization string for A/D Card
   TCHAR Message[100], StrInfo[64];
   BOOL WriteStatus,ReadStatus;
   OVERLAPPED osWrite = {0};
   DWORD BytesWrite, BytesRead;
   SYSTEMTIME SysTime;
   long tm;
   struct tm *nt;
   
   //
   /* ---------------------------------------------------------- */
   /* Send Messages to the SADC10 card via the Seial Comm's Port */
   /* ---------------------------------------------------------- */
   //
      WriteSystemLog(TEXT("SADC10 Driver"), 3); // Color Red

   // 
   //
   // Write to A/D the Speed of the channels in this case TURN OFF
   //
   for ( i = 0; i < 60; i++) Hold[i] = 0x00;

	   Hold[0] = 0x44; // 68 = D what does this mean?
	   Hold[1] = 0x55; // 85 = U
	   Hold[2] = 0x4D; // 77 = M
	   Hold[3] = 0x4D; // 77 = M
	   Hold[4] = 0x59; // 89 = y as above
	   Hold[5] = 0x84; // Control byte for speed

   //	  
   // Send: Stop sending data from the card
   //
   for ( i = 0; i < 11; i++)
   {
      setup[0] = Hold[i];     // Control Data 
      WriteStatus=WriteFile(SI->hComm, &setup[0], 1, &BytesWrite, &osWrite);
	  Sleep(6);
	  if (WriteStatus == false) WriteSystemLog(TEXT("Bad Write Status To Card"), 1);
   }
 
   Sleep(50); // wait 50 millisecond

   //
   // Send a request to the SADC10 card for a version number
   //
   for ( i = 40; i < 60; i++) Hold[i] = 0x00;
   Hold[40] = 0x81; // Request Version number
   PurgeComm(SI->hComm, PURGE_RXCLEAR);      // Clear the input buffer.
   //
   // Send: The requset 0x81 0x00 0x00 0x00 0x00
   //
   for ( i = 40; i < 46; i++)
   {
      setup[0] = Hold[i];     // Control Data 
      WriteStatus=WriteFile(SI->hComm, &setup[0], 1, &BytesWrite, &osWrite);
	  Sleep(20);
	  if (WriteStatus == false) WriteSystemLog(TEXT("Bad Write Status To Card"), 1);
   }
   Sleep(150); // Wait a bit before reading the serial data

   // Try reading 15 Bytes into the buffer.
   ReadStatus=ReadFile(SI->hComm, &setup[0], 15 , &BytesRead, NULL);		
   //
   //
   //
   Version = 0; // Default
   //
   // Look For "V" is Serial data
   //
    for ( i=0,j=-1;i< (int) BytesRead;i++)  if(setup[i] == 'V') j = i;
	  if  (j > -1) // make version number
	    for (i=1;i<4;i++) Version += ((int)(setup[i+j])- 48)* (int)(pow( 10,3-i));
   //
   // Send Version Message
   //

   if (Version >= 140)
   {
     wcscpy(Message, TEXT("SADC10 Version: "));
     swprintf(StrInfo,TEXT("%.*f "),2,(double)Version/100);
     wcscat(Message, StrInfo);
     WriteSystemLog(Message, 2); // Color Blue
   }
   else WriteSystemLog(TEXT("SADC10 Version MUST be Higher or Equal to,  1.4"), 3);
   Sleep(50);
 
   //
   // Sync the SADC10 card with the PC time.
   // It Sends GMT Time.
   //
   if (Version >= 140) WriteSystemLog(TEXT("Sending PC Time to SADC10"), 1);
   //
   // Write time and GMT info to SADC10
   //
   // Get PC System Time
   //
   //time(&tm); 
   //nt    = gmtime(&tm);
   GetSystemTime(&SysTime);
   //Hold[48] = (byte)nt->tm_sec;
   Hold[48] = (byte) SysTime.wSecond;
   //
   // Synch The SADC10 card With PC time. Wait For chang Of Second
   //
   do
   { 
	   //time(&tm); 
       //nt    = gmtime(&tm);
	   GetSystemTime(&SysTime);
   } while (Hold[48] == (byte) SysTime.wSecond);
   //
   // Send Time to SADC10 Card
   //
    for ( i = 20; i < 36; i++) Hold[i] = 0x00;
       //
	   // Get PC time in GMT 
	   //
       Hold[20] = 0x83;     // Control Byte for time
      // Hold[21] = (byte)nt->tm_sec;
      // Hold[22] = (byte)nt->tm_min;
      // Hold[23] = (byte)nt->tm_hour;
	   Hold[21] = (byte) SysTime.wSecond;
       Hold[22] = (byte) SysTime.wMinute;
       Hold[23] = (byte) SysTime.wHour;
	   // [24],[25] are 0x00
       Hold[26] = 0x82;     // Control Byte for GMT
       // [27] -> [31] ar 0x00  // GMT Time Used
       for ( i = 20; i < 32; i++)
	   {
          setup[0] = Hold[i];     // Control Data 
          WriteStatus=WriteFile(SI->hComm, &setup[0], 1, &BytesWrite, &osWrite);		
		  Sleep(6);
	      if (WriteStatus == false) WriteSystemLog(TEXT("Bad Write To SADC10 on Time "), 1);
	   }
	   Sleep(2);
	   // 
  	   // Determine SPS for the SADC10 Card and Set Cards SPS rate.
	   // Note: Channel 4 of the SADC10 card is not used in this driver.
       //
	   Hold[6] = (BYTE) 100 / SampleRate; // Determine sample rate Required

      if ((NumChannels == 3) || (NumChannels == 4))
      {
	    // All Channel SPS rates are the same
        Hold[7] = Hold[6]; 
        Hold[8] = Hold[6];   
		if (NumChannels == 4) Hold[9] = Hold[6];
      }
     //
     // Send: SPS info to the SADC10 Card
     //
     //
     // Send Data for Sample rate and this turn on the card.
     //
	 for ( i = 0; i < 11; i++)
     {
        setup[0] = Hold[i];     // Control Data 
		if (i == 4)  PurgeComm(SI->hComm, PURGE_RXCLEAR);      // Clear the input buffer.
        WriteStatus=WriteFile(SI->hComm, &setup[0], 1, &BytesWrite, &osWrite);		
		Sleep(1);
	    if (WriteStatus == false) WriteSystemLog(TEXT("Bad Write Status To Card"), 1);
      }

     //
     // Write Sample Rate information
     //
	 if (Version >= 140)  {
		  wcscpy(Message, TEXT("SADC10:  "));
			if (NumChannels > 1) swprintf(StrInfo,TEXT("%d Samples per Second For %d Channels"),SampleRate,NumChannels);
			else 		         swprintf(StrInfo,TEXT("%d Samples per Second For %d Channel"),SampleRate,NumChannels);
		  wcscat(Message, StrInfo);
		  WriteSystemLog(Message, 2); // Color Blue
          WriteSystemLog(TEXT("Please Wait.. Gathering data."),1);
	   
		   if (((NumChannels *4)*SampleRate )+6 > speed)
			{
				wcscpy(Message, TEXT("SADC10:   Sample Rate to High for the SADC10 cards baud Rate. Reduce Sample Rate"));
				WriteSystemLog(Message, 3); // Color Blue
			}
	 }
 }

/**********************************************************************/
/* Write Line2 change of State Levels                                 */
/**********************************************************************/

void SADC10Line2(PBYTE Block_Time, BOOL &Lines)

{
	  // Detect and Display the Status bits from the SADC10 card. Line 2 is displayed
	  // A valid time Packet or Data Packet must have been detected before this display is sent.
	  // This is sent out only at the start of the program. Once.
	  //
	  if (( Lines == TRUE) && ( *(Block_Time + 6) > 0x00))  {
		  
		 *(Block_Time + 8) = 0xFF; // force later write
         WriteSystemLog(TEXT("SADC10 : Line 2 state is displayed in the Log."), 1);
         WriteSystemLog(TEXT("SADC10 : Line 2 is the Third pin on the SADC10 Card. Grounding this pin will lower its state"), 1);
 	     Lines = FALSE;  // Dont Display this bloc again
	  }

      //
      //  Display Line 2 State if a change is detected. This is called every second.
	  // *(Block_Time + 5) = Line 2 state
	  // *(Block_Time + 8) = Line 2 Hold state
	  //  This is active all the time.
	  //
	  if (*(Block_Time + 8) != *(Block_Time + 5))
	  {
	     if (*(Block_Time + 5) == 0x01) WriteSystemLog(TEXT("SADC10 : Line 2 High."), 2);
		 else                           WriteSystemLog(TEXT("SADC10 : Line 2 Low"), 2);
		 *(Block_Time + 8) = *(Block_Time + 5);
	  }
}